@@ -65,6 +65,7 @@ struct cube_reader
   gchar        *comment;
   double        box[6];
   float         shift[3];
+  float         rot[3][3];
   int           ntype, natom;
   GHashTable   *elements;
   VisuElement **nodeTypes;
@@ -215,6 +216,7 @@ static gboolean read_box(struct cube_reader *rd, GError **error)
 {
   int i;
   double cart[3][3];
+  float boxMat[3][3], fileMat[3][3], fileMatInv[3][3];
   gchar dir[3] = "XYZ";
 
   /* Read the box definition. */
@@ -258,13 +260,46 @@ static gboolean read_box(struct cube_reader *rd, GError **error)
 			   _("Wrong Cube format, primitive vectors are not 3D.\n"));
       return FALSE;
     }
+  /* We create the rotation matrix that pass from the cartesian
+     coordinates of the file to the cartesian coordinates of the box. */
+  boxMat[0][0] = rd->box[0];
+  boxMat[0][1] = rd->box[1];
+  boxMat[0][2] = rd->box[3];
+  boxMat[1][0] = 0.f;
+  boxMat[1][1] = rd->box[2];
+  boxMat[1][2] = rd->box[4];
+  boxMat[2][0] = 0.f;
+  boxMat[2][1] = 0.f;
+  boxMat[2][2] = rd->box[5];
+  fileMat[0][0] = (float)cart[0][0];
+  fileMat[0][1] = (float)cart[1][0];
+  fileMat[0][2] = (float)cart[2][0];
+  fileMat[1][0] = (float)cart[0][1];
+  fileMat[1][1] = (float)cart[1][1];
+  fileMat[1][2] = (float)cart[2][1];
+  fileMat[2][0] = (float)cart[0][2];
+  fileMat[2][1] = (float)cart[1][2];
+  fileMat[2][2] = (float)cart[2][2];
+  if (!matrix_invert(fileMatInv, fileMat))
+    {
+      *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
+			   _("Wrong Cube format, cannot invert primitive vectors.\n"));
+      return FALSE;
+    }
+  matrix_productMatrix(rd->rot, boxMat, fileMatInv);
+  DBG_fprintf(stderr, "Cube: rotation matrix %8g %8g %8g\n"
+	      "                      %8g %8g %8g\n"
+	      "                      %8g %8g %8g\n",
+	      rd->rot[0][0], rd->rot[0][1], rd->rot[0][2],
+	      rd->rot[1][0], rd->rot[1][1], rd->rot[1][2],
+	      rd->rot[2][0], rd->rot[2][1], rd->rot[2][2]);
   return TRUE;
 }
 
 static gboolean read_coords(struct cube_reader *rd, GError **error)
 {
   int i, zele, nb;
-  float rcov, trash;
+  float rcov, trash, coord[3];
   gchar *name, *ptChar;
   VisuElement *type;
   struct dataAscii *infos;
@@ -286,17 +321,17 @@ static gboolean read_coords(struct cube_reader *rd, GError **error)
 	  return FALSE;
 	}
       nb = sscanf(rd->line->str, "%d %f %f %f %f\n", &zele, &trash,
-		  rd->coords + 3 * i, rd->coords + 3 * i + 1,
-		  rd->coords + 3 * i + 2);
+		  coord, coord + 1, coord + 2);
       if (nb != 5)
 	{
 	  *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
 			       _("Wrong Cube format, can't read coordinates.\n"));
 	  return FALSE;
 	}
-      rd->coords[3 * i + 0] -= rd->shift[0];
-      rd->coords[3 * i + 1] -= rd->shift[1];
-      rd->coords[3 * i + 2] -= rd->shift[2];
+      coord[0] -= rd->shift[0];
+      coord[1] -= rd->shift[1];
+      coord[2] -= rd->shift[2];
+      matrix_productVector(rd->coords + 3 * i, rd->rot, coord);
       /* Try to find a z number instead of a name. */
       rcov = -1.f;
       toolElementsGet_element(&ptChar, &rcov, zele);
