@@ -54,6 +54,7 @@
 #include <visu_tools.h>
 
 static Display *dpy = (Display*)0;
+static gboolean Xerror = FALSE;
 
 struct DumpImage_struct
 {
@@ -65,6 +66,8 @@ struct DumpImage_struct
 
 int handler(Display *dpy _U_, XErrorEvent *error _U_)
 {
+  DBG_fprintf(stderr, "Visu GLX: ouch one X error.\n");
+  Xerror = TRUE;
   return 0;
 }
 
@@ -102,7 +105,7 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
   int screenId, res;
   XVisualInfo *visualInfo;
 
-  DBG_fprintf(stderr, "Visu GLX : creating a off-screen buffer (%dx%d).\n", width, height);
+  DBG_fprintf(stderr, "Visu GLX: creating a off-screen buffer (%dx%d).\n", width, height);
   image = g_malloc(sizeof(DumpImage));
   image->pixmap = (Pixmap)0;
   image->glxPixmap = (GLXPixmap)0;
@@ -117,7 +120,7 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
       g_free(image);
       return (DumpImage*)0;
     }
-  DBG_fprintf(stderr, "Visu GLX : display opened.\n");
+  DBG_fprintf(stderr, "Visu GLX: display opened.\n");
 
   screenId = DefaultScreen(dpy);
   visualInfo = visuOpenGLGet_visualInfo(dpy, screenId);
@@ -133,6 +136,7 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
       XFree(visualInfo);
       return (DumpImage*)0;
     }
+  DBG_fprintf(stderr, "Visu GLX: X pixmap created.\n");
 
   image->glxPixmap = glXCreateGLXPixmap(dpy, visualInfo, image->pixmap);
   if (!image->glxPixmap)
@@ -143,6 +147,7 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
       XFree(visualInfo);
       return (DumpImage*)0;
     }
+  DBG_fprintf(stderr, "Visu GLX: GLX pixmap created.\n");
 
   image->context = glXCreateContext(dpy, visualInfo, 0, GL_FALSE);
   if (!image->context)
@@ -154,10 +159,13 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
       XFree(visualInfo);
       return (DumpImage*)0;
     }
+  DBG_fprintf(stderr, "Visu GLX: new OpenGL context created.\n");
 
+  Xerror = FALSE;
   XSetErrorHandler(handler);
   res = glXMakeCurrent(dpy, image->glxPixmap, image->context);
-  if (!res)
+  XSetErrorHandler(NULL);
+  if (!res || Xerror)
     {
       g_warning("Cannot make current the pixmap context.");
       XFreePixmap(dpy, image->pixmap);
@@ -166,9 +174,12 @@ DumpImage* visuOpenGLNew_pixmapContext(guint width, guint height)
       XFree(visualInfo);
       return (DumpImage*)0;
     }
-  XSetErrorHandler(NULL);
+  DBG_fprintf(stderr, "Visu GLX: pixmap context is now current.\n");
 
   XFree(visualInfo);
+
+  glXWaitX();
+
   return image;
 }
 
@@ -178,30 +189,40 @@ void visuOpenGLFree_pixmapContext(DumpImage *dumpData)
 
   if (dpy)
     {
-      if (dumpData->pixmap)
-	XFreePixmap(dpy, dumpData->pixmap);
+      DBG_fprintf(stderr, "Visu GLX: free the GLX pixmap 0x%x.\n",
+		  (int)dumpData->glxPixmap);
       if (dumpData->glxPixmap)
 	glXDestroyGLXPixmap(dpy, dumpData->glxPixmap);
+
+      DBG_fprintf(stderr, "Visu GLX: free the X pixmap 0x%x.\n",
+		  (int)dumpData->pixmap);
+      if (dumpData->pixmap)
+	XFreePixmap(dpy, dumpData->pixmap);
+
+      DBG_fprintf(stderr, "Visu GLX: free the context.\n");
       if (dumpData->context)
 	glXDestroyContext(dpy, dumpData->context);
+
       /* We do NOT close the display since it is shared
 	 by all the application and thus dpy structure
 	 is unique and will be closed by the main program when quiting. */
       /*   XCloseDisplay(dumpData->dpy); */
+      glXWaitX();
     }
   g_free(dumpData);
 }
 
-GLuint visuOpenGLinit_fontList()
+GLuint visuOpenGLinit_fontList(guint size)
 {
-  static XFontStruct *fontInfo = (XFontStruct*)0;
+  XFontStruct *fontInfo = (XFontStruct*)0;
   unsigned int first, last; 
-  char name[] =  
-    "-adobe-helvetica-medium-r-normal--18-180-75-75-p-98-iso8859-1"; 
+  char name[256];
   GLuint BASE;
   static Display *dpy = (Display*)0;
+
+  sprintf(name, "-adobe-helvetica-medium-r-normal-*-%d-*-*-*-p-*-iso8859-1", size); 
         
-  DBG_fprintf(stderr, "Visu GLX : initialise fonts.\n");
+  DBG_fprintf(stderr, "Visu GLX: initialise fonts (%s).\n", name);
   if (!fontInfo)
     {
       if (!dpy)
@@ -215,15 +236,20 @@ GLuint visuOpenGLinit_fontList()
 	      g_error("Fixed font not available.\n"); 
 	    }
 	}
+      DBG_fprintf(stderr, " | Found XFontStruct.\n");
     }
-
   first = fontInfo->min_char_or_byte2; 
   last = fontInfo->max_char_or_byte2; 
+  DBG_fprintf(stderr, " | XFontStruct OK (%d %d).\n", first, last);
  
   /* this creates display lists 1 to 256 (not checked) */ 
-  if ( (BASE = glGenLists(last+1)) == 0 )
+  BASE = glGenLists(last+1);
+  if (!BASE)
     return 0;
     
+  DBG_fprintf(stderr, " | list id %d\n", BASE);
   glXUseXFont(fontInfo->fid, first, last-first+1, BASE+first); 
+
+/*   XFreeFont(dpy, fontInfo); */
   return BASE; 
 }
