@@ -74,10 +74,19 @@ GList *color_storageArray;
 /******************/
 /* Storing colors */
 /******************/
-void initColorStorage()
+void toolColorInit_storage()
 {
   color_storageArray = (GList*)0;
 }
+void toolColorFree_storage()
+{
+  GList *lst;
+  
+  for (lst = color_storageArray; lst; lst = g_list_next(lst))
+    g_free(lst->data);
+  if (lst)
+    g_list_free(lst);
+}
 Color* colorNew_floatRGBA(float rgba[4])
 {
   Color *color;
@@ -174,7 +183,7 @@ Color* colorAdd_color(Color* color)
       color_storageArray = g_list_append(color_storageArray, (gpointer)color);
       DBG_fprintf(stderr, "Visu Tools : adding a new color at %p (%d).\n",
 		  (gpointer)color_storageArray, g_list_length(color_storageArray));
-      g_signal_emit(VISU_INSTANCE, VISU_GET_CLASS(VISU_INSTANCE)->colorNewAvailable_signal_id,
+      g_signal_emit(VISU_INSTANCE, VISU_SIGNALS[COLORNEWAVAILABLE_SIGNAL],
 		    0 , (gpointer)color, NULL);
     }
   return color;
@@ -200,7 +209,7 @@ Color* colorAdd_floatRGBA(float rgba[4], int *position)
     *position = g_list_length(color_storageArray);
   DBG_fprintf(stderr, "Visu Tools : adding a new color at %p (%d).\n",
 	      (gpointer)color_storageArray, g_list_length(color_storageArray));
-  g_signal_emit(VISU_INSTANCE, VISU_GET_CLASS(VISU_INSTANCE)->colorNewAvailable_signal_id,
+  g_signal_emit(VISU_INSTANCE, VISU_SIGNALS[COLORNEWAVAILABLE_SIGNAL],
 		0 , (gpointer)color, NULL);
   return color;
 }
@@ -226,7 +235,7 @@ Color* colorAdd_intRGBA(int rgba[4])
   color_storageArray = g_list_append(color_storageArray, (gpointer)color);
   DBG_fprintf(stderr, "Visu Tools : adding a new color at %p (%d).\n",
 	      (gpointer)color_storageArray, g_list_length(color_storageArray));
-  g_signal_emit(VISU_INSTANCE, VISU_GET_CLASS(VISU_INSTANCE)->colorNewAvailable_signal_id,
+  g_signal_emit(VISU_INSTANCE, VISU_SIGNALS[COLORNEWAVAILABLE_SIGNAL],
 		0 , (gpointer)color, NULL);
   return color;
 }
@@ -334,3 +343,43 @@ void color_HSLtoRGB(float *rgb, float *hsl)
       rgb[2] = Hue_2_RGB( var_1, var_2, hsl[0] - ( 1. / 3. ) );
     }  
 }
+
+void color_RGBtoHSL(float *hsl, float *rgb)
+{
+  float var_Min, var_Max, del_Max, del_R, del_G, del_B;
+
+  var_Min = MIN(MIN( rgb[0], rgb[1]), rgb[2] );
+  var_Max = MAX(MAX( rgb[0], rgb[1]), rgb[2] );
+  del_Max = var_Max - var_Min;
+  
+  hsl[2] = ( var_Max + var_Min ) / 2;
+
+  if ( del_Max == 0 )
+    {
+      hsl[0] = 0.f;
+      hsl[1] = 0.f;
+    }
+  else
+    {
+      if ( hsl[2] < 0.5f )
+	hsl[1] = del_Max / ( var_Max + var_Min );
+      else
+	hsl[1] = del_Max / ( 2.f - var_Max - var_Min );
+
+      del_R = ( ( ( var_Max - rgb[0] ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
+      del_G = ( ( ( var_Max - rgb[1] ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
+      del_B = ( ( ( var_Max - rgb[2] ) / 6 ) + ( del_Max / 2 ) ) / del_Max;
+
+      if ( rgb[0] == var_Max )
+	hsl[0] = del_B - del_G;
+      else if ( rgb[1] == var_Max )
+	hsl[0] = ( 1.f / 3.f ) + del_R - del_B;
+      else if ( rgb[2] == var_Max )
+	hsl[0] = ( 2.f / 3.f ) + del_G - del_R;
+ 
+      if ( hsl[0] < 0.f )
+	hsl[0] += 1;
+      if ( hsl[0] > 1.f )
+	hsl[0] -= 1;
+    }
+}
