@@ -111,7 +111,8 @@ typedef struct _Color
  * This method must be called once at the beginning of the program, if
  * you want to use the color storage system.
  */
-void initColorStorage();
+void toolColorInit_storage();
+void toolColorFree_storage();
 /**
  * colorNew_floatRGBA:
  * @rgba: four values between 0. and 1. that represent [Red, Green, Blue, Alpha].
@@ -237,6 +238,7 @@ void color_HSVtoRGB(float* rgb, float* hsv);
  * This methods convert a HSL color to a RGB one.
  */
 void color_HSLtoRGB(float *rgb, float *hsl);
+void color_RGBtoHSL(float *hsl, float *rgb);
 
 
 G_END_DECLS
