@@ -104,8 +104,6 @@ typedef enum
     FILEFORMAT_ERROR_UNKNOWN_FORMAT
   } FileFormatErrorFlag;
 
-void fileFormatInit();
-
 /**
  * fileFormatNew:
  * @descr: a short string to label a new file format.
@@ -175,6 +173,8 @@ typedef enum
     fileFormat_type_string,
     fileFormat_nb_type
   } FileFormatPropertyTypeId;
+FileFormatProperty* fileFormatGet_propertyByName(FileFormat *format,
+						 const gchar *name);
 /**
  * fileFormatGet_propertiesList:
  * @format: a #FileFormat.
