@@ -75,14 +75,14 @@ typedef struct Option_stuct Option;
  * Returns: a newly created table. Use toolOptionsFree_table() to destroy it.
  */
 #define toolOptionsNew_table() g_hash_table_new_full(g_str_hash, g_str_equal, NULL, \
-						     toolOptionsFree_option);
+						     toolOptionsFree_option)
 /**
  * toolOptionsFree_table:
  * @table: the table to free.
  * 
  * Free all memory used by a table of options, and free also its elements.
  */
-#define toolOptionsFree_table(table) g_hash_table_destroy((GHashTable*)table);
+#define toolOptionsFree_table(table) g_hash_table_destroy((GHashTable*)table)
 
 /**
  * toolOptionsGet_optionFromTable:
@@ -93,7 +93,7 @@ typedef struct Option_stuct Option;
  * it returns NULL.
  */
 #define toolOptionsGet_optionFromTable(table, option) g_hash_table_lookup((GHashTable*)table, \
-									  (gchar*)option);
+									  (gchar*)option)
 /**
  * toolOptionsAdd_optionToTable:
  * @table: the table to add the option to ;
@@ -103,7 +103,7 @@ typedef struct Option_stuct Option;
  */
 #define toolOptionsAdd_optionToTable(table, option) \
   g_hash_table_insert((GHashTable*)table, (gpointer)toolOptionsGet_name(option), \
-		      (gpointer)option);
+		      (gpointer)option)
 
 /**
  * OptionTypes:
