@@ -57,6 +57,17 @@
 
 #include <glib.h>
 
+/**
+ * SECTION:dumpToPsAndPdf
+ * @short_description: add an export capability into PS and PDF files
+ * (encapsulating bitmaps).
+ *
+ * This provides a write routine to export V_Sim views into bitmaps
+ * encapsulated into PS or PDF files. The PostScript file can be
+ * either in 256 colours or in full True Colour. For a vectorial
+ * export, see #dumpToSVG.
+ */
+
 static void PrintByte(unsigned long lcode);
 static void OutputCode(int code);
 
@@ -72,22 +83,22 @@ static gpointer waitData;
 static voidDataFunc waitFunc;
 
 
-int writeViewInPsFormat(FileFormat *format, GString *buffer,
-			char* filename, int width, int height,
-			VisuData *dataObj, guchar* imageData,
-			voidDataFunc functionWait, gpointer data);
-int writeViewInPs256Format(GString *buffer, char* filename,
-			   int width, int height,
-			   VisuData *dataObj, guchar* imageData,
-			   voidDataFunc functionWait, gpointer data);
-int writeViewInPdfFormat(FileFormat *format, GString *buffer,
-			 char* filename, int width, int height,
-			 VisuData *dataObj, guchar* imageData,
-			 voidDataFunc functionWait, gpointer data);
-int writeViewInPsTrueColorFormat(GString *buffer, char* filename,
-				 int width, int height,
-				 VisuData *dataObj, guchar* imageData,
+static gboolean writeViewInPsFormat(FileFormat *format, const char* filename,
+				    int width, int height, VisuData *dataObj,
+				    guchar* imageData, GError **error,
+				    voidDataFunc functionWait, gpointer data);
+static gboolean writeViewInPs256Format(const char* filename,
+				       int width, int height, VisuData *dataObj,
+				       guchar* imageData, GError **error,
+				       voidDataFunc functionWait, gpointer data);
+static gboolean writePsTrueColor(const char* filename,
+				 int width, int height, VisuData *dataObj,
+				 guchar* imageData, GError **error,
 				 voidDataFunc functionWait, gpointer data);
+static gboolean writeViewInPdfFormat(FileFormat *format, const char* filename,
+				     int width, int height, VisuData *dataObj,
+				     guchar* imageData, GError **error,
+				     voidDataFunc functionWait, gpointer data);
 
 DumpType* dumpToPS_init()
 {
@@ -96,17 +107,15 @@ DumpType* dumpToPS_init()
   #define descrPS _("Postscript (v3.0) file")
   FileFormat* fmt;
 
-  ps = malloc(sizeof(DumpType));
-  if (!ps)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  ps = g_malloc(sizeof(DumpType));
   fmt = fileFormatNew(descrPS, typePS);
   if (!fmt)
-    g_error("Can't initialize the PS dump module, aborting.\n");
+    {
+      g_error("Can't initialize the PS dump module, aborting.\n");
+    }
 
   ps->bitmap = TRUE;
+  ps->hasAlpha = FALSE;
   ps->fileType = fmt;
   ps->writeFunc = writeViewInPsFormat;
 
@@ -125,17 +134,15 @@ DumpType* dumpToPDF_init()
 #define descrPDF _("PDF file (v. 1.2)")
   FileFormat* fmt;
 
-  pdf = malloc(sizeof(DumpType));
-  if (!pdf)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  pdf = g_malloc(sizeof(DumpType));
   fmt = fileFormatNew(descrPDF, typePDF);
   if (!fmt)
-    g_error("Can't initialize the PS dump module, aborting.\n");
+    {
+      g_error("Can't initialize the PS dump module, aborting.\n");
+    }
 
   pdf->bitmap = TRUE;
+  pdf->hasAlpha = FALSE;
   pdf->fileType = fmt;
   pdf->writeFunc = writeViewInPdfFormat;
   
@@ -340,10 +347,10 @@ static void OutputDataPS256() {
 
 /******************************************************************************/
 
-int writeViewInPsFormat(FileFormat *format, GString *buffer,
-			char* filename, int width, int height,
-			VisuData *dataObj, guchar* imageData,
-			voidDataFunc functionWait, gpointer data)
+static gboolean writeViewInPsFormat(FileFormat *format, const char* filename,
+				    int width, int height, VisuData *dataObj,
+				    guchar* imageData, GError **error,
+				    voidDataFunc functionWait, gpointer data)
 {
   GList *prop;
   gchar* name;
@@ -355,51 +362,47 @@ int writeViewInPsFormat(FileFormat *format, GString *buffer,
       if (strcmp(name, "reduced_colormap") == 0)
 	{
 	  if (fileFormatGet_propertyBoolean((FileFormatProperty*)(prop->data)))
-	    return writeViewInPs256Format(buffer, filename, width,
-					  height, dataObj, imageData, functionWait, data);
+	    return writeViewInPs256Format(filename, width, height, dataObj,
+					  imageData, error, functionWait, data);
 	  else
-	    return writeViewInPsTrueColorFormat(buffer, filename, width,
-						height, dataObj, imageData, functionWait, data);
+	    return writePsTrueColor(filename, width, height, dataObj,
+				    imageData, error, functionWait, data);
 	}
       prop = g_list_next(prop);
     }
-  return writeViewInPsTrueColorFormat(buffer, filename, width,
-				      height, dataObj, imageData, functionWait, data);
+  return writePsTrueColor(filename, width, height, dataObj,
+			  imageData, error, functionWait, data);
 }
 
 /******************************************************************************/
 
-int writeViewInPsTrueColorFormat(GString *buffer,
-				 char* filename, int width, int height,
-				 VisuData *dataObj, guchar* imageData,
-				 voidDataFunc functionWait, gpointer data)
+static gboolean writePsTrueColor(const char* filename,
+				 int width, int height, VisuData *dataObj _U_,
+				 guchar* imageData, GError **error,
+				 voidDataFunc functionWait _U_, gpointer data _U_)
 {
   static int x = 10, y = 10;
   time_t timer;
   float facx, facy, fac;
   int wu, hu;
          
-  image = imageData;
-  if(!image)
-    {
-      g_string_append(buffer, _("Internal dumpImageToPS error :\n"
-				"image is not allocated"));
-      return 1;
-    }
+  g_return_val_if_fail(error && !*error, FALSE);
+  g_return_val_if_fail(imageData, FALSE);
+
+  image    = imageData;
+  PSwidth  = width;
+  PSheight = height;
 
   DBG_fprintf(stderr, "Dump PS & PDF : begin PS True Color export in %dx%d : %s.\n",
 	      width, height, filename);
-  PSwidth = width;
-  PSheight = height;
 
   out = fopen(filename, "w");
   if(!out)
     {
-      g_string_append(buffer, _("Cannot open file (to write in)\n"));
-      free(image);
-      return 1;
-    }
-                  
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_FILE,
+			   _("Cannot open file (to write in)."));
+      return FALSE;
+    }                  
    
    facx = (552.0f - x - 5)/PSwidth;
    facy = (796.0f - y - 5)/PSheight;
@@ -467,15 +470,15 @@ int writeViewInPsTrueColorFormat(GString *buffer,
    (void)fprintf(out, "%%%%EOF\n");
     
    (void)fclose(out);
-  return 0;
+  return TRUE;
 }
 
 /******************************************************************************/
 
-int writeViewInPdfFormat(FileFormat *format, GString *buffer,
-			 char* filename, int width, int height,
-			 VisuData *dataObj, guchar* imageData,
-			 voidDataFunc functionWait, gpointer data)
+static gboolean writeViewInPdfFormat(FileFormat *format _U_, const char* filename,
+				     int width, int height, VisuData *dataObj _U_,
+				     guchar* imageData, GError **error,
+				     voidDataFunc functionWait _U_, gpointer data _U_)
 {
   time_t timer;
    
@@ -484,25 +487,22 @@ int writeViewInPdfFormat(FileFormat *format, GString *buffer,
   long int sz[10];
   int i;    
          
-  image = imageData;
-  if(!image)
-    {
-      g_string_append(buffer, _("Internal dumpImageToPS error :\n"
-				"image is not allocated"));
-      return 1;
-    }
+  g_return_val_if_fail(error && !*error, FALSE);
+  g_return_val_if_fail(imageData, FALSE);
+
+  image    = imageData;
+  PSwidth  = width;
+  PSheight = height;
 
   DBG_fprintf(stderr, "Dump PS & PDF : begin PDF export in %dx%d : %s.\n",
 	      width, height, filename);
-  PSwidth = width;
-  PSheight = height;
 
   out = fopen(filename, "w");
   if(!out)
     {
-      g_string_append(buffer, _("Cannot open file (to write in)\n"));
-      free(image);
-      return 1;
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_FILE,
+			   _("Cannot open file (to write in)."));
+      return FALSE;
     }
                   
   timer = time((time_t *) NULL);
@@ -597,15 +597,15 @@ int writeViewInPdfFormat(FileFormat *format, GString *buffer,
   (void)fprintf(out, "%%%%EOF\n");
     
   (void)fclose(out);
-  return 0;
+  return TRUE;
 }
 
 /******************************************************************************/
 
-int writeViewInPs256Format(GString *buffer,
-			   char* filename, int width, int height,
-			   VisuData *dataObj, guchar* imageData,
-			   voidDataFunc functionWait, gpointer data)
+static gboolean writeViewInPs256Format(const char* filename,
+				       int width, int height, VisuData *dataObj _U_,
+				       guchar* imageData, GError **error,
+				       voidDataFunc functionWait, gpointer data)
 {
   static int x = 10, y = 10;
   time_t timer;
@@ -615,31 +615,23 @@ int writeViewInPs256Format(GString *buffer,
   register ColorPacket *q;
   register unsigned char *p;
          
-  image = imageData;
-  if(!image)
-    {
-      g_string_append(buffer, _("Internal dumpImageToPS error :\n"
-				"image is not allocated"));
-      return 1;
-    }
+  g_return_val_if_fail(error && !*error, FALSE);
+  g_return_val_if_fail(imageData, FALSE);
+
+  image    = imageData;
+  PSwidth  = width;
+  PSheight = height;
 
   DBG_fprintf(stderr, "Dump PS & PDF : begin PS 256c. export in %dx%d : %s.\n",
 	      width, height, filename);
-  PSwidth = width;
-  PSheight = height;
 
   img=g_malloc(sizeof(Image));
+  img->colormap = (ColorPacket*)0;
 
   img->columns = PSwidth;
   img->rows = PSheight;
   img->packets=img->columns*img->rows;
-  img->pixels=(ColorPacket *)
-    malloc((unsigned int) img->packets*sizeof(ColorPacket));
-  if (!img->pixels)
-    {
-      g_string_append(buffer, _("Memory allocation error for img->packets\n"));
-      return 1;
-    }
+  img->pixels=g_malloc(img->packets*sizeof(ColorPacket));
   q=img->pixels;
   p=image;
   for (i=0; i < img->packets; i++) {
@@ -651,16 +643,25 @@ int writeViewInPs256Format(GString *buffer,
   }
   dumpToGif_setImage(img);
   
-  if(dumpToGif_quantizeImage(256, buffer, functionWait, data))
-    return 1;
+  if(dumpToGif_quantizeImage(256, error, functionWait, data))
+    {
+      g_free(img->pixels);
+      if (img->colormap)
+	g_free(img->colormap);
+      g_free(img);
+      return FALSE;
+    }
   dumpToGif_syncImage();
 
   out = fopen(filename, "w");
   if(!out)
     {
-      g_string_append(buffer, _("Cannot open file (to write in)\n"));
-      free(image);
-      return 1;
+      *error = g_error_new(VISU_ERROR_DUMP, DUMP_ERROR_FILE,
+			   _("Cannot open file (to write in)."));
+      g_free(img->pixels);
+      g_free(img->colormap);
+      g_free(img);
+      return FALSE;
     }
    
    facx = (552.0f - x - 5)/img->columns;
@@ -738,5 +739,10 @@ int writeViewInPs256Format(GString *buffer,
    (void)fprintf(out, "%%%%EOF\n");
 
   (void)fclose(out);
-  return 0;
+
+  g_free(img->pixels);
+  g_free(img->colormap);
+  g_free(img);
+
+  return TRUE;
 }
