@@ -59,7 +59,11 @@
 #include <openGLFunctions/light.h>
 #include <openGLFunctions/view.h>
 #include <extensions/box.h>
+#include <extensions/fogAndBGColor.h>
 #include <extensions/axes.h>
+#include <extensions/legend.h>
+
+#define DEBUG_USE_FOG 0
 
 /**
  * SECTION:dumpToSVG
@@ -77,6 +81,7 @@
 
 struct _pairs
 {
+  VisuPairData *data;
   VisuNode *node1, *node2;
   VisuElement *ele1, *ele2;
   float alpha;
@@ -105,6 +110,9 @@ DumpType* dumpToSVG_init()
   svg->hasAlpha  = FALSE;
   svg->fileType  = fmt;
   svg->writeFunc = writeViewInSvgFormat;
+
+  fileFormatAdd_propertyBoolean(fmt, "use_flat_rendering",
+				_("Use flat colours for scheme rendering"), FALSE);
   
   return svg;
 }
@@ -139,40 +147,53 @@ static void sort_by_z(float *coordinates, float *buffer,
   sort_by_z(coordinates, buffer, n, z, middle + 1, end);
 }
 
-gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
-				 GError **error, voidDataFunc functionWait _U_,
+gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, FileFormat *format,
+				 VisuData *dataObj, GError **error,
+				 voidDataFunc functionWait _U_,
 				 gpointer user_data _U_)
 {
   cairo_t *cr;
   cairo_status_t status;
-  cairo_pattern_t *pat;
+  cairo_pattern_t **pat;
 #define FONT_SIZE 16.
   cairo_matrix_t scale = {1., 0., 0., -1., 0., 0.};
   cairo_matrix_t scaleFont = {FONT_SIZE, 0., 0., -FONT_SIZE, 0., 0.};
   VisuDataIter iter;
-  int i, nNodes;
-  float xyz[3], radius, norm, *rgb;
-  GLfloat *coordinates, *box, *pairs, *axes;
-  GLint nValuesBox, nValues, nValuesAxes;
+  int i, nNodes, nEle;
+  float xyz[3], radius, norm, *rgb, rgbaFog[4], rgbaBg[4];
+  GLfloat *coordinates, *box, *pairs, *axes, *legend;
+  GLint nValuesBox, nValues, nValuesAxes, nValuesLegend;
   OpenGLView *view;
   float modelView[16];
-  int viewport[4];
-  RenderingMethod *method;
+  int viewport[4], *width;
+  VisuRendering *method;
   GList *lights;
   Light *light;
-#define NVERT 4
+#define NVERT 2
 #define NVALS 8
 #define NPASS 1
 #define NBUFF (NVERT * NVALS + NPASS * 2)
   float tmpFloat[NBUFF];
+#define PAIRS_NVERT 2
+#define PAIRS_NVALS 4
+#define PAIRS_NPASS 5
+#define PAIRS_NBUFF (PAIRS_NVERT * PAIRS_NVALS + PAIRS_NPASS * 2)
+#define PAIRS_NCMPT (4 + 1 + PAIRS_NPASS)
   VisuDataIter iter1, iter2;
   float d2, d2min, d2max, d2min_buffered, d2max_buffered, l;
   VisuPairData *data;
-  float xyz1[3], xyz2[3], u[3], alpha;
-  GList *pairsLst, *tmpLst;
+  float xyz1[3], xyz2[3], u[3], alpha, hsl[3], rgba[3], lum;
+  GList *pairsLst, *tmpLst, *prop;
   struct _pairs *pairData;
   int nPairs, iPairs;
   float tmpPairs[6];
+  Color *color;
+  gboolean flat;
+  gchar *name;
+  VisuElement *ele;
+  gchar *lbl;
+  VisuNodeArray *array;
+  float lgWidth, lgHeight, max;
   
   cr = cairo_create(cairo_surf);
   status = cairo_status(cr);
@@ -185,13 +206,39 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
     }
   cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
 
+  flat = FALSE;
+  prop = fileFormatGet_propertiesList(format);
+  while (prop)
+    {
+      name = fileFormatGet_propertyName((FileFormatProperty*)(prop->data));
+      if (strcmp(name, "use_flat_rendering") == 0)
+	flat = fileFormatGet_propertyBoolean((FileFormatProperty*)(prop->data));
+      prop = g_list_next(prop);
+    }
+
   visuDataIter_new(dataObj, &iter);
   view = visuDataGet_openGLView(dataObj);
-  method = getRenderingMethodInUse();
+  method = visuRenderingClassGet_current();
   lights = lightEnvironnementGet_lightList(openGLGet_currentLights());
   g_return_val_if_fail(lights, FALSE);
   light = (Light*)lights->data;
 
+  /* We get the fog color. */
+  if (fogGet_isOn() && DEBUG_USE_FOG)
+    {
+      if (fogGet_useSpecificColor())
+	fogGet_value(rgbaFog);
+      else
+	bgGet_value(rgbaFog);
+    }
+  else
+    {
+      rgbaFog[0] = 0.;
+      rgbaFog[1] = 0.;
+      rgbaFog[2] = 0.;
+      rgbaFog[3] = 0.;
+    }
+
   /* We create a feedback mode to get node coordinates. */
   coordinates = g_malloc(sizeof(GLfloat) * (iter.nAllStoredNodes * NBUFF));
   glFeedbackBuffer((iter.nAllStoredNodes * NBUFF), GL_3D_COLOR, coordinates);
@@ -214,36 +261,15 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       glVertex3fv(xyz);
       /* We compute the node apparent radius using the real radius in
 	 X direction in ModelView. */
-      radius = renderingMethodGet_sizeOfElement(method, iter.element);
+      radius = visuRenderingGet_sizeOfElement(method, iter.element);
       glVertex3f(xyz[0] + modelView[0] * radius,
 		 xyz[1] + modelView[4] * radius,
 		 xyz[2] + modelView[8] * radius);
-      /* We get the color of the specular point on sphere and of the
-	 opposite point. */
-      glCallList(visuElementGet_identifierMaterial(iter.element));
-      norm = light->position[0] * light->position[0] +
-	light->position[1] * light->position[1] +
-	light->position[2] * light->position[2];
-      norm = sqrt(norm);
-      glNormal3f(light->position[0],
-		 light->position[1],
-		 light->position[2]);
-      glVertex3f(xyz[0] + (light->position[0]) * radius / norm,
-		 xyz[1] + (light->position[1]) * radius / norm,
-		 xyz[2] + (light->position[2]) * radius / norm);
-      glNormal3f(-light->position[0],
-		 -light->position[1],
-		 -light->position[2]);
-      glVertex3f(xyz[0] - (light->position[0]) * radius / norm,
-		 xyz[1] - (light->position[1]) * radius / norm,
-		 xyz[2] - (light->position[2]) * radius / norm);
       glEnd();
 
-      /* We compute z coordinates in eyes basis set. */
-      glPassThrough(modelView[2] * xyz[0] + 
-		    xyz[1] * modelView[6] + xyz[2] * modelView[10]);
+      /* We store the number of the VisuElement. */
+      glPassThrough(iter.iElement);
     }
-  glPopMatrix();
 
   /* We sort the coordinates along z. */
   nValues = glRenderMode(GL_RENDER);
@@ -253,9 +279,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
   while (i < nValues)
     {
       if (coordinates[i] == GL_POINT_TOKEN &&
-	  coordinates[i + NVALS] == GL_POINT_TOKEN &&
-	  coordinates[i + 2 * NVALS] == GL_POINT_TOKEN &&
-	  coordinates[i + 3 * NVALS] == GL_POINT_TOKEN)
+	  coordinates[i + NVALS] == GL_POINT_TOKEN)
 	{
 /* 	  fprintf(stderr, "Found a complete node %d, at %gx%g.\n", */
 /* 		  i, coordinates[i + 1], coordinates[i + 2]); */
@@ -356,6 +380,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 #else
 			  pairData = g_malloc(sizeof(struct _pairs));
 #endif
+			  pairData->data  = data;
 			  pairData->node1 = iter1.node;
 			  pairData->node2 = iter2.node;
 			  pairData->ele1  = iter1.element;
@@ -372,11 +397,10 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 
       if (nPairs > 0)
 	{
-	  pairs = g_malloc(sizeof(GLfloat) * nPairs * 10);
-	  glFeedbackBuffer(nPairs * 10, GL_3D, pairs);
+	  pairs = g_malloc(sizeof(GLfloat) * nPairs * PAIRS_NBUFF);
+	  glFeedbackBuffer(nPairs * PAIRS_NBUFF, GL_3D, pairs);
 	  glRenderMode(GL_FEEDBACK);
 	  glPushMatrix();
-	  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
 
 	  /* Render the list of pairs and free them each time. */
 	  tmpLst = pairsLst;
@@ -393,11 +417,11 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 	      u[0] /= norm;
 	      u[1] /= norm;
 	      u[2] /= norm;
-	      radius = renderingMethodGet_sizeOfElement(method, pairData->ele1);
+	      radius = visuRenderingGet_sizeOfElement(method, pairData->ele1);
 	      xyz1[0] += radius * u[0];
 	      xyz1[1] += radius * u[1];
 	      xyz1[2] += radius * u[2];
-	      radius = renderingMethodGet_sizeOfElement(method, pairData->ele2);
+	      radius = visuRenderingGet_sizeOfElement(method, pairData->ele2);
 	      xyz2[0] -= radius * u[0];
 	      xyz2[1] -= radius * u[1];
 	      xyz2[2] -= radius * u[2];
@@ -405,8 +429,18 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 	      glVertex3fv(xyz1);
 	      glVertex3fv(xyz2);
 	      glEnd();
-	      /* We save alpha channel as passthrough. */
+	      /* We save colour channel as passthrough. */
+	      color = visuPairGet_color(pairData->data);
+	      glPassThrough(color->rgba[0]);
+	      glPassThrough(color->rgba[1]);
+	      glPassThrough(color->rgba[2]);
 	      glPassThrough(pairData->alpha);
+	      /* We save the width in a passthrough. */
+	      width = (int*)visuPairGet_linkProperty(pairData->data, "width");
+	      if (width)
+		glPassThrough((float)*width);
+	      else
+		glPassThrough((float)boxGet_lineWidth());
 	  
 	      /* Free the data. */
 #if GLIB_MINOR_VERSION > 9
@@ -427,8 +461,9 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 	  while (i < nValues)
 	    {
 	  
-	      if (pairs[i] == GL_POINT_TOKEN && pairs[i + 4] == GL_POINT_TOKEN &&
-		  pairs[i + 8] == GL_PASS_THROUGH_TOKEN)
+	      if (pairs[i] == GL_POINT_TOKEN && 
+		  pairs[i + PAIRS_NVALS] == GL_POINT_TOKEN &&
+		  pairs[i + 2 * PAIRS_NVALS] == GL_PASS_THROUGH_TOKEN)
 		{
 /* 	      DBG_fprintf(stderr, " | pair %d at (%f;%f;%f) - (%f;%f;%f) %g\n", */
 /* 			  nPairs, */
@@ -437,31 +472,41 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 		  /* Copy all these values into the beginning of the pairs
 		     array. */
 		  norm = (pairs[i + 3] + pairs[i + 7]) / 2.f;
-		  pairs[nPairs * 6 + 0] = pairs[i + 1];
-		  pairs[nPairs * 6 + 1] = pairs[i + 2];
-		  pairs[nPairs * 6 + 2] = pairs[i + 5];
-		  pairs[nPairs * 6 + 3] = pairs[i + 6];
-		  pairs[nPairs * 6 + 4] = pairs[i + 9];
-		  pairs[nPairs * 6 + 5] = norm;
-		  i += 10;
+		  pairs[nPairs * PAIRS_NCMPT + 0] = pairs[i + 1]; /* x1 */
+		  pairs[nPairs * PAIRS_NCMPT + 1] = pairs[i + 2]; /* y1 */
+		  pairs[nPairs * PAIRS_NCMPT + 2] = pairs[i + 5]; /* x2 */
+		  pairs[nPairs * PAIRS_NCMPT + 3] = pairs[i + 6]; /* y2 */
+		  pairs[nPairs * PAIRS_NCMPT + 4] = norm;         /* altitude */
+		  pairs[nPairs * PAIRS_NCMPT + 5] = pairs[i + PAIRS_NVALS * PAIRS_NVERT + 7]; /* alpha */
+		  pairs[nPairs * PAIRS_NCMPT + 6] = pairs[i + PAIRS_NVALS * PAIRS_NVERT + 1]; /* red */
+		  pairs[nPairs * PAIRS_NCMPT + 7] = pairs[i + PAIRS_NVALS * PAIRS_NVERT + 3]; /* green */
+		  pairs[nPairs * PAIRS_NCMPT + 8] = pairs[i + PAIRS_NVALS * PAIRS_NVERT + 5]; /* blue */
+		  pairs[nPairs * PAIRS_NCMPT + 9] = pairs[i + PAIRS_NVALS * PAIRS_NVERT + 9]; /* width */
+		  i += PAIRS_NBUFF;
 		  nPairs += 1;
 		}
 	      else if (pairs[i] == GL_POINT_TOKEN &&
-		       pairs[i + 4] == GL_PASS_THROUGH_TOKEN)
+		       pairs[i + PAIRS_NVALS] == GL_PASS_THROUGH_TOKEN)
 		{
 		  DBG_fprintf(stderr, "| uncomplete pair for i=%d\n", i);
-		  i += 6;
+		  i += PAIRS_NVALS + 2 * PAIRS_NPASS;
 		}
 	      else if (pairs[i] == GL_PASS_THROUGH_TOKEN)
 		{
 		  DBG_fprintf(stderr, "| no pair for i=%d\n", i);
-		  i += 2;
+		  i += 2 * PAIRS_NPASS;
 		}
 	    }
 	  glPopMatrix();
-	  sort_by_z(pairs, tmpPairs, 6, 5, 0, nPairs - 1);
+	  sort_by_z(pairs, tmpPairs, PAIRS_NCMPT, 4, 0, nPairs - 1);
+	  DBG_fprintf(stderr, " | will draw %d pairs.\n", nPairs);
 	}
     }
+  
+  /* We draw the background colour. */
+  bgGet_value(rgbaBg);
+  cairo_set_source_rgba(cr, rgbaBg[0], rgbaBg[1], rgbaBg[2], rgbaBg[3]);
+  cairo_paint(cr);
 
   /* Draw the back box. */
   nValuesBox = -1;
@@ -471,7 +516,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       box = g_malloc(sizeof(GLfloat) * 7000);
       glFeedbackBuffer(7000, GL_3D, box);
       glRenderMode(GL_FEEDBACK);
-      OpenGLExtensionCall_list(EXT_BOX_ID);
+      OpenGLExtensionCall_list(EXT_BOX_ID, FALSE);
       nValuesBox = glRenderMode(GL_RENDER);
       cairo_set_line_width(cr, (double)boxGet_lineWidth());
       rgb = boxGet_RGBValues();
@@ -487,29 +532,71 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 	  }
     }
 
+  /* We create the pattern for the different elements. */
+  pat = g_malloc(sizeof(cairo_pattern_t*) * dataObj->ntype);
+  for (i = 0; i < (int)dataObj->ntype; i++)
+    {
+      if (flat)
+	{
+	  pat[i] =
+	    cairo_pattern_create_rgba(dataObj->fromIntToVisuElement[i]->rgb[0],
+				      dataObj->fromIntToVisuElement[i]->rgb[1],
+				      dataObj->fromIntToVisuElement[i]->rgb[2],
+				      dataObj->fromIntToVisuElement[i]->rgb[3]);
+	}
+      else
+	{
+	  pat[i] = cairo_pattern_create_radial(.4f, .4f, .1f, 0.f, 0.f, 1.f);
+	  /* We get the Element colour in HSL. */
+	  color_RGBtoHSL(hsl, dataObj->fromIntToVisuElement[i]->rgb);
+	  lum = hsl[2];
+	  hsl[2] = CLAMP(lum + 0.2f, 0.f, 1.f);
+	  color_HSLtoRGB(rgba, hsl);
+	  cairo_pattern_add_color_stop_rgba(pat[i], 0, rgba[0], rgba[1], rgba[2], 1.f);
+	  hsl[2] = CLAMP(lum + 0.05f, 0.f, 1.f);
+	  color_HSLtoRGB(rgba, hsl);
+	  cairo_pattern_add_color_stop_rgba(pat[i], 0.3, rgba[0], rgba[1], rgba[2], 1.f);
+	  hsl[2] = CLAMP(lum - 0.05f, 0.f, 1.f);
+	  color_HSLtoRGB(rgba, hsl);
+	  cairo_pattern_add_color_stop_rgba(pat[i], 0.7, rgba[0], rgba[1], rgba[2], 1.f);
+	  hsl[2] = CLAMP(lum - 0.2f, 0.f, 1.f);
+	  color_HSLtoRGB(rgba, hsl);
+	  cairo_pattern_add_color_stop_rgba(pat[i], 1, rgba[0], rgba[1], rgba[2], 1.f);
+	}
+    }
+
   cairo_set_line_width(cr, 1.);
   iPairs = 0;
+  alpha = 0.f;
   for (i = 0; i < nNodes; i+= NBUFF)
     {
-      if (nPairs > 0 && iPairs < 6 * nPairs)
+      if (nPairs > 0 && iPairs < PAIRS_NCMPT * nPairs)
 	{
 	  /* We draw the pairs in between. */
-	  cairo_set_line_width(cr, (double)boxGet_lineWidth());
-	  while (pairs[iPairs + 5] > coordinates[i + 3] && iPairs < 6 * nPairs)
+	  while (pairs[iPairs + 4] > coordinates[i + 3] &&
+		 iPairs < PAIRS_NCMPT * nPairs)
 	    {
 	      DBG_fprintf(stderr, " | pair %d at (%f;%f) - (%f;%f) %g,%g\n",
-			  iPairs / 6,
+			  iPairs / PAIRS_NCMPT,
 			  pairs[iPairs + 0], pairs[iPairs + 1],
 			  pairs[iPairs + 2], pairs[iPairs + 3],
 			  pairs[iPairs + 4], pairs[iPairs + 5]);
-	      cairo_set_source_rgba(cr, 0, 0, 0, pairs[iPairs + 4]);
+	      cairo_set_line_width(cr, pairs[iPairs + 9]);
+	      cairo_set_source_rgba(cr, pairs[iPairs + 6], pairs[iPairs + 7],
+				    pairs[iPairs + 8], pairs[iPairs + 5]);
 	      cairo_move_to(cr, pairs[iPairs + 0], pairs[iPairs + 1]);
 	      cairo_line_to(cr, pairs[iPairs + 2], pairs[iPairs + 3]);
 	      cairo_stroke(cr);
-	      iPairs += 6;
+	      iPairs += PAIRS_NCMPT;
 	    }
+	  cairo_set_line_width(cr, 1.);
 	}
 
+      /* Compute the alpha for the fog. */
+      if (fogGet_isOn() && DEBUG_USE_FOG)
+	alpha = 1.f - CLAMP((fogGet_end() - coordinates[i + 3]) /
+			    (fogGet_end() - fogGet_start()), 0.f, 1.f);
+      
       radius = (coordinates[i + NVALS + 1] - coordinates[i + 1]) *
 	(coordinates[i + NVALS + 1] - coordinates[i + 1]) +
 	(coordinates[i + NVALS + 2] - coordinates[i + 2]) *
@@ -517,41 +604,49 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       /*       fprintf(stderr, "%gx%g %g %g %g\n", coordinates[i + 1], coordinates[i + 2], */
       /* 	      sqrt(radius), coordinates[i + 3], coordinates[i + NBUFF - 1]); */
       radius = sqrt(radius);
-      cairo_arc(cr, coordinates[i + 1], coordinates[i + 2],
-		radius, 0., 2 * G_PI);
-      /*       fprintf(stderr, "%gx%g %g, %gx%g %g\n", coordinates[i + 2 * NVALS + 1], */
-      /* 					coordinates[i + 2 * NVALS + 2], */
-      /* 					radius * 0.2f, */
-      /* 					coordinates[i + 1], */
-      /* 					coordinates[i + 2], */
-      /* 					radius); */
-      pat = cairo_pattern_create_radial(coordinates[i + 1] + 0.5f * radius,
-					coordinates[i + 2] + 0.5f * radius,
-					radius * 0.2f,
-					coordinates[i + 1],
-					coordinates[i + 2],
-					radius);
-      /*       fprintf(stderr,"%g %g %g\n", coordinates[i + 2 * NVALS + 4], */
-      /* 	      coordinates[i + 2 * NVALS + 5], */
-      /* 	      coordinates[i + 2 * NVALS + 6]); */
-      cairo_pattern_add_color_stop_rgba(pat, 0,
-					coordinates[i + 2 * NVALS + 4],
-					coordinates[i + 2 * NVALS + 5],
-					coordinates[i + 2 * NVALS + 6],
-					coordinates[i + 2 * NVALS + 7]);
-      /*       fprintf(stderr,"%g %g %g\n", coordinates[i + 3 * NVALS + 4], */
-      /* 	      coordinates[i + 3 * NVALS + 5], */
-      /* 	      coordinates[i + 3 * NVALS + 6]); */
-      cairo_pattern_add_color_stop_rgba(pat, 1,
-					coordinates[i + 3 * NVALS + 4],
-					coordinates[i + 3 * NVALS + 5],
-					coordinates[i + 3 * NVALS + 6],
-					coordinates[i + 3 * NVALS + 7]);
-      cairo_set_source(cr, pat);
-      cairo_fill_preserve(cr);
-      cairo_pattern_destroy(pat);
-      cairo_set_source_rgb(cr, 0, 0, 0);
+
+      cairo_save(cr);
+      cairo_translate(cr, coordinates[i + 1], coordinates[i + 2]);
+      cairo_arc(cr, 0.f, 0.f, radius, 0., 2 * G_PI);
+      cairo_save(cr);
+      cairo_scale(cr, radius, radius);
+      if (alpha < 1.f)
+	{
+	  cairo_set_source(cr, pat[(int)coordinates[i + NVERT * NVALS + 1]]);
+	  cairo_fill_preserve(cr);
+	}
+      if (alpha > 0.f)
+	{
+	  fprintf(stderr, "%g %g %g %g.\n", rgbaFog[0], rgbaFog[1], rgbaFog[2], alpha);
+	  cairo_set_source_rgba(cr, rgbaFog[0], rgbaFog[1], rgbaFog[2], alpha);
+	  cairo_fill_preserve(cr);
+	}
+      cairo_restore(cr);
+      cairo_set_source_rgb(cr, rgbaFog[0] * alpha, rgbaFog[1] * alpha,
+			   rgbaFog[2] * alpha);
       cairo_stroke(cr);
+      cairo_restore(cr);
+    }
+  /* We draw the remaining pairs. */
+  if (nPairs > 0 && iPairs < PAIRS_NCMPT * nPairs)
+    {
+      /* We draw the pairs in between. */
+      while (iPairs < PAIRS_NCMPT * nPairs)
+	{
+	  DBG_fprintf(stderr, " | pair %d at (%f;%f) - (%f;%f) %g,%g\n",
+		      iPairs / PAIRS_NCMPT,
+		      pairs[iPairs + 0], pairs[iPairs + 1],
+		      pairs[iPairs + 2], pairs[iPairs + 3],
+		      pairs[iPairs + 4], pairs[iPairs + 5]);
+	  cairo_set_line_width(cr, pairs[iPairs + 9]);
+	  cairo_set_source_rgba(cr, pairs[iPairs + 6], pairs[iPairs + 7],
+				pairs[iPairs + 8], pairs[iPairs + 5]);
+	  cairo_move_to(cr, pairs[iPairs + 0], pairs[iPairs + 1]);
+	  cairo_line_to(cr, pairs[iPairs + 2], pairs[iPairs + 3]);
+	  cairo_stroke(cr);
+	  iPairs += PAIRS_NCMPT;
+	}
+      cairo_set_line_width(cr, 1.);
     }
   if (nPairs > 0)
     g_free(pairs);
@@ -584,7 +679,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       axes = g_malloc(sizeof(GLfloat) * 7000);
       glFeedbackBuffer(7000, GL_3D, axes);
       glRenderMode(GL_FEEDBACK);
-      OpenGLExtensionCall_list(EXT_AXES_ID);
+      OpenGLExtensionCall_list(EXT_AXES_ID, TRUE);
       nValuesAxes = glRenderMode(GL_RENDER);
 
       cairo_set_line_width(cr, (double)axesGet_lineWidth());
@@ -597,6 +692,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       if (nValuesAxes >= 21 && axes[0] == GL_LINE_RESET_TOKEN &&
 	  axes[7] == GL_LINE_RESET_TOKEN && axes[14] == GL_LINE_RESET_TOKEN)
 	{
+	  DBG_fprintf(stderr, "Dump SVG: output axes.\n");
 	  cairo_move_to(cr, axes[0 + 1], axes[0 + 2]);
 	  cairo_line_to(cr, axes[0 + 4], axes[0 + 5]);
 	  cairo_stroke(cr);
@@ -612,6 +708,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
       if (nValuesAxes >= 33 && axes[21] == GL_BITMAP_TOKEN &&
 	  axes[25] == GL_BITMAP_TOKEN && axes[29] == GL_BITMAP_TOKEN)
 	{
+	  DBG_fprintf(stderr, "Dump SVG: output axes names.\n");
 	  cairo_move_to(cr, axes[21 + 1], axes[21 + 2]);
 	  cairo_show_text(cr, "x");
 	  cairo_move_to(cr, axes[25 + 1], axes[25 + 2]);
@@ -621,6 +718,78 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
 	}
       g_free(axes);
     }
+
+  /* We draw a legend, if required. */
+  nValuesLegend = -1;
+  legend = (GLfloat*)0;
+  if (extLegendGet_isOn())
+    {
+      legend = g_malloc(sizeof(GLfloat) * 70000);
+      glFeedbackBuffer(70000, GL_3D, legend);
+      glRenderMode(GL_FEEDBACK);
+      OpenGLExtensionCall_list(EXT_LEGEND_ID, TRUE);
+      nValuesLegend = glRenderMode(GL_RENDER);
+
+      cairo_select_font_face(cr, "Serif", CAIRO_FONT_SLANT_NORMAL,
+			     CAIRO_FONT_WEIGHT_NORMAL);
+      cairo_set_font_size(cr, 12.0);
+      /* The two first polygons are the white frame. */
+      if (nValuesLegend >= 22)
+	{
+	  cairo_set_source_rgba(cr, 1.f, 1.f, 1.f, 0.4f);
+	  lgWidth  = legend[5] - legend[2];
+	  lgHeight = legend[9] - legend[3];
+	  cairo_rectangle(cr, legend[2], legend[3], lgWidth, lgHeight);
+	  cairo_fill(cr);
+	  /* Run to the names and the elements... */
+	  nEle = 0;
+	  array = visuDataGet_nodeArray(dataObj);
+	  max = visuDataGet_allElementExtens(dataObj);
+	  for (i = 0; i < nValuesLegend; i++)
+	    if (legend[i] == GL_BITMAP_TOKEN)
+	      {
+		ele = dataObj->fromIntToVisuElement[nEle];
+
+		/* Write the name. */
+		cairo_move_to(cr, legend[i + 1], legend[i + 2]);
+		lbl = g_strdup_printf("%s (%d)", ele->name,
+				      array->numberOfStoredNodes[nEle]);
+		cairo_set_source_rgb(cr, 0.f, 0.f, 0.f);
+		cairo_set_font_matrix(cr, &scaleFont);
+		cairo_show_text(cr, lbl);
+		g_free(lbl);
+
+		/* Draw the element. */
+		radius = 0.4f * lgHeight *
+		  visuRenderingGet_sizeOfElement(method, ele) / max;
+		cairo_new_path(cr);
+		cairo_save(cr);
+		cairo_translate(cr, legend[i + 1] - 0.6f * lgHeight,
+				legend[3] + 0.5f * lgHeight);
+		cairo_arc(cr, 0.f, 0.f, radius, 0., 2 * G_PI);
+		cairo_save(cr);
+		cairo_scale(cr, radius, radius);
+		cairo_set_source(cr, pat[nEle]);
+		cairo_fill_preserve(cr);
+		cairo_restore(cr);
+		cairo_set_source_rgb(cr, 0.f, 0.f, 0.f);
+		cairo_stroke(cr);
+		cairo_restore(cr);
+
+		nEle += 1;
+		do
+		  i += 4;
+		while (legend[i] == GL_BITMAP_TOKEN && i < nValuesLegend);
+	      }
+	}
+      g_free(legend);
+    }
+
+  for (i = 0; i < (int)dataObj->ntype; i++)
+    cairo_pattern_destroy(pat[i]);
+  g_free(pat);
+
+  glPopMatrix();
   
   cairo_show_page(cr);
   cairo_destroy(cr);
@@ -628,7 +797,7 @@ gboolean writeDataToCairoSurface(cairo_surface_t *cairo_surf, VisuData *dataObj,
   return TRUE;
 }
 
-static gboolean writeViewInSvgFormat(FileFormat *format _U_, const char* filename,
+static gboolean writeViewInSvgFormat(FileFormat *format, const char* filename,
 				     int width, int height, VisuData *dataObj,
 				     guchar* imageData _U_, GError **error,
 				     voidDataFunc functionWait, gpointer data)
@@ -658,7 +827,7 @@ static gboolean writeViewInSvgFormat(FileFormat *format _U_, const char* filenam
   old_width = view->window->width;
   old_height = view->window->height;
   OpenGLViewSet_windowSize(view, (guint)width, (guint)height);
-  writeDataToCairoSurface(svg_surface, dataObj, error, functionWait, data);
+  writeDataToCairoSurface(svg_surface, format, dataObj, error, functionWait, data);
   cairo_surface_destroy(svg_surface);
 
   /* Set back the viewport. */
