@@ -51,10 +51,12 @@
 #include "dumpToPsAndPdf.h"
 #include "dumpThroughGdkPixbuf.h"
 #include "dumpToAscii.h"
+#include "dumpToXyz.h"
 #include "dumpToSVG.h"
 
 static VisuDumpInitFunc listInitDumpModuleFunc[] = {
   dumpToAscii_init,
+  dumpToXyz_init,
   dumpToGif_init,
   dumpToJPEG_init,
   dumpToPDF_init,
