@@ -123,6 +123,7 @@ static void onAxesConfigChange(GObject *visu, VisuData *dataObj, gpointer data);
 static void rebuildAxesOnResourcesLoaded(GObject *visu, VisuData *dataObj,
 					 gpointer data);
 static void onDataNewSignal(GObject *visu, VisuData *dataObj, gpointer data);
+static void onDataReady(GObject *obj, VisuData *dataObj, gpointer data);
 
 OpenGLExtension* initExtensionAxes()
 {
@@ -166,6 +167,8 @@ OpenGLExtension* initExtensionAxes()
 
   g_signal_connect(VISU_INSTANCE, "dataNew",
 		   G_CALLBACK(onDataNewSignal), (gpointer)0);
+  g_signal_connect(VISU_INSTANCE, "dataReadyForRendering",
+		   G_CALLBACK(onDataReady), (gpointer)0);
 
   g_signal_connect(VISU_INSTANCE, "SpinAxes",
  		   G_CALLBACK(onAxesConfigChange), (gpointer)0);
@@ -282,11 +285,16 @@ static void onDataNewSignal(GObject *visu _U_, VisuData *dataObj,
   g_signal_connect(G_OBJECT(dataObj), "OpenGLWidthHeight",
 		   G_CALLBACK(onAxesParametersChange), (gpointer)0);
 }
+static void onDataReady(GObject *obj _U_, VisuData *dataObj, gpointer data)
+{
+  if (dataObj)
+    onAxesParametersChange(dataObj, (OpenGLView*)0, data);
+}
 static void onAxesConfigChange(GObject *visu _U_, VisuData *dataObj, gpointer data)
 {
   g_return_if_fail(dataObj);
 
-  onAxesParametersChange(dataObj, visuDataGet_openGLView(dataObj), data);
+  onAxesParametersChange(dataObj, (OpenGLView*)0, data);
 }
 
 static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view _U_,
