@@ -50,6 +50,8 @@
 #include "axes.h"
 #include "fogAndBGColor.h"
 #include "extInfos.h"
+#include "legend.h"
+#include "rings.h"
 
 /**
  * OpenGLExtensionInitFunc:
@@ -71,5 +73,7 @@ OpenGLExtensionInitFunc listInitExtensionFunc[] = {
   initExtensionBox,
   initExtensionAxes,
   extInfosInit,
+  extLegendInit,
+  extRingsInit,
   (OpenGLExtensionInitFunc)0};
 #endif
