@@ -102,15 +102,14 @@ static gboolean readBgColor(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesFog(GString *data, int *nbLinesWritten,
-				   VisuData *dataObj);
+static void exportResourcesFog(GString *data, VisuData *dataObj);
 
 static float fogRGBDefault[4] = {0., 0., 0., 1.};
 static float fogStartEndDefault[2] = {0.3, 0.7};
 static float bgRGBDefault[4] = {0., 0., 0., 0.};
 
 static guchar *bgImage;
-static gboolean bgImageAlpha;
+static gboolean bgImageAlpha, bgImageFit;
 static guint bgImageW, bgImageH;
 static gchar *bgImageTitle;
 
@@ -124,6 +123,7 @@ static void rebuildOpenGLValues(VisuData *dataObj);
 static void onDataReadySignal(GObject *visu, VisuData *dataObj, gpointer data);
 static void onDataNewSignal(GObject *visu, VisuData *dataObj, gpointer data);
 static void onFogParameterChanged(VisuData *dataObj, OpenGLView *view, gpointer data);
+static void onBgImageRescale(VisuData *dataObj, OpenGLView *view, gpointer data);
 static void rebuildFogOnResources(GObject *obj, VisuData *dataObj, gpointer data);
 
 /* Local methods */
@@ -239,7 +239,9 @@ gboolean fogSet_isOn(gboolean value)
   if (value)
     {
       glEnable(GL_FOG);
-      glFogi(GL_FOG_MODE, GL_LINEAR); 
+      glFogi(GL_FOG_MODE, GL_LINEAR);
+/*       glFogi(GL_FOG_MODE, GL_EXP);  */
+/*       glFogf(GL_FOG_DENSITY, 0.03f);  */
     }
   else
     glDisable(GL_FOG);
@@ -270,11 +272,7 @@ gboolean fogSet_startEndValues(float startEnd[2], int mask)
   
   if (mask & MASK_FOG_START && fog_start != startEnd[0])
     {
-      fog_start = startEnd[0];
-      if (fog_start < 0.)
-	fog_start = 0.;
-      if (fog_start > 1.)
-	fog_start = 1.;
+      fog_start = CLAMP(startEnd[0], 0., 1.);
       if (mask & MASK_FOG_END)
 	{
 	  if (fog_start >= startEnd[1])
@@ -287,11 +285,7 @@ gboolean fogSet_startEndValues(float startEnd[2], int mask)
     }
   if (mask & MASK_FOG_END && fog_end != startEnd[1])
     {
-      fog_end = startEnd[1];
-      if (fog_end < 0.)
-	fog_end = 0.;
-      if (fog_end > 1.)
-	fog_end = 1.;
+      fog_end = CLAMP(startEnd[1], 0., 1.);
       if (fog_end <= fog_start)
 	fog_end = fog_start + 0.001;
       diff = 1;
@@ -386,7 +380,7 @@ void bgGet_value(float rgba[4])
 }
 
 void bgSet_image(const guchar *imageData, guint width, guint height,
-		 gboolean alpha, const gchar *title)
+		 gboolean alpha, const gchar *title, gboolean fit)
 {
   guint n;
 
@@ -413,6 +407,7 @@ void bgSet_image(const guchar *imageData, guint width, guint height,
   bgImageAlpha = alpha;
   if (title)
     bgImageTitle = g_strdup_printf(_("Background: %s"), title);
+  bgImageFit = fit;
 
   createBgImage();
 }
@@ -436,6 +431,8 @@ static void onDataNewSignal(GObject *visu _U_, VisuData *dataObj,
 {
   g_signal_connect(G_OBJECT(dataObj), "OpenGLNearFar",
 		   G_CALLBACK(onFogParameterChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(dataObj), "OpenGLWidthHeight",
+		   G_CALLBACK(onBgImageRescale), (gpointer)0);
 }
 static void onFogParameterChanged(VisuData *dataObj, OpenGLView *view _U_,
 				  gpointer data _U_)
@@ -454,6 +451,14 @@ static void rebuildFogOnResources(GObject *obj _U_, VisuData *dataObj,
     return;
   fogCreate(dataObj);
 }
+static void onBgImageRescale(VisuData *dataObj _U_, OpenGLView *view _U_,
+			     gpointer data _U_)
+{
+  DBG_fprintf(stderr, "Extension Bg: caught the 'OpenGLWidthHeight' signal.\n");
+
+  if (bgImage)
+    createBgImage();
+}
 
 void fogCreate(VisuData *data)
 {
@@ -506,8 +511,15 @@ static void createBgImage()
   openGLText_initFontList();
   
   glGetIntegerv(GL_VIEWPORT, viewport);
-  x = (float)viewport[2] / (float)bgImageW;
-  y = (float)viewport[3] / (float)bgImageH;
+  if (bgImageFit)
+    {
+      x = (float)viewport[2] / (float)bgImageW;
+      y = (float)viewport[3] / (float)bgImageH;
+    }
+  else
+    {
+      x = y = 1.f;
+    }
   zoom = MIN(x, y);
   x = ((float)viewport[2] - zoom * (float)bgImageW) / 2.f;
   y = ((float)viewport[3] - zoom * (float)bgImageH) / 2.f;
@@ -538,7 +550,7 @@ static void createBgImage()
       glDisable(GL_LIGHTING);
       glColor4f(1.f - bgRGB[0], 1.f - bgRGB[1], 1.f - bgRGB[2], 1.f);
       glRasterPos2f(5.f, 5.f);
-      openGLText_drawChars(bgImageTitle); 
+      openGLText_drawChars(bgImageTitle, TEXT_NORMAL); 
     }
   glDepthMask(1);
 
@@ -708,8 +720,7 @@ static gboolean readBgColor(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesFog(GString *data, int *nbLinesWritten,
-				   VisuData *dataObj _U_)
+static void exportResourcesFog(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_FOG_USED);
   g_string_append_printf(data, "%s:\n    %d\n", FLAG_RESOURCE_FOG_USED,
@@ -717,7 +728,6 @@ static gboolean exportResourcesFog(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_FOG_SPECIFIC);
   g_string_append_printf(data, "%s:\n    %d\n", FLAG_RESOURCE_FOG_SPECIFIC,
 			 fogColorSpecific);
-  *nbLinesWritten = 6;
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_FOG_COLOR);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_FOG_COLOR);
   g_string_append_printf(data, "    %4.3f %4.3f %4.3f %4.3f\n",
@@ -727,13 +737,9 @@ static gboolean exportResourcesFog(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "    %4.3f %4.3f\n",
 			 fog_start, fog_end);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 7;
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_BG_COLOR);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_BG_COLOR);
   g_string_append_printf(data, "    %4.3f %4.3f %4.3f %4.3f\n",
 			 bgRGB[0], bgRGB[1], bgRGB[2], bgRGB[3]);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 4;
-
-  return TRUE;
 }
