@@ -107,8 +107,7 @@ static gboolean readScaleDefinition(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesScale(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj);
+static void exportResourcesScale(GString *data, VisuData *dataObj);
 
 #define SCALE_AUTO_LEGEND "[auto]"
 
@@ -634,7 +633,6 @@ void scalesDraw(VisuData *dataObj)
   ScaleClass *klass;
   GList *tmpLst;
   int nlat;
-  OpenGLView *view;
   float radius = 0.3;
 
   /* Nothing to draw; */
@@ -651,8 +649,7 @@ void scalesDraw(VisuData *dataObj)
   scaleHasBeenBuilt = TRUE;
 
   openGLText_initFontList();
-  view = visuDataGet_openGLView(dataObj);
-  nlat = OpenGLViewGet_numberOfFacettes(view, radius);
+  nlat = OpenGLViewGet_numberOfFacettes(visuDataGet_openGLView(dataObj), radius);
   
   glDeleteLists(klass->ext->objectListId, 1);
 
@@ -663,8 +660,6 @@ void scalesDraw(VisuData *dataObj)
   glDisable(GL_FOG);
 
   glMatrixMode(GL_MODELVIEW);
-  glPushMatrix();
-  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
 
   glLineWidth(klass->width);
   glColor4fv(klass->rgba);
@@ -681,7 +676,6 @@ void scalesDraw(VisuData *dataObj)
       tmpLst = g_list_next(tmpLst);
     }
 
-  glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
 
   glEndList();
@@ -815,9 +809,7 @@ static gboolean readScaleDefinition(gchar **lines, int nbLines,
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-static gboolean exportResourcesScale(GString *data, 
-				     int *nbLinesWritten,
-				     VisuData *dataObj _U_)
+static void exportResourcesScale(GString *data, VisuData *dataObj _U_)
 {
   ScaleClass *klass;
   GList *tmpLst;
@@ -840,7 +832,6 @@ static gboolean exportResourcesScale(GString *data,
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_SCALE_STIPPLE);
   g_string_append_printf(data, "    %d\n", klass->stipple);
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_SCALE_DEFINITION);
-  *nbLinesWritten += 13;
   tmpLst = klass->scales;
   while (tmpLst)
     {
@@ -854,14 +845,9 @@ static gboolean exportResourcesScale(GString *data,
 			     scale->origin[0], scale->origin[1], scale->origin[2],
 			     scale->direction[0], scale->direction[1], scale->direction[2],
 			     scale->length, legend);
-      *nbLinesWritten += 2;
       tmpLst = g_list_next(tmpLst);
     }
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 1;
-
-  return TRUE;
-
 }
 
 
@@ -905,8 +891,8 @@ static void lineDraw(VisuData *dataObj _U_, int nlat, Scale *scale)
   glVertex3f(x2, y2, z2);
   glEnd();
 
-  gluDeleteQuadric(obj);   
-
   glRasterPos3fv(scale->origin);
-  openGLText_drawChars(scale->legend->str); /*affichage de la  legende*/
+  openGLText_drawChars(scale->legend->str, TEXT_NORMAL); /*affichage de la
+					      legende*/
+  gluDeleteQuadric(obj);
 }
