@@ -49,6 +49,7 @@
 #include <openGLFunctions/objectList.h>
 #include <renderingBackend/visu_windowInterface.h>
 #include <coreTools/toolColor.h>
+#include <coreTools/toolShade.h>
 
 /**
  * SECTION:dataFile
@@ -85,14 +86,11 @@ struct DataFile_struct
   /* Set to TRUE to colorize. */
   gboolean used;
 
-  /* Transformation vectors. */
-  float vectA[3], vectB[3];
+  Shade *shade;
 
   /* Columns to be used. */
   int colUsed[3];
-
-  /* RGB or HSV. */
-  DataFileColorModeId colorType;
+  int scaleUsed;
 
   /* Scheme to scale : manual or auto. */
   DataFileInputScaleId scaleType;
@@ -112,13 +110,16 @@ struct DataFile_struct
 typedef struct DataFile_struct DataFile;
 
 /* Cache pointers for data from currentVisuData.
-   This cache is usefull to avoid too much visuDataGet_property().
+   This cache is usefull to avoid too much g_object_get_data().
    This cache is refresh as soon as currentVisuData is changed. */
 DataFile *cacheDataFile;
 VisuData *cacheVisuData;
 
 /* Default values */
-#define DATAFILE_NB_COLUMN_DEFAULT  -1
+#define DATAFILE_NB_COLUMN_DEFAULT  -4
+#define DATAFILE_COLUMN_X  -3
+#define DATAFILE_COLUMN_Y  -2
+#define DATAFILE_COLUMN_Z  -1
 #define DATAFILE_SCALE_TYPE_DEFAULT dataFile_normalize
 #define DATAFILE_MIN_NORM_DEFAULT   -1.
 #define DATAFILE_MAX_NORM_DEFAULT   +1.
@@ -131,10 +132,15 @@ static DataNode *dataNode;
 static GValue dataValue = {0, {{0}, {0}}};
 
 /* local methods. */
-void dataFileFree(gpointer data);
-DataFile* dataFileNew(VisuData *visuData);
-void colorFromUserData(VisuData *visuData, float rgba[4], VisuElement *ele, VisuNode* node);
-float dataFileGet_valuesFromData(VisuData *visuData, int column, float fromVal);
+static void dataFileFree(gpointer data);
+static DataFile* dataFileNew(VisuData *visuData);
+static DataFile* _get_dataColor(VisuData *data, gboolean create, gboolean *new);
+
+static void colorFromUserData(VisuData *visuData, float rgba[4],
+			      VisuElement *ele, VisuNode* node);
+static float radiusFromUserData(VisuData *visuData, VisuNode *node);
+static float dataFileGet_valuesFromData(VisuData *visuData,
+					int column, float fromVal);
 static void freeData(gpointer obj, gpointer data);
 static gpointer newOrCopyData(gconstpointer orig, gpointer user_data);
 
@@ -159,32 +165,33 @@ void dataFileReDraw(VisuData *data)
 }
 
 
-DataFile* dataFileNew(VisuData *visuData)
+static DataFile* dataFileNew(VisuData *visuData)
 {
-  int i;
   DataFile *dataFile;
+  GList *lst;
 
-  DBG_fprintf(stderr, "Visu dataFile : allocate data informations for"
+  DBG_fprintf(stderr, "Visu dataFile: allocate data informations for"
 	      " given VisuData object %p.\n", (gpointer)visuData);
   g_return_val_if_fail(visuData, (DataFile*)0);
 
+  lst = toolShadeListGet();
+
   dataFile = g_malloc(sizeof(DataFile));
-  dataFile->used            = TRUE;
+  dataFile->used            = FALSE;
   dataFile->file            = (gchar*)0;
   dataFile->readMinMax      = (float*)0;
-  dataFile->nbColumns       = DATAFILE_NB_COLUMN_DEFAULT;
+  dataFile->nbColumns       = 0;
   dataFile->scaleType       = DATAFILE_SCALE_TYPE_DEFAULT;
-  dataFile->colorType       = DATAFILE_COLOR_TYPE_DEFAULT;
   dataFile->minMaxValues[0] = DATAFILE_MIN_NORM_DEFAULT;
   dataFile->minMaxValues[1] = DATAFILE_MAX_NORM_DEFAULT;
-  for (i = 0; i < 3; i++)
-    {
-      dataFile->vectA[i]    = 1.;
-      dataFile->vectB[i]    = 0.;
-      dataFile->colUsed[i]  = -1;
-    }
-  visuDataSet_propertyWithDestroyFunc(visuData, "dataColor_parameters",
-				      (gpointer)dataFile, dataFileFree);
+  dataFile->shade           = (lst)?shadeCopy((Shade*)lst->data):(Shade*)0;
+  dataFile->colUsed[0]      = DATAFILE_NB_COLUMN_DEFAULT;
+  dataFile->colUsed[1]      = DATAFILE_NB_COLUMN_DEFAULT;
+  dataFile->colUsed[2]      = DATAFILE_NB_COLUMN_DEFAULT;
+  dataFile->scaleUsed       = DATAFILE_NB_COLUMN_DEFAULT;
+  g_object_set_data_full(G_OBJECT(visuData), "dataColor_parameters",
+			 (gpointer)dataFile, dataFileFree);
+  DBG_fprintf(stderr, " | %p\n", (gpointer)dataFile);
 
   /* Set the cache. */
   cacheDataFile = dataFile;
@@ -193,7 +200,7 @@ DataFile* dataFileNew(VisuData *visuData)
   return dataFile;
 }
 
-void dataFileFree(gpointer data)
+static void dataFileFree(gpointer data)
 {
   DataFile *dataFile;
 
@@ -204,6 +211,8 @@ void dataFileFree(gpointer data)
     g_free(dataFile->readMinMax);
   if (dataFile->file)
     g_free(dataFile->file);
+  if (dataFile->shade)
+    shadeFree(dataFile->shade);
   g_free(dataFile);
 
   /* Update the cache. */
@@ -223,11 +232,13 @@ void dataFileActivate(VisuData *visuData, gboolean status)
     {
       visuElementSet_updateNodesOnMaterialChange();
       visuDataSet_ColorFunc(visuData, colorFromUserData);
+      visuDataSet_nodeScalingFunc(visuData, radiusFromUserData);
     }
   else
     {
       visuElementUnset_updateNodesOnMaterialChange();
       visuDataSet_ColorFunc(visuData, (setColorFunc)0);
+      visuDataSet_nodeScalingFunc(visuData, (VisuDataScalingFunc)0);
     }
 }
 
@@ -287,7 +298,7 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
   /* Test if this VisuData has already been colorized by testing
      the existence of the 'dataColor_used' property. If not, all required
      memory is allocated. */
-  dataFile = visuDataGet_property(attachedVisuData, "dataColor_parameters");
+  dataFile = g_object_get_data(G_OBJECT(attachedVisuData), "dataColor_parameters");
   if (!dataFile)
     dataFile = dataFileNew(attachedVisuData);
   else
@@ -299,7 +310,7 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
   *errors = 0;
 
   /* erase previously stored dataFile informations. */
-  dataFile->nbColumns = DATAFILE_NB_COLUMN_DEFAULT;
+  dataFile->nbColumns = 0;
   for (i = 0; i < 3; i++)
     {
       colUsed[i] = dataFile->colUsed[i];
@@ -368,7 +379,7 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
       voidLine = (line[0] == '#' || line[0] == '!' || line[0] == '\0');
 
       /* If still not done, we test the number of columns. */
-      if (dataFile->nbColumns == DATAFILE_NB_COLUMN_DEFAULT)
+      if (dataFile->nbColumns == 0)
 	{
 	  if (!voidLine)
 	    {
@@ -385,7 +396,7 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
 	      DBG_fprintf(stderr, "Visu dataFile : detected number of columns : %d\n",
 			  dataFile->nbColumns);
 	    }
-	  if (dataFile->nbColumns <= 0 || voidLine)
+	  if (dataFile->nbColumns == 0 || voidLine)
 	    {
 	      g_string_append_printf(message, _("Can't find any column of"
 						" data in the given file.\n"));
@@ -454,7 +465,7 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
   fclose(file);
   dataFile->file = g_strdup(filename);
 
-  if (dataFile->nbColumns <= 0)
+  if (dataFile->nbColumns == 0)
     {
       g_string_append_printf(message,
 			     _("Can't find any columns with numbers.\n"
@@ -486,38 +497,52 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename,
 
   return 1;
 }
+
+static DataFile* _get_dataColor(VisuData *data, gboolean create, gboolean *new)
+{
+  DataFile *dataFile;
+
+  g_return_val_if_fail(data, (DataFile*)0);
+
+  DBG_fprintf(stderr, "Visu DataFile: get data for %p (%p %p).\n",
+	      (gpointer)data, (gpointer)cacheVisuData, (gpointer)cacheDataFile);
+  if (new)
+    *new = FALSE;
+  if (data == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)g_object_get_data(G_OBJECT(data), "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = data;
+    }
+  if (!dataFile && create)
+    {
+      DBG_fprintf(stderr, "Visu DataFile: get data and create new.\n");
+      dataFile = dataFileNew(data);
+      if (new)
+	*new = TRUE;
+    }
+  return dataFile;
+}
+
 gchar* dataFileGet_file(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);
+
   if (dataFile)
     return dataFile->file;
   else
     return (gchar*)0;
 }
-gboolean dataFileGet_fileMinMaxFromColumn(VisuData *visuData, float minMax[2], int column)
+gboolean dataFileGet_fileMinMaxFromColumn(VisuData *visuData,
+					  float minMax[2], int column)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData && minMax, FALSE);
-
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);
   g_return_val_if_fail(dataFile, FALSE);
   g_return_val_if_fail(column >= 0 && column < dataFile->nbColumns, FALSE);
 
@@ -526,157 +551,100 @@ gboolean dataFileGet_fileMinMaxFromColumn(VisuData *visuData, float minMax[2], i
   return TRUE;
 }
 
-
-int dataFileSet_used(VisuData *data, int val)
+gboolean dataFileSet_used(VisuData *visuData, int val)
 {
   DataFile *dataFile;
+  gboolean new;
 
-  DBG_fprintf(stderr, "Visu dataFile : set using flag for dataFile module to %d .\n", val);
+  DBG_fprintf(stderr, "Visu dataFile: set using flag for"
+	      " dataFile module to %d .\n", val);
 
-  if (data == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = data;
-    }
-  if (!dataFile)
-    return 0;
+  dataFile = _get_dataColor(visuData, TRUE, &new);
+  g_return_val_if_fail(dataFile, FALSE);
 
   if (val == dataFile->used)
-    return 0;
+    return FALSE;
 
   dataFile->used = val;
 
-  dataFileActivate(data, dataFile->used && dataFile->file);
-  if (dataFile->file)
-    return 1;
-  else
-    return 0;
+  dataFileActivate(visuData, dataFile->used);
+  return !new;
 }
-int dataFileGet_used(VisuData *data)
+gboolean dataFileGet_used(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  if (data == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = data;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);
   
   if (!dataFile)
-    return 0;
+    return FALSE;
   else
-    return (int)dataFile->used;
+    return dataFile->used;
 }
 
-int dataFileSet_scaleType(VisuData *visuData, DataFileInputScaleId scale)
+gboolean dataFileSet_scaleType(VisuData *visuData, DataFileInputScaleId scale)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);  
+  g_return_val_if_fail(dataFile, FALSE);
 
-  DBG_fprintf(stderr, "Visu dataFile : set the scale type to %d (previuosly %d).\n",
+  DBG_fprintf(stderr, "Visu dataFile: set the scale type to %d (previuosly %d).\n",
 	      scale, dataFile->scaleType);
   if (scale == dataFile->scaleType)
-    return 0;
+    return FALSE;
   dataFile->scaleType = scale;
 
-  return dataFile->used && dataFile->file;
+  return dataFile->used;
 }
 DataFileInputScaleId dataFileGet_scaleType(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, DATAFILE_SCALE_TYPE_DEFAULT);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
     return DATAFILE_SCALE_TYPE_DEFAULT;
   else
     return dataFile->scaleType;
 }
-int dataFileSet_min(VisuData *visuData, float min)
+gboolean dataFileSet_min(VisuData *visuData, float min)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);
+  g_return_val_if_fail(dataFile, FALSE);
 /*   g_return_val_if_fail(min < dataFile->minMaxValues[1], 0); */
 
-  DBG_fprintf(stderr, "Visu dataFile : set the min value to %f (previuosly %f).\n",
+  DBG_fprintf(stderr, "Visu dataFile: set the min value to %f (previuosly %f).\n",
 	      min, dataFile->minMaxValues[0]);
   if (dataFile->minMaxValues[0] == min)
-    return 0;
+    return FALSE;
   dataFile->minMaxValues[0] = min;
 
-  return dataFile->used && dataFile->file;
+  return dataFile->used;
 }
-int dataFileSet_max(VisuData *visuData, float max)
+gboolean dataFileSet_max(VisuData *visuData, float max)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);  
+  g_return_val_if_fail(dataFile, FALSE);
 /*   g_return_val_if_fail(max > dataFile->minMaxValues[0], 0); */
 
-  DBG_fprintf(stderr, "Visu dataFile : set the max value to %f (previuosly %f).\n",
+  DBG_fprintf(stderr, "Visu dataFile: set the max value to %f (previuosly %f).\n",
 	      max, dataFile->minMaxValues[1]);
   if (dataFile->minMaxValues[1] == max)
-    return 0;
+    return FALSE;
   dataFile->minMaxValues[1] = max;
 
-  return dataFile->used && dataFile->file;
+  return dataFile->used;
 }
 float dataFileGet_min(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, DATAFILE_MIN_NORM_DEFAULT);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
     return DATAFILE_MIN_NORM_DEFAULT;
@@ -687,15 +655,7 @@ float dataFileGet_max(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, DATAFILE_MAX_NORM_DEFAULT);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
     return DATAFILE_MAX_NORM_DEFAULT;
@@ -707,15 +667,7 @@ int dataFileGet_nbColumns(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, DATAFILE_NB_COLUMN_DEFAULT);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
     return DATAFILE_NB_COLUMN_DEFAULT;
@@ -723,204 +675,89 @@ int dataFileGet_nbColumns(VisuData *visuData)
     return dataFile->nbColumns;
 }
 
-int dataFileSet_vectA(VisuData *visuData, float val, int pos)
+gboolean dataFileSet_colUsed(VisuData *visuData, int val, int pos)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
+  g_return_val_if_fail(pos >= 0 && pos < 3, FALSE);
 
-  DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectA to %f (previuosly %f).\n",
-	      pos, val, dataFile->vectA[pos]);
-  if (dataFile->vectA[pos] == val)
-    return 0;
-  dataFile->vectA[pos] = val;
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);  
+  g_return_val_if_fail(dataFile, FALSE);
 
-  return dataFile->used && dataFile->file;
-}
-int dataFileSet_vectB(VisuData *visuData, float val, int pos)
-{
-  DataFile *dataFile;
-
-  g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
-
-  DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectB to %f (previuosly %f).\n",
-	      pos, val, dataFile->vectB[pos]);
-  if (dataFile->vectB[pos] == val)
-    return 0;
-  dataFile->vectB[pos] = val;
-
-  return dataFile->used && dataFile->file;
-}
-float* dataFileGet_vectA(VisuData *visuData)
-{
-  DataFile *dataFile;
-
-  g_return_val_if_fail(visuData, (float*)0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-
-  if (!dataFile)
-    return (float*)0;
-  else
-    return dataFile->vectA;
-}
-float* dataFileGet_vectB(VisuData *visuData)
-{
-  DataFile *dataFile;
-
-  g_return_val_if_fail(visuData, (float*)0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-
-  if (!dataFile)
-    return (float*)0;
-  else
-    return dataFile->vectB;
-}
-int dataFileSet_colUsed(VisuData *visuData, int val, int pos)
-{
-  DataFile *dataFile;
-
-  g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
-
-  g_return_val_if_fail(val < dataFile->nbColumns || val == DATAFILE_NB_COLUMN_DEFAULT, 0);
-
-  DBG_fprintf(stderr, "Visu dataFile : channel %d uses column %d (previuosly %d).\n",
+  DBG_fprintf(stderr, "Visu dataFile: channel %d uses column %d (previuosly %d).\n",
 	      pos, val, dataFile->colUsed[pos]);
+  g_return_val_if_fail(val < dataFile->nbColumns &&
+		       val >= DATAFILE_NB_COLUMN_DEFAULT, FALSE);
+
   if (dataFile->colUsed[pos] == val)
-    return 0;
+    return FALSE;
   dataFile->colUsed[pos] = val;
 
-  return dataFile->used && dataFile->file;
+  return dataFile->used;
 }
 int* dataFileGet_colUsed(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, (int*)0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
     return (int*)0;
   else
     return dataFile->colUsed;
 }
-int dataFileSet_colorType(VisuData *visuData, DataFileColorModeId color)
+gboolean dataFileSet_scalingUsed(VisuData *visuData, int val)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, 0);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
-  g_return_val_if_fail(dataFile, 0);
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);  
+  g_return_val_if_fail(dataFile, FALSE);
 
-  DBG_fprintf(stderr, "Visu dataFile : set the color type to %d (previuosly %d).\n",
-	      color, dataFile->colorType);
-  if (color == dataFile->colorType)
-    return 0;
-  dataFile->colorType = color;
+  DBG_fprintf(stderr, "Visu dataFile: scaling uses column %d (previuosly %d).\n",
+	      val, dataFile->scaleUsed);
+  g_return_val_if_fail((val < dataFile->nbColumns &&
+			val >= 0) || val == DATAFILE_NB_COLUMN_DEFAULT, FALSE);
 
-  return dataFile->used && dataFile->file;
+  if (dataFile->scaleUsed == val)
+    return FALSE;
+  dataFile->scaleUsed = val;
+
+  return dataFile->used;
 }
-DataFileColorModeId dataFileGet_colorType(VisuData *visuData)
+int dataFileGet_scalingUsed(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData, DATAFILE_COLOR_TYPE_DEFAULT);
-  if (visuData == cacheVisuData)
-    dataFile = cacheDataFile;
-  else
-    {
-      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-      cacheDataFile = dataFile;
-      cacheVisuData = visuData;
-    }
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);  
 
   if (!dataFile)
-    return DATAFILE_COLOR_TYPE_DEFAULT;
+    return DATAFILE_NB_COLUMN_DEFAULT;
   else
-    return dataFile->colorType;
+    return dataFile->scaleUsed;
 }
-int dataFileSet_shade(VisuData *visuData, Shade *shade)
+gboolean dataFileSet_shade(VisuData *visuData, Shade *shade)
 {
-  float *vectA, *vectB;
-  gboolean *vectX, ok;
-  int i, res;
   DataFile *dataFile;
 
-  g_return_val_if_fail(visuData && shade, (int)0);
+  DBG_fprintf(stderr, "Visu DataFile: set shade.\n");
+  dataFile = _get_dataColor(visuData, TRUE, (gboolean*)0);
+  g_return_val_if_fail(dataFile, FALSE);
 
-  res = dataFileSet_colorType(visuData, shadeGet_colorMode(shade));
-  ok = shadeGet_colorTransformation(shade, &vectA, &vectB, &vectX);
-  if (!ok)
-    {
-      g_warning("Can't get the color transformation.");
-      return 0;
-    }
-  for (i = 0; i < 3; i++)
-    {
-      res = dataFileSet_vectA(visuData, vectA[i], i) || res;
-      res = dataFileSet_vectB(visuData, vectB[i], i) || res;
-      if (vectX[i])
-	res = dataFileSet_colUsed(visuData, 0, i) || res;
-      else
-	res = dataFileSet_colUsed(visuData, DATAFILE_NB_COLUMN_DEFAULT, i) || res;
-    }
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-  g_return_val_if_fail(dataFile, 0);
-  return dataFile->used && res;
+  shadeFree(dataFile->shade);
+  dataFile->shade = shadeCopy(shade);
+
+  return dataFile->used;
+}
+Shade* dataFileGet_shade(VisuData *visuData)
+{
+  DataFile *dataFile;
+
+  dataFile = _get_dataColor(visuData, FALSE, (gboolean*)0);
+
+  if (!dataFile)
+    return (Shade*)0;
+  else
+    return dataFile->shade;
 }
 gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float value)
 {
@@ -939,10 +776,10 @@ gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float valu
       return FALSE;
     }
   
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  dataFile = (DataFile*)g_object_get_data(G_OBJECT(visuData), "dataColor_parameters");
   if (!dataFile)
     return FALSE;
-  if (dataFile->nbColumns == DATAFILE_NB_COLUMN_DEFAULT)
+  if (dataFile->nbColumns == 0)
     return FALSE;
 
   g_return_val_if_fail(column > DATAFILE_NB_COLUMN_DEFAULT && column < dataFile->nbColumns, FALSE);
@@ -974,7 +811,8 @@ gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float valu
    pointing to the right place in the visuData object that needs to be rendered. */
 
 /* Normalization method */
-float dataFileGet_valuesFromData(VisuData *visuData _U_, int column, float fromVal)
+static float dataFileGet_valuesFromData(VisuData *visuData _U_,
+					int column, float fromVal)
 {
   float res;
 
@@ -986,64 +824,94 @@ float dataFileGet_valuesFromData(VisuData *visuData _U_, int column, float fromV
       res = ( (fromVal - cacheDataFile->readMinMax[column * 2 + 0]) /
 	      (cacheDataFile->readMinMax[column * 2 + 1] -
 	       cacheDataFile->readMinMax[column * 2 + 0]) );
+      DBG_fprintf(stderr, "Visu DataFile: normalise %f -> %f.\n", fromVal, res);
       return res;
     case dataFile_minMax:
       res = ( (fromVal - cacheDataFile->minMaxValues[0]) /
 	      (cacheDataFile->minMaxValues[1] - cacheDataFile->minMaxValues[0]) );
+      DBG_fprintf(stderr, "Visu DataFile: normalise %f -> %f.\n",
+		  fromVal, CLAMP(res, 0., 1.));
       return CLAMP(res, 0., 1.);
     }
   return 0.;
 }
-/* Colourization method */
-void dataFileGet_valuesTransformedInRGB(VisuData *visuData,
-					float* valuesTransformed, float* valuesFrom)
-{
-  int i;
-
-  g_return_if_fail(visuData && valuesTransformed && valuesFrom);
-
-  for (i = 0; i < 3; i++)
-    {
-      valuesTransformed[i] = cacheDataFile->vectA[i] * valuesFrom[i] + cacheDataFile->vectB[i];
-      valuesTransformed[i] = (valuesTransformed[i] < 0.)?0.:valuesTransformed[i];
-      valuesTransformed[i] = (valuesTransformed[i] > 1.)?1.:valuesTransformed[i];
-    }
-  switch(cacheDataFile->colorType)
-    {
-    case dataFile_rgb:
-      return;
-    case dataFile_hsv:
-      color_HSVtoRGB(valuesTransformed, valuesTransformed);
-      break;
-    }
-}
-
 
 /*******************/
 /* Color functions */
 /*******************/
-
-void colorFromUserData(VisuData *visuData, float rgba[4], VisuElement *ele, VisuNode* node)
+static void colorFromUserData(VisuData *visuData, float rgba[4],
+			      VisuElement *ele, VisuNode* node)
 {
-  float val[3];
+  float val[3], red[3], coord[3], delta;
   int i;
   float *storedValues;
+  gboolean useCoord, useData;
 
   g_return_if_fail(visuData && node && ele && rgba);
 
-  visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
-			    DATAFILE_ID, &dataValue);
-  storedValues = (float*)g_value_get_pointer(&dataValue);
-  if (!storedValues)
-    return;
+  useCoord = (cacheDataFile->colUsed[0] == DATAFILE_COLUMN_X ||
+	      cacheDataFile->colUsed[0] == DATAFILE_COLUMN_Y ||
+	      cacheDataFile->colUsed[0] == DATAFILE_COLUMN_Z ||
+	      cacheDataFile->colUsed[1] == DATAFILE_COLUMN_X ||
+	      cacheDataFile->colUsed[1] == DATAFILE_COLUMN_Y ||
+	      cacheDataFile->colUsed[1] == DATAFILE_COLUMN_Z ||
+	      cacheDataFile->colUsed[2] == DATAFILE_COLUMN_X ||
+	      cacheDataFile->colUsed[2] == DATAFILE_COLUMN_Y ||
+	      cacheDataFile->colUsed[2] == DATAFILE_COLUMN_Z);
+  useData = (cacheDataFile->colUsed[0] >= 0 ||
+	     cacheDataFile->colUsed[1] >= 0 ||
+	     cacheDataFile->colUsed[2] >= 0);
+
+  storedValues = (float*)0;
+  if (useData)
+    {
+      visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
+				DATAFILE_ID, &dataValue);
+      storedValues = (float*)g_value_get_pointer(&dataValue);
+      g_return_if_fail(storedValues);
+    }
+  if (useCoord)
+    {
+      visuDataGet_nodePosition(visuData, node, coord);
+      visuDataConvert_XYZtoBoxCoordinates(visuData, red, coord);
+      if (cacheDataFile->scaleType == dataFile_minMax)
+	{
+	  delta = cacheDataFile->minMaxValues[1] - cacheDataFile->minMaxValues[0];
+	  red[0] = CLAMP((red[0] - cacheDataFile->minMaxValues[0]) / delta, 0.f, 1.f);
+	  red[1] = CLAMP((red[1] - cacheDataFile->minMaxValues[0]) / delta, 0.f, 1.f);
+	  red[2] = CLAMP((red[2] - cacheDataFile->minMaxValues[0]) / delta, 0.f, 1.f);
+	}
+    }
 
   for (i = 0; i < 3; i++)
-    if (cacheDataFile->colUsed[i] == -1)
+    if (cacheDataFile->colUsed[i] == DATAFILE_NB_COLUMN_DEFAULT)
       val[i] = 1.;
+    else if (cacheDataFile->colUsed[i] == DATAFILE_COLUMN_X)
+      val[i] = fModulo(red[0], 1);
+    else if (cacheDataFile->colUsed[i] == DATAFILE_COLUMN_Y)
+      val[i] = fModulo(red[1], 1);
+    else if (cacheDataFile->colUsed[i] == DATAFILE_COLUMN_Z)
+      val[i] = fModulo(red[2], 1);
     else
       val[i] = dataFileGet_valuesFromData(visuData, cacheDataFile->colUsed[i],
 					  storedValues[cacheDataFile->colUsed[i]]);
 
-  dataFileGet_valuesTransformedInRGB(visuData, rgba, val);
-  rgba[3] = ele->rgb[3];
+  shadeGet_RGBFromValues(cacheDataFile->shade, rgba, val);
+/*   fprintf(stderr, "%f %f %f  ->  %f %f %f %f\n", */
+/* 	  val[0], val[1], val[2], rgba[0], rgba[1], rgba[2], rgba[3]); */
+}
+static float radiusFromUserData(VisuData *visuData, VisuNode *node)
+{
+  float *storedValues;
+
+  if (cacheDataFile->scaleUsed == DATAFILE_NB_COLUMN_DEFAULT)
+    return 1.f;
+
+  storedValues = (float*)0;
+  visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
+			    DATAFILE_ID, &dataValue);
+  storedValues = (float*)g_value_get_pointer(&dataValue);
+  return dataFileGet_valuesFromData(visuData, cacheDataFile->scaleUsed,
+				    storedValues[cacheDataFile->scaleUsed]);
+
 }
