@@ -269,7 +269,8 @@ DataNode* nodeDataNew(const gchar *name, GType type)
 
   g_return_val_if_fail(name, (DataNode*)0);
   g_return_val_if_fail(type == G_TYPE_INT || type == G_TYPE_FLOAT ||
-		       type == G_TYPE_BOOLEAN, (DataNode*)0);
+		       type == G_TYPE_BOOLEAN || type == G_TYPE_STRING,
+		       (DataNode*)0);
 
   /* Create the object with defaulty values. */
   data = DATA_NODE(g_object_new(DATA_NODE_TYPE, NULL));
@@ -351,6 +352,9 @@ void nodeDataSet_used(DataNode *data, VisuData *dataObj, gint nb)
   storage = getInternalStorage(data, dataObj);
   if (nb > 0)
     {
+      DBG_fprintf(stderr, "Data Node: associating %p to the node property '%s' (%d).\n",
+		  (gpointer)dataObj, data->id, g_list_length(data->lstVisuData));
+
       /* Add a new associated #VisuData or change an already
 	 associated one. */
       if (!storage)
@@ -362,16 +366,14 @@ void nodeDataSet_used(DataNode *data, VisuData *dataObj, gint nb)
 						     G_CALLBACK(onVisuDataFreed),
 						     (gpointer)data);
 	  data->lstVisuData = g_list_prepend(data->lstVisuData, (gpointer)storage);
+
+	  DBG_fprintf(stderr, "Data Node: %p emit the 'propertyUsed'.\n",
+		      (gpointer)data);
+	  g_signal_emit(data, dataNode_signals[DATA_NODE_USED_SIGNAL],
+			0 , dataObj, NULL);
 	}
       else
 	storage->dimension = nb;
-
-      DBG_fprintf(stderr, "Data Node : associating %p to the node property '%s' (%d).\n",
-		  (gpointer)dataObj, data->id, g_list_length(data->lstVisuData));
-
-      DBG_fprintf(stderr, "Data Node : %p emit the 'propertyUsed'.\n", (gpointer)data);
-      g_signal_emit(data, dataNode_signals[DATA_NODE_USED_SIGNAL],
-		    0 , dataObj, NULL);
     }
   else
     {
@@ -420,6 +422,8 @@ gboolean nodeDataSet_valueAsString(DataNode *data, VisuData *dataObj,
   DBG_fprintf(stderr, "Node Data: read value from '%s' for data '%s'.\n",
 	      labelIn, data->id);
   valid = data->setAsString(data, dataObj, node, labelIn, labelOut, &modify);
+  DBG_fprintf(stderr, "Node Data: read %d to '%s' (%d).\n",
+	      valid, *labelOut, modify);
 
   /* Call the callback if values have been modified. */
   if (valid && modify)
@@ -492,11 +496,11 @@ static gchar* valueAsString(DataNode *data, VisuData *dataObj,
   g_return_val_if_fail(IS_DATA_NODE_TYPE(data) && dataObj && node, (gchar*)0);
 
   /* Check if the given property has an association with the given VisuData. */
-  DBG_fprintf(stderr, "Data Node : get label for node property '%s'.\n", data->id);
+  DBG_fprintf(stderr, "Data Node: get label for node property '%s'.\n", data->id);
   storage = getInternalStorage(data, dataObj);
   if (!storage || storage->dimension <= 0)
     {
-      DBG_fprintf(stderr, "Data Node : no associated data.\n");
+      DBG_fprintf(stderr, "Data Node: no associated data.\n");
       return (gchar*)0;
     }
 
@@ -505,10 +509,17 @@ static gchar* valueAsString(DataNode *data, VisuData *dataObj,
 			    (gchar*)data->id, &val);
   dataValues = g_value_get_pointer(&val);
   if (!dataValues)
-    return (gchar*)0;
+    {
+      if (data->type != G_TYPE_STRING)
+	return (gchar*)0;
+      else
+	return g_strdup("");
+    }
 
   /* Set the format attribute. */
-  str = g_string_new("( ");
+  str = g_string_new("");
+  if (storage->dimension > 1)
+    g_string_append(str, "( ");
   for (i = 0; i < storage->dimension; i++)
     {
       switch (data->type)
@@ -527,17 +538,21 @@ static gchar* valueAsString(DataNode *data, VisuData *dataObj,
 	    /* F for False. */
 	    g_string_append(str, _("F"));
 	  break;
+	case G_TYPE_STRING:
+	  g_string_append(str, (gchar*)dataValues);
+	  break;
 	default:
 	  g_warning("Unsupported type for DataNode");
 	}
       if (i < storage->dimension - 1)
 	g_string_append(str, " ; ");
     }
-  g_string_append(str, " )");
+  if (storage->dimension > 1)
+    g_string_append(str, " )");
   value = str->str;
   g_string_free(str, FALSE);
 
-  DBG_fprintf(stderr, "DataNode : get values '%s'.\n", value);
+  DBG_fprintf(stderr, "Data Node: get values '%s'.\n", value);
 
   return value;
 }
@@ -546,7 +561,7 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
 				gchar **labelOut, gboolean *modify)
 {
   int res, ln, i, valueInt;
-  gchar **datas;
+  gchar **datas, *str;
   gpointer dataValues;
   gboolean error;
   struct internalUsedVisuData_struct *storage;
@@ -560,8 +575,8 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
   if (!data->editable)
     {
       *labelOut = g_strdup(_("No data"));
-      g_error("Can't call 'nodeDataSet_valueAsString' since the property"
-	      " '%s' is not editable.", data->id);
+      g_warning("Can't call 'nodeDataSet_valueAsString' since the property"
+		" '%s' is not editable.", data->id);
       return FALSE;
     }
 
@@ -570,8 +585,8 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
   if (!storage)
     {
       *labelOut = g_strdup(_("No data"));
-      g_error("Can't call 'nodeDataSet_valueAsString' since the property"
-	      " '%s' has not been associated with the given VisuData.", data->id);
+      g_warning("Can't call 'nodeDataSet_valueAsString' since the property"
+		" '%s' has not been associated with the given VisuData.", data->id);
       return FALSE;
     }
   
@@ -580,11 +595,11 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
   visuNodeGet_propertyValue(visuDataGet_nodeArray(dataObj), node,
 			    (gchar*)data->id, &val);
   dataValues = g_value_get_pointer(&val);
-  if (!dataValues)
+  if (data->type != G_TYPE_STRING && !dataValues)
     {
       *labelOut = g_strdup(_("No data"));
-      g_error("Can't call 'nodeDataSet_valueAsString' since %p has no"
-	      " node property labelled '%s'", (gpointer)dataObj, data->id);
+      g_warning("Can't call 'nodeDataSet_valueAsString' since %p has no"
+		" node property labelled '%s'", (gpointer)dataObj, data->id);
       return FALSE;
     }
 
@@ -599,6 +614,7 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
   *modify = FALSE;
   for (i = 0; datas[i]; i++)
     {
+      DBG_fprintf(stderr, "Data Node: reading value '%s'.\n", datas[i]);
       error = FALSE;
       switch (data->type)
 	{
@@ -646,6 +662,13 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
 	  else
 	    error = TRUE;
 	  break;
+	case G_TYPE_STRING:
+	  str = g_strdup(datas[i]);
+	  g_strstrip(str);
+	  g_value_set_pointer(&val, (gpointer)str);
+	  visuNodeSet_propertyValue(visuDataGet_nodeArray(dataObj), node,
+				    (gchar*)data->id, &val);
+	  break;
 	default:
 	  g_warning("Unsupported type for DataNode");
 	  error = TRUE;
@@ -657,13 +680,21 @@ static gboolean valueFromString(DataNode *data, VisuData *dataObj,
 	  return FALSE;
 	}
     }
+  g_strfreev(datas);
+  /* Special case of empty string. */
+  if (data->type == G_TYPE_STRING && i == 0)
+    {
+      g_value_set_pointer(&val, (gpointer)0);
+      visuNodeSet_propertyValue(visuDataGet_nodeArray(dataObj), node,
+				(gchar*)data->id, &val);
+      i = 1;
+    }
   if (i != storage->dimension)
     error = TRUE;
   else
     error = FALSE;
 
   *labelOut = valueAsString(data, dataObj, node);
-  g_strfreev(datas);
 
   return !error;
 }
