@@ -55,6 +55,12 @@
 #include "surfaces_points.h"
 #include "isoline.h"
 
+#ifdef HAVE_CAIRO
+#include <cairo.h>
+#include <cairo-svg.h>
+#include <cairo-pdf.h>
+#endif
+
 /**
  * SECTION:scalarFields
  * @short_description:Gives capabilities to load a scalar field.
@@ -137,6 +143,8 @@ enum
     MAP_X,
     MAP_Y,
     MAP_Z,
+    MAP_U,
+    MAP_V,
     MAP_VISIBILITY,
     MAP_VALUE,
     MAP_R,
@@ -267,7 +275,7 @@ static gboolean scalarFieldLoad_fromAscii(const gchar *filename, GList **fieldLi
   gchar *comment;
   int res, i, j, k;
   int size[3];
-  float box[6];
+  double box[6];
   ScalarField *field;
   gboolean periodic;
   ScalarField_meshflag meshtype;
@@ -305,7 +313,7 @@ static gboolean scalarFieldLoad_fromAscii(const gchar *filename, GList **fieldLi
     }
 
   (void)fgets(rep, MAX_LINE_LENGTH, in);
-  res = sscanf(rep, "%f %f %f", box, box + 1, box + 2);
+  res = sscanf(rep, "%lf %lf %lf", box, box + 1, box + 2);
   if (res != 3)
     {
       /* Not a valid ASCII format. */
@@ -314,7 +322,7 @@ static gboolean scalarFieldLoad_fromAscii(const gchar *filename, GList **fieldLi
       return FALSE;
     }
   (void)fgets(rep, MAX_LINE_LENGTH, in); 
-  res = sscanf(rep, "%f %f %f", box + 3, box + 4, box + 5);
+  res = sscanf(rep, "%lf %lf %lf", box + 3, box + 4, box + 5);
   if (res != 3)
     {
       /* Not a valid ASCII format. */
@@ -561,12 +569,13 @@ void scalarFieldSet_gridSize(ScalarField *field, int grid[3])
       field->nElements[2] == grid[2])
     return;
   
-  DBG_fprintf(stderr, "ScalarField : changing size from (%d ; %d ; %d)"
+  DBG_fprintf(stderr, "ScalarField: changing size from (%d ; %d ; %d)"
                       " to (%d ; %d ; %d).\n", field->nElements[0], field->nElements[1],
                       field->nElements[2], grid[0], grid[1], grid[2]);
 
   if (field->mesh_type == nonuniform)
     {
+      DBG_fprintf(stderr, " |free the previous mesh allocation.\n");
       /* If mesh was already allocated, we free it. */
       if (field->meshx)
         g_free(field->meshx);
@@ -581,6 +590,7 @@ void scalarFieldSet_gridSize(ScalarField *field, int grid[3])
   /* If data was already allocated, we free it. */
   if (field->data)
     {
+      DBG_fprintf(stderr, " |free the previous data allocation.\n");
       for (i = 0; i < field->nElements[0]; i++)
 	      {
           for (j = 0; j < field->nElements[1]; j++)
@@ -597,17 +607,17 @@ void scalarFieldSet_gridSize(ScalarField *field, int grid[3])
 
   if (field->mesh_type == nonuniform)
     {
-      DBG_fprintf(stderr, "ScalarField : allocating meshx array.\n");
+      DBG_fprintf(stderr, "ScalarField: allocating meshx array.\n");
       field->meshx = g_malloc(sizeof(double) * grid[0]);
 
-      DBG_fprintf(stderr, "ScalarField : allocating meshy array.\n");
+      DBG_fprintf(stderr, "ScalarField: allocating meshy array.\n");
       field->meshy = g_malloc(sizeof(double) * grid[1]);
 
-      DBG_fprintf(stderr, "ScalarField : allocating meshz array.\n");
+      DBG_fprintf(stderr, "ScalarField: allocating meshz array.\n");
       field->meshz = g_malloc(sizeof(double) * grid[2]);
     }
 
-  DBG_fprintf(stderr, "ScalarField : allocating data array.\n");
+  DBG_fprintf(stderr, "ScalarField: allocating data array.\n");
   field->data = g_malloc(sizeof(double **) * grid[0]);
   for(i = 0; i < grid[0]; i++)
     {
@@ -615,6 +625,7 @@ void scalarFieldSet_gridSize(ScalarField *field, int grid[3])
       for(j = 0; j < grid[1]; j++)
         field->data[i][j] = g_malloc(sizeof(double) * grid[2]);
     }
+  DBG_fprintf(stderr, " | allocation done.\n");
 }
 
 void scalarFieldGet_minMax(ScalarField *field, double minmax[2])
@@ -643,7 +654,7 @@ double scalarFieldGet_secondaryMin(ScalarField *field)
     }
   return field->secondMin;
 }
-void scalarFieldSet_data(ScalarField *field, double *data)
+void scalarFieldSet_data(ScalarField *field, double *data, gboolean xyzOrder)
 {
   int i, j, k, ii;
   
@@ -652,59 +663,146 @@ void scalarFieldSet_data(ScalarField *field, double *data)
   field->min = G_MAXFLOAT;
   field->max = -G_MAXFLOAT;
   ii = 0;
-  for (k = 0 ; k < field->nElements[2] ; k++)
-    for (j = 0 ; j < field->nElements[1] ; j++)
-      for (i = 0 ; i < field->nElements[0] ; i++)
-        {
-          field->data[i][j][k] = data[ii];
-          ii += 1;
-	  field->min = MIN(field->data[i][j][k], field->min);
-	  field->max = MAX(field->data[i][j][k], field->max);
-        }
+  if (xyzOrder)
+    for (k = 0 ; k < field->nElements[2] ; k++)
+      for (j = 0 ; j < field->nElements[1] ; j++)
+	for (i = 0 ; i < field->nElements[0] ; i++)
+	  {
+	    field->data[i][j][k] = data[ii];
+	    field->min = MIN(data[ii], field->min);
+	    field->max = MAX(data[ii], field->max);
+	    ii += 1;
+	  }
+  else
+    for (i = 0 ; i < field->nElements[0] ; i++)
+      for (j = 0 ; j < field->nElements[1] ; j++)
+	for (k = 0 ; k < field->nElements[2] ; k++)
+	  {
+	    field->data[i][j][k] = data[ii];
+	    field->min = MIN(data[ii], field->min);
+	    field->max = MAX(data[ii], field->max);
+	    ii += 1;
+	  }
 }
-void scalarFieldDraw_map(ScalarField *field, Plane *plane, Shade *shade,
-                         OpenGLView *view, float precision, float drawnMinMax[2],
-			 gboolean logScale, guint nIsoLines)
+
+static void planeTo3D(float xyz[3], float uv[2], float basis[2][3], float origin[3])
 {
-  float xyz[2][3], normal[3], spherical[3], center[3], uMinMax[2], vMinMax[2];
-  float dist, Xcoeff, Ycoeff, coeff, uCoord, vCoord;
-  SurfacesPoints points;
-  double value, minmax[2], minlog;
-  float valMinMax[2];
+  xyz[0] = origin[0] + uv[0] * basis[0][0] + uv[1] * basis[1][0];
+  xyz[1] = origin[1] + uv[0] * basis[0][1] + uv[1] * basis[1][1];
+  xyz[2] = origin[2] + uv[0] * basis[0][2] + uv[1] * basis[1][2];
+}
+static gboolean setValue(ScalarField *field, Shade *shade, matrixGet_scaleVal get_val,
+			 double minmax[2], double logval, float drawnMinMax[2],
+			 float extension[3], float xyz[3], float *val, float rgba[4])
+{
+  gboolean inside;
+  double value;
+
+  inside = scalarFieldGet_value(field, xyz, &value, extension);
+  if (inside)
+    {
+      *val = (float)get_val(value, minmax, logval);
+      shadeGet_valueTransformedInRGB(shade, rgba, *val);
+      drawnMinMax[0] = MIN(drawnMinMax[0], (float)value);
+      drawnMinMax[1] = MAX(drawnMinMax[1], (float)value);
+    }
+  return inside;
+}
+
+void drawMapAndLines(SurfacesPoints *points, Plane *plane, guint nIsoLines,
+		     Shade *shade, float *rgb, float valMinMax[2])
+{
+  int i;
   GList *inter;
-  int alpha, i, j, k, uAlpha[2], vAlpha[2], nU, nV, nPolys, nPoints;
-  int iPolys[4];
+  float v, rgba[4];
   Line *line;
-  #if DEBUG == 1
-  GTimer *timer;
-  gulong fractionTimer;
-  #endif
+
+  glDisable(GL_CULL_FACE);
+  glDisable(GL_LIGHTING);
+/*   glShadeModel(GL_FLAT); */
+/*   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); */
+  DBG_fprintf(stderr, "ScalarFields: plot triangles (%d).\n", points->num_polys);
+  for (i = 0; i < points->num_polys; i++)
+    {
+      if (points->poly_surf_index[i] > 0)
+	{
+	  glBegin(GL_TRIANGLE_STRIP);
+	  glColor3fv(points->poly_points_data[points->poly_vertices[i][0]] + MAP_R);
+	  glVertex3fv(points->poly_points_data[points->poly_vertices[i][0]]);
+	  glColor3fv(points->poly_points_data[points->poly_vertices[i][1]] + MAP_R);
+	  glVertex3fv(points->poly_points_data[points->poly_vertices[i][1]]);
+	  glColor3fv(points->poly_points_data[points->poly_vertices[i][2]] + MAP_R);
+	  glVertex3fv(points->poly_points_data[points->poly_vertices[i][2]]);
+	  glEnd();
+	}
+    }
+/*   for (i = 0; i < points.num_points; i++) */
+/*     { */
+/*       sprintf(str, "%6.4f", points.poly_points_data[i][MAP_VALUE]); */
+/*       glRasterPos3fv(points.poly_points_data[i]); openGLText_drawChars(str); */
+/*     } */
+  DBG_fprintf(stderr, "ScalarFields: plot lines.\n");
+  for (i = 1; i <= (int)nIsoLines; i++)
+    {
+      v = valMinMax[0] + (valMinMax[1] - valMinMax[0]) *
+	(float)i / (float)(nIsoLines + 1);
+      if (isolineBuild(&line, points, MAP_VALUE, MAP_VISIBILITY, v))
+	{
+	  if (rgb)
+	    isolineDraw(line, rgb);
+	  else
+	    {
+	      shadeGet_valueTransformedInRGB(shade, rgba, v);
+	      rgba[0] = 1.f - rgba[0];
+	      rgba[1] = 1.f - rgba[1];
+	      rgba[2] = 1.f - rgba[2];
+	      rgba[3] = 1.f;
+	      isolineDraw(line, rgba);
+	    }
+	  isolineFree(line);
+	}
+    }
+
+  DBG_fprintf(stderr, "ScalarFields: plot box.\n");
+  glLineWidth(1.f);
+  glColor3f(0.f, 0.f, 0.f);
+  glBegin(GL_LINE_LOOP);
+  for (inter = planeGet_intersection(plane); inter; inter = g_list_next(inter))
+    glVertex3fv((float*)inter->data);
+  glEnd();
+
+  glEnable(GL_CULL_FACE);
+  glEnable(GL_LIGHTING);
+}
+static void computeMap(SurfacesPoints *points, float viewport[4],
+		       float valMinMax[2], float drawnMinMax[2],
+		       ScalarField *field, Plane *plane, Shade *shade,
+		       OpenGLView *view, float precision,
+		       matrix_scaleflag scale, float *inputMinMax)
+{
+  matrixGet_scaleVal get_val;
+  double logval;
+  float xyz[2][3], center[3], uMinMax[2], vMinMax[2];
+  float dist, coeff, *red, **data, a[2], *b, I[2], lambda;
+  double minmax[2];
+  GList *inter, *interRed;
+  int alpha, i, j, k, n, uAlpha[2], vAlpha[2];
+  int nU, nV, nPolys, nPoints, visible, hidden;
+  int *linkedPoints;
+  int iPolys[4];
 
   g_return_if_fail(field && shade && plane);
   g_return_if_fail(view && precision > 0);
 
-  #if DEBUG == 1
-    timer = g_timer_new();
-    g_timer_start(timer);
-  #endif
+  points->num_polys = 0;
 
-  planeGet_nVectUser(plane, normal);
-  matrix_cartesianToSpherical(spherical, normal);
-
-  planeGet_barycentre(plane, center);
-  /* We compute the min and max in the 3D basis. */
   inter = planeGet_intersection(plane);
   /* In case of no intersections, we build an empty list. */
   if (!inter)
     return;
   
   /* We compute the plane basis. */
-  xyz[0][0] = cos(spherical[1] * PI180) * cos(spherical[2] * PI180);
-  xyz[0][1] = cos(spherical[1] * PI180) * sin(spherical[2] * PI180);
-  xyz[0][2] = -sin(spherical[1] * PI180);
-  xyz[1][0] = -sin(spherical[2] * PI180);
-  xyz[1][1] =  cos(spherical[2] * PI180);
-  xyz[1][2] = 0.;
+  planeGet_basis(plane, xyz, center);
 
   /* We look for the min and max position in the plane basis for the
      different intersections. */
@@ -713,19 +811,20 @@ void scalarFieldDraw_map(ScalarField *field, Plane *plane, Shade *shade,
   uMinMax[1] = 0.;
   vMinMax[0] = 0.;
   vMinMax[1] = 0.;
-  while(inter)
+  for (interRed = (GList*)0; inter; inter = g_list_next(inter))
     {
-      uCoord = xyz[0][0] * (((float*)inter->data)[0] - center[0]) +
+      red = g_malloc(sizeof(float) * 2);
+      red[0] = xyz[0][0] * (((float*)inter->data)[0] - center[0]) +
 	xyz[0][1] * (((float*)inter->data)[1] - center[1]) +
 	xyz[0][2] * (((float*)inter->data)[2] - center[2]);
-      uMinMax[0] = MIN(uCoord, uMinMax[0]);
-      uMinMax[1] = MAX(uCoord, uMinMax[1]);
-      vCoord = xyz[1][0] * (((float*)inter->data)[0] - center[0]) +
+      uMinMax[0] = MIN(red[0], uMinMax[0]);
+      uMinMax[1] = MAX(red[0], uMinMax[1]);
+      red[1] = xyz[1][0] * (((float*)inter->data)[0] - center[0]) +
 	xyz[1][1] * (((float*)inter->data)[1] - center[1]) +
 	xyz[1][2] * (((float*)inter->data)[2] - center[2]);
-      vMinMax[0] = MIN(vCoord, vMinMax[0]);
-      vMinMax[1] = MAX(vCoord, vMinMax[1]);
-      inter = g_list_next(inter);
+      vMinMax[0] = MIN(red[1], vMinMax[0]);
+      vMinMax[1] = MAX(red[1], vMinMax[1]);
+      interRed = g_list_append(interRed, red);
     }
   /* So, in plane coordinates, X range in uMinMax and Y in vMinMax. */
 
@@ -737,76 +836,100 @@ void scalarFieldDraw_map(ScalarField *field, Plane *plane, Shade *shade,
 	      view->box->p7[1] * view->box->p7[1] +
 	      view->box->p7[2] * view->box->p7[2]);
 /*   dist = MAX(uMinMax[1] - uMinMax[0], vMinMax[1] - vMinMax[0]); */
-  uAlpha[0] = (int)(uMinMax[0] / dist * (float)alpha) - 1;
-  uAlpha[1] = (int)(uMinMax[1] / dist * (float)alpha) + 2;
-  vAlpha[0] = (int)(vMinMax[0] / dist * (float)alpha * 1.154700538379251448) - 1;
-  vAlpha[1] = (int)(vMinMax[1] / dist * (float)alpha * 1.154700538379251448) + 2;
-  scalarFieldGet_minMax(field, minmax);
+  uAlpha[0] = (int)(uMinMax[0] / dist * (float)alpha) - 2;
+  uAlpha[1] = (int)(uMinMax[1] / dist * (float)alpha) + 4;
+  vAlpha[0] = (int)(vMinMax[0] / dist * (float)alpha * 1.154700538379251448) - 2;
+  vAlpha[1] = (int)(vMinMax[1] / dist * (float)alpha * 1.154700538379251448) + 3;
+  if (!inputMinMax)
+    scalarFieldGet_minMax(field, minmax);
+  else
+    {
+      minmax[0] = inputMinMax[0];
+      minmax[1] = inputMinMax[1];
+    }
   drawnMinMax[0] = minmax[1];
   drawnMinMax[1] = minmax[0];
 
-  DBG_fprintf(stderr, "Panel Map: allocating storing values (%dx%d).\n",
+  /* In log scale, we need the second minimum value. */
+  switch (scale)
+    {
+    case linear:
+      logval = 0.;
+      get_val = matrixGet_linear;
+      break;
+    case logarithm:
+      logval = log10((scalarFieldGet_secondaryMin(field) - minmax[0]) /
+		     (minmax[1] - minmax[0]));
+      get_val = matrixGet_logarithm;
+      break;
+    case zeroCentredLog:
+      logval = 0;
+      get_val = matrixGet_zeroCenteredLog;
+      break;
+    default:
+      logval = 0.;
+      get_val = (matrixGet_scaleVal)0;
+      break;
+    }
+  g_return_if_fail(get_val);
+
+  DBG_fprintf(stderr, "ScalarFields: allocating storing values (%dx%d).\n",
 	      vAlpha[0] + vAlpha[1], uAlpha[0] + uAlpha[1]);
-  /* We store 6 additional data per point.
-     - 0: used or not ;
-     - 1: field value ;
-     - 2~5: RGBA. */
-  isosurfacesPointsInit(&points, 6);
+  /* We store 8 additional data per point.
+     - MAP_U: the x coordinate in plane ;
+     - MAP_V: the y coordinate in plane ;
+     - MAP_VISIBILITY: used or not ;
+     - MAP_VALUE: field value ;
+     - MAP_R...: RGBA. */
+  isosurfacesPointsInit(points, 8);
   nU = uAlpha[1] - uAlpha[0];
   nV = vAlpha[1] - vAlpha[0];
-  isosurfacesPointsAllocate(&points, 1, (nV - 1) * (2 * (nU - 1) - 2), nV * nU);
-
-  /* In log scale, we need the second minimum value. */
-  minlog = minmax[0];
-  if (logScale)
-    minlog = scalarFieldGet_secondaryMin(field);
+  isosurfacesPointsAllocate(points, 1, (nV - 1) * (2 * (nU - 1) - 2), nV * nU);
 
   /* We use an hexagonal padding for equilateral triangles. */
-  minlog = log10((minlog - minmax[0]) / (minmax[1] - minmax[0]));
   coeff =  dist / (float)alpha;
   nPoints = 0;
+  viewport[0] = ((float)uAlpha[0] + 0.5f) * coeff;
+  viewport[1] = (float)vAlpha[0] * coeff * 0.8660254f;
+  viewport[2] = ((float)(uAlpha[1] - uAlpha[0]) + 0.5f) * coeff;
+  viewport[3] = (float)(vAlpha[1] - vAlpha[0]) * coeff * 0.8660254f;
   for (i = 0; i < vAlpha[1] - vAlpha[0]; i++)
     {
-      Ycoeff = (float)(i + vAlpha[0]) * coeff * 0.8660254; /* the num coef is cos 30 */
+      I[1] = (float)(i + vAlpha[0]) * coeff * 0.8660254f; /* the num coef is cos 30 */
       for (j = 0; j < uAlpha[1] - uAlpha[0]; j++)
 	{
 	  if (i%2)
-	    Xcoeff = (float)(j + uAlpha[0]) * coeff;
+	    I[0] = (float)(j + uAlpha[0]) * coeff;
 	  else
-	    Xcoeff = ((float)(j + uAlpha[0]) + 0.5) * coeff;
-	  points.poly_points_data[nPoints][MAP_X] = center[0] +
-	    Xcoeff * xyz[0][0] + Ycoeff * xyz[1][0];
-	  points.poly_points_data[nPoints][MAP_Y] = center[1] +
-	    Xcoeff * xyz[0][1] + Ycoeff * xyz[1][1];
-	  points.poly_points_data[nPoints][MAP_Z] = center[2] +
-	    Xcoeff * xyz[0][2] + Ycoeff * xyz[1][2];
-	  points.poly_points_data[nPoints][MAP_VISIBILITY] =
-	    (float)scalarFieldGet_value(field, points.poly_points_data[nPoints],
-					&value, view->box->extension);
-	  if ((gboolean)points.poly_points_data[nPoints][MAP_VISIBILITY])
-	    {
-	      points.poly_points_data[nPoints][MAP_VALUE] =
-		(float)((CLAMP(value, minmax[0], minmax[1]) - minmax[0]) /
-			(minmax[1] - minmax[0]));
-	      if (logScale)
-		points.poly_points_data[nPoints][MAP_VALUE] =
-		  (points.poly_points_data[nPoints][MAP_VALUE] == 0.f)?0.f:
-		  - (log10(points.poly_points_data[nPoints][MAP_VALUE])
-		     - minlog) / minlog;
-	      shadeGet_valueTransformedInRGB
-		(shade,
-		 points.poly_points_data[nPoints] + MAP_R,
-		 points.poly_points_data[nPoints][MAP_VALUE]);
-	      drawnMinMax[0] = MIN(drawnMinMax[0], (float)value);
-	      drawnMinMax[1] = MAX(drawnMinMax[1], (float)value);
-	    }
-/* 	  DBG_fprintf(stderr, " | %d (%g, %g, %g) -> %g\n", (gboolean)values[i][j][0], */
-/* 		      values[i][j][1], values[i][j][2], values[i][j][3], values[i][j][4]); */
+	    I[0] = ((float)(j + uAlpha[0]) + 0.5f) * coeff;
+	  planeTo3D(points->poly_points_data[nPoints] + MAP_X, I, xyz, center);
+	  points->poly_points_data[nPoints][MAP_U] = I[0];
+	  points->poly_points_data[nPoints][MAP_V] = I[1];
+	  points->poly_points_data[nPoints][MAP_VISIBILITY] =
+	    (float)setValue(field, shade, get_val, minmax, logval, drawnMinMax,
+			    view->box->extension,
+			    points->poly_points_data[nPoints] + MAP_X,
+			    points->poly_points_data[nPoints] + MAP_VALUE,
+			    points->poly_points_data[nPoints] + MAP_R);
+	  points->poly_points_data[nPoints][MAP_A] = points->poly_points_data[nPoints][MAP_VALUE];
+/* 	  DBG_fprintf(stderr, " | %d (%g, %g, %g) -> %g %g\n", */
+/* 		      (gboolean)points->poly_points_data[nPoints][MAP_VISIBILITY], */
+/* 		      points->poly_points_data[nPoints][MAP_X], */
+/* 		      points->poly_points_data[nPoints][MAP_Y], */
+/* 		      points->poly_points_data[nPoints][MAP_Z], */
+/* 		      points->poly_points_data[nPoints][MAP_VALUE], */
+/* 		      points->poly_points_data[nPoints][MAP_A]); */
 	  nPoints += 1;
 	}
     }
+
+  linkedPoints = g_malloc(sizeof(int) * points->num_points);
+  for (i = 0; i < points->num_points; i++)
+    linkedPoints[i] = -1;
   nPolys = 0;
-  points.num_polys_surf[0] = 0;
+  points->num_polys_surf[0] = 0;
+  points->num_polys_surf[1] = 0;
+  data = points->poly_points_data;
   for (i = 0; i < vAlpha[1] - vAlpha[0] - 1; i++)
     {
       iPolys[3] = 0;
@@ -818,109 +941,282 @@ void scalarFieldDraw_map(ScalarField *field, Plane *plane, Shade *shade,
 	      iPolys[iPolys[3]%3] = (i + (i + k)%2) * nU + j;
 	      if (iPolys[3] > 2)
 		{
-		  /* The triangle is visible if all of its vertices are
-		     visible. */
-		  if ((gboolean)points.poly_points_data[iPolys[0]][MAP_VISIBILITY] &&
-		      (gboolean)points.poly_points_data[iPolys[1]][MAP_VISIBILITY] &&
-		      (gboolean)points.poly_points_data[iPolys[2]][MAP_VISIBILITY])
+		  /* n is the number of visible points. */
+		  n  = ((gboolean)data[iPolys[0]][MAP_VISIBILITY])?1:0;
+		  n += ((gboolean)data[iPolys[1]][MAP_VISIBILITY])?1:0;
+		  n += ((gboolean)data[iPolys[2]][MAP_VISIBILITY])?1:0;
+		  switch (n)
 		    {
-		      points.poly_surf_index[nPolys] = 1;
-		      points.num_polys_surf[0]      += 1;
+		    case 1:
+		      visible = ((gboolean)data[iPolys[0]][MAP_VISIBILITY])?
+			0:((gboolean)data[iPolys[1]][MAP_VISIBILITY])?1:2;
+		      linkedPoints[iPolys[(visible + 1) % 3]] = iPolys[visible];
+		      linkedPoints[iPolys[(visible + 2) % 3]] = iPolys[visible];
+
+		      points->poly_surf_index[nPolys] = 1;
+		      points->num_polys_surf[0]      += 1;
+		      break;
+		    case 2:
+		      hidden = (!(gboolean)data[iPolys[0]][MAP_VISIBILITY])?
+			0:(!(gboolean)data[iPolys[1]][MAP_VISIBILITY])?1:2;
+		      linkedPoints[iPolys[hidden]] = iPolys[(hidden + 1) % 3];
+
+		      points->poly_surf_index[nPolys] = 1;
+		      points->num_polys_surf[0]      += 1;
+		      break;
+		    case 3:
+		      points->poly_surf_index[nPolys] = 1;
+		      points->num_polys_surf[0]      += 1;
+		      break;
+		    default:
+		      points->poly_surf_index[nPolys] = -1;
 		    }
-		  else
-		    points.poly_surf_index[nPolys] = -1;
-		  points.poly_num_vertices[nPolys] = 3;
-		  points.poly_vertices[nPolys] = g_malloc(sizeof(int) * 3);
-		  points.poly_vertices[nPolys][0] = iPolys[0];
-		  points.poly_vertices[nPolys][1] = iPolys[1];
-		  points.poly_vertices[nPolys][2] = iPolys[2];
+		  points->poly_num_vertices[nPolys] = 3;
+		  points->poly_vertices[nPolys] = g_malloc(sizeof(int) * 3);
+		  points->poly_vertices[nPolys][0] = iPolys[0];
+		  points->poly_vertices[nPolys][1] = iPolys[1];
+		  points->poly_vertices[nPolys][2] = iPolys[2];
 		  nPolys += 1;
 		}
 	    }
 	}
     }
+  /* We now modify the position of the vertices defined in
+     linkedPoints array. */
+  data = points->poly_points_data;
+  for (i = 0; i < points->num_points; i++)
+    if (linkedPoints[i] >= 0)
+      {
+	a[0] = data[linkedPoints[i]][MAP_U];
+	a[1] = data[linkedPoints[i]][MAP_V];
+	b = data[i] + MAP_U;
+	matrixGet_inter2DFromList(I, &lambda, a, b, interRed);
+	b[0] = I[0];
+	b[1] = I[1];
+	planeTo3D(data[i] + MAP_X, b, xyz, center);
+	data[i][MAP_VISIBILITY] = TRUE;
+	data[i][MAP_VALUE] = data[linkedPoints[i]][MAP_VALUE];
+	data[i][MAP_R] = data[linkedPoints[i]][MAP_R];
+	data[i][MAP_G] = data[linkedPoints[i]][MAP_G];
+	data[i][MAP_B] = data[linkedPoints[i]][MAP_B];
+	data[i][MAP_A] = data[linkedPoints[i]][MAP_A];
+      }
+  g_free(linkedPoints);
 #if DEBUG == 1
-  isosurfacesPointsCheck(&points);
+  isosurfacesPointsCheck(points);
+#endif
+  for (inter = interRed; inter; inter = g_list_next(inter))
+    g_free(inter->data);
+  g_list_free(interRed);
+
+  valMinMax[0] = (float)get_val(drawnMinMax[0], minmax, logval);
+  valMinMax[1] = (float)get_val(drawnMinMax[1], minmax, logval);
+}
+
+void scalarFieldDraw_map(ScalarField *field, Plane *plane, Shade *shade,
+                         OpenGLView *view, float precision, float drawnMinMax[2],
+			 matrix_scaleflag scale, guint nIsoLines, float *rgb,
+			 float *inputMinMax)
+{
+  SurfacesPoints points;
+  float valMinMax[2], viewport[4];
+  #if DEBUG == 1
+/*   gchar str[5]; */
+  GTimer *timer;
+  gulong fractionTimer;
+  #endif
+
+#if DEBUG == 1
+  timer = g_timer_new();
+  g_timer_start(timer);
 #endif
 
-  glDisable(GL_CULL_FACE);
-  glDisable(GL_LIGHTING);
-  glPushMatrix();
-  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
-/*   glBegin(GL_LINES); */
-/*   glVertex3d(center[0] - dist * xyz[0][0] - dist * xyz[1][0], */
-/* 	     center[1] - dist * xyz[0][1] - dist * xyz[1][1], */
-/* 	     center[2] - dist * xyz[0][2] - dist * xyz[1][2]); */
-/*   glVertex3d(center[0] + dist * xyz[0][0] - dist * xyz[1][0], */
-/* 	     center[1] + dist * xyz[0][1] - dist * xyz[1][1], */
-/* 	     center[2] + dist * xyz[0][2] - dist * xyz[1][2]); */
-/*   glVertex3d(center[0] + dist * xyz[0][0] - dist * xyz[1][0], */
-/* 	     center[1] + dist * xyz[0][1] - dist * xyz[1][1], */
-/* 	     center[2] + dist * xyz[0][2] - dist * xyz[1][2]); */
-/*   glVertex3d(center[0] + dist * xyz[0][0] + dist * xyz[1][0], */
-/* 	     center[1] + dist * xyz[0][1] + dist * xyz[1][1], */
-/* 	     center[2] + dist * xyz[0][2] + dist * xyz[1][2]); */
-/*   glVertex3d(center[0] + dist * xyz[0][0] + dist * xyz[1][0], */
-/* 	     center[1] + dist * xyz[0][1] + dist * xyz[1][1], */
-/* 	     center[2] + dist * xyz[0][2] + dist * xyz[1][2]); */
-/*   glVertex3d(center[0] - dist * xyz[0][0] + dist * xyz[1][0], */
-/* 	     center[1] - dist * xyz[0][1] + dist * xyz[1][1], */
-/* 	     center[2] - dist * xyz[0][2] + dist * xyz[1][2]); */
-/*   glVertex3d(center[0] - dist * xyz[0][0] + dist * xyz[1][0], */
-/* 	     center[1] - dist * xyz[0][1] + dist * xyz[1][1], */
-/* 	     center[2] - dist * xyz[0][2] + dist * xyz[1][2]); */
-/*   glVertex3d(center[0] - dist * xyz[0][0] - dist * xyz[1][0], */
-/* 	     center[1] - dist * xyz[0][1] - dist * xyz[1][1], */
-/* 	     center[2] - dist * xyz[0][2] - dist * xyz[1][2]); */
-/*   glEnd(); */
+  computeMap(&points, viewport, valMinMax, drawnMinMax,
+	     field, plane, shade, view,
+	     precision, scale, inputMinMax);
+  DBG_fprintf(stderr, "ScalarFields: map computed.\n");
+  drawMapAndLines(&points, plane, nIsoLines, shade, rgb, valMinMax);
+  DBG_fprintf(stderr, "ScalarFields: map drawn.\n");
+  isosurfacesPointsFree(&points);
+  DBG_fprintf(stderr, "ScalarFields: memory free.\n");
+
+#if DEBUG == 1
+  g_timer_stop(timer);
+  fprintf(stderr, "ScalarFields: Draw map in %g micro-s.\n",
+	  g_timer_elapsed(timer, &fractionTimer)/1e-6);
+  g_timer_destroy(timer);
+#endif
+}
+
+gboolean scalarFieldExport_map(ScalarField *field, Plane *plane, Shade *shade,
+			       OpenGLView *view, float precision,
+			       float drawnMinMax[2], matrix_scaleflag scale,
+			       guint nIsoLines, float *rgb, float *inputMinMax,
+			       gchar *filename, ScalarField_exportFormat format,
+			       GError **error)
+{
+#ifdef HAVE_CAIRO
+#define fact 25.f
+  SurfacesPoints points;
+  float valMinMax[2], viewport[4], **data;
+  int i, j;
+  float v, rgba[4];
+  Line *line;
+  float *uvs;
+  gint nVals;
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  cairo_status_t status;
+  cairo_matrix_t mat = {fact, 0., 0., fact, 0., 0.};
+  
+  g_return_val_if_fail(error && *error == (GError*)0, FALSE);
+
+  computeMap(&points, viewport, valMinMax, drawnMinMax,
+	     field, plane, shade, view,
+	     precision, scale, inputMinMax);
+
+  DBG_fprintf(stderr, "Scalar Field: begin export to PDF/SVG.\n");
+
+  switch (format)
+    {
+    case sf_svg:
+      surface = cairo_svg_surface_create(filename,
+					 (double)(viewport[2] * fact),
+					 (double)(viewport[3] * fact));
+      break;
+    case sf_pdf:
+      surface = cairo_pdf_surface_create(filename,
+					 (double)(viewport[2] * fact),
+					 (double)(viewport[3] * fact));
+      break;
+    default:
+      surface = (cairo_surface_t*)0;
+    }
+  status = cairo_surface_status(surface);
+  if (status != CAIRO_STATUS_SUCCESS)
+    {
+      *error = g_error_new(G_FILE_ERROR, G_FILE_ERROR_FAILED,
+			   cairo_status_to_string(status));
+      cairo_surface_destroy(surface);
+      isosurfacesPointsFree(&points);
+      return FALSE;
+    }
+
+  cr = cairo_create(surface);
+  status = cairo_status(cr);
+  if (status != CAIRO_STATUS_SUCCESS)
+    {
+      *error = g_error_new(G_FILE_ERROR, G_FILE_ERROR_FAILED,
+			   cairo_status_to_string(status));
+      cairo_destroy(cr);
+      cairo_surface_destroy(surface);
+      isosurfacesPointsFree(&points);
+      return FALSE;
+    }
+  cairo_set_line_cap(cr, CAIRO_LINE_CAP_BUTT);
+  cairo_set_line_join(cr, CAIRO_LINE_JOIN_BEVEL);
+  cairo_set_line_width(cr, 0.01);
+  mat.x0 = -(double)viewport[0] * fact;
+  mat.y0 = -(double)viewport[1] * fact;
+  cairo_set_matrix(cr, &mat);
+
   for (i = 0; i < points.num_polys; i++)
     {
       if (points.poly_surf_index[i] > 0)
 	{
-	  glBegin(GL_TRIANGLE_STRIP);
-	  glColor3fv(points.poly_points_data[points.poly_vertices[i][0]] + MAP_R);
-	  glVertex3fv(points.poly_points_data[points.poly_vertices[i][0]]);
-	  glColor3fv(points.poly_points_data[points.poly_vertices[i][1]] + MAP_R);
-	  glVertex3fv(points.poly_points_data[points.poly_vertices[i][1]]);
-	  glColor3fv(points.poly_points_data[points.poly_vertices[i][2]] + MAP_R);
-	  glVertex3fv(points.poly_points_data[points.poly_vertices[i][2]]);
-	  glEnd();
+	  data = points.poly_points_data;
+	  cairo_set_source_rgba(cr,
+				(data[points.poly_vertices[i][0]][MAP_R] +
+				 data[points.poly_vertices[i][1]][MAP_R] +
+				 data[points.poly_vertices[i][2]][MAP_R]) / 3.f,
+				(data[points.poly_vertices[i][0]][MAP_G] +
+				 data[points.poly_vertices[i][1]][MAP_G] +
+				 data[points.poly_vertices[i][2]][MAP_G]) / 3.f,
+				(data[points.poly_vertices[i][0]][MAP_B] +
+				 data[points.poly_vertices[i][1]][MAP_B] +
+				 data[points.poly_vertices[i][2]][MAP_B]) / 3.f,
+				1.f);
+	  cairo_move_to(cr,
+			data[points.poly_vertices[i][0]][MAP_U],
+			data[points.poly_vertices[i][0]][MAP_V]);
+	  cairo_line_to(cr,
+			data[points.poly_vertices[i][1]][MAP_U],
+			data[points.poly_vertices[i][1]][MAP_V]);
+	  cairo_line_to(cr,
+			data[points.poly_vertices[i][2]][MAP_U],
+			data[points.poly_vertices[i][2]][MAP_V]);
+	  cairo_line_to(cr,
+			data[points.poly_vertices[i][0]][MAP_U],
+			data[points.poly_vertices[i][0]][MAP_V]);	
+	  cairo_fill_preserve(cr);
+	  cairo_stroke(cr);
 	}
     }
 
-  valMinMax[0] =
-    (float)((CLAMP(drawnMinMax[0], minmax[0], minmax[1]) - minmax[0]) /
-	    (minmax[1] - minmax[0]));
-  if (logScale)
-    valMinMax[0] = (valMinMax[0] == 0.f)?0.f:
-      - (log10(valMinMax[0]) - minlog) / minlog;
-  valMinMax[1] =
-    (float)((CLAMP(drawnMinMax[1], minmax[0], minmax[1]) - minmax[0]) /
-	    (minmax[1] - minmax[0]));
-  if (logScale)
-    valMinMax[1] = (valMinMax[1] == 0.f)?0.f:
-      - (log10(valMinMax[1]) - minlog) / minlog;
+  /* Add isolines. */
+  DBG_fprintf(stderr, "ScalarFields: export isolines.\n");
   for (i = 1; i <= (int)nIsoLines; i++)
-    if (isolineBuild(&line, &points, MAP_VALUE, MAP_VISIBILITY,
-		     valMinMax[0] + (valMinMax[1] - valMinMax[0]) *
-		     (float)i / (float)(nIsoLines + 1)))
-      {
-	isolineDraw(line);
-	isolineFree(line);
-      }
+    {
+      v = valMinMax[0] + (valMinMax[1] - valMinMax[0]) *
+	(float)i / (float)(nIsoLines + 1);
+      if (isolineBuild(&line, &points, MAP_VALUE, MAP_VISIBILITY, v))
+	{
+	  DBG_fprintf(stderr, "ScalarFields: set color for value %g.\n", v);
+	  if (!rgb)
+	    {
+	      shadeGet_valueTransformedInRGB(shade, rgba, v);
+	      rgba[0] = 1.f - rgba[0];
+	      rgba[1] = 1.f - rgba[1];
+	      rgba[2] = 1.f - rgba[2];
+	      rgba[3] = 1.f;
+	      rgb = rgba;
+	    }
+	  cairo_set_source_rgb(cr, rgb[0], rgb[1], rgb[2]);
 
-  glPopMatrix();  
-  glEnable(GL_CULL_FACE);
-  glEnable(GL_LIGHTING);
+	  DBG_fprintf(stderr, "ScalarFields: project line on plane %p.\n",
+		      (gpointer)plane);
+	  uvs = isolineProject(line, plane, &nVals);
+	  for (j = 0; j < nVals; j++)
+	    {
+	      cairo_move_to(cr, uvs[j * 4 + 0], uvs[j * 4 + 1]);
+	      cairo_line_to(cr, uvs[j * 4 + 2], uvs[j * 4 + 3]);
+	      cairo_stroke(cr);
+	    }
+	  g_free(uvs);
 
+	  isolineFree(line);
+	}
+    }
+
+  /* Add frame. */
+  DBG_fprintf(stderr, "ScalarFields: export the frame.\n");
+  uvs = planeGet_reducedIntersection(plane, &nVals);
+  if (uvs)
+    {
+      cairo_set_source_rgb(cr, 0., 0., 0.);
+
+      cairo_move_to(cr, uvs[(nVals - 1) * 2 + 0], uvs[(nVals - 1) * 2 + 1]);
+      for (j = 0; j < nVals; j++)
+	cairo_line_to(cr, uvs[j * 2 + 0], uvs[j * 2 + 1]);
+      cairo_stroke(cr);
+      g_free(uvs);
+    }
+
+  /* Add legend. */
+/*   toolShadeExport(shade, (double)(viewport[2] * fact), */
+/* 		  (double)(viewport[3] * fact)); */
+
+  /* Finalising */
+  cairo_show_page(cr);
+  cairo_destroy(cr);
+  cairo_surface_destroy(surface);
   isosurfacesPointsFree(&points);
 
-  #if DEBUG == 1
-    g_timer_stop(timer);
-    fprintf(stderr, "ScalarFields : Draw Map in %g micro-s.\n",
-	    g_timer_elapsed(timer, &fractionTimer)/1e-6);
-    g_timer_destroy(timer);
-  #endif
+  return TRUE;
+#else
+  g_error("Not compiled with CAIRO not able to export.");
+  return FALSE;
+#endif
 }
 
 gboolean scalarFieldGet_value(ScalarField *field, float xyz[3],
@@ -1006,21 +1302,6 @@ gboolean scalarFieldGet_value(ScalarField *field, float xyz[3],
 	return FALSE;
     }
 
-  /* We linearly interpolate the value. */
-  *value = field->data[ijk[0]][ijk[1]][ijk[2]];
-  for (l = 0; l < 3; l++)
-    {
-      dijk[0] = ijk[0];
-      dijk[1] = ijk[1];
-      dijk[2] = ijk[2];
-      if (ijk[l] == field->nElements[l] - 1)
-	dijk[l] = 0;
-      else
-	dijk[l] += 1;
-      *value += (field->data[dijk[0]][dijk[1]][dijk[2]] -
-		 field->data[ijk[0]][ijk[1]][ijk[2]]) * factor[l];
-    }
-
   /* lower left is ijk. */
   /* upper right is dijk. */
   dijk[0] = (ijk[0] + 1) % field->nElements[0];
@@ -1046,7 +1327,7 @@ gboolean scalarFieldGet_value(ScalarField *field, float xyz[3],
 
   return TRUE;
 }
-void scalarFieldSet_box(ScalarField *field, float box[6])
+void scalarFieldSet_box(ScalarField *field, double box[6])
 {
   int i;
 
@@ -1085,14 +1366,14 @@ void scalarFieldSet_box(ScalarField *field, float box[6])
   field->fromXYZtoReducedCoord[2][2] = 1 /
     field->box[VISU_DATA_BOX_DZZ];
 }
-void scalarFieldGet_box(ScalarField *field, float box[6])
+void scalarFieldGet_box(ScalarField *field, double box[6])
 {
   int i;
 
   g_return_if_fail(field);
 
   for (i = 0; i < 6; i++)
-    box[i] = field->box[i];
+    box[i] = (double)field->box[i];
 }
 void scalarFieldGet_gridSize(ScalarField *field, int grid[3])
 {
@@ -1127,14 +1408,15 @@ double*** scalarFieldGet_data(ScalarField *field)
 
 void scalarFieldSet_fitToBox(VisuData *data, ScalarField *field)
 {
-  float box[6];
+  double box[6];
   int i;
 
   g_return_if_fail(data);
 
-  DBG_fprintf(stderr, "ScalarField : change the current box to fit to %p.\n", (gpointer) data);
+  DBG_fprintf(stderr, "ScalarField: change the current box to fit to %p.\n",
+	      (gpointer) data);
   for (i = 0; i < 6; i++)
-    box[i] = visuDataGet_boxGeometry(data, i);
+    box[i] = (double)visuDataGet_boxGeometry(data, i);
   scalarFieldSet_box(field, box);
 }  
 GList* scalarFieldGet_allOptions(ScalarField *field)
