@@ -160,8 +160,7 @@ static gboolean drawIntra = FALSE;
 /* Local methods. */
 static gboolean isosurfaces_read_intra(gchar **lines, int nbLines, int position,
 					    VisuData *dataObj, GError **error);
-static gboolean isosurfaces_export_resources(GString *data, int *nbLinesWritten,
-					     VisuData *dataObj);
+static void isosurfaces_export_resources(GString *data, VisuData *dataObj);
 
 static void freeSurfacesProperties(gpointer data);
 static void propertiesRemoveSurf(gpointer key, gpointer value, gpointer data);
@@ -360,12 +359,19 @@ void isosurfacesAddSurfaces(Surfaces *surf, int nsurf, int npolys, int npoints)
 		       GINT_TO_POINTER(surf->nsurf));
 }
 
-void isosurfacesSet_box(Surfaces *surf, float box[6])
+void isosurfacesSet_box(Surfaces *surf, double box[6])
 {
   g_return_if_fail(surf);
 
   DBG_fprintf(stderr, "Surfaces: copy box.\n");
-  memcpy(surf->local_box, box, sizeof(float) * 6);
+  memcpy(surf->local_box, box, sizeof(double) * 6);
+}
+double* isosurfacesGet_box(Surfaces *surf)
+{
+  g_return_val_if_fail(surf, (double*)0);
+
+  DBG_fprintf(stderr, "Surfaces: get box.\n");
+  return surf->local_box;
 }
 static void freeSurfacesProperties(gpointer data)
 {
@@ -1126,8 +1132,7 @@ void isosurfacesOrder_polygons(SurfacesOrder *order, Surfaces *surf[])
 }
 
 
-void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
-			      Surfaces *surf[], SurfacesOrder *order)
+void isosurfacesDraw_surfaces(int OpenGLId, Surfaces *surf[], SurfacesOrder *order)
 {
   int i,j, ip, itp, idSurf;
   int nb;
@@ -1136,19 +1141,15 @@ void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
   SurfaceResource *res;
   float rgba[4];
    
-  g_return_if_fail(view && surf && order);
+  g_return_if_fail(surf && order);
    
   DBG_fprintf(stderr, "Isosurfaces : rebuilding surfaces list\n");
 
   glNewList(OpenGLId, GL_COMPILE);  
-  glPushMatrix();
   if (drawIntra)
     glEnable(GL_CULL_FACE);
   else
     glDisable(GL_CULL_FACE);
-  glTranslated(- view->box->dxxs2,
-	       - view->box->dyys2,
-	       - view->box->dzzs2);
    
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_BLEND);   
@@ -1196,7 +1197,7 @@ void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
 	    rgba[3] = res->color->rgba[3];
 	  }
 	else
-	  setOpenGlMaterial(res->material, res->color->rgba);
+	  openGLSet_color(res->material, res->color->rgba);
 
 	/*          glCallList(lists_location + itp); */
 	itp_old = itp;
@@ -1205,7 +1206,7 @@ void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
     /* This is where to find the points and the normals. */
     glBegin(GL_POLYGON);
     if (drawIntra)
-      setOpenGlMaterial(res->material, res->color->rgba);
+      openGLSet_color(res->material, res->color->rgba);
     for(j = 0; j < points->poly_num_vertices[ip]; j++ ) {
       glNormal3fv(points->poly_points_data[points->poly_vertices[ip][j]] +
 		  SurfacesPoints_normalOffset);
@@ -1216,7 +1217,7 @@ void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
     if (drawIntra)
       {
 	glBegin(GL_POLYGON);
-	setOpenGlMaterial(res->material, rgba);
+	openGLSet_color(res->material, rgba);
 	for(j = points->poly_num_vertices[ip] - 1; j >= 0; j-- )
 	  {
 	    glNormal3f(-points->poly_points_data[points->poly_vertices[ip][j]][SurfacesPoints_normalOffset + 0],
@@ -1228,8 +1229,6 @@ void isosurfacesDraw_surfaces(int OpenGLId, OpenGLView *view,
       }
   }
    
-
-  glPopMatrix();
   glEndList();
 }
 
@@ -1472,7 +1471,8 @@ void isoSurfacesSet_fitToBox(VisuData *data, Surfaces *surf)
 
   g_return_if_fail(data && surf);
 
-  DBG_fprintf(stderr, "Isosurfaces : change the current box to fit to %p.\n", (gpointer) data);
+  DBG_fprintf(stderr, "Isosurfaces: change the current box to fit to %p.\n",
+	      (gpointer) data);
 
   inverted_local_box_matrix[0][0] = 1 / surf->local_box[_DXX];
   inverted_local_box_matrix[0][1] = - surf->local_box[_DYX] /
@@ -1497,7 +1497,15 @@ void isoSurfacesSet_fitToBox(VisuData *data, Surfaces *surf)
 	(boxCoordinatestoXYZ[i][0] * inverted_local_box_matrix[0][j] +
 	 boxCoordinatestoXYZ[i][1] * inverted_local_box_matrix[1][j] +
 	 boxCoordinatestoXYZ[i][2] * inverted_local_box_matrix[2][j]);
+  DBG_fprintf(stderr, " | (%6.3f %6.3f %6.3f)\n", transform_box_matrix[0][0],
+	      transform_box_matrix[0][1], transform_box_matrix[0][2]);
+  DBG_fprintf(stderr, " | (%6.3f %6.3f %6.3f)\n", transform_box_matrix[1][0],
+	      transform_box_matrix[1][1], transform_box_matrix[1][2]);
+  DBG_fprintf(stderr, " | (%6.3f %6.3f %6.3f)\n", transform_box_matrix[2][0],
+	      transform_box_matrix[2][1], transform_box_matrix[2][2]);
 
+  DBG_fprintf(stderr, " | apply change to %d points.\n",
+	      surf->basePoints.num_points);
   for (i = 0; i < surf->basePoints.num_points; i++)
     {
       old_poly_points[0] = surf->basePoints.poly_points_data[i][0];
@@ -1511,6 +1519,12 @@ void isoSurfacesSet_fitToBox(VisuData *data, Surfaces *surf)
       matrix_productVector(surf->basePoints.poly_points_data[i] + SurfacesPoints_normalOffset,
 			   transform_box_matrix, old_poly_points + 3);
     }
+  surf->local_box[_DXX] = (double)boxCoordinatestoXYZ[0][0];
+  surf->local_box[_DYX] = (double)boxCoordinatestoXYZ[0][1];
+  surf->local_box[_DYY] = (double)boxCoordinatestoXYZ[1][1];
+  surf->local_box[_DZX] = (double)boxCoordinatestoXYZ[0][2];
+  surf->local_box[_DZY] = (double)boxCoordinatestoXYZ[1][2];
+  surf->local_box[_DZZ] = (double)boxCoordinatestoXYZ[2][2];
 }  
 
 /***************/
@@ -1528,13 +1542,9 @@ static gboolean isosurfaces_read_intra(gchar **lines, int nbLines, int position,
   drawIntra = intra;
   return TRUE;
 }
-static gboolean isosurfaces_export_resources(GString *data, int *nbLinesWritten,
-					     VisuData *dataObj _U_)
+static void isosurfaces_export_resources(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_INTRA);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_INTRA);
   g_string_append_printf(data, "    %d\n\n", drawIntra);
-  *nbLinesWritten += 4;
- 
-  return TRUE;
 }
