@@ -621,7 +621,11 @@ static void colorComboBox_changed(GtkWidget *widget _U_, ColorComboBox *colorCom
   else
     {
       /* Return the combobox to the previously selected color. */
-      colorComboBoxSet_selectionByColor(colorComboBox, colorComboBox->previouslySelectedColor);
+      if (colorComboBox->previouslySelectedColor)
+	colorComboBoxSet_selectionByColor(colorComboBox,
+					  colorComboBox->previouslySelectedColor);
+      else
+	gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), -1);
     }
   gtk_widget_destroy(selection);
 }
