@@ -47,8 +47,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkliststore.h>
+#include <gtk/gtk.h>
 
 #include <coreTools/toolColor.h>
 
