@@ -41,11 +41,9 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
-#include <gtk/gtksignal.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #include <visu_gtk.h>
-#include <gtk_main.h>
 #include "gtk_dumpDialogWidget.h"
 #include <opengl.h>
 
@@ -111,6 +109,8 @@ struct _DumpDialogClass
   guint width, height;
   gchar *current_dir;
 
+  guint formatId;
+
   void (*dumpDialog) (DumpDialog *dump);
 };
 
@@ -130,6 +130,7 @@ static void dumpDialog_class_init(DumpDialogClass *klass)
   klass->width  = 0;
   klass->height = 0;
   klass->current_dir = (gchar*)0;
+  klass->formatId = 0;
 
   G_OBJECT_CLASS(klass)->dispose = dumpDialog_dispose;
   G_OBJECT_CLASS(klass)->finalize = dumpDialog_finalize;
@@ -198,6 +199,9 @@ GtkWidget* dumpDialog_new(VisuData *dataObj, GtkWindow *parent,
   else
     view = (OpenGLView*)0;
 
+  if (!parent)
+    parent = visuGtkGet_render();
+
   DBG_fprintf(stderr, "Gtk DumpDialog : creating a new DumpDialog object.\n");
 
   dumpDialog = DUMP_DIALOG(g_object_new(dumpDialog_get_type(), NULL));
@@ -230,7 +234,8 @@ GtkWidget* dumpDialog_new(VisuData *dataObj, GtkWindow *parent,
   filename = (const gchar*)0;
   dumpDialog->dataObj = dataObj;
   if (dataObj)
-    filename = (const gchar*)visuDataGet_property(dataObj, "dumpDialog_filename");
+    filename = (const gchar*)g_object_get_data(G_OBJECT(dataObj),
+					       "dumpDialog_filename");
   if (!filename)
     filename = suggestedFilename;
   if (filename)
@@ -254,7 +259,6 @@ GtkWidget* dumpDialog_new(VisuData *dataObj, GtkWindow *parent,
   dumpDialog->comboType = gtk_combo_box_new_text();
   /* Set the combo entry that let V_Sim detect the desired format through the extension. */
   gtk_combo_box_append_text(GTK_COMBO_BOX(dumpDialog->comboType), _("Autodetect format"));
-  gtk_combo_box_set_active(GTK_COMBO_BOX(dumpDialog->comboType), 0);
   gtk_box_pack_start(GTK_BOX(hbox), dumpDialog->comboType, FALSE, FALSE, 0);
   /* Add an expander for file format options. */
   dumpDialog->expanderFileFormat = gtk_expander_new(_("File format option:"));
@@ -322,7 +326,7 @@ GtkWidget* dumpDialog_new(VisuData *dataObj, GtkWindow *parent,
 
   /* Setting default values. */
   gtk_widget_set_name(GTK_WIDGET(dumpDialog), "filesel");
-  directory = gtkMainClassGet_lastOpenDirectory();
+  directory = visuGtkGet_lastOpenDirectory();
   if (klass->current_dir)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dumpDialog->fileChooser),
 					klass->current_dir);
@@ -337,6 +341,7 @@ GtkWidget* dumpDialog_new(VisuData *dataObj, GtkWindow *parent,
 	gtk_combo_box_append_text(GTK_COMBO_BOX(dumpDialog->comboType), labelType);
       format = g_list_next(format);
     }
+  gtk_combo_box_set_active(GTK_COMBO_BOX(dumpDialog->comboType), klass->formatId);
 
   gtk_widget_show_all(GTK_DIALOG(dumpDialog)->vbox);
   return GTK_WIDGET(dumpDialog);
@@ -371,6 +376,7 @@ static void onComboFileFormatChange(GtkComboBox *combo, gpointer data)
 
   dumpTypes = visuDumpGet_allModules();
   formatInt = gtk_combo_box_get_active(combo);
+  DUMP_DIALOG_GET_CLASS(data)->formatId = (guint)formatInt;
   DBG_fprintf(stderr, "Gtk DumpDialog : On file format change (%d).\n", formatInt);
   /* Select auto file format -> no properties. */
   if (formatInt == 0)
@@ -436,6 +442,8 @@ static void onComboFileFormatChange(GtkComboBox *combo, gpointer data)
 
       formatProperties = g_list_next(formatProperties);
     }
+  /* Expand the option expander. */
+  gtk_expander_set_expanded(GTK_EXPANDER(DUMP_DIALOG(data)->expanderFileFormat), TRUE);
 }
 static void onDumpDialogResponse(GtkDialog *dialog, gint id, gpointer *data)
 {
@@ -512,8 +520,8 @@ static void onDumpDialogResponse(GtkDialog *dialog, gint id, gpointer *data)
 		  dumpDialog->selectedFileFormat->fileType->description);
       /* Save the filename in a property of the VisuData object. */
       filename = g_path_get_basename(dumpDialog->dumpFileName);
-      visuDataSet_propertyWithDestroyFunc(dumpDialog->dataObj, "dumpDialog_filename",
-					  (gpointer)filename, g_free);
+      g_object_set_data_full(G_OBJECT(dumpDialog->dataObj), "dumpDialog_filename",
+			     (gpointer)filename, g_free);
 
       /* Save the dirname in the class. */
       klass = DUMP_DIALOG_CLASS(G_OBJECT_GET_CLASS(dumpDialog));
