@@ -41,14 +41,13 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
-#include <gtk/gtksignal.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
+#include <gtk/gtk.h>
 
 #include "gtk_shadeComboBoxWidget.h"
 #include <visu_tools.h>
+#include <visu_data.h>
+#include <visu_configFile.h>
+#include <coreTools/toolConfigFile.h>
 
 /**
  * SECTION:shadecombobox
@@ -81,6 +80,8 @@ enum
     COLUMN_SHADE_PIXBUF,
     /* This is a pointer to a label that describes the shade. */
     COLUMN_SHADE_LABEL,
+    /* This is a label with the shade id '(n)'. */
+    COLUMN_SHADE_ID,
     /* This a pointer to the #Shade as defined in toolShade.h */
     COLUMN_SHADE_POINTER_TO,
 
@@ -92,6 +93,11 @@ enum
 #define SHADE_BOX_HEIGHT 16
 #define SHADE_BOX_BITS   8
 
+/* Entries in the parameter file. */
+#define FLAG_PARAMETER_FAVSHADE   "presetShade"
+#define DESC_PARAMETER_FAVSHADE   "The id of a shade used as preset one in the shade selectors ; an integer ranging from 0"
+#define PARAMETER_FAVSHADE_DEFAULT    0
+
 static void shadeComboBox_dispose (GObject *obj);
 static void shadeComboBox_finalize(GObject *obj);
 
@@ -119,8 +125,11 @@ struct _ShadeComboBoxClass
      as TreeModel for the combobox in the widget. */
   GtkListStore *listStoredShades;
 
+  guint favShade;
+
   gulong shadeAddedSignalId;
 };
+static ShadeComboBoxClass *my_class = (ShadeComboBoxClass*)0;
 
 /* Local callbacks. */
 static void shadeComboBox_changed(GtkWidget *widget, ShadeComboBox *shadeComboBox);
@@ -129,6 +138,9 @@ static void shadeComboBox_changed(GtkWidget *widget, ShadeComboBox *shadeComboBo
 static void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass,
 			    Shade* shade);
 static void buildWidgets(ShadeComboBox *shadeComboBox);
+static gboolean readFavShade(gchar **lines, int nbLines, int position,
+			     VisuData *dataObj, GError **error);
+static void exportParameters(GString *data, VisuData* dataObj);
 
 G_DEFINE_TYPE(ShadeComboBox, shadeComboBox, GTK_TYPE_COMBO_BOX)
 
@@ -136,6 +148,7 @@ static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
 {
   GtkTreeIter iter;
   GList *shadeLst;
+  VisuConfigFileEntry *resourceEntry;
   
   DBG_fprintf(stderr, "Gtk ShadeComboBox : creating the class of the widget.\n");
   DBG_fprintf(stderr, "                     - adding new signals ;\n");
@@ -163,6 +176,7 @@ static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
   klass->listStoredShades = gtk_list_store_new(N_COLUMN_SHADE,
 					       GDK_TYPE_PIXBUF,
 					       G_TYPE_STRING,
+					       G_TYPE_STRING,
 					       G_TYPE_POINTER);
   shadeLst = toolShadeBuild_presetList();
   while (shadeLst)
@@ -170,10 +184,22 @@ static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
       addShadeToModel(&iter, klass, (Shade*)shadeLst->data);
       shadeLst = g_list_next(shadeLst);
     }
+  klass->favShade = 0;
+
+  /* Dealing with parameters. */
+  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
+					  FLAG_PARAMETER_FAVSHADE,
+					  DESC_PARAMETER_FAVSHADE,
+					  1, readFavShade);
+  visuConfigFileSet_version(resourceEntry, 3.5f);
+  visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_PARAMETER,
+				   exportParameters);
 
   /* Connect freeing methods. */
-  G_OBJECT_CLASS(klass)->dispose = shadeComboBox_dispose;
+  G_OBJECT_CLASS(klass)->dispose  = shadeComboBox_dispose;
   G_OBJECT_CLASS(klass)->finalize = shadeComboBox_finalize;
+
+  my_class = klass;
 }
 
 static void shadeComboBox_dispose(GObject *obj)
@@ -224,11 +250,22 @@ static void buildWidgets(ShadeComboBox *shadeComboBox)
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(shadeComboBox), renderer,
 				"pixbuf", COLUMN_SHADE_PIXBUF);
   renderer = gtk_cell_renderer_text_new();
+  g_object_set(G_OBJECT(renderer), "foreground", "#505050", NULL);
+  g_object_set(G_OBJECT(renderer), "xpad", 3, NULL);
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(shadeComboBox), renderer, FALSE);
+  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(shadeComboBox), renderer,
+				"markup", COLUMN_SHADE_ID);
+  renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(shadeComboBox), renderer, FALSE);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(shadeComboBox), renderer,
 				"text", COLUMN_SHADE_LABEL);
 
-  gtk_combo_box_set_active(GTK_COMBO_BOX(shadeComboBox), -1);
+  if ((gint)SHADE_COMBOX_CLASS(klass)->favShade < gtk_tree_model_iter_n_children
+      (GTK_TREE_MODEL(SHADE_COMBOX_CLASS(klass)->listStoredShades), (GtkTreeIter*)0))
+    gtk_combo_box_set_active(GTK_COMBO_BOX(shadeComboBox),
+			     SHADE_COMBOX_CLASS(klass)->favShade);
+  else
+    gtk_combo_box_set_active(GTK_COMBO_BOX(shadeComboBox), -1);
   g_signal_connect(G_OBJECT(shadeComboBox), "changed",
 		   G_CALLBACK(shadeComboBox_changed), (gpointer)shadeComboBox);
 }
@@ -342,16 +379,26 @@ GdkPixbuf* shadeComboBoxBuild_shadeStamp(Shade *shade, GdkPixbuf *pixbuf)
 static void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass,
 			    Shade* shade)
 {
+  GtkTreePath *path;
+  gint *ids;
+  gchar *idLbl;
+
   g_return_if_fail(iter && klass && shade);
 
   DBG_fprintf(stderr, "Gtk ShadeComboBox : appending a new shade '%s'.\n",
 	      shadeGet_label(shade));
   gtk_list_store_append(klass->listStoredShades, iter);
+  path = gtk_tree_model_get_path(GTK_TREE_MODEL(klass->listStoredShades), iter);
+  ids = gtk_tree_path_get_indices(path);
+  idLbl = g_strdup_printf("<span size=\"small\">(%d)</span>", ids[0]);
+  gtk_tree_path_free(path);
   gtk_list_store_set(klass->listStoredShades, iter,
 		     COLUMN_SHADE_PIXBUF      , shadeComboBoxBuild_shadeStamp(shade, (GdkPixbuf*)0),
 		     COLUMN_SHADE_LABEL       , shadeGet_label(shade),
+		     COLUMN_SHADE_ID          , idLbl,
 		     COLUMN_SHADE_POINTER_TO  , (gpointer)shade,
 		     -1);
+  g_free(idLbl);
   DBG_fprintf(stderr, "Gtk ShadeComboBox : appending a new shade '%s' at position '%s'.\n",
 	      shadeGet_label(shade), gtk_tree_model_get_string_from_iter
 	      (GTK_TREE_MODEL(SHADE_COMBOX_CLASS(klass)->listStoredShades), iter));
@@ -376,7 +423,7 @@ gboolean shadeComboBoxSet_selectionByShade(ShadeComboBox* shadeComboBox, Shade *
       gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
 			 COLUMN_SHADE_POINTER_TO, &tmpShade,
 			 -1);
-      if (tmpShade == shade)
+      if (tmpShade == shade || shadeCompare(tmpShade, shade))
 	{
 	  gtk_combo_box_set_active_iter(GTK_COMBO_BOX(shadeComboBox), &iter);
 	  return TRUE;
@@ -452,3 +499,34 @@ GdkPixbuf* shadeComboBoxGet_pixbufFromShade(ShadeComboBox *shadeComboBox, Shade
     }
   return (GdkPixbuf*)0;
 }
+
+
+static gboolean readFavShade(gchar **lines, int nbLines, int position,
+			     VisuData *dataObj _U_, GError **error)
+{
+  int val;
+  
+  g_return_val_if_fail(my_class, FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_integer(lines[0], position, &val, 1, error))
+    return FALSE;
+  if (val < 0)
+    {
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: prefered shade must be positive.\n"),
+			   position);
+      return FALSE;
+    }
+  my_class->favShade = (guint)val;
+
+  return TRUE;
+}
+static void exportParameters(GString *data, VisuData* dataObj _U_)
+{
+  g_return_if_fail(my_class);
+
+  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_FAVSHADE);
+  g_string_append_printf(data, "%s[gtk]: %d\n\n", FLAG_PARAMETER_FAVSHADE,
+			 my_class->favShade);
+}
