@@ -42,7 +42,8 @@
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
 
-#include <gtk/gtkalignment.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 #include <string.h>
 
 #include "gtk_toolPanelWidget.h"
@@ -50,6 +51,9 @@
 #include <support.h>
 #include <visu_tools.h>
 #include <gtk_main.h>
+#include <visu_gtk.h>
+#include <gtk_renderingWindowWidget.h>
+#include "renderingBackend/visu_windowInterface.h"
 
 /**
  * SECTION:toolpanel
@@ -183,8 +187,11 @@ static GtkWidget* buildMainMenu(DockWindow *window);
 static DockWindow* buildDockWindow(gchar *name, gboolean withWindow);
 
 /* Local callbacks. */
+static gboolean onHomePressed(GtkWidget *widget _U_, GdkEventKey *event,
+			      gpointer data);
 static void onDockButtonClicked(ToolPanel *toolPanel, gpointer data);
 static void onMainButtonClicked(GtkButton *button, gpointer data);
+static void onRaiseButtonClicked(GtkButton *button, gpointer data);
 static void onDockMenuClicked(GtkMenuItem *menuitem, gpointer user_data);
 static void onDockMenuNewClicked(GtkMenuItem *menuitem, gpointer user_data);
 static void onDockMenuHideClicked(GtkMenuItem *menuitem, gpointer user_data);
@@ -356,7 +363,9 @@ GtkWidget* toolPanelNew_withIconFromStock(gchar *id, gchar* name, gchar *tabName
 GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
 {
   GtkWidget *label, *wd, *image, *align;
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips;
+#endif
 
   g_return_val_if_fail(toolPanel, (GtkWidget*)0);
 
@@ -364,14 +373,16 @@ GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
      we do it now. */
   if (!toolPanel->headerWidget)
     {
+#if GTK_MINOR_VERSION < 12
       tooltips = gtk_tooltips_new ();
+#endif
 
       toolPanel->headerWidget = gtk_hbox_new(FALSE, 0);
       wd = gtk_event_box_new();
       gtk_event_box_set_visible_window(GTK_EVENT_BOX(wd), FALSE);
       gtk_box_pack_start(GTK_BOX(toolPanel->headerWidget),
 			 wd, FALSE, FALSE, 0);
-      gtk_tooltips_set_tip(tooltips, wd, toolPanel->comboLabel, NULL);
+      gtk_widget_set_tooltip_text(wd, toolPanel->comboLabel);
       if (toolPanel->icon)
 	gtk_container_add(GTK_CONTAINER(wd), toolPanel->icon);
       else
@@ -394,9 +405,9 @@ GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
 	  gtk_container_add(GTK_CONTAINER(wd), image);
 	  gtk_container_add(GTK_CONTAINER(align), wd);
 
-	  gtk_tooltips_set_tip(tooltips, wd,
+	  gtk_widget_set_tooltip_text(wd,
 			       _("Manage this subpanel: attach/detach"
-				 " or hide it."), NULL);
+				 " or hide it."));
 	}
       else
 	align = (GtkWidget*)0;
@@ -540,6 +551,7 @@ static GtkWidget* buildDockMenu(ToolPanel *toolPanel, GList *listOfDocks)
 {
   GtkWidget *menu, *item;
   DockWindow *window;
+  gchar *lbl;
 
   menu = gtk_menu_new();
   
@@ -549,7 +561,9 @@ static GtkWidget* buildDockMenu(ToolPanel *toolPanel, GList *listOfDocks)
       window = (DockWindow*)listOfDocks->data;
       if (window->show)
 	{
-	  item = gtk_menu_item_new_with_label(window->name);
+	  lbl = g_strdup_printf(_("Send to '%s'"), window->name);
+	  item = gtk_menu_item_new_with_label(lbl);
+	  g_free(lbl);
 	  g_signal_connect(G_OBJECT(item), "activate",
 			   G_CALLBACK(onDockMenuClicked), listOfDocks->data);
 	  g_object_set_qdata_full(G_OBJECT(item), CURRENT_TOOLPANEL_POINTER,
@@ -568,7 +582,9 @@ static GtkWidget* buildDockMenu(ToolPanel *toolPanel, GList *listOfDocks)
 		   G_CALLBACK(onDockMenuNewClicked), (gpointer)toolPanel);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
   /* Remove the current tool panel. */
-  item = gtk_menu_item_new_with_label(_("Hide tool"));
+  lbl = g_strdup_printf(_("Hide tool '%s'"), toolPanel->tabLabel);
+  item = gtk_menu_item_new_with_label(lbl);
+  g_free(lbl);
   g_signal_connect(G_OBJECT(item), "activate",
 		   G_CALLBACK(onDockMenuHideClicked), (gpointer)toolPanel);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -625,12 +641,17 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 {
   DockWindow *window;
   GtkWidget *wd, *image, *align;
-  GtkTooltips *tooltips;
   GtkCellRenderer *renderer;
+/*   GtkBindingSet *keys; */
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
+
+  tooltips = gtk_tooltips_new ();
+#endif
 
   window = g_malloc(sizeof(DockWindow));
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : create a new dock window %p (%s).\n",
+  DBG_fprintf(stderr, "Gtk ToolPanel: create a new dock window %p (%s).\n",
 	      (gpointer)window, name);
 
   window->name = name;
@@ -650,6 +671,8 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 		       G_CALLBACK(onKillDockWindowEvent), (gpointer)window);
       g_signal_connect(G_OBJECT(window->window), "destroy-event",
 		       G_CALLBACK(onKillDockWindowEvent), (gpointer)window);
+      g_signal_connect(G_OBJECT(window->window), "key-press-event",
+		       G_CALLBACK(onHomePressed), (gpointer)window);
 
       gtk_container_set_border_width(GTK_CONTAINER(window->window), 3);
       gtk_container_add(GTK_CONTAINER(window->window), window->vbox);
@@ -663,9 +686,8 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
   gtk_box_pack_end(GTK_BOX(window->vbox), align, TRUE, TRUE, 0);
   
   window->notebook = gtk_notebook_new();
-  gtk_notebook_set_tab_border(GTK_NOTEBOOK(window->notebook), 1);
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(window->notebook), TRUE);
-  gtk_container_add(GTK_CONTAINER(align), window->notebook);
+  gtk_container_add(GTK_CONTAINER(align), window->notebook);  
   window->notebookChanged =
     g_signal_connect(G_OBJECT(window->notebook), "switch-page",
 		     G_CALLBACK(onPageChanged), (gpointer)window);
@@ -673,33 +695,22 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
   window->hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(window->vbox), window->hbox, FALSE, FALSE, 0);
 
-  tooltips = gtk_tooltips_new ();
+  /* Define the key bindings. */
+/*   keys = gtk_binding_set_new("commandPanel"); */
+/*   gtk_binding_entry_add_signal(keys, GDK_space, 0, "clicked", 1, (gpointer)window); */
 
-  /* Add a help tooltips. */
-  wd = gtk_event_box_new();
-  gtk_event_box_set_visible_window(GTK_EVENT_BOX(wd), FALSE);
-  gtk_tooltips_set_tip(tooltips, wd,
-		       _("Positions, sizes, names, contains... of dock" \
-			 " windows are stored in the parameters file, " \
-			 "see the 'Config. files' button on the command panel."), NULL);
-  gtk_box_pack_end(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
-  image = gtk_image_new_from_stock(GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
-  gtk_container_add(GTK_CONTAINER(wd), image);
-
-  /* Add a menu button. */
+  /* Raise render area. */
   wd = gtk_button_new();
-  gtk_button_set_relief(GTK_BUTTON(wd), GTK_RELIEF_NONE);
-  gtk_box_pack_end(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
+  g_object_set(G_OBJECT(wd), "can-default", TRUE, "can-focus", TRUE,
+	       "has-default", FALSE, "has-focus", FALSE, NULL);
+/*   gtk_button_set_relief(GTK_BUTTON(wd), GTK_RELIEF_NONE); */
+  gtk_box_pack_start(GTK_BOX(window->hbox), wd, FALSE, FALSE, 0);
   g_signal_connect(G_OBJECT(wd), "clicked",
-		   G_CALLBACK(onMainButtonClicked), (gpointer)window);
-  image = create_pixmap((GtkWidget*)0, "stock-menu-detach.png");
+		   G_CALLBACK(onRaiseButtonClicked), (gpointer)window);
+  image = gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU);
   gtk_container_add(GTK_CONTAINER(wd), image);
-  gtk_tooltips_set_tip(tooltips, wd,
-		       _("Manage hidden subpanels and dock windows."), NULL);
-
-  /* Little image. */
-  image = create_pixmap((GtkWidget*)0, "icone-tool.png");
-  gtk_box_pack_start(GTK_BOX(window->hbox), image, FALSE, FALSE, 0);
+  gtk_widget_set_tooltip_text(wd, _("Raise the rendering window.\n"
+				    "  Use <home> as key binding."));
 
   /* The Label to introduce the combo list. */
   wd = gtk_label_new(_("Tool: "));
@@ -725,6 +736,28 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 					  G_CALLBACK(onComboChanged),
 					  (gpointer)window);
 
+  /* Add a menu button. */
+  wd = gtk_button_new();
+  gtk_button_set_relief(GTK_BUTTON(wd), GTK_RELIEF_NONE);
+  gtk_box_pack_start(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
+  g_signal_connect(G_OBJECT(wd), "clicked",
+		   G_CALLBACK(onMainButtonClicked), (gpointer)window);
+  image = create_pixmap((GtkWidget*)0, "stock-menu-detach.png");
+  gtk_container_add(GTK_CONTAINER(wd), image);
+  gtk_widget_set_tooltip_text(wd,
+		       _("Manage hidden subpanels and dock windows."));
+
+  /* Add a help tooltips. */
+  wd = gtk_event_box_new();
+  gtk_event_box_set_visible_window(GTK_EVENT_BOX(wd), FALSE);
+  gtk_widget_set_tooltip_text(wd,
+		       _("Positions, sizes, names, contains... of dock" \
+			 " windows are stored in the parameters file, " \
+			 "see the 'Config. files' button on the command panel."));
+  gtk_box_pack_end(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
+  image = gtk_image_new_from_stock(GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
+  gtk_container_add(GTK_CONTAINER(wd), image);
+
   gtk_widget_show_all(window->vbox);
   return window;
 }
@@ -733,7 +766,8 @@ static gboolean emitPageEnter(gpointer data)
 {
   g_return_val_if_fail(IS_TOOL_PANEL(data), FALSE);
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : '%p' emits 'page-entered' signal.\n", data);
+  DBG_fprintf(stderr, "Gtk ToolPanel: '%s' (%p) emits 'page-entered' signal.\n",
+	      TOOL_PANEL(data)->id, data);
   g_signal_emit(G_OBJECT(data), toolPanel_signals[NOTEBOOK_ENTERED],
 		0, NULL);
   
@@ -759,6 +793,17 @@ static void changeHeaderVisibility(ToolPanel *panel, gboolean status)
   g_list_free(list);
 }
 
+static gboolean onHomePressed(GtkWidget *widget _U_, GdkEventKey *event,
+			      gpointer data)
+{
+  DBG_fprintf(stderr, "Gtk ToolPanel: get key pressed.\n");
+  if(event->keyval == GDK_Home)
+    {
+      onRaiseButtonClicked((GtkButton*)0, data);
+      return TRUE;
+    }
+  return FALSE;
+}
 static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page _U_,
 			  guint num, gpointer data)
 {
@@ -766,21 +811,26 @@ static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page _U_,
   DockWindow *window;
   int numOld;
 
-  wd = gtk_notebook_get_nth_page(book, num);
-  g_return_if_fail(IS_TOOL_PANEL(wd));
-
   window = (DockWindow*)data;
   g_return_if_fail(window);
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : caught 'page-switch' signal, jump to %d.\n", num);
+  wd = gtk_notebook_get_nth_page(book, num);
+  g_return_if_fail(IS_TOOL_PANEL(wd));
+
+  DBG_fprintf(stderr, "Gtk ToolPanel: caught 'page-switch' signal, jump to %d.\n", num);
   /* Change the visibility of the header labels. */
   if (!local_class->showHeader)
     {
       numOld = gtk_combo_box_get_active(GTK_COMBO_BOX(window->combo));
-      changeHeaderVisibility
-	(TOOL_PANEL(gtk_notebook_get_nth_page(book, numOld)), FALSE);
-      changeHeaderVisibility
-	(TOOL_PANEL(gtk_notebook_get_nth_page(book, num)), TRUE);
+      if ((gint)num != numOld)
+	{
+	  /* We don't come from the combo so we cahnge the
+	     visibility. */
+	  changeHeaderVisibility
+	    (TOOL_PANEL(gtk_notebook_get_nth_page(book, numOld)), FALSE);
+	  changeHeaderVisibility
+	    (TOOL_PANEL(gtk_notebook_get_nth_page(book, num)), TRUE);
+	}
     }
   
   /* Change the combo to the right one. */
@@ -788,7 +838,6 @@ static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page _U_,
   gtk_combo_box_set_active(GTK_COMBO_BOX(window->combo), num);
   g_signal_handler_unblock(G_OBJECT(window->combo), window->comboChanged);
 
-
   g_idle_add(emitPageEnter, wd);
 }
 
@@ -824,7 +873,7 @@ static gboolean onKillDockWindowEvent(GtkWidget *widget, GdkEvent *event _U_,
 {
   g_return_val_if_fail(user_data, TRUE);
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : delete or destroy event cancelled.\n");
+  DBG_fprintf(stderr, "Gtk ToolPanel: delete or destroy event cancelled.\n");
   gtk_widget_hide(widget);
   ((DockWindow*)user_data)->show = FALSE;
   return TRUE;
@@ -836,7 +885,7 @@ void toolPanelAttach(ToolPanel *toolPanel, DockWindow *dock)
   ToolPanelClass *klass;
   GdkPixbuf *pixbuf;
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : attach a toolPanel (%p) to a dock"
+  DBG_fprintf(stderr, "Gtk ToolPanel: attach a toolPanel (%p) to a dock"
 	      " window (%s).\n", (gpointer)toolPanel, dock->name);
 
   g_return_if_fail(dock && toolPanel);
@@ -906,7 +955,7 @@ void toolPanelDetach(ToolPanel *toolPanel)
   if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(window->notebook)) == 0 &&
       window != TOOL_PANEL_CLASS(G_OBJECT_GET_CLASS(toolPanel))->commandPanel)
     {
-      DBG_fprintf(stderr, "Gtk ToolPanel : destroying dock window %p (%s).\n",
+      DBG_fprintf(stderr, "Gtk ToolPanel: destroying dock window %p (%s).\n",
 		  (gpointer)window, window->name);
       gtk_widget_destroy(window->window);
       TOOL_PANEL_CLASS(G_OBJECT_GET_CLASS(toolPanel))->hostingWindows =
@@ -921,7 +970,7 @@ void toolPanelDetach(ToolPanel *toolPanel)
   klass = TOOL_PANEL_CLASS(G_OBJECT_GET_CLASS(toolPanel));
   klass->orphanToolPanel = g_list_prepend(klass->orphanToolPanel,
 					  (gpointer)toolPanel);
-  DBG_fprintf(stderr, "Gtk ToolPanel : number of orphan tool panels %d.\n",
+  DBG_fprintf(stderr, "Gtk ToolPanel: number of orphan tool panels %d.\n",
 	      g_list_length(klass->orphanToolPanel));
 }
 
@@ -1009,7 +1058,7 @@ static GtkWidget* buildMainMenu(DockWindow *window)
       gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     }
   
-  DBG_fprintf(stderr, "Gtk ToolPanel : create the main menu %p.\n", (gpointer)menu);
+  DBG_fprintf(stderr, "Gtk ToolPanel: create the main menu %p.\n", (gpointer)menu);
 
   return menu;
 }
@@ -1025,7 +1074,7 @@ static void onMainMenuClicked(GtkMenuItem *menuitem, gpointer user_data)
 
   window = (DockWindow*)user_data;
   
-  DBG_fprintf(stderr, "Gtk ToolPanel : attaching panel '%s' (%p) to %p.\n",
+  DBG_fprintf(stderr, "Gtk ToolPanel: attaching panel '%s' (%p) to %p.\n",
 	      toolPanel->tabLabel, (gpointer)toolPanel, (gpointer)window);
   toolPanelAttach(toolPanel, window);
   g_object_unref(G_OBJECT(toolPanel));
@@ -1054,9 +1103,18 @@ static void onMainMenuHideClicked(GtkMenuItem *menuitem _U_, gpointer user_data)
 }
 static void onMainMenuSelected(GtkMenuShell *menushell, gpointer user_data _U_)
 {
-  DBG_fprintf(stderr, "Gtk ToolPanel : destroy the main menu %p.\n", (gpointer)menushell);
+  DBG_fprintf(stderr, "Gtk ToolPanel: destroy the main menu %p.\n", (gpointer)menushell);
   gtk_widget_destroy(GTK_WIDGET(menushell));
 }
+static void onRaiseButtonClicked(GtkButton *button _U_, gpointer data _U_)
+{
+  GtkWindow *window;
+
+  window = visuGtkGet_render();
+  g_return_if_fail(window);
+  /* We raised the rendering window, if required. */
+  gtk_window_present(window);
+}
 
 
 /******************/
@@ -1186,7 +1244,7 @@ ToolPanel* toolPanelClassGet_toolPanelById(const gchar *id)
       g_object_unref(dummy);
     }
 
-  DBG_fprintf(stderr, "Gtk ToolPanel : search panel '%s'.\n", id);
+  DBG_fprintf(stderr, "Gtk ToolPanel: search panel '%s'.\n", id);
   panel = g_hash_table_lookup(local_class->allToolPanels, id);
   if (!panel)
     return (ToolPanel*)0;
@@ -1242,7 +1300,7 @@ GList* toolPanelClassGet_allToolPanels()
       g_object_unref(dummy);
     }
   
-  DBG_fprintf(stderr, "Gtk ToolPanel : create list of all panels.\n");
+  DBG_fprintf(stderr, "Gtk ToolPanel: create list of all panels.\n");
   returnlst = (GList*)0;
   tmplst = local_class->hostingWindows;
   while (tmplst)
