@@ -41,10 +41,13 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
+#include <gdk/gdkkeysyms.h>
+
 #include "support.h"
 #include "interface.h"
 #include "gtk_interactive.h"
 #include "gtk_main.h"
+#include "visu_gtk.h"
 #include "visu_object.h"
 #include "visu_pickMesure.h"
 
@@ -56,35 +59,54 @@
 #include "extraGtkFunctions/gtk_numericalEntryWidget.h"
 #include "extraGtkFunctions/gtk_orientationChooser.h"
 
-static GtkInteractiveActionId currentAction;
+/* Local types. */
+typedef struct ActionInterface_
+{
+  guint id;
+
+  gchar *label;
+  gchar *help;
+
+  GtkWidget *radio;
+
+  GtkInteractiveBuild build;
+  VisuInteractiveId mode;
+  GtkInteractiveStartStop onStart, onStop;
+} ActionInterface;
+
+/* Local variables. */
+static ActionInterface* currentAction;
+static guint n_actions = 0;
+static GList *actions;
+static gulong onStop_id;
 
 /* Callbacks. */
+static gboolean onHomePressed(GtkWidget *widget _U_, GdkEventKey *event,
+			      gpointer data);
 static void onNewData(GObject *obj, VisuData *dataObj, gpointer data);
 static void onVisuData(GObject *obj, VisuData *dataObj, gpointer data);
 static void observeMethodChanged(GtkToggleButton* button, gpointer data);
 static void radioObserveToggled(GtkToggleButton *togglebutton, gpointer user_data);
-static void radioPickToggled(GtkToggleButton *togglebutton, gpointer user_data);
-static void radioMoveToggled(GtkToggleButton *togglebutton, gpointer user_data);
-static void radioMarkToggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void onCloseButtonClicked(GtkButton *button, gpointer user_data);
 static void refreshCameraSpinValues(VisuData *obj, OpenGLView *view, gpointer data);
 static void onTabActionChanged(GtkNotebook *book, GtkNotebookPage *page,
 			       guint num, gpointer data);
-static void onEraseMarksClicked(GtkButton *button, gpointer user_data);
 static void onOrientationChooser(GtkButton *button, gpointer data);
 static void onAngleChanged(GtkSpinButton *spin, gpointer data);
 static void onTranslationChanged(GtkSpinButton *spin, gpointer data);
 static void onZoomChanged(GtkSpinButton *spin, gpointer data);
 static void onPerspChanged(GtkSpinButton *spin, gpointer data);
-static void gtkObservePick_marks(PickMesure *pickMesure, gpointer data);
 static gboolean onKillWindowEvent(GtkWidget *widget, GdkEvent *event,
 				  gpointer user_data);
+static void onRadioToggled(GtkToggleButton *toggle, gpointer data);
+static void onObserveStart(RenderingWindow *window);
+static void onObserveStop(RenderingWindow *window);
 
 /* Local methods. */
-static void setSensitiveActionSensitive(GtkInteractiveActionId id);
 static void connectSignalsObservePick();
 static void setNamesGtkWidgetObservePick(GtkMain *main);
-
+static GtkWidget* gtkObserveBuild_interface(GtkMain *main, gchar **label,
+					    gchar **help, GtkWidget **radio);
 
 /* Widgets */
 static GtkWidget *observeWindow;
@@ -97,180 +119,180 @@ static GtkWidget *spinPersp;
 
 /* Help message used in the help area. */
 #define GTK_PICKOBSERVE_OBSERVE_INFO \
-  _("left-[Control]-button\t\t\t: rotations "				\
-    "(\316\270, \317\206, [Control]\317\211)\n"				\
+  _("left-[control]-button\t\t\t: rotations "				\
+    "(\316\270, \317\206, [control]\317\211)\n"				\
     "shift-left-button\t\t\t\t: translations (dx, dy)\n"		\
-    "middle-[Shift]-button or wheel\t: zoom or [Shift]perspective\n"	\
+    "middle-[shift]-button or wheel\t: zoom or [shift]perspective\n"	\
     "key 's' / 'r'\t\t\t\t\t: save/restore camera position\n"		\
     "right-button\t\t\t\t\t: switch to current tabbed action")
-#define GTK_PICKOBSERVE_PICK_INFO				\
-  _("left-button\t\t\t: standard pick\n"			\
-    "middle-button\t\t: pick 1st reference\n"			\
-    "shift-middle-button\t: pick 2nd reference\n"		\
-    "drag-left-button\t\t: make a rectangular selection\n"	\
-    "right-button\t\t\t: switch to observe")
-#define GTK_PICKOBSERVE_MOVE_INFO				\
-  _("left-button\t\t: drag node(s) in the screen plane\n"	\
-    "shift-left-button\t: drag node(s) along x or y axis\n"	\
-    "control-left-button\t: drag node(s) along z axis\n"	\
-    "right-button\t\t: switch to observe")
-#define GTK_PICKOBSERVE_MARK_INFO		\
-  _("left-button\t: select (unselect) a node\n"	\
-    "right-button\t: switch to observe")
-static char *markupObserveHelp, *markupPickHelp, *markupMoveHelp, *markupMarkHelp;
-
-/* Flag used to block callbacks when changing are done. */
-static gboolean flagDisableChangePickObserve;
-
 
 /* Initialise the observe/pick window, connect the
    signals, give names to widgets... */
 void gtkInteractiveInit()
 {
   orientationChooser = (GtkWidget*)0;
+  /* Set the actions tabs. */
+  actions = (GList*)0;
+
+  gtkInteractiveAdd_action(gtkObserveBuild_interface,
+			   onObserveStart, onObserveStop);
+  currentAction = (ActionInterface*)actions->data;
+
+  gtkInteractiveAdd_action(gtkPickBuild_interface,
+			   gtkPickStart, gtkPickStop);
+
+  gtkInteractiveAdd_action(gtkMoveBuild_interface,
+			   gtkMoveStart, gtkMoveStop);
+}
+
+guint gtkInteractiveAdd_action(GtkInteractiveBuild build,
+			       GtkInteractiveStartStop start,
+			       GtkInteractiveStartStop stop)
+{
+  ActionInterface *action;
+
+  g_return_val_if_fail(build && start && stop, 0);
+
+  action = g_malloc(sizeof(ActionInterface));
+  action->id      = n_actions;
+  action->build   = build;
+  action->onStart = start;
+  action->onStop  = stop;
+
+  actions       = g_list_append(actions, action);
+  n_actions += 1;
+
+  return action->id;
+}
+
+static GtkWidget* gtkObserveBuild_interface(GtkMain *main _U_, gchar **label,
+					    gchar **help, GtkWidget **radio)
+{
+  *label = g_strdup("Observe");
+  *help  = g_strdup(GTK_PICKOBSERVE_OBSERVE_INFO);
+  *radio = lookup_widget(observeWindow, "radioObserve");
+
+  return (GtkWidget*)0;
 }
 
 void gtkInteractiveBuild_window(GtkMain *main)
 {
-  GtkWidget *labelHelp;
+  GtkWidget *labelHelp, *wd;
   RenderingWindow *window;
   VisuData *data;
+  GList *tmpLst;
+  ActionInterface *action;
+  gchar *msg;
+  GSList *lst;
  
   window = visuRenderingWindowGet_current();
   g_return_if_fail(window);
   data = renderingWindowGet_visuData(window);
   g_return_if_fail(data);
 
-  DBG_fprintf(stderr, "Gtk observePick : creating the window.\n");
+  DBG_fprintf(stderr, "Gtk observePick: creating the window.\n");
   main->interactiveDialog = create_observeDialog();
   observeWindow = main->interactiveDialog;
   gtk_window_set_type_hint(GTK_WINDOW(main->interactiveDialog),
 			   GDK_WINDOW_TYPE_HINT_NORMAL);
   setNamesGtkWidgetObservePick(main);
-  /* Create and Set the help text. */
-  markupObserveHelp = g_markup_printf_escaped("<span size=\"smaller\">%s</span>",
-					      GTK_PICKOBSERVE_OBSERVE_INFO);
-  markupPickHelp = g_markup_printf_escaped("<span size=\"smaller\">%s</span>",
-					    GTK_PICKOBSERVE_PICK_INFO);
-  markupMoveHelp = g_markup_printf_escaped("<span size=\"smaller\">%s</span>",
-					    GTK_PICKOBSERVE_MOVE_INFO);
-  markupMarkHelp = g_markup_printf_escaped("<span size=\"smaller\">%s</span>",
-					    GTK_PICKOBSERVE_MARK_INFO);
-  labelHelp = lookup_widget(main->interactiveDialog, "labelInfoObservePick");
-  gtk_label_set_markup(GTK_LABEL(labelHelp), markupObserveHelp);
-  currentAction = action_observe;
+  g_signal_connect(G_OBJECT(observeWindow), "key-press-event",
+		   G_CALLBACK(onHomePressed), (gpointer)observeWindow);
 
-  /* Create the pick/move parts. */
-  gtkPickBuild_interface(main);
-  gtkMoveBuild_interface(main);
+  /* Create the actions parts. */
+  lst = (GSList*)0;
+  for (tmpLst = actions; tmpLst; tmpLst = g_list_next(tmpLst))
+    {
+      action = ((ActionInterface*)tmpLst->data);
+      wd = action->build(main, &action->label, &msg, &action->radio);
+      action->help = g_markup_printf_escaped("<span size=\"smaller\">%s</span>", msg);
+      g_free(msg);
+      if (wd)
+	gtk_notebook_append_page
+	  (GTK_NOTEBOOK(lookup_widget(observeWindow, "notebookAction")),
+	   wd, gtk_label_new(action->label));
+      if (action->id > 0)
+	gtk_radio_button_set_group(GTK_RADIO_BUTTON(action->radio), lst);
+      lst = gtk_radio_button_get_group(GTK_RADIO_BUTTON(action->radio));
+      g_signal_connect(G_OBJECT(action->radio), "toggled",
+		       G_CALLBACK(onRadioToggled), (gpointer)action);
+    }
+
+  /* Create and Set the help text. */
+  action = ((ActionInterface*)actions->data);
+  labelHelp = lookup_widget(main->interactiveDialog, "labelInfoObservePick");
+  gtk_label_set_markup(GTK_LABEL(labelHelp), action->help);
 
   /* connect the signals to the spins. */
   connectSignalsObservePick(main);
+
   onNewData((GObject*)0, data, (gpointer)0);
   /* set the initial values for the spins. */
   refreshCameraSpinValues((VisuData*)0, visuDataGet_openGLView(data), (gpointer)0);
 }
 
 /* Methods dealing with observe & pick session. */
-void stopPickObserveGtk()
+static void onStop(VisuInteractive *inter _U_, gpointer data)
 {
   GtkWidget *wd;
-  gint id;
+  guint id;
+  ActionInterface *action;
 
-  if (flagDisableChangePickObserve)
-    return;
-
-  wd = (GtkWidget*)0;
-  if (currentAction == action_observe)
+  if (currentAction->id == action_observe)
     {
       wd = lookup_widget(observeWindow, "notebookAction");
-      id = gtk_notebook_get_current_page(GTK_NOTEBOOK(wd));
-      switch (id + 1)
-	{
-	case action_pick:
-	  wd = lookup_widget(observeWindow, "radioPick");
-	  break;
-	case action_move:
-	  wd = lookup_widget(observeWindow, "radioMove");
-	  break;
-	case action_mark:
-	  wd = lookup_widget(observeWindow, "radioSelect");
-	  break;
-	default:
-	  wd = (GtkWidget*)0;
-	};
+      id = gtk_notebook_get_current_page(GTK_NOTEBOOK(wd)) + 1;
     }
   else
-    wd = lookup_widget(observeWindow, "radioObserve");
+    id = action_observe;
+  action = (ActionInterface*)g_list_nth_data(actions, id);
 
-  if (wd)
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
+  /* We stop the current action. */
+  currentAction->onStop(RENDERING_WINDOW(data));
+
+  /* We toggle on the radio of the new action. */
+  if (action->radio)
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(action->radio), TRUE);
 }
 
-static void gtkObservePick_marks(PickMesure *pickMesure, gpointer data _U_)
+void gtkInteractiveInit_session(RenderingWindow *window, VisuInteractive *inter)
 {
-  if (currentAction == action_pick)
-    gtkPickUpdate_informations(pickMesure);
-  else if (currentAction == action_move)
-    gtkMoveUpdate_informations(pickMesure);
-  else if (currentAction == action_mark)
-    g_idle_add(visuObjectRedraw, GINT_TO_POINTER(1));
+  DBG_fprintf(stderr, "Gtk Interactive: initialise session.\n");
+
+  /* We attach the new signals. */
+  onStop_id = g_signal_connect(G_OBJECT(inter), "stop",
+			       G_CALLBACK(onStop), (gpointer)window);
+
+  /* We empty the status bar. */
+  renderingWindowPush_message(window, "");
+
+  /* We start the new mode. */
+  currentAction->onStart(window);
 }
 
-void gtkInteractiveInit_session()
+static void gtkInteractiveStop()
 {
   VisuInteractive *inter;
-  VisuData *data;
+  RenderingWindow *window;
 
-  DBG_fprintf(stderr, "Gtk Interactive: initialise session.\n");
-  renderingWindowBlock_defaultIneractiveMode(RENDERING_WINDOW(currentRenderingWindow));
-  renderingWindowPush_message(RENDERING_WINDOW(currentRenderingWindow), "");
+  window = RENDERING_WINDOW(currentRenderingWindow);
+  inter = renderingWindowGet_interactive(window);
 
-  data = renderingWindowGet_visuData(RENDERING_WINDOW(currentRenderingWindow));
-  g_return_if_fail(data);
+  currentAction->onStop(window);
 
-  inter = openGLInteractiveInit_session((GenericRenderingWindow)currentRenderingWindow,
-					gtkObservePick_marks, (gpointer)data,
-					stopPickObserveGtk);
-  pickMesureSet_formatedOutput(openGLInteractiveGet_pickMesure(inter), TRUE);
-  g_object_set_data(G_OBJECT(observeWindow), "visuInteractive_data", (gpointer)inter);
+  /* We reset the status bar. */
+  renderingWindowPop_message(window);
+  /* We unblock the default mode. */
+  renderingWindowUnblock_defaultIneractiveMode(window);
 
-  setSensitiveActionSensitive(currentAction);
-  switch (currentAction)
-    {
-    case action_observe:
-      openGLInteractiveBegin_observe(inter);
-      break;
-    case action_pick:
-      openGLInteractiveBegin_pick(inter);
-      break;
-    case action_move:
-      openGLInteractiveBegin_move(inter);
-      break;
-    case action_mark:
-      openGLInteractiveBegin_mark(inter);
-      break;
-    }
-  flagDisableChangePickObserve = FALSE;
-}
-
-static void stopInteractive()
-{
-  gtkPick_onClose();
-
-  flagDisableChangePickObserve = TRUE;
-  openGLInteractiveEnd_session((VisuInteractive*)g_object_get_data
-			       (G_OBJECT(observeWindow), "visuInteractive_data"));
-
-  renderingWindowPop_message(RENDERING_WINDOW(currentRenderingWindow));
-  renderingWindowUnblock_defaultIneractiveMode(RENDERING_WINDOW(currentRenderingWindow));
+  g_signal_handler_disconnect(G_OBJECT(inter), onStop_id);
 }
 static gboolean onKillWindowEvent(GtkWidget *widget _U_, GdkEvent *event _U_,
 				  gpointer user_data _U_)
 {
   DBG_fprintf(stderr, "Gtk interactive: window killed.\n");
 
-  stopInteractive();
+  gtkInteractiveStop();
 
   return FALSE;
 }
@@ -280,47 +302,9 @@ static void onCloseButtonClicked(GtkButton *button _U_, gpointer user_data _U_)
 
   if (!GTK_WIDGET_VISIBLE(observeWindow))
     return;
-
-  stopInteractive();
+  gtkInteractiveStop();
 }
 
-static void setSensitiveObserve(gboolean value)
-{
-  GtkWidget *wd;
-
-  wd = lookup_widget(observeWindow, "hboxObserve");
-  gtk_widget_set_sensitive(wd, value);
-  wd = lookup_widget(observeWindow, "tableObserve");
-  gtk_widget_set_sensitive(wd, value);
-  if (openGLInteractiveGet_preferedObserveMethod() == OPENGL_OBSERVE_CONSTRAINED)
-    gtk_widget_set_sensitive(spinOmega, FALSE);
-  else
-    gtk_widget_set_sensitive(spinOmega, value);
-}
-static void setSensitiveMove(gboolean value)
-{
-  GtkWidget *wd;
-
-  wd = lookup_widget(observeWindow, "alignment27");
-  gtk_widget_set_sensitive(wd, value);
-  wd = lookup_widget(observeWindow, "hboxAddNode");
-  gtk_widget_set_sensitive(wd, value);
-}
-static void setSensitiveMark(gboolean value _U_)
-{
-/*   wd = lookup_widget(observeWindow, "tableMovePick"); */
-/*   gtk_widget_set_sensitive(wd, value); */
-}
-
-static void setSensitiveActionSensitive(GtkInteractiveActionId id)
-{
-  currentAction = id;
-  setSensitiveObserve((id == action_observe));
-  setSensitiveMove((id == action_move));
-  setSensitiveMark((id == action_mark));
-}
-
-
 /****************/
 /* Private part */
 /****************/
@@ -346,15 +330,6 @@ static void connectSignalsObservePick()
   wd = lookup_widget(observeWindow, "radioObserve");
   g_signal_connect(G_OBJECT(wd), "toggled",
 		   G_CALLBACK(radioObserveToggled), (gpointer)0);
-  wd = lookup_widget(observeWindow, "radioPick");
-  g_signal_connect(G_OBJECT(wd), "toggled",
-		   G_CALLBACK(radioPickToggled), (gpointer)0);
-  wd = lookup_widget(observeWindow, "radioMove");
-  g_signal_connect(G_OBJECT(wd), "toggled",
-		   G_CALLBACK(radioMoveToggled), (gpointer)0);
-  wd = lookup_widget(observeWindow, "radioSelect");
-  g_signal_connect(G_OBJECT(wd), "toggled",
-		   G_CALLBACK(radioMarkToggled), (gpointer)0);
 
   wd = lookup_widget(observeWindow, "buttonOrientationChooser");
   g_signal_connect(G_OBJECT(wd), "clicked",
@@ -384,18 +359,14 @@ static void connectSignalsObservePick()
 		   G_CALLBACK(onPerspChanged), (gpointer)0);
   wd = lookup_widget(observeWindow, "radioObserveConstrained");
   g_signal_connect(G_OBJECT(wd), "toggled", G_CALLBACK(observeMethodChanged),
-		   GINT_TO_POINTER(OPENGL_OBSERVE_CONSTRAINED));
+		   GINT_TO_POINTER(interactive_constrained));
   wd = lookup_widget(observeWindow, "radioObserveWalker");
-  g_signal_connect(G_OBJECT(wd), "toggled",
-		   G_CALLBACK(observeMethodChanged), GINT_TO_POINTER(OPENGL_OBSERVE_WALKER));
+  g_signal_connect(G_OBJECT(wd), "toggled", G_CALLBACK(observeMethodChanged),
+		   GINT_TO_POINTER(interactive_walker));
 
   wd = lookup_widget(observeWindow, "notebookAction");
   g_signal_connect(G_OBJECT(wd), "switch-page",
 		   G_CALLBACK(onTabActionChanged), (gpointer)0);
-
-  wd = lookup_widget(observeWindow, "buttonEraseMarks");
-  g_signal_connect(G_OBJECT(wd), "clicked",
-		   G_CALLBACK(onEraseMarksClicked), (gpointer)0);
 }
 
 static void onNewData(GObject *obj _U_, VisuData *dataObj, gpointer data _U_)
@@ -415,17 +386,28 @@ static void onVisuData(GObject *obj _U_, VisuData *dataObj, gpointer data)
     gtk_button_clicked(GTK_BUTTON(data));
 }
 
+static void onObserveStart(RenderingWindow *window)
+{
+  visuInteractiveSet_type(renderingWindowGet_interactive(window),
+			  interactive_observe);
+}
+static void onObserveStop(RenderingWindow *window)
+{
+  visuInteractiveSet_type(renderingWindowGet_interactive(window),
+			  interactive_none);
+}
+
 static void observeMethodChanged(GtkToggleButton* button, gpointer data)
 {
-  int method;
+  VisuInteractiveMethod method;
   RenderingWindow *window;
 
   if (!gtk_toggle_button_get_active(button))
     return;
 
-  method = GPOINTER_TO_INT(data);
-  openGLInteractiveSet_preferedObserveMethod(method);
-  if (method == OPENGL_OBSERVE_CONSTRAINED)
+  method = (VisuInteractiveMethod)GPOINTER_TO_INT(data);
+  visuInteractiveClassSet_preferedObserveMethod(method);
+  if (method == interactive_constrained)
     {
       window = RENDERING_WINDOW(visuRenderingWindowGet_current());
       g_return_if_fail(window);
@@ -481,8 +463,6 @@ static void setNamesGtkWidgetObservePick(GtkMain *main)
   gtk_widget_set_name(wd, "message_radio");
   wd = lookup_widget(main->interactiveDialog, "radioMove");
   gtk_widget_set_name(wd, "message_radio");
-  wd = lookup_widget(main->interactiveDialog, "radioSelect");
-  gtk_widget_set_name(wd, "message_radio");
 
   spinTheta = lookup_widget(main->interactiveDialog, "spinTheta");
   spinPhi = lookup_widget(main->interactiveDialog, "spinPhi");
@@ -492,23 +472,38 @@ static void setNamesGtkWidgetObservePick(GtkMain *main)
   spinGross = lookup_widget(main->interactiveDialog, "spinGross");
   spinPersp = lookup_widget(main->interactiveDialog, "spinPersp");
 
-  if (openGLInteractiveGet_preferedObserveMethod() == OPENGL_OBSERVE_CONSTRAINED)
+  method = visuInteractiveClassGet_preferedObserveMethod();
+  if (method == interactive_constrained)
     gtk_widget_set_sensitive(spinOmega, FALSE);
 
   wd = lookup_widget(main->interactiveDialog, "notebookAction");
   gtk_widget_set_name(wd, "message_notebook");
 
-  method = openGLInteractiveGet_preferedObserveMethod();
   wd = lookup_widget(main->interactiveDialog, "radioObserveConstrained");
   gtk_widget_set_name(wd, "message_radio");
-  if (method == OPENGL_OBSERVE_CONSTRAINED)
+  if (method == interactive_constrained)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
   wd = lookup_widget(main->interactiveDialog, "radioObserveWalker");
   gtk_widget_set_name(wd, "message_radio");
-  if (method == OPENGL_OBSERVE_WALKER)
+  if (method == interactive_walker)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
 }
+static gboolean onHomePressed(GtkWidget *widget _U_, GdkEventKey *event,
+			      gpointer data _U_)
+{
+  GtkWindow *window;
 
+  DBG_fprintf(stderr, "Gtk Interactive: get key pressed.\n");
+  if(event->keyval == GDK_Home)
+    {
+      window = visuGtkGet_render();
+      g_return_val_if_fail(window, FALSE);
+      /* We raised the rendering window, if required. */
+      gtk_window_present(window);
+      return TRUE;
+    }
+  return FALSE;
+}
 static void onAngleChanged(GtkSpinButton *spin, gpointer data)
 {
   int reDrawNeeded;
@@ -579,120 +574,53 @@ static void onPerspChanged(GtkSpinButton *spin, gpointer data _U_)
     g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
+static void onRadioToggled(GtkToggleButton *toggle, gpointer data)
+{
+  GtkWidget *labelHelp;
+
+  if (!gtk_toggle_button_get_active(toggle))
+    return;
+
+  currentAction = (ActionInterface*)data;
+  DBG_fprintf(stderr, "Gtk Interactive: set the action %d current.\n",
+	      currentAction->id);
+
+  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
+  gtk_label_set_markup(GTK_LABEL(labelHelp), currentAction->help);
+
+  currentAction->onStart(RENDERING_WINDOW(currentRenderingWindow));
+}
+
 static void radioObserveToggled(GtkToggleButton *togglebutton, gpointer user_data _U_)
 {
-  GtkWidget *labelHelp;
+  gboolean value;
+  GtkWidget *wd;
 
-  if (!gtk_toggle_button_get_active(togglebutton))
-    return;
+  value = gtk_toggle_button_get_active(togglebutton);
 
-  DBG_fprintf(stderr, "Gtk observePick : Toggle of the radio button in observe window.\n");
-
-  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
-  gtk_label_set_markup(GTK_LABEL(labelHelp), markupObserveHelp);
-
-  flagDisableChangePickObserve = TRUE;
-  setSensitiveActionSensitive(action_observe);
-  openGLInteractiveBegin_observe((VisuInteractive*)g_object_get_data
-				 (G_OBJECT(observeWindow), "visuInteractive_data"));
-
-  flagDisableChangePickObserve = FALSE;
-}
-static void radioPickToggled(GtkToggleButton *togglebutton, gpointer user_data _U_)
-{
-  GtkWidget *labelHelp;
-
-  if (!gtk_toggle_button_get_active(togglebutton))
-    return;
-
-  DBG_fprintf(stderr, "Gtk observePick : Toggle of the radio button in observe window.\n");
-
-  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
-  gtk_label_set_markup(GTK_LABEL(labelHelp), markupPickHelp);
-
-  setSensitiveActionSensitive(action_pick);
-  openGLInteractiveBegin_pick((VisuInteractive*)g_object_get_data
-			      (G_OBJECT(observeWindow), "visuInteractive_data"));
-
-  flagDisableChangePickObserve = FALSE;
-}
-static void radioMoveToggled(GtkToggleButton *togglebutton, gpointer user_data _U_)
-{
-  GtkWidget *labelHelp;
-
-  if (!gtk_toggle_button_get_active(togglebutton))
-    return;
-
-  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
-  gtk_label_set_markup(GTK_LABEL(labelHelp), markupMoveHelp);
-
-  setSensitiveActionSensitive(action_move);
-
-  openGLInteractiveBegin_move((VisuInteractive*)g_object_get_data
-			      (G_OBJECT(observeWindow), "visuInteractive_data"));
-
-  flagDisableChangePickObserve = FALSE;
-}
-static void radioMarkToggled(GtkToggleButton *togglebutton, gpointer user_data _U_)
-{
-  GtkWidget *labelHelp;
-  RenderingWindow *window;
-
-  if (!gtk_toggle_button_get_active(togglebutton))
-    return;
-
-  labelHelp = lookup_widget(observeWindow, "labelInfoObservePick");
-  gtk_label_set_markup(GTK_LABEL(labelHelp), markupMarkHelp);
-
-  setSensitiveActionSensitive(action_mark);
-
-  window = RENDERING_WINDOW(visuRenderingWindowGet_current());
-  g_return_if_fail(window);
-  openGLInteractiveBegin_mark((VisuInteractive*)g_object_get_data
-			      (G_OBJECT(observeWindow), "visuInteractive_data"));
-
-  flagDisableChangePickObserve = FALSE;
+  wd = lookup_widget(observeWindow, "hboxObserve");
+  gtk_widget_set_sensitive(wd, value);
+  wd = lookup_widget(observeWindow, "tableObserve");
+  gtk_widget_set_sensitive(wd, value);
+  if (visuInteractiveClassGet_preferedObserveMethod() == interactive_constrained)
+    gtk_widget_set_sensitive(spinOmega, FALSE);
+  else
+    gtk_widget_set_sensitive(spinOmega, value);
 }
 
 static void onTabActionChanged(GtkNotebook *book _U_, GtkNotebookPage *page _U_,
 			       guint num, gpointer data _U_)
 {
-  GtkWidget *wd;
+  ActionInterface *action;
 
-  DBG_fprintf(stderr, "Gtk Observe/Pick : change the action tab to %d.\n", num);
-  switch (num + 1)
-    {
-    case action_pick:
-      wd = lookup_widget(observeWindow, "radioPick");
-      break;
-    case action_move:
-      gtkPick_onClose();
-      wd = lookup_widget(observeWindow, "radioMove");
-      break;
-    case action_mark:
-      gtkPick_onClose();
-      wd = lookup_widget(observeWindow, "radioSelect");
-      break;
-    default:
-      wd = (GtkWidget*)0;
-    };
+  DBG_fprintf(stderr, "Gtk Interactive: change the action tab to %d.\n", num);
 
-  if (wd)
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wd), TRUE);
-}
-static void onEraseMarksClicked(GtkButton *button _U_, gpointer user_data _U_)
-{
-  RenderingWindow *window;
-  PickMesure *pickMesure;
+  /* We stop the current action. */
+  currentAction->onStop(RENDERING_WINDOW(currentRenderingWindow));
 
-  window = RENDERING_WINDOW(visuRenderingWindowGet_current());
-  pickMesure = (PickMesure*)visuDataGet_property(renderingWindowGet_visuData(window),
-						 "pickMesure_data");
-  g_return_if_fail(pickMesure);
-
-  DBG_fprintf(stderr, "Gtk RenderingWindow: remove all highlights.\n");
-  if (pickMesureRemove_allHighlights(pickMesure))
-    g_idle_add(visuObjectRedraw, (gpointer)0);
+  action = (ActionInterface*)g_list_nth_data(actions, num + 1);
+  if (action->radio)
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(action->radio), TRUE);
 }
 static void onOrientationChanged(OrientationChooser *orientationChooser,
 				 gpointer data _U_)
