@@ -45,13 +45,14 @@
 #include <config.h>
 #endif
 
-#include "gtk/gtkwidget.h"
+#include "gtk/gtk.h"
 
 #include <GL/gl.h>
 
 #include "gtk_openGLWidget.h"
 #include "OSOpenGL/visu_openGL.h"
 #include "visu_tools.h"
+#include "openGLFunctions/text.h"
 
 /* OpenGL implementation dependencies */
 #ifdef HAVE_GTKGLEXT
@@ -92,7 +93,7 @@ struct _OpenGLWidget
 
   /* Redraw method and user data. */
   RedrawMethod redraw;
-  gpointer redrawData;
+  VisuData *redrawData;
 
   /* OpenGL part, OS dependent. */
   gboolean isContextDirect;
@@ -225,7 +226,7 @@ static void openGLWidgetEvent_dispose(GObject *obj)
 }
 static void openGLWidgetEvent_finalize(GObject *obj)
 {
-  DBG_fprintf(stderr, "Gtk OpenGL (events) : finalize event for object %p.\n", (gpointer)obj);
+  DBG_fprintf(stderr, "Gtk OpenGL (events): finalize event for object %p.\n", (gpointer)obj);
 
   openGLWidgetFree_openGL(OPENGL_WIDGET(obj));
 
@@ -274,7 +275,7 @@ static void openGLWidgetEvent_sizeRequest(GtkWidget *widget, GtkRequisition *req
   OpenGLWidget *render;
   int width, height;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (events) : size request event (%dx%d).\n", requisition->width, requisition->height);
+  DBG_fprintf(stderr, "Gtk OpenGL (events): size request event (%dx%d).\n", requisition->width, requisition->height);
 
   render = OPENGL_WIDGET(widget);
   if (render->sizeAllocation_has_run)
@@ -301,7 +302,7 @@ static void openGLWidgetEvent_sizeAllocate(GtkWidget *widget, GtkAllocation *all
       (widget->allocation.height == allocation->height))
     return;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (events) : size allocation event (%dx%d).\n", allocation->width, allocation->height);
+  DBG_fprintf(stderr, "Gtk OpenGL (events): size allocation event (%dx%d).\n", allocation->width, allocation->height);
 
   render->sizeAllocation_has_run = TRUE;
   /* Chain up to default that simply reads current requisition */
@@ -342,6 +343,7 @@ static void openGLWidgetEvent_realise(GtkWidget *widget)
   attributes.visual = gdk_colormap_get_visual(colormap);
   attributes.colormap = colormap;
   attributes.event_mask = GDK_EXPOSURE_MASK |
+    GDK_VISIBILITY_NOTIFY_MASK |
     GDK_BUTTON_PRESS_MASK |
     GDK_BUTTON_RELEASE_MASK |
     GDK_SCROLL_MASK |
@@ -409,7 +411,7 @@ static void openGLWidgetSet_viewport(OpenGLWidget *render, guint width,
       openGLWidgetSwap_buffers(render);
     }
 }
-void openGLWidgetSet_redraw(OpenGLWidget *render, RedrawMethod method, gpointer data)
+void openGLWidgetSet_redraw(OpenGLWidget *render, RedrawMethod method, VisuData *data)
 {
   g_return_if_fail(IS_OPENGL_WIDGET(render));
   
@@ -480,10 +482,10 @@ guchar* openGLWidgetGet_pixmapData(OpenGLWidget *render, int *width,
   render->redraw((const char**)0, render->redrawData);
   /* We copy the pixmap into generic data. */
   image = visuOpenGLGet_pixmapData((guint)*width, (guint)*height, hasAlpha);
-  /* We free the pixmap context. */
-  visuOpenGLFree_pixmapContext(dumpData);
   /* We change back the context to the current rendering area. */
   openGLWidgetSet_current(render, TRUE);
+  /* We free the pixmap context. */
+  visuOpenGLFree_pixmapContext(dumpData);
 
   return image;
 }
@@ -566,6 +568,7 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
   DBG_fprintf(stderr, "Gtk OpenGL (glext): %p is set current.\n", (gpointer)render);
   res = gdk_gl_drawable_make_current(GDK_GL_DRAWABLE(render->glwindow),
 				     render->context);
+  openGLText_onNewContext();
 
   if (!res)
     {
@@ -686,6 +689,10 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
   if (!force && OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render)
     return TRUE;
 
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): unset the context.\n");
+  glXMakeCurrent(render->dpy, None, NULL);
+  glXWaitX();
+  
   DBG_fprintf(stderr, "Gtk OpenGL (X11): widget visualID %d.\n",
 	      (int)gdk_x11_visual_get_xvisual
 	      (gtk_widget_get_visual(GTK_WIDGET(render)))->visualid);
@@ -693,7 +700,7 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
   DBG_fprintf(stderr, "Gtk OpenGL (X11): %p is set current.\n", (gpointer)render);
 
   windowId = gdk_x11_drawable_get_xid(GDK_DRAWABLE(GTK_WIDGET(render)->window));
-  DBG_fprintf(stderr, "Gtk OpenGL (X11): widget windowID %d.\n", (int)windowId);
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): widget windowID 0x%x.\n", (int)windowId);
 
   res = glXMakeCurrent(render->dpy, (GLXDrawable)windowId,
 		       render->context);
@@ -703,7 +710,9 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
 		(gpointer)render);
       return FALSE;
     }
+  glXWaitX();
   DBG_fprintf(stderr, "Gtk OpenGL (X11): OK context changed.\n");
+  openGLText_onNewContext();
 
   /* Now that the glx tunnel has been added, we need
      to specify again that we want a backing store because until now
@@ -772,6 +781,7 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force _U_)
   DBG_fprintf(stderr, "Gtk OpenGL (action) : %p is set current.\n", (gpointer)render);
   wglMakeCurrent(NULL, NULL);
   wglMakeCurrent(render->hdc, render->context);
+  openGLText_onNewContext();
 
   OPENGL_WIDGET_GET_CLASS(render)->contextCurrent = render;
 
