@@ -43,7 +43,6 @@
 */
 #include "gtk_pairs.h"
 #include "pairsModeling/externalGtkPairsExtensions.h"
-#include "gtk_main.h"
 #include "extraGtkFunctions/gtk_colorComboBoxWidget.h"
 #include "renderingBackend/visu_windowInterface.h"
 #include "gtk_renderingWindowWidget.h"
@@ -57,8 +56,8 @@
 #include "visu_pairs.h"
 
 
-GtkWidget *vboxPairsDialog;
-GtkWidget *specificWindow;
+static GtkWidget *vboxPairsDialog;
+static GtkWidget *specificWindow;
 
 
 /* Current pair list is out of date. */
@@ -75,7 +74,7 @@ struct specBuildFunc_struct
 static GtkWidget *gtkPairs_treeView;
 static GtkWidget *widgetColor;
 static GtkWidget *hboxCheckWidgets;
-static GtkWidget *comboFilter, *btAdd, *btRemove;
+static GtkWidget *comboFilter, *btAdd, *btRemove, *btHide;
 static gulong widgetColorSignalId;
 
 static gulong gtkPairs_spinMinSignalId, gtkPairs_spinMaxSignalId;
@@ -120,7 +119,6 @@ GtkWidget *gtkPairs_comboWidget;
 GtkPairsSetValuesFunc currentSignalSelectedFunc;
 
 /* Local routines. */
-void addLabelToColorCombo(GtkListStore *list, GtkTreeIter *iter, Color* color);
 static void minMaxChanged(GtkSpinButton *spin, GtkTreeSelection *tree, int minMax);
 static void addPairAllData(GtkPairsGetValuesAsLabelFunc labelFunc,
 			   VisuElement *ele1, VisuElement *ele2);
@@ -148,6 +146,7 @@ static gboolean pairsRowIsVisible(GtkTreeModel *model, GtkTreeIter *iter,
 				  gpointer data);
 static void onBtAdd(GtkButton *button, gpointer user_data);
 static void onBtRemove(GtkButton *button, gpointer user_data);
+static void onBtHide(GtkToggleButton *button, gpointer user_data);
 static void onDistanceAutoSet(GtkButton *button, gpointer data);
 
 #define GTK_PAIRS_HELP_TEXT _("Modifications, as color or wire width, are applied only to selected rows. Use <control> to select more than one row at a time.")
@@ -155,7 +154,7 @@ static void onDistanceAutoSet(GtkButton *button, gpointer data);
 /* Create the label for the distance, need to free result. */
 gchar* createDistanceLabel(VisuPairData *data);
 /* Create the area for specific widgets of current rendering pairs method. */
-void setSpecificArea(struct specBuildFunc_struct *specBuild);
+static void setSpecificArea(struct specBuildFunc_struct *specBuild);
 
 /* Connect the signal and initialise values. */
 void gtkPairsInit()
@@ -203,8 +202,8 @@ void gtkPairsInit()
 
 void gtkPairsBuild_window(GtkMain *main)
 {
-  GtkWidget *wd, *hbox, *image;
-  GtkWidget *viewport;
+  GtkWidget *wd, *image, *vbox;
+  GtkWidget *viewport, *align;
   PairsExtension* pairs, *tmpPairs;
   GtkWidget *label;
   GtkObject *adj;
@@ -215,10 +214,14 @@ void gtkPairsBuild_window(GtkMain *main)
   gboolean validIter;
   struct specBuildFunc_struct *specBuild;
   char* markup;
-  GtkTooltips *tooltips;
   GtkTreeSelection *sel;
-
+#if GTK_MINOR_VERSION > 11
+  GList *lst;
+#endif
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
   tooltips = gtk_tooltips_new ();
+#endif
 
   /* Create the listModel for pairs data*/
   pairsTreeStore = gtk_tree_store_new(N_GTK_PAIRS_COLUMN, G_TYPE_BOOLEAN,
@@ -282,49 +285,52 @@ void gtkPairsBuild_window(GtkMain *main)
   gtk_tree_view_set_model(GTK_TREE_VIEW(gtkPairs_treeView), pairsSortable);
 
   /* Create a filter selector for the shown pairs. */
-  hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(vboxPairsDialog), hbox, FALSE, FALSE, 0);
+  wd = lookup_widget(main->pairsDialog, "hbox73");
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(wd), vbox, FALSE, FALSE, 0);
 
-  label = gtk_label_new(_("Show pairs with: "));
-  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-  comboFilter = gtk_combo_box_new_text();
-  gtk_box_pack_start(GTK_BOX(hbox), comboFilter, FALSE, FALSE, 0);
-  g_signal_connect(G_OBJECT(comboFilter), "changed",
-		   G_CALLBACK(filterChanged), (gpointer)pairsFilter);
-  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(pairsFilter),
-					 pairsRowIsVisible, (gpointer)comboFilter,
-					 (GtkDestroyNotify)0);
+#if GTK_MINOR_VERSION > 5
+  label = gtk_label_new(_("Manage links: "));
+  gtk_label_set_angle(GTK_LABEL(label), 90.);
+  gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+#endif
 
   btRemove = gtk_button_new();
   gtk_widget_set_sensitive(btRemove, FALSE);
-  image = gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
   gtk_container_add(GTK_CONTAINER(btRemove), image);
-  gtk_box_pack_end(GTK_BOX(hbox), btRemove, FALSE, FALSE, 0);  
+  gtk_box_pack_end(GTK_BOX(vbox), btRemove, FALSE, FALSE, 3);  
   g_signal_connect(G_OBJECT(btRemove), "clicked",
 		   G_CALLBACK(onBtRemove), (gpointer)0);
   gtk_tooltips_set_tip(tooltips, btRemove,
 		       _("Remove a link definition."), NULL);
 
   btAdd = gtk_button_new();
-  image = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
+  gtk_widget_set_sensitive(btAdd, FALSE);
+  image = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
   gtk_container_add(GTK_CONTAINER(btAdd), image);
-  gtk_box_pack_end(GTK_BOX(hbox), btAdd, FALSE, FALSE, 3);  
+  gtk_box_pack_end(GTK_BOX(vbox), btAdd, FALSE, FALSE, 3);  
   g_signal_connect(G_OBJECT(btAdd), "clicked",
 		   G_CALLBACK(onBtAdd), (gpointer)0);
   gtk_tooltips_set_tip(tooltips, btAdd,
-		       _("Add a new link definition using (0.,0.) if"
+		       _("Add a new link definition using (0., 0.) if"
 			 " it does not already exist."), NULL);
 
-  label = gtk_label_new(_("Manage links: "));
-  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  btHide = gtk_toggle_button_new();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btHide), FALSE);
+  image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
+  gtk_container_add(GTK_CONTAINER(btHide), image);
+  gtk_box_pack_end(GTK_BOX(vbox), btHide, FALSE, FALSE, 8);
+  g_signal_connect(G_OBJECT(btHide), "toggled",
+		   G_CALLBACK(onBtHide), (gpointer)pairsFilter);
+  gtk_tooltips_set_tip(tooltips, btHide,
+		       _("Show/hide the undrawn pairs."), NULL);
 
-  gtk_widget_show_all(hbox);
+  gtk_widget_show_all(vbox);
 
   /* Create the part for the link parameters. */
-  label = gtk_label_new(_("<b>Link parameters:</b>"));
+  label = gtk_label_new(_("<b>Link parameters:</b>\t <span size=\"small\">"
+			  "(apply to one or more selected rows)</span>"));
   gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_box_pack_start(GTK_BOX(vboxPairsDialog), label, FALSE, FALSE, 5);
@@ -332,39 +338,49 @@ void gtkPairsBuild_window(GtkMain *main)
   gtk_widget_show_all(label);
 
   /* Create widgets that apply to all checked pairs. */
-  hboxCheckWidgets = gtk_hbox_new (FALSE, 0);
+  align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 10, 10);
+  gtk_box_pack_start(GTK_BOX(vboxPairsDialog), align, FALSE, FALSE, 0);
+
+  hboxCheckWidgets = gtk_hbox_new(FALSE, 3);
   gtk_widget_set_sensitive(hboxCheckWidgets, FALSE);
-  gtk_box_pack_start (GTK_BOX (vboxPairsDialog), hboxCheckWidgets, FALSE, FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(align), hboxCheckWidgets);
 
   label = gtk_label_new(_("From: "));
-  gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), label, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), label, FALSE, FALSE, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-  adj = gtk_adjustment_new(0, 0, 100, 0.1, 0.5, 10);
-  gtkPairs_spinMin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 3);
+  adj = gtk_adjustment_new(0, 0, 100, 0.05, 0.2, 0);
+  gtkPairs_spinMin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(gtkPairs_spinMin), TRUE);
+  gtk_entry_set_width_chars(GTK_ENTRY(gtkPairs_spinMin), 4);
   gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), gtkPairs_spinMin, FALSE, FALSE, 0);
 
   label = gtk_label_new(_(" to: "));
   gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), label, FALSE, FALSE, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
-  adj = gtk_adjustment_new(0, 0, 100, 0.1, 0.5, 10);
-  gtkPairs_spinMax = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 3);
+  adj = gtk_adjustment_new(0, 0, 100, 0.05, 0.2, 0);
+  gtkPairs_spinMax = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 2);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(gtkPairs_spinMax), TRUE);
+  gtk_entry_set_width_chars(GTK_ENTRY(gtkPairs_spinMax), 4);
   gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), gtkPairs_spinMax, FALSE, FALSE, 0);
 
   wd = gtk_button_new_with_mnemonic(_("_Auto set"));
-  gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), wd, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), wd, FALSE, FALSE, 0);
   g_signal_connect(G_OBJECT(wd), "clicked",
 		   G_CALLBACK(onDistanceAutoSet), (gpointer)0);
+  gtk_tooltips_set_tip(tooltips, wd,
+		       _("Set the distance criterion to select the first"
+			 " pick in the distance distribution between all"
+			 " nodes of the selected types."), NULL);
 
-  label = gtk_label_new (_("Color: "));
-  gtk_box_pack_start (GTK_BOX (hboxCheckWidgets), label, TRUE, TRUE, 0);
-  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+/*   label = gtk_label_new (_("Color: ")); */
+/*   gtk_box_pack_start (GTK_BOX (hboxCheckWidgets), label, TRUE, TRUE, 0); */
+/*   gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5); */
 
   widgetColor = colorComboBox_new(FALSE);
-  gtk_box_pack_start(GTK_BOX(hboxCheckWidgets), widgetColor, FALSE, FALSE, 0);
+  gtk_box_pack_end(GTK_BOX(hboxCheckWidgets), widgetColor, FALSE, FALSE, 0);
 
-  gtk_widget_show_all(hboxCheckWidgets);
+  gtk_widget_show_all(align);
 
   /* Render the associated tree */
   renderer = gtk_cell_renderer_text_new ();
@@ -455,6 +471,7 @@ void gtkPairsBuild_window(GtkMain *main)
   gtk_combo_box_set_model(GTK_COMBO_BOX(gtkPairs_comboWidget),
 			  GTK_TREE_MODEL(gtkPairs_comboModel));
 
+  /* We create the pair model box. */
   wd = lookup_widget(main->pairsDialog, "hboxPairsModel");
   pairs = visuPairExtensionGet_current();
   if (!visuPairExtensionGet_allMethods())
@@ -484,6 +501,40 @@ void gtkPairsBuild_window(GtkMain *main)
     }
   gtk_box_pack_start(GTK_BOX(wd), gtkPairs_comboWidget, TRUE, TRUE, 0);
   
+  label = gtk_label_new("]");
+  gtk_box_pack_end(GTK_BOX(wd), label, FALSE, FALSE, 0);
+
+  align = gtk_alignment_new(0.5, 0.5, 1., 0.);
+  gtk_box_pack_end(GTK_BOX(wd), align, FALSE, FALSE, 0);
+
+  comboFilter = gtk_combo_box_new_text();
+#if GTK_MINOR_VERSION > 11
+  lst = gtk_cell_layout_get_cells(GTK_CELL_LAYOUT(comboFilter));
+  if (lst)
+    {
+      g_object_set(G_OBJECT(lst->data), "scale", 0.75, NULL);
+      g_list_free(lst);
+    }
+#endif
+/*   gtk_box_pack_end(GTK_BOX(wd), comboFilter, FALSE, FALSE, 0); */
+  gtk_container_add(GTK_CONTAINER(align), comboFilter);
+  g_signal_connect(G_OBJECT(comboFilter), "changed",
+		   G_CALLBACK(filterChanged), (gpointer)pairsFilter);
+  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(pairsFilter),
+					 pairsRowIsVisible, (gpointer)comboFilter,
+					 (GDestroyNotify)0);
+
+  label = gtk_label_new(_("filter: "));
+  gtk_box_pack_end(GTK_BOX(wd), label, FALSE, FALSE, 0);
+
+  align = gtk_alignment_new(0.5, 0.5, 1., 1.);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 10, 0);
+  gtk_box_pack_end(GTK_BOX(wd), align, FALSE, FALSE, 0);
+
+  label = gtk_label_new("[");
+  gtk_container_add(GTK_CONTAINER(align), label);
+
+  gtk_widget_show_all(wd);
 
   /* Connect local signals. */
   sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(gtkPairs_treeView));
@@ -636,8 +687,8 @@ void gtkPairsBuild_pairs(GtkMain *main, VisuData *dataObj, gboolean force)
   gboolean visible;
   GtkTreeModel *model;
   VisuDataIter iter1, iter2;
-  GtkTreeIter iter;
-  gboolean validIter;
+/*   GtkTreeIter iter; */
+/*   gboolean validIter; */
   GtkPairsGetValuesAsLabelFunc labelFunc;
 
   DBG_fprintf(stderr, "Gtk Pairs: build the list of pairs.\n");
@@ -663,7 +714,7 @@ void gtkPairsBuild_pairs(GtkMain *main, VisuData *dataObj, gboolean force)
   model = gtk_combo_box_get_model(GTK_COMBO_BOX(comboFilter));
   gtk_list_store_clear(GTK_LIST_STORE(model));
   /* We add the default value. */
-  gtk_combo_box_append_text(GTK_COMBO_BOX(comboFilter), _("all elements"));
+  gtk_combo_box_append_text(GTK_COMBO_BOX(comboFilter), _("none"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(comboFilter), 0);
 
   /* If there is no data, we return. */
@@ -687,9 +738,9 @@ void gtkPairsBuild_pairs(GtkMain *main, VisuData *dataObj, gboolean force)
   /* If we have only one element, we make the filter insensitive. */
   gtk_widget_set_sensitive(comboFilter, (dataObj->ntype > 1));
   /* We select the first link, if any. */
-  validIter = gtk_tree_model_get_iter_first(pairsSortable, &iter);
-  if (validIter)
-    gtk_tree_selection_select_iter(pairsSelection, &iter);
+/*   validIter = gtk_tree_model_get_iter_first(pairsSortable, &iter); */
+/*   if (validIter) */
+/*     gtk_tree_selection_select_iter(pairsSelection, &iter); */
 
   DBG_fprintf(stderr, "Gtk Pairs: Pairs are rebuilt.\n");
 }
@@ -724,11 +775,10 @@ gchar* createDistanceLabel(VisuPairData *data)
   return g_string_free(label, FALSE);
 }
 
-void setSpecificArea(struct specBuildFunc_struct *specBuild)
+static void setSpecificArea(struct specBuildFunc_struct *specBuild)
 {
   GtkPairsBuildWidgetsFunc buildSpecific;
-  GtkWidget *wdlabel;
-
+  GtkWidget *wdlabel, *align;
 
   /* change the specific area. */
   DBG_fprintf(stderr, "Gtk Pairs : previous specific widget : %d\n",
@@ -749,8 +799,15 @@ void setSpecificArea(struct specBuildFunc_struct *specBuild)
       wdlabel = gtk_label_new(_("No parameters for these kinds of pairs."));
       specificWindow = wdlabel;
     }
-  gtk_box_pack_start(GTK_BOX(vboxPairsDialog), specificWindow, FALSE, FALSE, 10);
   gtk_widget_show(specificWindow);
+
+  align = gtk_alignment_new(0.5, 0.5, 1., 1.);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 10, 10);
+  gtk_container_add(GTK_CONTAINER(align), specificWindow);
+  specificWindow = align;
+  
+  gtk_box_pack_start(GTK_BOX(vboxPairsDialog), align, FALSE, FALSE, 10);
+  gtk_widget_show(align);
 }
 
 void gtkPairsIter_startSelected(GtkPairsIter *iter)
@@ -758,6 +815,7 @@ void gtkPairsIter_startSelected(GtkPairsIter *iter)
   g_return_if_fail(iter);
 
   iter->data = (VisuPairData*)0;
+  iter->current = (GList*)0;
 
   iter->selected = gtk_tree_selection_get_selected_rows(pairsSelection, NULL);
   if (!iter->selected)
@@ -854,32 +912,43 @@ static gboolean pairsRowIsVisible(GtkTreeModel *model, GtkTreeIter *iter,
   gchar *label;
   gboolean visible;
   GtkTreeIter iterParent;
-
-  if (gtk_combo_box_get_active(GTK_COMBO_BOX(data)) == 0)
-    return TRUE;
+  float start, stop;
 
   if (!gtk_tree_model_iter_parent(model, &iterParent, iter))
     iterParent = *iter;
   gtk_tree_model_get(model, &iterParent,
 		     GTK_PAIRS_COLUMN_LABEL_ELE1, &name1,
 		     GTK_PAIRS_COLUMN_LABEL_ELE2, &name2,
+		     GTK_PAIRS_COLUMN_DISTANCE_FROM, &start,
+		     GTK_PAIRS_COLUMN_DISTANCE_TO, &stop,
 		     -1);
+
+  if (gtk_combo_box_get_active(GTK_COMBO_BOX(data)) > 0)
+    {
 #if GTK_MINOR_VERSION > 5
-  label = gtk_combo_box_get_active_text(GTK_COMBO_BOX(data));
+      label = gtk_combo_box_get_active_text(GTK_COMBO_BOX(data));
 #else
-  if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(data), &iterParent))
-    gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(data)),
-		       &iterParent, 0, &label, -1);
-  else
-    label = "error";
+      if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(data), &iterParent))
+	gtk_tree_model_get(gtk_combo_box_get_model(GTK_COMBO_BOX(data)),
+			   &iterParent, 0, &label, -1);
+      else
+	label = g_strdup("error");
 #endif
-  visible = !strcmp(label, name1) || !strcmp(label, name2);
+      visible = !strcmp(label, name1) || !strcmp(label, name2);
+      g_free(label);
+    }
+  else
+    visible = TRUE;
+
+  /* Add the visibility tag due to the hide button. */
+  visible = visible && (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btHide)) ||
+			(stop > start));
   DBG_fprintf(stderr, "Gtk Pairs: set visibility for '%s' '%s': %d\n",
 	  name1, name2, visible);
 
-  g_free(label);
   g_free(name1);
-  g_free(name2);  
+  g_free(name2);
+
   return visible;
 }
 static void pairsMethodChanged(GtkComboBox *combobox, gpointer userData _U_)
@@ -1157,7 +1226,7 @@ static void onDistanceAutoSet(GtkButton *button _U_, gpointer data _U_)
   RenderingWindow *window;
   VisuData *dataObj;
   float start, stop, min;
-  int i;
+  int i, sum;
   gboolean rebuild;
 
   rebuild = FALSE;
@@ -1187,19 +1256,38 @@ static void onDistanceAutoSet(GtkButton *button _U_, gpointer data _U_)
       dd = visuPairGet_distanceDistribution(pair, dataObj, -1.f, -1.f);
       g_return_if_fail(dd);
 
-      start = -1.f;
-      stop = -1.f;
-      min = MIN(0.5f * dd->nNodesEle1, 0.5f * dd->nNodesEle2);
-      for (i = 0 ; i < dd->nValues; i++)
+      min = 1.5f * MIN(dd->nNodesEle1, dd->nNodesEle2);
+      do
 	{
-	  if (dd->histo[i] >= min)
-	    start = dd->stepValue * i;
-	  if (start > 0.f && dd->histo[i] < min)
+	  min  *= 0.5f;
+	  start = -1.f;
+	  stop  = -1.f;
+	  sum   = 0;
+	  for (i = 0 ; i < dd->nValues; i++)
 	    {
-	      stop = dd->stepValue * i;
-	      break;
+	      if (start < 0.f && dd->histo[i] > 0)
+		{
+		  start = dd->stepValue * i;
+		  sum = dd->histo[i];
+		}
+	      else if (start > 0.f)
+		{
+		  if (dd->histo[i] == 0)
+		    {
+		      stop = dd->stepValue * i;
+		      if (sum >= min)
+			break;
+		      else
+			start = -1.f;
+		    }
+		  else
+		    sum += dd->histo[i];
+		}
 	    }
 	}
+      while (start < 0.f && min > 0.1f * MIN(dd->nNodesEle1, dd->nNodesEle2));
+      DBG_fprintf(stderr, "Gtk Pairs: set start and stop at %f, %f (%f %d).\n",
+		  start, stop, min, sum);
 
       if (start > 0.f && stop > 0.f)
 	{
@@ -1236,6 +1324,7 @@ static void onBtAdd(GtkButton *button _U_, gpointer user_data _U_)
   /* Go to last iter. */
   for (gtkPairsIter_startSelected(&iter); iter.current;
        gtkPairsIter_nextSelected(&iter));
+  g_return_if_fail(iter.data);
   
   /* Get the ancestor of this iter. */
   if (!gtk_tree_model_iter_parent(GTK_TREE_MODEL(pairsTreeStore), &iterParent,
@@ -1296,6 +1385,7 @@ static void onBtRemove(GtkButton *button _U_, gpointer user_data _U_)
   /* Go to last iter. */
   for (gtkPairsIter_startSelected(&iter); iter.current;
        gtkPairsIter_nextSelected(&iter));
+  g_return_if_fail(iter.data);
   
   /* Get the ancestor of this iter. */
   if (!gtk_tree_model_iter_parent(GTK_TREE_MODEL(pairsTreeStore), &iterParent,
@@ -1327,6 +1417,10 @@ static void onBtRemove(GtkButton *button _U_, gpointer user_data _U_)
 
   gtkPairs_createPairs();
 }
+static void onBtHide(GtkToggleButton *button _U_, gpointer user_data)
+{
+  gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(user_data));
+}
 
 void gtkPairs_createPairs()
 {
