@@ -2,21 +2,21 @@
 	Copyright CEA, contributeurs : Luc BILLARD et Damien
 	CALISTE, laboratoire L_Sim, (2001-2005)
   
-	Adresse ml :
-	BILLARD, non joignable par ml ;
+	Adresse mèl :
+	BILLARD, non joignable par mèl ;
 	CALISTE, damien P caliste AT cea P fr.
 
-	Ce logiciel est un programme informatique servant  visualiser des
+	Ce logiciel est un programme informatique servant à visualiser des
 	structures atomiques dans un rendu pseudo-3D. 
 
-	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
+	Ce logiciel est régi par la licence CeCILL soumise au droit français et
 	respectant les principes de diffusion des logiciels libres. Vous pouvez
 	utiliser, modifier et/ou redistribuer ce programme sous les conditions
-	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
+	de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA 
 	sur le site "http://www.cecill.info".
 
-	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
-	pris connaissance de la licence CeCILL, et que vous en avez accept les
+	Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
+	pris connaissance de la licence CeCILL, et que vous en avez accepté les
 	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
 */
 
@@ -46,6 +46,7 @@
 
 #include <glib.h>
 
+#include "view.h"
 #include <visu_nodes.h>
 #include <visu_data.h>
 #include <visu_pickMesure.h>
@@ -58,7 +59,6 @@
  * In this mode, the movements are along meridians when the mouse is
  * dragged along y axis and along parallels when the movement is along x axis.
  */
-#define OPENGL_OBSERVE_CONSTRAINED 0
 /**
  * OPENGL_OBSERVE_WALKER
  *
@@ -67,7 +67,63 @@
  * when mouse move along y axis, the ant goes strait on, when mouse is
  * dragged along x axis, the ant translates on its right or on its left.
  */
-#define OPENGL_OBSERVE_WALKER 1
+typedef enum
+  {
+    interactive_constrained,
+    interactive_walker
+  } VisuInteractiveMethod;
+
+typedef enum
+  {
+    interactive_none,
+    interactive_observe,
+    interactive_pickAndObserve,
+    interactive_pick,
+    interactive_move,
+    interactive_mark
+  } VisuInteractiveId;
+
+/**
+ * VISU_INTERACTIVE_TYPE:
+ *
+ * return the type of #VisuInteractive.
+ */
+#define VISU_INTERACTIVE_TYPE	     (visuInteractive_get_type ())
+/**
+ * VISU_INTERACTIVE:
+ * @obj: a #GObject to cast.
+ *
+ * Cast the given @obj into #VisuInteractive type.
+ */
+#define VISU_INTERACTIVE(obj)	     (G_TYPE_CHECK_INSTANCE_CAST(obj, VISU_INTERACTIVE_TYPE, VisuInteractive))
+/**
+ * VISU_INTERACTIVE_CLASS:
+ * @klass: a #GClassObject to cast.
+ *
+ * Cast the given @klass into #VisuInteractiveClass.
+ */
+#define VISU_INTERACTIVE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST(klass, VISU_INTERACTIVE_TYPE, VisuInteractiveClass))
+/**
+ * IS_VISU_INTERACTIVE_TYPE:
+ * @obj: a #GObject to test.
+ *
+ * Test if the given @ogj is of the type of #VisuInteractive object.
+ */
+#define IS_VISU_INTERACTIVE(obj)    (G_TYPE_CHECK_INSTANCE_TYPE(obj, VISU_INTERACTIVE_TYPE))
+/**
+ * IS_VISU_INTERACTIVE_CLASS:
+ * @klass: a #GClassObject to test.
+ *
+ * Test if the given @klass is of the type of #VisuInteractiveClass class.
+ */
+#define IS_VISU_INTERACTIVE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE(klass, VISU_INTERACTIVE_TYPE))
+/**
+ * VISU_INTERACTIVE_GET_CLASS:
+ * @obj: a #GObject to get the class of.
+ *
+ * It returns the class of the given @obj.
+ */
+#define VISU_INTERACTIVE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS(obj, VISU_INTERACTIVE_TYPE, VisuInteractiveClass))
 
 /**
  * VisuInteractive:
@@ -75,6 +131,16 @@
  * All fields are private.
  */
 typedef struct _VisuInteractive VisuInteractive;
+typedef struct _VisuInteractiveClass VisuInteractiveClass;
+
+/**
+ * visu_interactive_get_type:
+ *
+ * This method returns the type of #VisuInteractive, use VISU_INTERACTIVE_TYPE instead.
+ *
+ * Returns: the type of #VisuInteractive.
+ */
+GType visuInteractive_get_type(void);
 
 /**
  * openGLInteractiveSet_preferedObserveMethod:
@@ -85,7 +151,7 @@ typedef struct _VisuInteractive VisuInteractive;
  * two methods are described in the commentary of the keys #OPENGL_OBSERVE_CONSTRAINED
  * an d#OPENGL_OBSERVE_WALKER.
  */
-void openGLInteractiveSet_preferedObserveMethod(int method);
+void visuInteractiveClassSet_preferedObserveMethod(VisuInteractiveMethod method);
 /**
  * openGLInteractiveGet_preferedObserveMethod:
  *
@@ -96,32 +162,8 @@ void openGLInteractiveSet_preferedObserveMethod(int method);
  * Returns: an integer that identify the method, see #OPENGL_OBSERVE_CONSTRAINED,
  *          and #OPENGL_OBSERVE_WALKER flags.
  */
-int openGLInteractiveGet_preferedObserveMethod();
+VisuInteractiveMethod visuInteractiveClassGet_preferedObserveMethod();
 
-/**
- * callBackOnNodeSelectedFunc:
- * @mesureData: the object that stores the pick information ;
- * @data: a pointer to user define data.
- *
- * This interface defines functions that are called whenever a
- * node is selected.
- */
-typedef void (*callBackOnNodeSelectedFunc)(PickMesure *mesureData, gpointer data);
-
-/**
- * callBackOnEventFunc:
- * @event: the event that triggered the action ;
- * @inter: the #VisuInteractive object the callback deals with.
- *
- * An interface to methods that can be called whenever an action occurs.
- *
- * Returns: TRUE if action should be stopped after this call.
- */
-typedef gboolean (*callBackOnEventFunc)(SimplifiedEvents *event,
-					VisuInteractive *inter);
-
-
-/* observe and pick functions. */
 /**
  * openGLInteractiveInit_session:
  * @window: a pointer to the window that ask to initialize
@@ -142,18 +184,8 @@ typedef gboolean (*callBackOnEventFunc)(SimplifiedEvents *event,
  * associated to the #VisuData handled by the @window and will be
  * destroyed with it.
  */
-VisuInteractive* openGLInteractiveInit_session(GenericRenderingWindow window,
-					       callBackOnNodeSelectedFunc handler,
-					       gpointer data,
-					       GDestroyNotify stopPickObserve);
-/**
- * openGLInteractiveEnd_session:
- * @inter: a #VisuInteractive object.
- *
- * Stop the period when pick or observe sessions are allowed. Basically,
- * for the user point of view it returns the cursor to its initial shape.
- */
-void openGLInteractiveEnd_session(VisuInteractive *inter);
+VisuInteractive* visuInteractiveNew(GenericRenderingWindow window);
+
 /**
  * openGLInteractiveBegin_observe:
  * @inter: a #VisuInteractive object.
@@ -162,44 +194,7 @@ void openGLInteractiveEnd_session(VisuInteractive *inter);
  * by a call to this method. It actually stops also a possibly running
  * pick session.
  */
-void openGLInteractiveBegin_observe(VisuInteractive *inter);
-/**
- * openGLInteractiveBegin_pick:
- * @inter: a #VisuInteractive object.
- * 
- * During a pick or observe session, the client can begin a pick session
- * by a call to this method. It actually stops also a possibly running
- * observe session.
- */
-void openGLInteractiveBegin_pick(VisuInteractive *inter);
-/**
- * openGLInteractiveBegin_move:
- * @inter: a #VisuInteractive object.
- * 
- * During an interactive session, the client can begin a move session
- * by a call to this method. It enables the press-button (and release), the motion-notify
- * and the scroll event. It returns the displacement and the node selected.
- */
-void openGLInteractiveBegin_move(VisuInteractive *inter);
-/**
- * openGLInteractiveBegin_mark:
- * @inter: a #VisuInteractive object.
- * 
- * During an interactive session, the client can begin a mark session
- * by a call to this method. It enables the press-button (and
- * release), the motion-notify and the scroll event. A mark is drawn
- * around the selected element (or removed) when clicked. To begin an
- * interactive session, see openGLInteractiveInit_session().
- */
-void openGLInteractiveBegin_mark(VisuInteractive *inter);
-/**
- * openGLInteractiveBegin_pickAndObserve:
- * @inter: a #VisuInteractive object.
- * 
- * During a pick or observe session, the client can both observe (normal way)
- * and do minimal pick with the right button.
- */
-void openGLInteractiveBegin_pickAndObserve(VisuInteractive *inter);
+void visuInteractiveSet_type(VisuInteractive *inter, VisuInteractiveId type);
 
 /**
  * openGLInteractiveHandle_event:
@@ -209,7 +204,7 @@ void openGLInteractiveBegin_pickAndObserve(VisuInteractive *inter);
  * This routine should be called by the rendering window when some
  * event is raised on the rendering surface.
  */
-void openGLInteractiveHandle_event(SimplifiedEvents *ev, VisuInteractive *inter);
+void visuInteractiveHandle_event(VisuInteractive *inter, SimplifiedEvents *ev);
 
 /**
  * openGLInteractiveStart_move:
@@ -221,7 +216,7 @@ void openGLInteractiveHandle_event(SimplifiedEvents *ev, VisuInteractive *inter)
  * from the classical #VisuData OpenGL lists. A redraw is
  * automatically asked.
  */
-void openGLInteractiveStart_move(VisuInteractive *inter, GList *nodes);
+void visuInteractiveStart_move(VisuInteractive *inter, GList *nodes);
 /**
  * openGLInteractiveStop_move:
  * @inter: a #VisuInteractive object ;
@@ -231,7 +226,7 @@ void openGLInteractiveStart_move(VisuInteractive *inter, GList *nodes);
  * recreated in the OpenGL lists of #VisuData. A redraw is
  * automatically asked.
  */
-void openGLInteractiveStop_move(VisuInteractive *inter, GList *nodes);
+void visuInteractiveStop_move(VisuInteractive *inter, GList *nodes);
 /**
  * openGLInteractiveMove:
  * @inter: a #VisuInteractive object ;
@@ -240,7 +235,7 @@ void openGLInteractiveStop_move(VisuInteractive *inter, GList *nodes);
  *
  * Update the internal of @inter when the given list is dragged.
  */
-void openGLInteractiveMove(VisuInteractive *inter, GList *nodes, float drag[3]);
+void visuInteractiveMove(VisuInteractive *inter, GList *nodes, float drag[3]);
 
 /**
  * openGLInteractiveGet_pickMesure:
@@ -250,7 +245,7 @@ void openGLInteractiveMove(VisuInteractive *inter, GList *nodes, float drag[3]);
  *
  * Returns: a #PickMesure object, private.
  */
-PickMesure* openGLInteractiveGet_pickMesure(VisuInteractive *inter);
+PickMesure* visuInteractiveGet_pickMesure(VisuInteractive *inter);
 /**
  * openGLInteractiveSet_visuData:
  * @inter: a #VisuInteractive object ;
@@ -258,14 +253,28 @@ PickMesure* openGLInteractiveGet_pickMesure(VisuInteractive *inter);
  *
  * Change the #VisuData the interactive session is dealing with.
  */
-void openGLInteractiveSet_visuData(VisuInteractive *inter, VisuData *data);
+void visuInteractiveSet_visuData(VisuInteractive *inter, VisuData *data);
 
+OpenGLCamera* visuInteractivePop_savedCamera(VisuInteractive *inter);
+void visuInteractivePush_savedCamera(VisuInteractive *inter, OpenGLCamera *camera);
+void visuInteractiveGet_savedCameras(VisuInteractive *inter,
+				       GList **cameras, GList **head);
 
 /**
- * openGLInteractive_init:
+ * getNodes2DCoordinates:
+ * @dataObj: a #VisuData object ;
+ * nodeIds: an array of #VisuNode ids ;
+ * nNodes: the number of nodes in the array ;
+ * coordinates2D: an array of nodes 2D coordinates ;
+ * size: the size of the coordinates array.
  *
- * Used to initialise this part of code, don't use it.
+ * From nodes ids, compute and store in @coordinates2D their 2D coordinates.
  */
-void openGLInteractive_init();
+void getNodes2DCoordinates(VisuData *dataObj, unsigned int *nodeIds,
+	unsigned int nNodes, float *coordinates2D, unsigned int *size);
+
+void defineNodeVertices(int nVert, double radius, double xn, double yn, double *nodeVertices);
+
+void addVerticesToGlobalArray(int nVert, double *nodeVertices, double *vertices, int i);
 
 #endif
