@@ -171,47 +171,61 @@ int openGLViewSet_thetaPhiOmega(OpenGLView *view, float valueTheta,
 /* Change the value of the camera position. */
 int openGLViewSet_XsYs(OpenGLView *view, float valueX, float valueY, int mask)
 {
-  float valX, valY;
-  int idem;
+  float valX, valY, dX, dY/* , z */;
+/*   float O[3]; */
   
   g_return_val_if_fail(view && view->camera, 0);
   
-  idem = 1;
+  dX = 0.f;
+  dY = 0.f;
   if (mask & MASK_XS)
     {
       valX = valueX;
       if (valX < -3.)
 	valX = -3.;
-      while (valX > 3.)
+      if (valX > 3.)
 	valX = 3.;
 
       if (view->camera->xs != valX)
 	{
-	  idem = 0;
+	  dX = valX - view->camera->xs;
 	  view->camera->xs = valX;
 	}
     }
   if (mask & MASK_YS)
     {
       valY = valueY;
-      while (valY < -3.)
+      if (valY < -3.)
 	valY = -3.;
-      while (valY > 3.)
+      if (valY > 3.)
 	valY = 3.;
 
       if (view->camera->ys != valY)
 	{
-	  idem = 0;
+	  dY = valY - view->camera->ys;
 	  view->camera->ys = valY;
 	}
     }
-  if (idem)
+  if (dX == 0.f && dY == 0.f)
     return 0;
 
 /*   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLXsYs_signal_id, */
 /* 		 0 , NULL); */
+
   project(view);
 
+  /* Change the eyes position. */
+/*   O[0] = view->camera->centre[0] + view->box->dxxs2; */
+/*   O[1] = view->camera->centre[1] + view->box->dyys2; */
+/*   O[2] = view->camera->centre[2] + view->box->dzzs2; */
+/*   z = openGLViewGet_zCoordinate(view, O); */
+/*   DBG_fprintf(stderr, "OpenGL View: new window coordinates (%f;%f;%f).\n", */
+/* 	      0.5f * view->window->width, 0.5f * view->window->height, z); */
+/*   openGLViewGet_realCoordinates(view, view->camera->centre, */
+/* 				0.5f * view->window->width, */
+/* 				0.5f * view->window->height, z, FALSE); */
+/*   modelize(view); */
+
   return 1;
 }
 
@@ -267,7 +281,7 @@ int openGLViewSet_persp(OpenGLView *view, float value)
 }
 int OpenGLViewSet_windowSize(OpenGLView *view, guint width, guint height)
 {
-  DBG_fprintf(stderr, "OpenGL View : set viewport size (%dx%d) for view %p.\n",
+  DBG_fprintf(stderr, "OpenGL View: set viewport size (%dx%d) for view %p.\n",
 	      width, height, (gpointer)view);
   
   g_return_val_if_fail(view, 0);
@@ -285,14 +299,21 @@ int OpenGLViewSet_windowSize(OpenGLView *view, guint width, guint height)
   return 0;
 }
 
+void openGLViewSet_upAxis(OpenGLView *view, ViewAxis upAxis)
+{
+  g_return_if_fail(view && view->camera);
+
+  view->camera->upAxis = upAxis;
+}
 
 static void modelize(OpenGLView *view)
 {
   double theta_rad;
   double phi_rad; 
   double sth, cth, sph, cph, com, som;
-  double eyex, eyey, eyez;
-  double distance;   
+  double eye[3];
+  double distance;
+  int permut[3][3] = {{1,2,0}, {2,0,1}, {0,1,2}};
 
   g_return_if_fail(view && view->camera);
  
@@ -311,23 +332,30 @@ static void modelize(OpenGLView *view)
   /* La matrice de rotation est la suivante pour passer
      des coordonnes transformes aux coordonnes de l'cran :
      /cph.cth -sph cph.sth\
-     |sph.cth  cph sph.sth|
+     |sph.cth  cph sph.sth| (for z as north axis)
      \   -sth   0      cth/
+
+     / cph -sph.sth sph.cth\
+     |  0       cth     sth| (for y as north axis)
+     \-sph -sth.cph cph.cth/
      Ainsi la camra qui est situ en (0,0,Distance) dans le repre transform
      devient dans le repre de l'cran : */
-  eyex = distance*sth*cph;
-  eyey = distance*sth*sph;
-  eyez = distance*cth;
+  eye[permut[view->camera->upAxis][0]] = distance*sth*cph;
+  eye[permut[view->camera->upAxis][1]] = distance*sth*sph;
+  eye[permut[view->camera->upAxis][2]] = distance*cth;
    
   /* Vecteur donnant la direction verticale.
      Dans le repre transform il est (-1,0,0). */
-  view->camera->up[0] = -cth*cph*com + sph*som;
-  view->camera->up[1] = -cth*sph*com - cph*som;
-  view->camera->up[2] = sth*com;
-   
+  view->camera->up[permut[view->camera->upAxis][0]] = -cth*cph*com + sph*som;
+  view->camera->up[permut[view->camera->upAxis][1]] = -cth*sph*com - cph*som;
+  view->camera->up[permut[view->camera->upAxis][2]] = sth*com;
+
   glMatrixMode(GL_MODELVIEW); 
   glLoadIdentity();
-  gluLookAt(eyex, eyey, eyez, 0.0, 0.0, 0.0,
+  gluLookAt(eye[0], eye[1], eye[2],
+	    view->camera->centre[0],
+	    view->camera->centre[1],
+	    view->camera->centre[2],
 	    view->camera->up[0],
 	    view->camera->up[1],
 	    view->camera->up[2]);
@@ -339,7 +367,7 @@ static void project(OpenGLView *view)
   double rap_win;
  
   g_return_if_fail(view && view->camera && view->box && view->window);
-
+  
   fact = view->camera->d_red * view->camera->length0;
   view->window->near = MAX(0.01, fact - view->box->extens);
   view->window->far  = fact + view->box->extens;
@@ -386,8 +414,9 @@ void openGLViewCompute_matrixAndView(OpenGLView* view)
 {
   g_return_if_fail(view);
 
-  DBG_fprintf(stderr, "OpenGL View : modelize and project view %p.\n", (gpointer)view);
+  DBG_fprintf(stderr, "OpenGL View: modelize view %p.\n", (gpointer)view);
   modelize(view);
+  DBG_fprintf(stderr, "OpenGL View: project view %p.\n", (gpointer)view);
   project(view);
 }
 
@@ -413,6 +442,13 @@ OpenGLView* OpenGLViewNew()
   view->camera->gross   = grossDefault;
   view->camera->d_red   = perspDefault;
   view->camera->length0 = -1.;
+  view->camera->upAxis  = VIEW_Z;
+  view->camera->centre[0] = 0.f;
+  view->camera->centre[1] = 0.f;
+  view->camera->centre[2] = 0.f;
+
+  view->box->margin     = 0.f;
+  view->box->bc         = BOX_PERIODIC;
 
   DBG_fprintf(stderr, "OpenGL view : create a new view (%p).\n", (gpointer)view);
   DBG_fprintf(stderr, " | theta = %g\n", view->camera->theta);
@@ -465,7 +501,12 @@ OpenGLView* OpenGLViewCopy(OpenGLView *view)
   viewCopy->camera->gross   = view->camera->gross;
   viewCopy->camera->d_red   = view->camera->d_red;
   viewCopy->camera->length0 = view->camera->length0;
+  viewCopy->camera->upAxis  = view->camera->upAxis;
+  viewCopy->camera->centre[0] = view->camera->centre[0];
+  viewCopy->camera->centre[1] = view->camera->centre[1];
+  viewCopy->camera->centre[2] = view->camera->centre[2];
   
+  viewCopy->box->bc      = view->box->bc;
   viewCopy->box->extens  = view->box->extens;
   viewCopy->box->dxxs2   = view->box->dxxs2;
   viewCopy->box->dyys2   = view->box->dyys2;
@@ -486,6 +527,26 @@ OpenGLView* OpenGLViewCopy(OpenGLView *view)
   return viewCopy;
 }
 
+void openGLViewSet_boundaryConditions(OpenGLView *view, OpenGLBoxBoundaries bc)
+{
+  g_return_if_fail(view && view->box);
+
+  view->box->bc = bc;
+  switch (bc)
+    {
+    case (BOX_PERIODIC):
+    case (BOX_FREE):
+    case (BOX_SURFACE_XY):
+      openGLViewSet_upAxis(view, VIEW_Z);
+      break;
+    case (BOX_SURFACE_YZ):
+      openGLViewSet_upAxis(view, VIEW_X);
+      break;
+    case (BOX_SURFACE_ZX):
+      openGLViewSet_upAxis(view, VIEW_Y);
+      break;
+    }
+}
 void openGLViewGet_boxSpan(OpenGLView *view, float span[2])
 {
   float diag[3], ext[3];
@@ -544,7 +605,8 @@ static float computeExtens(float basisSet[6], float extension[3])
 
   return sqrt(su);
 }
-void openGLViewSet_box(OpenGLView *view, float basisSet[6], float extension[3])
+void openGLViewSet_box(OpenGLView *view, float basisSet[6],
+		       float extension[3], float margin)
 {
   g_return_if_fail(view && view->box);
 
@@ -597,14 +659,16 @@ void openGLViewSet_box(OpenGLView *view, float basisSet[6], float extension[3])
   view->box->p8[1] = view->box->p5[1] + view->box->p4[1];
   view->box->p8[2] = view->box->p5[2] + view->box->p4[2];
 
-  view->box->extens = computeExtens(basisSet, extension) * 0.55f;
+  if (margin >= 0.f)
+    view->box->margin = margin;
+  view->box->extens = computeExtens(basisSet, extension) * 0.5f + view->box->margin;
   DBG_fprintf(stderr, " | extens = %g.\n", view->box->extens);
 }
 gboolean openGLViewSet_refLength(OpenGLView *view, float length)
 {
   float basisSet[6], ext[3], l0;
 
-  g_return_val_if_fail(view && view->camera, FALSE);
+  g_return_val_if_fail(view && view->camera && length >= 0.f, FALSE);
 
   if (length > 0.f)
     l0 = length;
@@ -619,7 +683,7 @@ gboolean openGLViewSet_refLength(OpenGLView *view, float length)
       basisSet[VISU_DATA_BOX_DZX] = view->box->p5[0];
       basisSet[VISU_DATA_BOX_DZY] = view->box->p5[1];
       basisSet[VISU_DATA_BOX_DZZ] = view->box->p5[2];
-      l0 = computeExtens(basisSet, ext) * 0.55f;
+      l0 = computeExtens(basisSet, ext) * 0.5f;
     }
   if (l0 == view->camera->length0)
     return FALSE;
@@ -829,6 +893,69 @@ void OpenGLViewGet_screenAxes(OpenGLView *view, float xAxis[3], float yAxis[3])
   axis[2] = 0.;
   matrix_productVector(yAxis, matRes2, axis);
 }
+float openGLViewGet_zCoordinate(OpenGLView *view, float xyz[3])
+{
+  GLdouble model[16], project[16];
+  GLint viewport[4];
+  GLdouble xyzGL[3], winGL[3];
+
+  g_return_val_if_fail(view, 0.5f);
+
+  glMatrixMode(GL_MODELVIEW); 
+  glPushMatrix();
+  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
+
+  glGetDoublev(GL_MODELVIEW_MATRIX, model);
+  glGetDoublev(GL_PROJECTION_MATRIX, project);
+  glGetIntegerv(GL_VIEWPORT, viewport);
+  
+  xyzGL[0] = (GLdouble)xyz[0];
+  xyzGL[1] = (GLdouble)xyz[1];
+  xyzGL[2] = (GLdouble)xyz[2];
+  gluProject(xyzGL[0], xyzGL[1], xyzGL[2], model, project, viewport,
+	     winGL, winGL + 1, winGL + 2);
+  DBG_fprintf(stderr, "OpenGL View: get z coordinates from %gx%gx%g: %g.\n",
+	      xyz[0], xyz[1], xyz[2], (float)winGL[2]);
+  DBG_fprintf(stderr, " | win coords: %gx%g\n",
+	      (float)winGL[0], (float)winGL[1]);
+  glPopMatrix();
+  return (float)winGL[2];
+}
+void openGLViewGet_realCoordinates(OpenGLView *view, float xyz[3],
+				   float winx, float winy, float winz,
+				   gboolean centred)
+{
+  GLdouble model[16], project[16];
+  GLint viewport[4];
+  GLdouble xyzGL[3], winGL[3];
+
+  g_return_if_fail(view);
+
+  if (centred)
+    {
+      glMatrixMode(GL_MODELVIEW); 
+      glPushMatrix();
+      glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
+    }
+
+  glGetDoublev(GL_MODELVIEW_MATRIX, model);
+  glGetDoublev(GL_PROJECTION_MATRIX, project);
+  glGetIntegerv(GL_VIEWPORT, viewport);
+  
+  winGL[0] = (GLdouble)winx;
+  winGL[1] = (GLdouble)(view->window->height - winy);
+  winGL[2] = (GLdouble)winz;
+  gluUnProject(winGL[0], winGL[1], winGL[2], model, project, viewport,
+	       xyzGL, xyzGL + 1, xyzGL + 2);
+  xyz[0] = (float)xyzGL[0];
+  xyz[1] = (float)xyzGL[1];
+  xyz[2] = (float)xyzGL[2];
+  DBG_fprintf(stderr, "OpenGL View: get real coordinates from %gx%gx%g: %gx%gx%g.\n",
+	      winx, winy, winz, xyz[0], xyz[1], xyz[2]);
+
+  if (centred)
+    glPopMatrix();
+}
 
 void openGLViewRotate_box(OpenGLView *view, float dTheta, float dPhi, float angles[2])
 {
@@ -1021,8 +1148,7 @@ void openGLViewRotate_camera(OpenGLView *view, float dTheta, float dPhi, float a
 #define PARAMETER_OPENGL_DETAILS_DEFAULT 100
 static gboolean readOpenGLPrecision(gchar **lines, int nbLines, int position,
 				    VisuData *dataObj, GError **error);
-static gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
-					   VisuData *dataObj);
+static void exportParametersOpenGLView(GString *data, VisuData *dataObj);
 
 #define FLAG_RESOURCE_OPENGL_ANGLES "opengl_theta_phi_omega"
 #define DESC_RESOURCE_OPENGL_ANGLES "2 real values (degrees) for user orientation with respect to sample"
@@ -1040,8 +1166,7 @@ static gboolean readOpenGLGross(gchar **lines, int nbLines, int position,
 #define DESC_RESOURCE_OPENGL_PERSP "reduced perspective distance (must be real > 1.0)"
 static gboolean readOpenGLPersp(gchar **lines, int nbLines, int position,
 				VisuData *dataObj, GError **error);
-static gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
-					  VisuData *dataObj);
+static void exportResourcesOpenGLView(GString *data, VisuData *dataObj);
 
 
 void OpenGLViewInit()
@@ -1101,14 +1226,11 @@ static gboolean readOpenGLPrecision(gchar **lines, int nbLines, int position,
   OpenGLViewSet_precision((float)val / 100.);
   return TRUE;
 }
-static gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
-					   VisuData *dataObj _U_)
+static void exportParametersOpenGLView(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_DETAILS);
   g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OPENGL_DETAILS,
 	  (int)(precisionOfRendering * 100.));
-  *nbLinesWritten = 3;
-  return TRUE;
 }
 static gboolean readOpenGLThetaPhi(gchar **lines, int nbLines, int position,
 				   VisuData *dataObj, GError **error)
@@ -1175,8 +1297,7 @@ static gboolean readOpenGLPersp(gchar **lines, int nbLines, int position,
 
   return TRUE;
 }
-static gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
-					  VisuData *dataObj _U_)
+static void exportResourcesOpenGLView(GString *data, VisuData *dataObj _U_)
 {
   GList *tmpLst;
   OpenGLView *view;
@@ -1216,8 +1337,5 @@ static gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
   else
     g_string_append_printf(data, "    %9.3f\n", perspDefault);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten = 10;
-
-  return TRUE;
 }
 
