@@ -64,7 +64,8 @@
  *
  * This method call glMaterial to create the right shiningness, emission, diffuse...
  */
-void setOpenGlMaterial(float* material, float* rgba);
+void openGLSet_color(float* material, float* rgba);
+void openGLSet_highlightColor(float material[5], float rgb[3], float alpha);
 
 
 /* Object to deal with quadrics. */
@@ -187,20 +188,20 @@ gboolean openGLGet_stereo();
 /**
  * openGL_reDraw:
  * @lists: a string array, NULL terminated.
- * @data: some associated data.
+ * @data: the #VisuData to redraw.
  *
  * Basic drawing method : it clears the OpenGL area and call all lists if @lists
  * is NULL, or draw only the given lists.
  */
-void openGL_reDraw(const char **lists, gpointer data);
+void openGL_reDraw(const char **lists, VisuData *data);
 /**
  * openGL_drawToEmpty:
- * @lists: a string array, NULL terminated.
- * @data: some associated data.
+ * @lists: always NULL.
+ * @data: always NULL.
  *
  * Basic drawing method when no data is loaded: it draws the V_Sim logo.
  */
-void openGL_drawToEmpty(const char **lists, gpointer data);
+void openGL_drawToEmpty(const char **lists, VisuData *data);
 /**
  * openGLInit_context:
  *
