@@ -96,8 +96,7 @@ static gboolean readLinkRadius(gchar **lines, int nbLines,
 			       int position, VisuData *dataObj, GError **error);
 static gboolean readCylinderColorType(gchar **lines, int nbLines, int position,
 				      VisuData *dataObj, GError **error);
-static gboolean exportResourcesCylinder(GString *data, int *nbLinesWritten,
-					VisuData *dataObj);
+static void exportResourcesCylinder(GString *data, VisuData *dataObj);
 
 
 PairsExtension* cylinderGet_static()
@@ -193,7 +192,7 @@ void setColorAndWidthForCylinder(VisuElement *ele1 _U_, VisuElement *ele2 _U_,
       rgba[1] = color->rgba[1];
       rgba[2] = color->rgba[2];
       rgba[3] = 1.;
-      setOpenGlMaterial(mm, rgba);
+      openGLSet_color(mm, rgba);
       break;
     case cylinder_element_color:
     default:
@@ -260,7 +259,7 @@ void drawCylinderPairs(VisuElement *ele1, VisuElement *ele2, VisuPairData *data,
       rgba[1] = color->rgba[1];
       rgba[2] = color->rgba[2];
       rgba[3] = alphaColour;
-      setOpenGlMaterial(mm, rgba);
+      openGLSet_color(mm, rgba);
 
       glTranslated(x1, y1, z1);
       glRotated(alpha, vNorm[0], vNorm[1], vNorm[2]);
@@ -280,8 +279,8 @@ void drawCylinderPairs(VisuElement *ele1, VisuElement *ele2, VisuPairData *data,
 		  (GLdouble)sqrt(d2) * 0.5f, (GLint)nlat, (GLint)1);
       glPopMatrix();
       glPushMatrix();
-      glTranslated(x2, y2, z2);
-      glRotated(alpha - 180., vNorm[0], vNorm[1], vNorm[2]);
+      glTranslated(0.5f * (x2 + x1), 0.5f * (y2 + y1), 0.5f * (z2 + z1));
+      glRotated(alpha, vNorm[0], vNorm[1], vNorm[2]);
       glCallList(mat2);
       gluCylinder(obj, (GLdouble)radius, (GLdouble)radius,
 		  (GLdouble)sqrt(d2) * 0.5f, (GLint)nlat, (GLint)1);
@@ -290,7 +289,7 @@ void drawCylinderPairs(VisuElement *ele1, VisuElement *ele2, VisuPairData *data,
       break;
     }
   glPopMatrix();
-  gluDeleteQuadric(obj);   
+  gluDeleteQuadric(obj);
 }
 
 PairsExtension* initPairsCylinder()
@@ -453,11 +452,9 @@ static void exportPairsRadius(VisuElement *ele1, VisuElement *ele2,
 			     "    %s %s  %4.3f %4.3f  %4.3f\n", FLAG_RESOURCES_LINK_RADIUS,
 			     ele1->name, ele2->name, data->minMax[PAIRS_MIN],
 			     data->minMax[PAIRS_MAX], *radius);
-      *str->nbLinesWritten += 2;
     }
 }
-static gboolean exportResourcesCylinder(GString *data, int *nbLinesWritten,
-				 VisuData *dataObj)
+static void exportResourcesCylinder(GString *data, VisuData *dataObj)
 {
   struct foreachFuncExport_struct str;
 
@@ -469,16 +466,11 @@ static gboolean exportResourcesCylinder(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_CYLINDER_RADIUS);
   g_string_append_printf(data, "%s:\n    %f\n",
 			 FLAG_RESOURCES_CYLINDER_RADIUS, cylinderRadius);
-  *nbLinesWritten = 6;
   str.data           = data;
-  str.nbLinesWritten = nbLinesWritten;
   str.dataObj        = dataObj;
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_PAIR_RADIUS);
   visuPairForeach(exportPairsRadius, &str);
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 2;
-
-  return TRUE;
 }
 
 
