@@ -51,13 +51,13 @@
  *
  * Minimum value for the radius of cylinder pairs.
  */
-#define RESOURCES_CYLINDER_RADIUS_MIN 0.01
+#define RESOURCES_CYLINDER_RADIUS_MIN 0.01f
 /**
  * RESOURCES_CYLINDER_RADIUS_MAX:
  *
  * Maximum value for the radius of cylinder pairs.
  */
-#define RESOURCES_CYLINDER_RADIUS_MAX 3.
+#define RESOURCES_CYLINDER_RADIUS_MAX 3.f
 
 enum
   {
@@ -66,7 +66,7 @@ enum
     cylinder_nb_color
   };
 
-PairsExtension* pointerToPairExtension_cylinder;
+PairsExtension* cylinderGet_static();
 
 /**
  * initPairsCylinder:
