@@ -307,14 +307,14 @@ int setWireWidth(VisuPairData *data, int val)
     {
       width = g_malloc(sizeof(int));
       visuPairSet_linkProperty(data, "width", (gpointer)width);
-      *width = val;
+      *width = CLAMP(val, 0, RESOURCES_WIRE_WIDTH_MAX);
       visuPairSet_outOfDate();
       return 1;
     }
   else
     if (*width != val)
       {
-	*width = val;
+	*width = CLAMP(val, 0, RESOURCES_WIRE_WIDTH_MAX);
 	visuPairSet_outOfDate();
 	return 1;
       }
