@@ -47,14 +47,13 @@
 
 #include <visu_gtk.h>
 #include <visu_elements.h>
-#include <renderingMethods/renderingAtomic.h>
 #include <renderingMethods/renderingSpin.h>
 #include <openGLFunctions/view.h>
 #include <coreTools/toolMatrix.h>
 #include "gtkAtomic.h"
 #include "panelElements.h"
-#include "visuConfig.h"
 #include "panelMethod.h"
+#include <support.h>
 
 #include <assert.h>
 #include <string.h>
@@ -75,7 +74,6 @@
  */
 
 #define LABEL_ARROW_SIZE      _("<i>Shape size and color properties:</i>")
-#define LABEL_ARROW_SIZE_MULT _("<i>Shape size and color properties <span style=\"italic\" size=\"smaller\" foreground=\"red\">\303\227 mult.</span>:</i>")
 #define LABEL_SPIN_POLICY     _("Drawing policy for spins with null modulus:")
 
 void set_view(GtkWidget* button, gpointer data);
@@ -99,6 +97,9 @@ static GtkWidget *gtkw_set_view;
 static GtkWidget *labelSizeProp;
 static GtkWidget *vboxArrowShape, *vboxElipsoidShape;
 static GtkWidget *radioPerTypeMod, *radioGlobalMod;
+static GtkWidget *expandAtomic;
+static GtkWidget *radioAlwaysSpin, *radioEmptySpin, *radioAtomicSpin;
+static GtkWidget *labelAtomic;
 
 static guint element_callback_ids[10];
 
@@ -114,6 +115,7 @@ struct FileChooserSpin_struct
   GtkRadioButton* radio[2];
   gchar* files[2];
   GtkFileChooserDialog *fileChooserDialog;
+  GtkWidget* apply[2];
   int fileKind;
 
   VisuGtkPreview preview;
@@ -121,7 +123,7 @@ struct FileChooserSpin_struct
 /* A special widget to select two kind of files at a time. */
 static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent);
 /* Its callbacks. */
-static void fileSelectedForSpinMethod(GtkButton *button, gpointer data);
+static void onFileSelected(GtkButton *button, gpointer data);
 static void fileKindChangedForPosition(GtkToggleButton *button, gpointer data);
 static void fileKindChangedForSpin(GtkToggleButton *button, gpointer data);
 static void fileUpdatePreview(GtkFileChooser *chooser,
@@ -131,17 +133,16 @@ static void fileUpdatePreview(GtkFileChooser *chooser,
 void global_resource_callback(GtkWidget* widget, gpointer data);
 void element_resource_callback(GtkWidget* widget, gpointer data);
 void changeSomethingSpin(GList *eleList);
-void onSpinBoundsChangeArrowSize(GObject *obj, gfloat mult, gpointer data);
 GtkWidget* createGtkInterfaceForSpinMethod();
 static void onPolicyChanged(GtkToggleButton *toggle, gpointer data);
 static void onPolicyAtomicChanged(GtkToggleButton *toggle, gpointer data);
 static void onPolicyModulusChanged(GtkToggleButton *toggle, gpointer data);
 
-GtkWidget* createGtkConfigForSpinMethod(void);
-void destroyGtkConfigForSpinMethod();
-
-void redraw(int i);
+static GtkWidget* createGtkConfigForSpinMethod(void);
+static void destroyGtkConfigForSpinMethod();
 
+/* Local routines. */
+static gboolean fileSelected(struct FileChooserSpin_struct *values);
 static void sync_global_resources(GObject *object, VisuData *dataObj, gpointer data);
 static void sync_local_resources(GObject *trash, gpointer data);
 
@@ -153,7 +154,8 @@ static void sync_local_resources(GObject *trash, gpointer data);
 /* See h file for more info */
 void panelElementSpinInit_gtkPanel()
 {
-  panelElements_setInterfaceMethods(renderingSpinGet_static(),
+  panelElements_setInterfaceMethods(visuRenderingClassGet_methodByName
+				    (VISU_RENDERING_SPIN),
 				    changeSomethingSpin,
 				    createGtkInterfaceForSpinMethod);
   currentListElement = (GList*)0;
@@ -162,7 +164,8 @@ void panelElementSpinInit_gtkPanel()
 /* See h file for more info */
 void panelElementSpinInit_gtkConfig()
 {
-  panelMethodSet_renderingInterface(renderingSpinGet_static(),
+  panelMethodSet_renderingInterface(visuRenderingClassGet_methodByName
+				    (VISU_RENDERING_SPIN),
 				    createGtkConfigForSpinMethod,
 				    destroyGtkConfigForSpinMethod);
 }
@@ -177,8 +180,12 @@ static GtkWidget *gtk_spin_create_direction(float *cartesian_values,
   GtkWidget* label_z = gtk_label_new(_("z:"));
   GtkWidget* label_theta = gtk_label_new("\316\270:");
   GtkWidget* label_phi = gtk_label_new("\317\206:");
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips;
 
+  tooltips = gtk_tooltips_new();
+#endif
+
   table = gtk_table_new(2, 6, FALSE);
 
   gtk_misc_set_alignment(GTK_MISC(label_x), 1., 0.5);
@@ -214,8 +221,6 @@ static GtkWidget *gtk_spin_create_direction(float *cartesian_values,
   gtk_table_attach(GTK_TABLE(table), gtkw_z, 5, 6, 0, 1,
 		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
 
-  tooltips = gtk_tooltips_new();
-
   gtk_misc_set_alignment(GTK_MISC(label_theta), 1., 0.5);
   gtk_misc_set_alignment(GTK_MISC(label_phi), 1., 0.5);
 
@@ -231,8 +236,9 @@ static GtkWidget *gtk_spin_create_direction(float *cartesian_values,
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(gtkw_cone_phi_angle), TRUE);
 
   gtkw_set_view = gtk_button_new_with_label (_("Set ortho."));
-  gtk_tooltips_set_tip(tooltips, gtkw_set_view,
-		       _("Set the cone orientation to be orthogonal to the screen."), NULL);
+  gtk_widget_set_tooltip_text(gtkw_set_view,
+			      _("Set the cone orientation to be orthogonal"
+				" to the screen."));
   g_signal_connect(gtkw_set_view, "clicked",
 		   G_CALLBACK(set_view), NULL);
 
@@ -278,7 +284,6 @@ static GtkWidget *gtk_spin_create_color_wheel(float color_wheel_angle)
 
 GtkWidget* createHidingModeRadioWidgets()
 {
-  GtkWidget *radioAlwaysSpin, *radioEmptySpin, *radioAtomicSpin;
   GtkWidget *hbox;
   GSList *radiobutton_group;
 
@@ -289,27 +294,18 @@ GtkWidget* createHidingModeRadioWidgets()
   gtk_box_pack_start(GTK_BOX(hbox), radioAlwaysSpin, TRUE, TRUE, 1);
   g_signal_connect(G_OBJECT(radioAlwaysSpin), "toggled",
 		   G_CALLBACK(onPolicyChanged), GINT_TO_POINTER(policyAlwaysSpin));
-/*   image1 = create_pixmap(mainWindow, "stock-go-around.png"); */
-/*   gtk_widget_show (image1); */
-/*   gtk_container_add(GTK_CONTAINER(radioGoAround), image1); */
   radioEmptySpin = gtk_radio_button_new_with_label(NULL, _("never"));
   gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioEmptySpin), radiobutton_group);
   radiobutton_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioEmptySpin));
   gtk_box_pack_start(GTK_BOX(hbox), radioEmptySpin, TRUE, TRUE, 1);
   g_signal_connect(G_OBJECT(radioEmptySpin), "toggled",
 		   G_CALLBACK(onPolicyChanged), GINT_TO_POINTER(policyHideNullSpin));
-/*   image1 = create_pixmap(mainWindow, "stock-go-once.png"); */
-/*   gtk_widget_show (image1); */
-/*   gtk_container_add(GTK_CONTAINER(radioGoOnce), image1); */
   radioAtomicSpin = gtk_radio_button_new_with_label(NULL, _("atomic"));
   gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioAtomicSpin), radiobutton_group);
   radiobutton_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioAtomicSpin));
   gtk_box_pack_start(GTK_BOX(hbox), radioAtomicSpin, TRUE, TRUE, 1);
   g_signal_connect(G_OBJECT(radioAtomicSpin), "toggled",
 		   G_CALLBACK(onPolicyChanged), GINT_TO_POINTER(policyAtomicNullSpin));
-/*   image1 = create_pixmap(mainWindow, "stock-go-and-back.png"); */
-/*   gtk_widget_show (image1); */
-/*   gtk_container_add(GTK_CONTAINER(radioGoAndBack), image1); */
   switch (rspin_getGlobalResource_uint(spin_globalHidingMode))
     {
     case policyAlwaysSpin:
@@ -330,7 +326,7 @@ GtkWidget* createHidingModeRadioWidgets()
 }
   
 /* Creates the vbox displayed in the config panel */
-GtkWidget* createGtkConfigForSpinMethod()
+static GtkWidget* createGtkConfigForSpinMethod()
 {
   GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
   GtkWidget *label2 = gtk_label_new(_("Color cone orientation:"));
@@ -356,9 +352,9 @@ GtkWidget* createGtkConfigForSpinMethod()
   label = gtk_label_new(_("Spin lenght is proportional to modulus: "));
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_widget_set_name(label, "label_head_2");
-  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
   check = gtk_check_button_new();
-  gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 10);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
 			       (modPol != policyNoneModulus));
   g_signal_connect(G_OBJECT(check), "toggled",
@@ -395,9 +391,9 @@ GtkWidget* createGtkConfigForSpinMethod()
   label = gtk_label_new(_("Use atomic rendering in addition to spin: "));
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_widget_set_name(label, "label_head_2");
-  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 3);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
   check = gtk_check_button_new();
-  gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 10);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
 			       rspin_getGlobalResource_boolean(spin_globalAtomic));
   g_signal_connect(G_OBJECT(check), "toggled",
@@ -439,33 +435,62 @@ GtkWidget* createGtkConfigForSpinMethod()
 
   return vbox;
 }
-void destroyGtkConfigForSpinMethod()
+static void destroyGtkConfigForSpinMethod()
 {
   DBG_fprintf(stderr, "Gtk Spin : remove signal from config rendering widget.\n");
   g_signal_handler_disconnect(VISU_INSTANCE, configResourcesLoadedId);
 }
 static void onPolicyChanged(GtkToggleButton *toggle, gpointer data)
 {
+  gboolean atomic;
+  VisuData *dataObj;
+
   if (!gtk_toggle_button_get_active(toggle))
     return;
 
-  if (rspin_setGlobalResource_uint(spin_globalHidingMode, (guint)GPOINTER_TO_INT(data)))
-    redraw(2);
+  if (!rspin_setGlobalResource_uint(spin_globalHidingMode,
+				    (guint)GPOINTER_TO_INT(data)))
+    return;
+
+  atomic = (gtk_toggle_button_get_active(toggle) ||
+	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioAtomicSpin)));
+  gtk_widget_set_sensitive(expandAtomic, atomic);	    
+  gtk_expander_set_expanded(GTK_EXPANDER(expandAtomic), atomic);
+  if (atomic)
+    gtk_widget_hide(labelAtomic);
+  else
+    gtk_widget_show(labelAtomic);
+
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
+  if (!dataObj)
+    return;
+  visuData_createAllNodes(dataObj);  
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 static void onPolicyAtomicChanged(GtkToggleButton *toggle, gpointer data _U_)
 {
   VisuData *dataObj;
+  gboolean atomic;
 
-  if (rspin_setGlobalResource_boolean(spin_globalAtomic,
-				      gtk_toggle_button_get_active(toggle)))
-    {
-      dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
-      if (!dataObj)
-	return;
+  if (!rspin_setGlobalResource_boolean(spin_globalAtomic,
+				       gtk_toggle_button_get_active(toggle)))
+    return;
 
-      visuData_createAllElements(dataObj);
-      redraw(2);
-    }
+  atomic = (gtk_toggle_button_get_active(toggle) ||
+	    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioAtomicSpin)));
+  gtk_widget_set_sensitive(expandAtomic, atomic);
+  gtk_expander_set_expanded(GTK_EXPANDER(expandAtomic), atomic);
+  if (atomic)
+    gtk_widget_hide(labelAtomic);
+  else
+    gtk_widget_show(labelAtomic);
+
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
+  if (!dataObj)
+    return;
+  visuData_createAllElements(dataObj);
+  visuData_createAllNodes(dataObj);  
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 static void onPolicyModulusChanged(GtkToggleButton *toggle, gpointer data)
 {
@@ -487,23 +512,24 @@ static void onPolicyModulusChanged(GtkToggleButton *toggle, gpointer data)
     modPol = (SpinModulusPolicy)data;
   gtk_widget_set_sensitive(radioGlobalMod, gtk_toggle_button_get_active(toggle));
   gtk_widget_set_sensitive(radioPerTypeMod, gtk_toggle_button_get_active(toggle));
-  if (rspin_setGlobalResource_uint(spin_globalModulus, modPol))
-    {
-      dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
-      if (!dataObj)
-	return;
+  if (!rspin_setGlobalResource_uint(spin_globalModulus, modPol))
+    return;
 
-      visuData_createAllElements(dataObj);
-      redraw(2);
-    }
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
+  if (!dataObj)
+    return;
+  visuData_createAllElements(dataObj);
+  visuData_createAllNodes(dataObj);  
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
 /* Initialise the gtk methods associated with
    the spin rendering method. */
 void panelElementSpinInit_gtkMain()
 {
-  gtkMainClassSet_renderingSpecificMethods(renderingSpinGet_static(),
-					   gtkSpinCreate_fileChooser);
+  visuGtkSet_renderingSpecificMethods(visuRenderingClassGet_methodByName
+				      (VISU_RENDERING_SPIN),
+				      gtkSpinCreate_fileChooser);
 }
 
 
@@ -512,7 +538,7 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
 {
   GtkWidget *fileSelection;
   GList *tmpLst, *tmpLst2;
-  GtkWidget *vbox;
+  GtkWidget *vbox, *wd;
   gchar* directory;
   GtkFileFilter *filterPos, *filterSpin, *filterAll;
   GtkWidget *table, *hbox, *align;
@@ -522,7 +548,13 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
   GtkWidget *hiding_mode;
   struct FileChooserSpin_struct fileChooserSpin;
   gint status;
+  VisuRendering *spin;
 
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
+
+  tooltips = gtk_tooltips_new ();
+#endif
 
   if (!data)
     return 0;
@@ -533,7 +565,7 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
 					      NULL);
-  directory = gtkMainClassGet_lastOpenDirectory();
+  directory = visuGtkGet_lastOpenDirectory();
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fileSelection), directory);
 
@@ -541,38 +573,41 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
   gtk_window_set_position(GTK_WINDOW(fileSelection), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);
 
-  tmpLst = renderingMethodGet_fileType(renderingAtomicGet_static(),
-				       FILE_KIND_POSITION);
+  spin = visuRenderingClassGet_methodByName(VISU_RENDERING_SPIN);
+
+  DBG_fprintf(stderr, "Gtk Spin: filter for position.\n");
+  tmpLst = visuRenderingGet_fileType(spin, FILE_KIND_POSITION);
   filterPos = gtk_file_filter_new ();
   while(tmpLst)
     {
       tmpLst2 = ((FileFormat*)tmpLst->data)->fileType;
       while (tmpLst2)
 	{
+	  DBG_fprintf(stderr, " | add pattern '%s'.\n", (char*)tmpLst2->data);
 	  gtk_file_filter_add_pattern (filterPos, (char*)tmpLst2->data);
 	  tmpLst2 = g_list_next(tmpLst2);
 	}
       tmpLst = g_list_next(tmpLst);
     }
-  gtk_file_filter_set_name(filterPos,
-			   renderingMethodGet_fileTypeName(renderingSpinGet_static(),
-							   FILE_KIND_POSITION));
+  gtk_file_filter_set_name(filterPos, visuRenderingGet_fileTypeName
+			   (spin, FILE_KIND_POSITION));
 
-  tmpLst = renderingMethodGet_fileType(renderingSpinGet_static(), FILE_KIND_SPIN);
+  DBG_fprintf(stderr, "Gtk Spin: filter for spin.\n");
+  tmpLst = visuRenderingGet_fileType(spin, FILE_KIND_SPIN);
   filterSpin = gtk_file_filter_new ();
   while(tmpLst)
     {
       tmpLst2 = ((FileFormat*)tmpLst->data)->fileType;
       while (tmpLst2)
 	{
+	  DBG_fprintf(stderr, " | add pattern '%s'.\n", (char*)tmpLst2->data);
 	  gtk_file_filter_add_pattern (filterSpin, (char*)tmpLst2->data);
 	  tmpLst2 = g_list_next(tmpLst2);
 	}
       tmpLst = g_list_next(tmpLst);
     }
   gtk_file_filter_set_name(filterSpin,
-			   renderingMethodGet_fileTypeName(renderingSpinGet_static(),
-							   FILE_KIND_SPIN));
+			   visuRenderingGet_fileTypeName(spin, FILE_KIND_SPIN));
   filterAll = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filterAll, "*");
   gtk_file_filter_set_name(filterAll, _("All files"));
@@ -594,7 +629,8 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(fileSelection), vbox);
   
-  label = gtk_label_new(_("Current files:"));
+  label = gtk_label_new(_("<b>Selected files are used for</b>:"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_widget_set_name(label, "label_head_2");
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 2);
@@ -603,51 +639,71 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
   /* The table for the two file types selections (spin & position). */
-  table = gtk_table_new(2, 3, FALSE);
+  table = gtk_table_new(2, 4, FALSE);
   gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 2);
   
   /* Position button. */
+  align = gtk_alignment_new(0., 1.0, 1., 0.);
   radioPosition = gtk_radio_button_new_with_label(NULL, _("position"));
-  gtk_table_attach(GTK_TABLE(table), radioPosition, 0, 1, 0, 1,
-		   GTK_SHRINK, GTK_SHRINK, 2, 0);
+  gtk_container_add(GTK_CONTAINER(align), radioPosition);
+  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1,
+		   GTK_FILL, GTK_SHRINK, 2, 0);
   g_signal_connect(G_OBJECT(radioPosition), "toggled",
 		   G_CALLBACK(fileKindChangedForPosition),
 		   (gpointer)&fileChooserSpin);
 
   /* Spin button */
+  align = gtk_alignment_new(0., 1.0, 1., 0.);
   radioSpin = gtk_radio_button_new_with_label_from_widget
     (GTK_RADIO_BUTTON(radioPosition), _("spin"));
-  gtk_table_attach(GTK_TABLE(table), radioSpin, 0, 1, 1, 2,
-		   GTK_SHRINK, GTK_SHRINK, 2, 0);
+  gtk_container_add(GTK_CONTAINER(align), radioSpin);
+  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2,
+		   GTK_FILL, GTK_SHRINK, 2, 0);
   g_signal_connect(G_OBJECT(radioSpin), "toggled",
 		   G_CALLBACK(fileKindChangedForSpin),
 		   (gpointer)&fileChooserSpin);
 
   /* Layout labels */
   label = gtk_label_new(":");
-  gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 5, 0);
+  gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1, GTK_SHRINK, GTK_SHRINK, 5, 0);
 
   label = gtk_label_new(":");
-  gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 5, 0);
+  gtk_table_attach(GTK_TABLE(table), label, 2, 3, 1, 2, GTK_SHRINK, GTK_SHRINK, 5, 0);
 
   labelPosition = gtk_label_new(_("None"));
   gtk_misc_set_alignment(GTK_MISC(labelPosition), 0., 0.5);
-  gtk_table_attach(GTK_TABLE(table), labelPosition, 2, 3, 0, 1,
+  gtk_table_attach(GTK_TABLE(table), labelPosition, 3, 4, 0, 1,
 		   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
 
   labelSpin = gtk_label_new(_("None"));
   gtk_misc_set_alignment(GTK_MISC(labelSpin), 0., 0.5);
-  gtk_table_attach(GTK_TABLE(table), labelSpin, 2, 3, 1, 2,
+  gtk_table_attach(GTK_TABLE(table), labelSpin, 3, 4, 1, 2,
 		   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
 
-  /* Right aligned apply button. */
-  align = gtk_alignment_new(0.5, 1.0, 0., 0.);
-  gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 2);
-  button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-  gtk_container_add(GTK_CONTAINER(align), button);
+  button = gtk_button_new();
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  wd = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
+  gtk_container_add(GTK_CONTAINER(button), wd);
+  gtk_table_attach(GTK_TABLE(table), button, 1, 2, 0, 1,
+		   GTK_SHRINK, GTK_SHRINK, 10, 0);
   g_signal_connect(G_OBJECT(button), "clicked",
-		   G_CALLBACK(fileSelectedForSpinMethod),
-		   (gpointer)&fileChooserSpin);
+		   G_CALLBACK(onFileSelected), (gpointer)&fileChooserSpin);
+  gtk_widget_set_tooltip_text(button,
+			      _("Set the selected file as position to load."));
+  fileChooserSpin.apply[0] = button;
+
+  button = gtk_button_new();
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  wd = gtk_image_new_from_stock(GTK_STOCK_APPLY, GTK_ICON_SIZE_MENU);
+  gtk_container_add(GTK_CONTAINER(button), wd);
+  gtk_table_attach(GTK_TABLE(table), button, 1, 2, 1, 2,
+		   GTK_SHRINK, GTK_SHRINK, 10, 0);
+  g_signal_connect(G_OBJECT(button), "clicked",
+		   G_CALLBACK(onFileSelected), (gpointer)&fileChooserSpin);
+  gtk_widget_set_tooltip_text(button,
+			       _("Set the selected file as spin to load."));
+  gtk_widget_set_sensitive(button, FALSE);
+  fileChooserSpin.apply[1] = button;
 
   /* Hiding mode button */
   hbox = gtk_hbox_new(FALSE, 0);
@@ -706,7 +762,8 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
 	}
       else
 	{
-	  fileSelectedForSpinMethod((GtkButton*)0, &fileChooserSpin);
+	  DBG_fprintf(stderr, "Gtk Spin: get response OK.\n");
+	  fileSelected(&fileChooserSpin);
 	  if (fileChooserSpin.files[FILE_KIND_POSITION] &&
 	      fileChooserSpin.files[FILE_KIND_SPIN])
 	    {
@@ -719,7 +776,7 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
 
 	      directory = gtk_file_chooser_get_current_folder
 		(GTK_FILE_CHOOSER(fileSelection));
-	      gtkMainClassSet_lastOpenDirectory((char*)directory);
+	      visuGtkSet_lastOpenDirectory((char*)directory, VISU_DIR_FILE);
 	      g_free(directory);
 	      
 	      gtk_widget_destroy(fileSelection);
@@ -731,42 +788,59 @@ static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
   return FALSE;
 }
 
-static void fileSelectedForSpinMethod(GtkButton *button _U_, gpointer data)
+static gboolean fileSelected(struct FileChooserSpin_struct *values)
 {
   gchar *filename = NULL, *filenameUTF8;
-  struct FileChooserSpin_struct *values;
   int res;
-  GString *message;
+  gchar *message;
 
-  values = (struct FileChooserSpin_struct*)data;
-  filename = gtk_file_chooser_get_filename
-    (GTK_FILE_CHOOSER(values->fileChooserDialog));
-  if (!g_file_test(filename, G_FILE_TEST_IS_DIR) && filename != NULL)
+  filename =
+    gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(values->fileChooserDialog));
+  if (!filename)
+    return FALSE;
+
+  if (!g_file_test(filename, G_FILE_TEST_IS_DIR))
     {
-      DBG_fprintf(stderr, "Gtk Spin : select '%s' as file %d\n", filename, values->fileKind);
+      DBG_fprintf(stderr, "Gtk Spin: select '%s' as file %d\n", filename, values->fileKind);
       res = 0;
+      if (values->files[values->fileKind])
+	g_free(values->files[values->fileKind]);
+      /* filename is not freed here, it will be later. */
       values->files[values->fileKind] = filename;
       filenameUTF8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
-      g_return_if_fail(filenameUTF8);
+      g_return_val_if_fail(filenameUTF8, TRUE);
 
-      message = g_string_new("");
       if (g_utf8_strlen(filenameUTF8, 51) > 50)
-	g_string_printf(message, "(...)%s",
-			 g_utf8_offset_to_pointer(filenameUTF8,
-						  g_utf8_strlen(filenameUTF8, -1) - 50));
+	message =
+	  g_strdup_printf("(...)%s", g_utf8_offset_to_pointer
+			  (filenameUTF8, g_utf8_strlen(filenameUTF8, -1) - 50));
       else
-	g_string_printf(message, "%s", filenameUTF8);
-      gtk_label_set_text(values->label[values->fileKind], message->str);
-      g_string_free(message, TRUE);
+	message = g_strdup(filenameUTF8);
+      gtk_label_set_text(values->label[values->fileKind], message);
+      g_free(message);
       g_free(filenameUTF8);
       if (values->fileKind == FILE_KIND_POSITION)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(values->radio[FILE_KIND_SPIN]), TRUE);
       else if (values->fileKind == FILE_KIND_SPIN)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(values->radio[FILE_KIND_POSITION]), TRUE);
       
-      if (values->files[FILE_KIND_POSITION] && values->files[FILE_KIND_SPIN])
-	gtk_dialog_response(GTK_DIALOG(values->fileChooserDialog), GTK_RESPONSE_OK);
+      DBG_fprintf(stderr, "Gtk Spin: interface updated after selection.\n");
+
+      return TRUE;
+/*       if (values->files[FILE_KIND_POSITION] && values->files[FILE_KIND_SPIN]) */
+/* 	{ */
+/* 	  DBG_fprintf(stderr, "Gtk Spin: emit response OK.\n"); */
+/* 	  gtk_dialog_response(GTK_DIALOG(values->fileChooserDialog), GTK_RESPONSE_OK); */
+/* 	} */
     }
+
+  g_free(filename);
+  return FALSE;
+}
+
+static void onFileSelected(GtkButton *button _U_, gpointer data)
+{
+  fileSelected((struct FileChooserSpin_struct*)data);
 }
 
 static void fileUpdatePreview(GtkFileChooser *chooser,
@@ -787,6 +861,9 @@ static void fileUpdatePreview(GtkFileChooser *chooser,
     filenames[1] = (char*)0;
   filenames[2] = (char*)0;
 
+  DBG_fprintf(stderr, "Gtk Spin: update preview with %s %s.\n",
+	      filenames[0], filenames[1]);
+
   /* We test if the selected filename is a directory or not. */
   if (filenames[0] && !g_file_test(filenames[0], G_FILE_TEST_IS_DIR) &&
       filenames[1] && !g_file_test(filenames[1], G_FILE_TEST_IS_DIR))
@@ -810,6 +887,9 @@ static void fileKindChangedForPosition(GtkToggleButton *button, gpointer data)
 			      (GtkFileFilter*)(tmpLst->data));
   param->fileKind = FILE_KIND_POSITION;
 
+  gtk_widget_set_sensitive(param->apply[0], TRUE);
+  gtk_widget_set_sensitive(param->apply[1], FALSE);
+
   if (param->files[0])
     gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(param->fileChooserDialog),
 				     param->files[0]);
@@ -828,6 +908,9 @@ static void fileKindChangedForSpin(GtkToggleButton *button, gpointer data)
 			      (GtkFileFilter*)(tmpLst->next->data));
   param->fileKind = FILE_KIND_SPIN;
 
+  gtk_widget_set_sensitive(param->apply[1], TRUE);
+  gtk_widget_set_sensitive(param->apply[0], FALSE);
+
   if (param->files[1])
     gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(param->fileChooserDialog),
 				     param->files[1]);
@@ -837,9 +920,10 @@ static void fileKindChangedForSpin(GtkToggleButton *button, gpointer data)
    positive values) and return it. */
 GtkWidget* createGtkInterfaceForSpinMethod()
 {
-  GtkWidget* hbox[7], *vbox, *label, *expand, *vboxAtomic;
+  GtkWidget* hbox[7], *vbox, *label, *vboxAtomic;
   GtkWidget* shapeLabel, *hLabel, *rhLabel, *lLabel, *rlLabel, *ratioLabel, *lengthLabel;
   int i, j = 0;
+  gboolean atomic;
 
   vbox = gtk_vbox_new(FALSE, 0);
 
@@ -868,10 +952,7 @@ GtkWidget* createGtkInterfaceForSpinMethod()
   gtk_widget_show(shapeLabel);
 
   labelSizeProp = gtk_label_new("");
-  if (panelConfigGet_spinBoundsValue() == 1.)
-    gtk_label_set_markup(GTK_LABEL(labelSizeProp), LABEL_ARROW_SIZE);
-  else
-    gtk_label_set_markup(GTK_LABEL(labelSizeProp), LABEL_ARROW_SIZE_MULT);
+  gtk_label_set_markup(GTK_LABEL(labelSizeProp), LABEL_ARROW_SIZE);
   gtk_misc_set_alignment(GTK_MISC(labelSizeProp), 0., 0.5);
   gtk_misc_set_padding(GTK_MISC(labelSizeProp), 10, 0);
   gtk_widget_show(labelSizeProp);
@@ -919,11 +1000,6 @@ GtkWidget* createGtkInterfaceForSpinMethod()
 					     GINT_TO_POINTER(spin_elementTailLength));
   gtk_widget_show(rlengthResourceSpin);
 
-  /* This callback is for multiplier */
-  g_signal_connect(G_OBJECT(gtkMainClassGet_currentPanel()), "spinBoundsChanged",
-		   G_CALLBACK(onSpinBoundsChangeArrowSize), (gpointer)0);
-
-
   /* Check box creation */
   label = gtk_label_new(_("Use element color on:"));
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
@@ -943,7 +1019,7 @@ GtkWidget* createGtkInterfaceForSpinMethod()
 
   /* Combo boxes creation */
   gtkw_element_shape_number = gtk_combo_box_new_text();
-  for (i=0; i<rspin_get_number_of_shapes(); i++)
+  for (i=0; i<arrow_shape_nb; i++)
     gtk_combo_box_append_text(GTK_COMBO_BOX(gtkw_element_shape_number),
 			      rspin_shape_number_to_translated_name(i));
   /* set callback for the combo button. */
@@ -977,11 +1053,25 @@ GtkWidget* createGtkInterfaceForSpinMethod()
   gtk_widget_show(useElementForElipsoid);
 
   /* Atomic options. */
-  expand = gtk_expander_new(_("Atomic rendering options"));
-  gtk_expander_set_expanded(GTK_EXPANDER(expand), FALSE);
-  gtk_widget_show(expand);
+  expandAtomic = gtk_expander_new(_("Atomic rendering options"));
+  gtk_widget_set_sensitive(expandAtomic, FALSE);
+  gtk_expander_set_expanded(GTK_EXPANDER(expandAtomic), FALSE);
+  gtk_widget_show(expandAtomic);
   vboxAtomic = panelElementAtomicCreate_gtkInterface();
-  gtk_container_add(GTK_CONTAINER(expand), vboxAtomic);
+  gtk_container_add(GTK_CONTAINER(expandAtomic), vboxAtomic);
+  labelAtomic = gtk_label_new(_("<i>Enable the atomic rendering in the method tab.</i>"));
+  gtk_widget_show(labelAtomic);
+  gtk_label_set_use_markup(GTK_LABEL(labelAtomic), TRUE);
+
+  atomic = (rspin_getGlobalResource_boolean(spin_globalAtomic) ||
+	    rspin_getGlobalResource_uint(spin_globalHidingMode) ==
+	    policyAtomicNullSpin);
+  gtk_widget_set_sensitive(expandAtomic, atomic);
+  gtk_expander_set_expanded(GTK_EXPANDER(expandAtomic), atomic);
+  if (atomic)
+    gtk_widget_hide(labelAtomic);
+  else
+    gtk_widget_show(labelAtomic);
 
   /* Displaying every widget created so far */
 
@@ -1017,7 +1107,8 @@ GtkWidget* createGtkInterfaceForSpinMethod()
 
   gtk_box_pack_start(GTK_BOX(hbox[j++]), useElementForElipsoid, FALSE, FALSE, 1);
 
-  gtk_box_pack_start(GTK_BOX(vbox), expand, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), expandAtomic, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), labelAtomic, FALSE, FALSE, 0);
 
   g_signal_connect(VISU_INSTANCE, "resourcesLoaded",
 		   G_CALLBACK(sync_local_resources), NULL);
@@ -1028,7 +1119,6 @@ GtkWidget* createGtkInterfaceForSpinMethod()
 /* This function is called whenever the current element is changed in the gtk panel. */
 void changeSomethingSpin(GList *eleList)
 {
-  float mult;
   VisuElement *ele;
 
   DBG_fprintf(stderr, "Gtk Spin : changing values on element change.\n");
@@ -1049,8 +1139,6 @@ void changeSomethingSpin(GList *eleList)
   ele = (VisuElement*)eleList->data;
   need_redraw = 0;
 
-  mult = panelConfigGet_spinBoundsValue();
-  
   g_signal_handler_block (heightResourceSpin, element_callback_ids[0] );  
   g_signal_handler_block (rheightResourceSpin, element_callback_ids[1] );  
   g_signal_handler_block (lengthResourceSpin, element_callback_ids[2] );  
@@ -1063,17 +1151,17 @@ void changeSomethingSpin(GList *eleList)
   g_signal_handler_block (useElementForElipsoid, element_callback_ids[9] );  
 
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(heightResourceSpin),
-			    rspin_getElementResource_float(ele, spin_elementHatRadius) / mult);
+			    rspin_getElementResource_float(ele, spin_elementHatRadius));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(rheightResourceSpin),
-			    rspin_getElementResource_float(ele, spin_elementTailRadius) / mult);
+			    rspin_getElementResource_float(ele, spin_elementTailRadius));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(lengthResourceSpin),
-			    rspin_getElementResource_float(ele, spin_elementHatLength) / mult);
+			    rspin_getElementResource_float(ele, spin_elementHatLength));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(rlengthResourceSpin),
-			    rspin_getElementResource_float(ele, spin_elementTailLength) / mult);
+			    rspin_getElementResource_float(ele, spin_elementTailLength));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(lengthElipsoidSpin),
-			    rspin_getElementResource_float(ele, spin_elementAAxis) / mult);
+			    rspin_getElementResource_float(ele, spin_elementAAxis));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(ratioElipsoidSpin),
-			    rspin_getElementResource_float(ele, spin_elementBAxis) / mult);
+			    rspin_getElementResource_float(ele, spin_elementBAxis));
   
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gtkw_use_element_color),
 			       rspin_getElementResource_boolean(ele, spin_elementTailColor));
@@ -1099,43 +1187,6 @@ void changeSomethingSpin(GList *eleList)
   need_redraw = 1;
 }
 
-void onSpinBoundsChangeArrowSize(GObject *obj _U_, gfloat mult, gpointer data _U_)
-{
-  unsigned int i, j;
-  float *val;
-  GType type;
-  SpinElementResources props[6] = {spin_elementHatLength, spin_elementHatRadius,
-				   spin_elementTailLength, spin_elementTailRadius,
-				   spin_elementAAxis, spin_elementBAxis};
-  VisuData *dataObj;
-  int id;
-
-  if (getRenderingMethodInUse() != renderingSpinGet_static())
-    return;
-
-  if (panelConfigGet_spinBoundsValue() == 1.)
-    gtk_label_set_text(GTK_LABEL(labelSizeProp), LABEL_ARROW_SIZE);
-  else
-    {
-      gtk_label_set_text(GTK_LABEL(labelSizeProp), LABEL_ARROW_SIZE_MULT);
-      gtk_label_set_use_markup(GTK_LABEL(labelSizeProp), TRUE);
-    }
-
-  dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
-  for (i = 0; i < dataObj->ntype; i++)
-    {
-      for (j = 0; j < 6; j++)
-	{
-	  val = (float*)rspin_getElementResource(dataObj->fromIntToVisuElement[i], props[j], &type);
-	  *val = *val * mult;
-	}
-      id = rspin_createShapeSpin(dataObj, dataObj->fromIntToVisuElement[i]);
-      dataObj->fromIntToVisuElement[i]->openGLIdentifier = id;
-    }
-}
-
-
-
 /* This is the callback func for each element resource. */
 void element_resource_callback(GtkWidget* widget, gpointer data)
 {
@@ -1145,7 +1196,9 @@ void element_resource_callback(GtkWidget* widget, gpointer data)
   SpinElementResources property;
   gboolean new_bool;
   guint new_int;
-  float new_float, mult;
+  float new_float;
+  VisuData *dataObj;
+  int id;
 
   property = GPOINTER_TO_INT(data);
   g_return_if_fail(property < spin_nbElementResources);
@@ -1172,7 +1225,7 @@ void element_resource_callback(GtkWidget* widget, gpointer data)
 	  DBG_fprintf(stderr, "GtkSpin : Set element '%d' property to %d\n",
 		      property, new_int);
 	  refresh = rspin_setElementResource_uint(ele, property, new_int) || refresh;
-	  if (new_int == 2)
+	  if (new_int == arrow_ellipsoid || new_int == arrow_torus)
 	    {
 	      gtk_widget_show(vboxElipsoidShape);
 	      gtk_widget_hide(vboxArrowShape);
@@ -1189,12 +1242,11 @@ void element_resource_callback(GtkWidget* widget, gpointer data)
 	case spin_elementTailRadius:
 	case spin_elementAAxis:
 	case spin_elementBAxis:
-	  mult = panelConfigGet_spinBoundsValue();
 	  new_float = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
 
 	  DBG_fprintf(stderr, "GtkSpin : Set element '%d' property to %f\n",
-		      property, new_float * mult);
-	  refresh = rspin_setElementResource_float(ele, property, new_float * mult) || refresh;
+		      property, new_float);
+	  refresh = rspin_setElementResource_float(ele, property, new_float) || refresh;
 
 	  break;
 	default:
@@ -1205,7 +1257,20 @@ void element_resource_callback(GtkWidget* widget, gpointer data)
     }
 
   if(need_redraw && refresh)
-    redraw(1);
+    {
+      dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
+      if (!dataObj)
+	return;
+
+      for(tmpLst = currentListElement; tmpLst; tmpLst = g_list_next(tmpLst))
+	{
+	  ele = (VisuElement*)tmpLst->data;
+	  id = rspin_createShapeSpin(dataObj, ele);
+	  ele->openGLIdentifier = id;
+	}
+      visuData_createAllNodes(dataObj);  
+      g_idle_add(visuObjectRedraw, (gpointer)0);
+    }
 }
 
 /* This is the callback func for each global resource. */
@@ -1214,6 +1279,7 @@ void global_resource_callback(GtkWidget* button, gpointer data)
   SpinGlobalResources property;
   int value;
   gboolean refresh;
+  VisuData *dataObj;
 
   value = GPOINTER_TO_INT(data);
   if (value >= 0)
@@ -1284,7 +1350,16 @@ void global_resource_callback(GtkWidget* button, gpointer data)
     }
 
   if(need_redraw == 1)
-    redraw(2);
+    {
+      dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
+      if (!dataObj)
+	return;
+
+      g_signal_emit(VISU_INSTANCE,VISU_SIGNALS[SPINAXES_SIGNAL],
+		    0 , dataObj, NULL);
+      visuData_createAllNodes(dataObj);  
+      g_idle_add(visuObjectRedraw, (gpointer)0);
+    }
 }
 
 void set_view(GtkWidget* button _U_, gpointer data _U_)
@@ -1313,44 +1388,6 @@ void set_view(GtkWidget* button _U_, gpointer data _U_)
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtkw_cone_phi_angle), phi);
 }
 
-/* i = 1 if called by a function which modified an element resource
-   i = 2 if called by a function which modified a global resource */
-void redraw(int i)
-{
-  VisuData *dataObj;
-  int id;
-  VisuElement *ele;
-  GList *tmpLst;
-
-  if(currentListElement == NULL)
-    return;
-
-  DBG_fprintf(stderr, "Gtk Spin : Updating rendering area.\n");
-
-  dataObj = toolPanelGet_visuData(TOOL_PANEL(PANEL_ELEMENTS));
-  if (!dataObj)
-    return;
-
-  if(i == 1)
-    {
-      tmpLst = currentListElement;
-      while (tmpLst)
-	{
-	  ele = (VisuElement*)tmpLst->data;
-	  id = rspin_createShapeSpin(dataObj, ele);
-	  ele->openGLIdentifier = id;
-	  tmpLst = g_list_next(tmpLst);
-	}
-    }
-
-  visuData_createAllNodes(dataObj);  
-  
-  if (i == 2)
-    g_signal_emit(VISU_INSTANCE, VISU_GET_CLASS(VISU_INSTANCE)->SpinAxes_signal_id,
-		  0 , dataObj, NULL);
-  g_idle_add(visuObjectRedraw, (gpointer)0);
-}
-
 static void sync_global_resources(GObject *object _U_, VisuData *dataObj _U_,
 				  gpointer data _U_)
 {
