@@ -45,6 +45,7 @@
 
 #include <string.h>
 
+#include <support.h>
 #include <visu_object.h>
 #include <openGLFunctions/view.h>
 #include <coreTools/toolColor.h>
@@ -52,6 +53,7 @@
 #include <extraGtkFunctions/gtk_lineObjectWidget.h>
 #include <extensions/axes.h>
 #include <extensions/scale.h>
+#include <extensions/legend.h>
 
 /* Sensitive widget in this subpanel. */
 static GtkWidget *panelAxes;
@@ -61,6 +63,7 @@ static GtkWidget *spinScaleLength;
 static GtkWidget *entryScale;
 static GtkWidget *originespin[3];
 static GtkWidget *orientationspin[3];
+static GtkWidget *checkLegend;
 
 static gboolean disableCallbacks;
 
@@ -81,6 +84,7 @@ static void scaleDistanceChanged(GtkSpinButton *spin,gpointer data);
 static void scaleOriginChanged(GtkSpinButton *spin,gpointer data);
 static void scaleOrientationChanged(GtkSpinButton *spin,gpointer data);
 static void onLegendChanged(GtkEntry *entry, gpointer data);
+static void onUseLegendChanged(GtkToggleButton *toggle, gpointer data);
 
 ToolPanel* panelAxes_init()
 {
@@ -108,15 +112,17 @@ static GtkWidget *createInteriorAxes()
   GtkWidget *table;
   GtkWidget *align;
   GtkObject *aju;
-  GtkTooltips *tooltips;
   float *xyz, *orientation;
   Scale *scale;
   gchar *lblXYZ[3] = {"X", "Y", "Z"};
   float xyzDefault[3] = {0.f, 0.f, 0.f};
   float orientationDefault[3] = {1.f, 0.f, 0.f};
   int i;
+#if GTK_MINOR_VERSION < 12
+  GtkTooltips *tooltips;
 
   tooltips = gtk_tooltips_new ();
+#endif
 
   vbox = gtk_vbox_new(FALSE, 0);
 
@@ -125,7 +131,7 @@ static GtkWidget *createInteriorAxes()
   /*************/
   lineAxes = lineObjectNew(_("Basis set"));
   align = gtk_alignment_new(0.5, 0., 1, 1);
-  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 25, 0, 0);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 12, 0, 0);
   gtk_container_add(GTK_CONTAINER(align), lineAxes);
   gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
   g_signal_connect(G_OBJECT(lineAxes), "use-changed",
@@ -137,6 +143,32 @@ static GtkWidget *createInteriorAxes()
   g_signal_connect(G_OBJECT(lineAxes), "stipple-changed",
 		   G_CALLBACK(onAxesStippleChanged), (gpointer)0);
 
+  /**************/
+  /* The legend */
+  /**************/
+  hbox = gtk_hbox_new(FALSE, 0);
+  align = gtk_alignment_new(0.5, 0., 1, 1);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 12, 0, 0);
+  gtk_container_add(GTK_CONTAINER(align), hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), align, FALSE, FALSE, 0);
+
+  /* The label. */
+  label = gtk_label_new(_("<b>Legend</b>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_widget_set_name(label, "label_head");
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+
+  /* The drawn checkbox. */
+  label = gtk_label_new(_("drawn:"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  checkLegend = gtk_check_button_new();
+  gtk_box_pack_start(GTK_BOX(hbox), checkLegend, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkLegend),
+			       extLegendGet_isOn());
+  g_signal_connect(G_OBJECT(checkLegend), "toggled",
+                   G_CALLBACK(onUseLegendChanged), (gpointer)0);
+
   /*************************/
   /* Adding scale widgets. */
   /*************************/
@@ -158,13 +190,13 @@ static GtkWidget *createInteriorAxes()
   if (scalesGet_scales())
     {
       scale = SCALE(scalesGet_scales()->data);
-      aju = gtk_adjustment_new(scaleGet_length(scale), 1., 1000., 1., 5., 5.);
+      aju = gtk_adjustment_new(scaleGet_length(scale), 1., 1000., 1., 5., 0.);
       xyz = scaleGet_origin(scale);
       orientation = scaleGet_orientation(scale);
     }
   else
     {
-      aju = gtk_adjustment_new(5., 1., 1000., 1., 5., 5.);
+      aju = gtk_adjustment_new(5., 1., 1000., 1., 5., 0.);
       xyz = xyzDefault;
       orientation = orientationDefault;
     }
@@ -178,9 +210,9 @@ static GtkWidget *createInteriorAxes()
   gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
   gtk_container_add(GTK_CONTAINER(align), label);
   entryScale = gtk_entry_new();
-  gtk_tooltips_set_tip(tooltips, entryScale,
+  gtk_widget_set_tooltip_text(entryScale,
 		       _("Use blank legend to print the default"
-			 " value with the distance."), NULL);
+			 " value with the distance."));
   gtk_box_pack_start(GTK_BOX(hbox), entryScale, TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT(entryScale), "activate",
 		   G_CALLBACK(onLegendChanged), (gpointer)0);
@@ -213,13 +245,13 @@ static GtkWidget *createInteriorAxes()
       gtk_table_attach(GTK_TABLE(table), gtk_label_new(lblXYZ[i]), 1 + i, 2 + i, 0, 1,
 		       (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
 
-      aju = gtk_adjustment_new(xyz[i], -10., 999.,1.,5.,5.);
+      aju = gtk_adjustment_new(xyz[i], -10., 999., 1., 5., 0.);
       originespin[i] = gtk_spin_button_new(GTK_ADJUSTMENT(aju), 0, 2);
       g_signal_connect(G_OBJECT(originespin[i]), "value-changed",
 		       G_CALLBACK(scaleOriginChanged), GINT_TO_POINTER(i));
       gtk_table_attach(GTK_TABLE(table), originespin[i], 1 + i, 2 + i, 1, 2,
 		       (GtkAttachOptions)0, (GtkAttachOptions)0, 2, 0);
-      aju = gtk_adjustment_new(orientation[i], -10., 999.,1.,5.,5.);
+      aju = gtk_adjustment_new(orientation[i], -10., 999., 1., 5., 0.);
       orientationspin[i] = gtk_spin_button_new(GTK_ADJUSTMENT(aju), 0, 2);
       g_signal_connect(G_OBJECT(orientationspin[i]), "value-changed",
 		       G_CALLBACK(scaleOrientationChanged), GINT_TO_POINTER(i));
@@ -331,6 +363,9 @@ static void onAxesEnter(ToolPanel *toolPanel _U_, gpointer data _U_)
 
   gtk_entry_set_text(GTK_ENTRY(entryScale), legend);
       
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkLegend),
+			       extLegendGet_isOn());
+
   disableCallbacks = FALSE;
 }
 
@@ -512,3 +547,21 @@ static void onLegendChanged(GtkEntry *entry, gpointer data _U_)
   if (res)
     scaleRedraw();
 }
+static void onUseLegendChanged(GtkToggleButton *toggle, gpointer data _U_)
+{
+  VisuData *dataObj;
+
+  if (disableCallbacks)
+    return;
+
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(panelAxes));
+
+  if (extLegendSet_isOn(gtk_toggle_button_get_active(toggle)))
+    {
+      if (!dataObj) 
+	return;
+
+      extLegendDraw(dataObj);
+    }
+  g_idle_add(visuObjectRedraw, (gpointer)0);
+}
