@@ -509,6 +509,7 @@ static void onVisuDataChanged(GObject *obj _U_, VisuData* visuData, gpointer dat
   float transBox[3];
   int i;
   VisuData *dataRef;
+  gboolean res;
   
   DBG_fprintf(stderr, "Panel Box: Catch 'dataLoaded' signal, update.\n");
 
@@ -537,8 +538,9 @@ static void onVisuDataChanged(GObject *obj _U_, VisuData* visuData, gpointer dat
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkDiff)) &&
       dataRef)
     {
-      geometryDiff(dataRef, visuData);
-      geometryRebuild_list(visuData);
+      res = geometryDiff(dataRef, visuData);
+      if (res)
+	geometryRebuild_list(visuData);
     }
 
   g_signal_connect(G_OBJECT(visuData), "ElementRenderedChanged",
