@@ -46,6 +46,7 @@
 
 #include <visu_data.h>
 #include <extraGtkFunctions/gtk_toolPanelWidget.h>
+#include <coreTools/toolShade.h>
 
 /**
  * panelDataFileInit:
@@ -82,6 +83,7 @@ gboolean panelDataFileLoad_file(VisuData *visuData, gchar *file, gboolean *new);
  * Set if the panel is used or not.
  */
 void panelDataFileSet_use(gboolean used);
+gboolean panelDataFileSet_presetShade(Shade *shade);
 
 /**
  * panelDataFileUpdate:
