@@ -98,7 +98,7 @@ struct GtkRenderingInterface_struct
 {
 /*   setSensitiveFunc sensitiveInterface; */
   panelElements_changeFunc newElementInterface;
-  createGtkWidgetFunc createInterface;
+  VisuGtkWidgetFunc createInterface;
 };
 typedef struct GtkRenderingInterface_struct GtkRenderingInterface;
 GHashTable *listOfRenderingInterfaces;
@@ -119,19 +119,19 @@ void changeRenderingMethodInterface(GtkWidget* interface);
    user defined values. */
 void setElementChangedFunc(panelElements_changeFunc func);
 /* It reads the value of the interface methods. */
-void getRenderingInterface(RenderingMethod *method,
-			   panelElements_changeFunc *change,
-			   createGtkWidgetFunc *create);
+static void getRenderingInterface(VisuRendering *method,
+				  panelElements_changeFunc *change,
+				  VisuGtkWidgetFunc *create);
 
 /* Callbacks */
-guint idRefreshComboElements;
+static guint idRefreshComboElements;
 static void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data);
 static void resourcesChanged(GObject *object, VisuData *dataObj, gpointer data);
 void colorVisuElementChanged(ColorComboBox *colorWd, Color *color, gpointer userData);
 static void checkRenderedChanged(GtkToggleButton *button, gpointer data);
 static void checkMaskedChanged(GtkToggleButton *button, gpointer data);
-void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData);
-static void onRenderingMethodChanged(GObject *obj, RenderingMethod *method,
+static void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData);
+static void onRenderingMethodChanged(GObject *obj, VisuRendering *method,
 				     gpointer userData);
 static void onColorChanged(ColorComboBox *colorComboBox, guint colorId, gpointer data);
 static void onMaterialChanged(ColorComboBox *colorComboBox, Material mat, gpointer data);
@@ -173,9 +173,11 @@ void createInteriorElements(GtkWidget *toolPanel)
   GtkWidget *align;
   GtkWidget *scrollView;
   GtkCellRenderer *renderer;
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips;
 
   tooltips = gtk_tooltips_new ();
+#endif
 
   /* We create the structure that store the VisuElements */
   listStoreVisuElements = gtk_list_store_new (N_COLUMNS,
@@ -228,9 +230,8 @@ void createInteriorElements(GtkWidget *toolPanel)
   gtk_container_add(GTK_CONTAINER(align), checkMasked);
   image = create_pixmap((GtkWidget*)0, "stock-masking.png");
   gtk_container_add(GTK_CONTAINER(checkMasked), image);
-  gtk_tooltips_set_tip(tooltips, checkMasked,
-		       _("Make nodes sensitive to the masking effect of planes."),
-		       NULL);
+  gtk_widget_set_tooltip_text(checkMasked,
+		              _("Make nodes sensitive to the masking effect of planes."));
 
   expand = colorComboBoxGet_rangesWidgets(COLOR_COMBOX(widgetVisuElementColor));
   gtk_box_pack_start(GTK_BOX(vBoxElements), expand, FALSE, FALSE, 0);
@@ -286,7 +287,7 @@ gint sortVisuElements(gpointer a, gpointer b)
 /*************/
 /* Callbacks */
 /*************/
-void refreshComboElements(GObject *obj _U_, VisuData *dataObj, gpointer userData _U_)
+static void refreshComboElements(GObject *obj _U_, VisuData *dataObj, gpointer userData _U_)
 {
   unsigned int i;
   GList *tmpLst, *currentListOfElements;
@@ -298,19 +299,19 @@ void refreshComboElements(GObject *obj _U_, VisuData *dataObj, gpointer userData
   VisuElement *prevSelectedElement;
   gboolean selected;
 
-  DBG_fprintf(stderr, "Panel Element : catch 'dataReadyForRendering' signal,"
+  DBG_fprintf(stderr, "Panel Element: catch 'dataReadyForRendering' signal,"
 	      " trying to rebuild element list.\n");
   if (dataObj)
     {
       if (visuDataGet_changeElementFlag(dataObj))
 	{
-	  DBG_fprintf(stderr,"Panel Element : not removing old elements"
+	  DBG_fprintf(stderr,"Panel Element: not removing old elements"
 		      " from the liststore of labels.\n");
 	  return;
 	}
     }
 
-  DBG_fprintf(stderr,"Panel Element : removing old elements"
+  DBG_fprintf(stderr,"Panel Element: removing old elements"
 	      " from the liststore of labels.\n");
 
   /* Get the selected iter. */
@@ -377,12 +378,12 @@ void refreshComboElements(GObject *obj _U_, VisuData *dataObj, gpointer userData
 
   DBG_fprintf(stderr,"Panel Element : Refresh OK.\n");
 }
-static void onRenderingMethodChanged(GObject *obj _U_, RenderingMethod *method,
+static void onRenderingMethodChanged(GObject *obj _U_, VisuRendering *method,
 				     gpointer userData _U_)
 {
 /*   setSensitiveFunc sensitive; */
   panelElements_changeFunc change;
-  createGtkWidgetFunc create;
+  VisuGtkWidgetFunc create;
   int sans;
 
   sans = 1;
@@ -393,7 +394,7 @@ static void onRenderingMethodChanged(GObject *obj _U_, RenderingMethod *method,
 	{
 	  sans = 0;
 	  DBG_fprintf(stderr,"Panel Element : set the gtk interface for"
-		      " the method '%s'.\n", method->name);
+		      " the method '%s'.\n", visuRenderingGet_name(method, FALSE));
 	  setElementChangedFunc(change);
 	  changeRenderingMethodInterface(create());
 	}
@@ -851,9 +852,9 @@ void changeRenderingMethodInterface(GtkWidget* interface)
 
 /* This method allows to initiate the method
    to deal with the interface. */
-void panelElements_setInterfaceMethods(RenderingMethod *method,
-				      panelElements_changeFunc change,
-				      createGtkWidgetFunc create)
+void panelElements_setInterfaceMethods(VisuRendering *method,
+				       panelElements_changeFunc change,
+				       VisuGtkWidgetFunc create)
 {
   GtkRenderingInterface *interface;
 
@@ -880,14 +881,14 @@ void panelElements_setInterfaceMethods(RenderingMethod *method,
     }
 }
 /* It reads the value of the interface methods. */
-void getRenderingInterface(RenderingMethod *method,
-			   panelElements_changeFunc *change,
-			   createGtkWidgetFunc *create)
+static void getRenderingInterface(VisuRendering *method,
+				  panelElements_changeFunc *change,
+				  VisuGtkWidgetFunc *create)
 {
   GtkRenderingInterface *interface;
 
   *change = (panelElements_changeFunc)0;
-  *create = (createGtkWidgetFunc)0;
+  *create = (VisuGtkWidgetFunc)0;
   if (!method)
     return;
 
