@@ -47,6 +47,7 @@
 #include <gtk/gtk.h>
 
 #include <gtk_main.h>
+#include <visu_gtk.h>
 #include <visu_rendering.h>
 #include <extraGtkFunctions/gtk_toolPanelWidget.h>
 
@@ -80,9 +81,9 @@ typedef void (*panelElements_changeFunc)(GList *eleList);
  * This method allows to initiate the methods to deal with the rendering
  * specific interfaces.
  */
-void panelElements_setInterfaceMethods(RenderingMethod *method,
+void panelElements_setInterfaceMethods(VisuRendering *method,
 				       panelElements_changeFunc change,
-				       createGtkWidgetFunc create);
+				       VisuGtkWidgetFunc create);
 
 #define PANEL_ELEMENTS panelElementsGet_static()
 GtkWidget* panelElementsGet_static();
