@@ -47,6 +47,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <support.h>
 #include <visu_tools.h>
 #include <visu_object.h>
 #include <visu_gtk.h>
@@ -119,6 +120,30 @@ ToolPanel* panelFogBgColor_init()
   return TOOL_PANEL(panelFogBgColor);
 }
 
+static void update_preview_cb(GtkFileChooser *file_chooser, gpointer data)
+{
+  GtkWidget *preview;
+  char *filename;
+  GdkPixbuf *pixbuf;
+  gboolean have_preview;
+
+  preview = GTK_WIDGET (data);
+  filename = gtk_file_chooser_get_preview_filename (file_chooser);
+
+  if (filename)
+    pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
+  else
+    pixbuf = (GdkPixbuf*)0;
+  have_preview = (pixbuf != NULL);
+  g_free (filename);
+
+  gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
+  if (pixbuf)
+    g_object_unref (pixbuf);
+
+  gtk_file_chooser_set_preview_widget_active(file_chooser, have_preview);
+}
+
 GtkWidget *createInteriorFogBgColor()
 {
   GtkWidget *vbox, *hbox;
@@ -131,17 +156,19 @@ GtkWidget *createInteriorFogBgColor()
   char *rgb[4];
   char *rgbName[4] = {"scroll_r", "scroll_g", "scroll_b", "scroll_a"};
   int i;
-  GtkWidget *dialog, *bt;
+  GtkWidget *dialog, *bt, *preview;
   GtkFileFilter *filters;
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips;
 
+  tooltips = gtk_tooltips_new ();
+#endif
+
   rgb[0] = RED_LABEL;
   rgb[1] = GREEN_LABEL;
   rgb[2] = BLUE_LABEL;
   rgb[3] = ALPHA_LABEL;
 
-  tooltips = gtk_tooltips_new ();
-
   vbox = gtk_vbox_new(FALSE, 0);
 
   table = gtk_table_new(5, 2, FALSE);
@@ -178,6 +205,11 @@ GtkWidget *createInteriorFogBgColor()
   filters = gtk_file_filter_new();
   gtk_file_filter_add_pixbuf_formats(filters);
   gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filters);
+  preview = gtk_image_new();
+  gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(dialog), preview);
+  gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(dialog), FALSE);
+  g_signal_connect(GTK_FILE_CHOOSER(dialog), "update-preview",
+		   G_CALLBACK(update_preview_cb), preview);
 #if GTK_MINOR_VERSION > 5
   bgImageWd = gtk_file_chooser_button_new_with_dialog(dialog);
 #if GTK_MINOR_VERSION > 11
@@ -194,8 +226,7 @@ GtkWidget *createInteriorFogBgColor()
 #endif
   gtk_box_pack_start(GTK_BOX(hbox), bgImageWd, TRUE, TRUE, 0);
   bt = gtk_button_new();
-  gtk_tooltips_set_tip(tooltips, bt,
-		       _("Remove the background image."), NULL);
+  gtk_widget_set_tooltip_text(bt, _("Remove the background image."));
   gtk_container_add(GTK_CONTAINER(bt),
 		    gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU));
   gtk_box_pack_start(GTK_BOX(hbox), bt, FALSE, FALSE, 5);
@@ -424,6 +455,7 @@ static void onBgImageSet(GtkFileChooserButton *filechooserbutton, gpointer data
 {
   gchar *filename;
 
+  DBG_fprintf(stderr, "Panel Fog & Bg: bg image changed.\n");
   filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filechooserbutton));
   if (!filename)
     return;
@@ -461,6 +493,7 @@ static gboolean loadBgFile(gpointer data)
   GError *error;
   GdkPixbuf *pixbuf;
   gchar *filename, *title;
+  gboolean fit;
 
   filename = (gchar*)data;
 
@@ -468,21 +501,28 @@ static gboolean loadBgFile(gpointer data)
 	      filename);
   error = (GError*)0;
   pixbuf = gdk_pixbuf_new_from_file(filename, &error);
-  title = g_path_get_basename(filename);
-  g_free(filename);
-
   if (!pixbuf)
     {
       visuGtkRaise_warning(_("Load image file"), error->message, (GtkWindow*)0);
       g_error_free(error);
-      g_free(title);
+      g_free(filename);
       return FALSE;
     }
+
+  fit = TRUE;
+  title = g_path_get_basename(filename);
+  g_free(filename);
+  if (!strcmp(title, "logo_grey.png"))
+    {
+      fit = FALSE;
+      g_free(title);
+      title = (gchar*)0;
+    }
   bgSet_image(gdk_pixbuf_get_pixels(pixbuf),
 	      gdk_pixbuf_get_width(pixbuf),
 	      gdk_pixbuf_get_height(pixbuf),
 	      gdk_pixbuf_get_has_alpha(pixbuf),
-	      title);
+	      title, fit);
   g_object_unref(pixbuf);
   g_free(title);
 
@@ -490,7 +530,7 @@ static gboolean loadBgFile(gpointer data)
 }
 static void onBgImageUnset(GtkButton *bt _U_, gpointer data)
 {
-  bgSet_image((guchar*)0, 0, 0, FALSE, (const gchar*)0);
+  bgSet_image((guchar*)0, 0, 0, FALSE, (const gchar*)0, TRUE);
 #if GTK_MINOR_VERSION > 5
   gtk_file_chooser_unselect_all(GTK_FILE_CHOOSER(data));
 #endif
@@ -500,6 +540,7 @@ void panelFogBgColorSet_bgImage(const gchar *filename)
 {
   gchar *path, *cur;
 
+  DBG_fprintf(stderr, "Panel Fog & Bg: set bg image to '%s'.\n", filename);
   if (!g_path_is_absolute(filename))
     {
       cur = g_get_current_dir();
@@ -512,7 +553,5 @@ void panelFogBgColorSet_bgImage(const gchar *filename)
   gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(bgImageWd), path);
 #endif
 
-#if GTK_MINOR_VERSION < 11
   loadBgFile(path);
-#endif
 }
