@@ -100,7 +100,7 @@ static gdouble directionDist;
 static gboolean disableCallbacks;
 
 /* Local callbacks. */
-void onPlaneUsed(GtkToggleButton *button, gpointer data);
+static void onPlaneUsed(GtkToggleButton *button, gpointer data);
 void onPlaneAdd(GtkButton *button, gpointer data);
 void onPlaneRemove(GtkButton *button, gpointer data);
 static void onSpinNVectChange(GtkSpinButton *spin, gpointer data);
@@ -717,14 +717,18 @@ Plane** panelPlanesGet_listOfAllPlanes(gboolean maskingPlanesOnly)
   GtkTreeIter iter;
   Plane *plane;
   int i;
+  gboolean skip;
   
   DBG_fprintf(stderr, "Panel Plane : building plane list...\n");
   /* Build the list of all planes. */
   planeList = g_malloc(sizeof(Plane*) *
 		       (gtk_tree_model_iter_n_children
 			(GTK_TREE_MODEL(listStoredPlanes), NULL) + 1));
+  skip = (maskingPlanesOnly &&
+	  (!checkUsePlanes ||
+	   !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkUsePlanes))));
   i = 0;
-  if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter))
+  if (!skip && gtk_tree_model_get_iter_first(GTK_TREE_MODEL(listStoredPlanes), &iter))
     do
       {
 	gtk_tree_model_get(GTK_TREE_MODEL(listStoredPlanes),
@@ -785,7 +789,7 @@ gboolean panelPlanesSet_use(VisuData *dataObj, gboolean value)
 
   return (gboolean)val;
 }
-void onPlaneUsed(GtkToggleButton *button, gpointer data _U_)
+static void onPlaneUsed(GtkToggleButton *button, gpointer data _U_)
 {
   VisuData *dataObj;
   gboolean res;
