@@ -50,6 +50,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <support.h>
 #include <visu_gtk.h>
 #include <visu_basic.h>
 #include <extraFunctions/pot2surf.h>
@@ -274,7 +275,7 @@ void surf_open_file_chooser()
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 				NULL);
-  gchar *directory = gtkMainClassGet_lastOpenDirectory();
+  gchar *directory = visuGtkGet_lastOpenDirectory();
 
 
   if (directory)
@@ -330,7 +331,7 @@ void surfmerge_target_file_chooser(GtkWidget *widget _U_, gpointer data _U_)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 				NULL);
-  gchar *directory = gtkMainClassGet_lastOpenDirectory();
+  gchar *directory = visuGtkGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -769,7 +770,9 @@ void surf_gogogo()
 GtkWidget *surf_file_manager()
 {
   enum { DXX, DYX, DYY, DZX, DZY, DZZ };
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips = gtk_tooltips_new ();
+#endif
   GtkWidget* scrolled_window_top = gtk_scrolled_window_new(NULL, NULL);
   GtkWidget* scrolled_window_bot = gtk_scrolled_window_new(NULL, NULL);
   GtkWidget *global_vbox = gtk_vbox_new(FALSE, 0);
@@ -821,24 +824,24 @@ GtkWidget *surf_file_manager()
   image_open = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
   gtk_container_add (GTK_CONTAINER (save_file), image_open);
 
-  gtk_tooltips_set_tip(tooltips, go_button, 
-		       _("Build specified new .surf file"), NULL);
-  gtk_tooltips_set_tip(tooltips, entry_current_file, 
-		       _("Contains the full path to the currently .surf selected file"), NULL);
-  gtk_tooltips_set_tip(tooltips, entry_target_file, 
-		       _("Contains the full path to the .surf file you want to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, open_file, 
-		       _("Allows you to select a .surf file"), NULL);
-  gtk_tooltips_set_tip(tooltips, save_file, 
-		       _("Selects the .surf file to write"), NULL);
-  gtk_tooltips_set_tip(tooltips, add_surf_button, 
-		       _("Moves selected surface to the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, move_down_surf_button, 
-		       _("Moves down selected surface in the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, move_up_surf_button, 
-		       _("Moves up selected surface in the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, remove_surf_button, 
-		       _("Removes selected surface from the list of surfaces to build"), NULL);
+  gtk_widget_set_tooltip_text(go_button, 
+		       _("Build specified new .surf file"));
+  gtk_widget_set_tooltip_text(entry_current_file, 
+		       _("Contains the full path to the currently .surf selected file"));
+  gtk_widget_set_tooltip_text(entry_target_file, 
+		       _("Contains the full path to the .surf file you want to build"));
+  gtk_widget_set_tooltip_text(open_file, 
+		       _("Allows you to select a .surf file"));
+  gtk_widget_set_tooltip_text(save_file, 
+		       _("Selects the .surf file to write"));
+  gtk_widget_set_tooltip_text(add_surf_button, 
+		       _("Moves selected surface to the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(move_down_surf_button, 
+		       _("Moves down selected surface in the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(move_up_surf_button, 
+		       _("Moves up selected surface in the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(remove_surf_button, 
+		       _("Removes selected surface from the list of surfaces to build"));
   
   for(i=0; i<6; i++)
     {
@@ -846,8 +849,8 @@ GtkWidget *surf_file_manager()
       gtk_entry_set_text(GTK_ENTRY(surfmerge_source_ddd[i]), "0.0000000");
       g_object_set(G_OBJECT(surfmerge_source_ddd[i]), "width-chars", 9, NULL);
       gtk_editable_set_editable(GTK_EDITABLE(surfmerge_source_ddd[i]), FALSE);
-      gtk_tooltips_set_tip(tooltips, surfmerge_source_ddd[i],
-			   _("The d__ of the current selected file"), NULL);
+      gtk_widget_set_tooltip_text(surfmerge_source_ddd[i],
+			   _("The d__ of the current selected file"));
     }
 
   for(i=0; i<6; i++)
@@ -855,8 +858,8 @@ GtkWidget *surf_file_manager()
       surfmerge_target_ddd[i] = gtk_entry_new();
       gtk_entry_set_text(GTK_ENTRY(surfmerge_target_ddd[i]), "0.0000000");
       g_object_set(G_OBJECT(surfmerge_target_ddd[i]), "width-chars", 9, NULL);
-      gtk_tooltips_set_tip(tooltips, surfmerge_target_ddd[i],
-			   _("The d__ of the file to build"), NULL);
+      gtk_widget_set_tooltip_text(surfmerge_target_ddd[i],
+			   _("The d__ of the file to build"));
     }
       
   surf_create_tree_views();
@@ -1128,7 +1131,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 /* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, */
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  gchar *directory = gtkMainClassGet_lastOpenDirectory();
+  gchar *directory = visuGtkGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -1170,7 +1173,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 	      g_string_free(other_files, TRUE);
 
 	      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-	      gtkMainClassSet_lastOpenDirectory(directory);
+	      visuGtkSet_lastOpenDirectory(directory, VISU_DIR_SURFACE);
 	      g_free(directory);
 	      gtk_list_store_clear(pot2surf_list_store);
 	      gtk_widget_destroy(file_chooser);
@@ -1182,7 +1185,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 	case GTK_RESPONSE_CANCEL:
 	default:
 	  directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-	  gtkMainClassSet_lastOpenDirectory(directory);
+	  visuGtkSet_lastOpenDirectory(directory, VISU_DIR_SURFACE);
 	  g_free(directory);
 	  gtk_widget_destroy (file_chooser);
 	  return;
@@ -1201,7 +1204,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 /* 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, */
 /* /\* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, *\/ */
 /* 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL); */
-/*   gchar *directory = gtkMainClassGet_lastOpenDirectory(); */
+/*   gchar *directory = visuGtkGet_lastOpenDirectory(); */
 
 /*   if (directory) */
 /*     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory); */
@@ -1223,7 +1226,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 /* 	    gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser)); */
 /* 	  gtk_entry_set_text(GTK_ENTRY(pot2surf_entry_source_instruc_file), selected_file); */
 /*       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)); */
-/*       gtkMainClassSet_lastOpenDirectory(directory); */
+/*       visuGtkSet_lastOpenDirectory(directory); */
 /*       g_free(directory); */
 /* 	  gtk_widget_destroy(file_chooser); */
 /* 	  return; */
@@ -1232,7 +1235,7 @@ void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 /* 	case GTK_RESPONSE_CANCEL: */
 /* 	default: */
 /*       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)); */
-/*       gtkMainClassSet_lastOpenDirectory(directory); */
+/*       visuGtkSet_lastOpenDirectory(directory); */
 /*       g_free(directory); */
 /* 	  gtk_widget_destroy (file_chooser); */
 /* 	  return; */
@@ -1251,7 +1254,7 @@ void pot2surf_select_surf_file(GtkWidget *widget _U_, gpointer data _U_)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 /* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, */
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  gchar *directory = gtkMainClassGet_lastOpenDirectory();
+  gchar *directory = visuGtkGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -1275,7 +1278,7 @@ void pot2surf_select_surf_file(GtkWidget *widget _U_, gpointer data _U_)
 	    gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
 	  gtk_entry_set_text(GTK_ENTRY(pot2surf_entry_target_surf_file), selected_file);
       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-      gtkMainClassSet_lastOpenDirectory(directory);
+      visuGtkSet_lastOpenDirectory(directory, VISU_DIR_SURFACE);
       g_free(directory);
 	  gtk_widget_destroy(file_chooser);
 	  return;
@@ -1284,7 +1287,7 @@ void pot2surf_select_surf_file(GtkWidget *widget _U_, gpointer data _U_)
 	case GTK_RESPONSE_CANCEL:
 	default:
       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-      gtkMainClassSet_lastOpenDirectory(directory);
+      visuGtkSet_lastOpenDirectory(directory, VISU_DIR_SURFACE);
       g_free(directory);
 	  gtk_widget_destroy (file_chooser);
 	  return;
@@ -1404,6 +1407,7 @@ void pot2surf_special_add_surf(GtkWidget *button _U_, gpointer data _U_)
   GtkTreeIter new_row;
   GtkTreeSelection* tree_selection = 
     gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));
+  GtkWidget *dialog;
 
   if(strcmp("", gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_source_pot_file))) == 0)
     return;
@@ -1413,9 +1417,13 @@ void pot2surf_special_add_surf(GtkWidget *button _U_, gpointer data _U_)
 
   values = (float*)0;
   name = (gchar*)0;
-  panelIsosurfacesGenerate_isoValues(&nbValues, &values, &name,
-				     pot_min, pot_max);
+  dialog = panelIsosurfacesGenerate_isoValues(&nbValues, &values, &name,
+					      pot_min, pot_max);
 
+  if (!dialog)
+    return;
+
+  gtk_widget_destroy(dialog);
   for (i = 0; i < nbValues; i++)
     {
       gtk_list_store_append(pot2surf_list_store, &new_row);
@@ -1563,7 +1571,7 @@ void pot2surf_select_instruc_file_to_save()
   GtkFileFilter *filter_instruc;
   GtkFileFilter *filter_none;
   GtkWidget* file_chooser;
-  char *directory = gtkMainClassGet_lastOpenDirectory();
+  char *directory = visuGtkGet_lastOpenDirectory();
   int n=0;
   const gchar *pot_file = gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_source_pot_file));
   const gchar *surf_file = gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_target_surf_file));
@@ -1620,7 +1628,7 @@ void pot2surf_select_instruc_file_to_save()
 					gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser)), n))
 	   
 	  break;
-	gtkMainClassSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
+	visuGtkSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)), VISU_DIR_SURFACE);
 	g_string_free(default_file, TRUE);
 	gtk_widget_destroy(file_chooser);
 	return;
@@ -1726,7 +1734,7 @@ gboolean pot2surf_build_surf_file(GtkWidget *button _U_, gpointer data)
     {
       if(panelIsosurfacesLoad_file(surf_file, 0,
 				   renderingWindowGet_visuData(window),
-				   (OptionTable*)0) == 0)
+				   (OptionTable*)0, (double**)0) == 0)
 	{
 	  for(i=0; i<n; i++)
 	    g_free(names[i]);
@@ -1904,7 +1912,7 @@ void pot2surf_select_instruc_file_to_load()
 				GTK_FILE_CHOOSER_ACTION_OPEN,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  char *directory = gtkMainClassGet_lastOpenDirectory();
+  char *directory = visuGtkGet_lastOpenDirectory();
 
   if(directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -1923,7 +1931,7 @@ void pot2surf_select_instruc_file_to_load()
       case GTK_RESPONSE_ACCEPT:
 	if(!pot2surf_load_instruc_file(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser))))
 	  break;
-	gtkMainClassSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
+	visuGtkSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)), VISU_DIR_SURFACE);
 	gtk_widget_destroy(file_chooser);
 	return;
       case GTK_RESPONSE_CANCEL:
@@ -1936,7 +1944,9 @@ void pot2surf_select_instruc_file_to_load()
 /* See header file for more info */
 GtkWidget* pot2surf_widget()
 {
+#if GTK_MINOR_VERSION < 12
   GtkTooltips *tooltips = gtk_tooltips_new ();
+#endif
 
   GtkToolItem *instruc_open = gtk_tool_button_new_from_stock(GTK_STOCK_OPEN);
   GtkToolItem *instruc_save = gtk_tool_button_new_from_stock(GTK_STOCK_SAVE_AS);
@@ -2001,38 +2011,38 @@ GtkWidget* pot2surf_widget()
   gtk_editable_set_editable(GTK_EDITABLE(pot2surf_entry_target_surf_file), FALSE);
 
   /* Widget properties edition */
-  gtk_tooltips_set_tip(tooltips, autoload_checkbox, 
-		       _("If checked, autoloads newly built file in V_Sim"), NULL);
-  gtk_tooltips_set_tip(tooltips, pot2surf_entry_source_pot_file, 
-		       _("Contains the full path to the currently .pot selected file"), NULL);
-  gtk_tooltips_set_tip(tooltips, pot2surf_entry_target_surf_file, 
-		       _("Contains the full path to the .surf file you want to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, pot2surf_potmin, 
-		       _("Contains the minimal value found in the selected .pot file"), NULL);
-  gtk_tooltips_set_tip(tooltips, pot2surf_potmax, 
-		       _("Contains the maximal value found in the selected .pot file"), NULL);
-  gtk_tooltips_set_tip(tooltips, build_surf_button, 
-		       _("Tries to build specified .surf file using specified parameters"), NULL);
-  gtk_tooltips_set_tip(tooltips, add_surf_button, 
-		       _("Adds a new surface to the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, special_add_button, 
-		       _("Adds many surfaces to the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, remove_surf_button, 
-		       _("Removes selected surface from the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, move_down_button, 
-		       _("Moves down selected surface in the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, move_up_button, 
-		       _("Moves up selected surface in the list of surfaces to build"), NULL);
-  gtk_tooltips_set_tip(tooltips, pot_open_button, 
-		       _("Selects the .pot file to use"), NULL);
-  gtk_tooltips_set_tip(tooltips, surf_open_button, 
-		       _("Selects the .surf file to write"), NULL);
-  gtk_tool_item_set_tooltip(instruc_open, tooltips, 
-			    _("Opens a .instruc file"), NULL);
-  gtk_tool_item_set_tooltip(instruc_save, tooltips, 
-			    _("Saves current settings as .instruc file"), NULL);
-  gtk_tool_item_set_tooltip(instruc_help, tooltips, 
-			    _("Show help"), NULL);
+  gtk_widget_set_tooltip_text(autoload_checkbox, 
+		       _("If checked, autoloads newly built file in V_Sim"));
+  gtk_widget_set_tooltip_text(pot2surf_entry_source_pot_file, 
+		       _("Contains the full path to the currently .pot selected file"));
+  gtk_widget_set_tooltip_text(pot2surf_entry_target_surf_file, 
+		       _("Contains the full path to the .surf file you want to build"));
+  gtk_widget_set_tooltip_text(pot2surf_potmin, 
+		       _("Contains the minimal value found in the selected .pot file"));
+  gtk_widget_set_tooltip_text(pot2surf_potmax, 
+		       _("Contains the maximal value found in the selected .pot file"));
+  gtk_widget_set_tooltip_text(build_surf_button, 
+		       _("Tries to build specified .surf file using specified parameters"));
+  gtk_widget_set_tooltip_text(add_surf_button, 
+		       _("Adds a new surface to the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(special_add_button, 
+		       _("Adds many surfaces to the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(remove_surf_button, 
+		       _("Removes selected surface from the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(move_down_button, 
+		       _("Moves down selected surface in the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(move_up_button, 
+		       _("Moves up selected surface in the list of surfaces to build"));
+  gtk_widget_set_tooltip_text(pot_open_button, 
+		       _("Selects the .pot file to use"));
+  gtk_widget_set_tooltip_text(surf_open_button, 
+		       _("Selects the .surf file to write"));
+  gtk_tool_item_set_tooltip_text(instruc_open,
+			    _("Opens a .instruc file"));
+  gtk_tool_item_set_tooltip_text(instruc_save,
+			    _("Saves current settings as .instruc file"));
+  gtk_tool_item_set_tooltip_text(instruc_help,
+			    _("Show help"));
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				 GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
