@@ -207,25 +207,6 @@ gpointer visuRenderingWindowGet_backgroundImage(GenericRenderingWindow window,
  */
 void visuRenderingWindowFree_backgroundImage(GenericRenderingWindow window, gpointer image);
 
-/**
- * raiseRenderWindow:
- * @window: a pointer on a valid rendering window object (see #RenderingWindow
- *          if GTK is used for rendering).
- *
- * Ask the window manager to raise the given window.
- */
-void raiseRenderWindow(GenericRenderingWindow window);
-
-/**
- * rename_window:
- * @window: a pointer on a valid rendering window object (see #RenderingWindow
- *          if GTK is used for rendering).
- * @nom: a string, null terminated.
- *
- * Change the nam of the given window.
- */
-void rename_window(GenericRenderingWindow window, char *nom);
-
 /**
  * visuRenderingWindowGet_current:
  *
@@ -234,5 +215,12 @@ void rename_window(GenericRenderingWindow window, char *nom);
  * Returns: a pointer to the current active window.
  */
 GenericRenderingWindow visuRenderingWindowGet_current();
+/**
+ * visuRenderingWindowSet_default:
+ * @window: a pointer to a rendering area to become default.
+ *
+ * Method used to set the window returned by visuRenderingWindowGet_current().
+ */
+void visuRenderingWindowSet_default(GenericRenderingWindow window);
 
 #endif
