@@ -109,6 +109,7 @@ static gboolean loadXyz(VisuData *data, const gchar* filename,
     return FALSE;
 
   res = read_Xyz_File(data, readFrom, nSet, error);
+  DBG_fprintf(stderr, "Atomic XYZ: parse return %d.\n", res);
   status = g_io_channel_shutdown(readFrom, TRUE, (GError**)0);
   g_io_channel_unref(readFrom);
   if (status != G_IO_STATUS_NORMAL)
@@ -211,7 +212,11 @@ static int read_Xyz_File(VisuData *data, GIOChannel *flux, int nSet, GError **er
       DBG_fprintf(stderr, "Atomic xyz: read node set number %d.\n", nSets);
       /*The Number Of Nodes*/
       nNodes=0;
-      if ( sscanf(line->str, "%d", &nNodes) != 1 ) 
+      res = sscanf(line->str, "%d", &nNodes);
+      if (res != 1 && nSets > nSet)
+	break;
+
+      if ( res != 1 ) 
 	{ 
 	  *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
 			       _("Wrong XYZ format, no number on first line.\n"));
@@ -324,11 +329,22 @@ static int read_Xyz_File(VisuData *data, GIOChannel *flux, int nSet, GError **er
 	    {
 	      nbcolumn = sscanf(line->str, "%s %lf %lf %lf %lf %lf %lf",
 				nomloc, &xu, &yu, &zu, &dx, &dy, &dz);
-	      if ( nbcolumn !=4  && nbcolumn != 7) 
+	      if (nbcolumn < 4)
 		{
 		  DBG_fprintf(stderr, "Atomic XYZ: can't read line values.\n");
 		  *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-				       _("Wrong XYZ format, 'Atom X Y Z' awaited.\n"));
+				       _("Wrong XYZ format, 'Atom X Y Z' awaited."));
+		}
+	      else if ((dcoord && nbcolumn != 7) ||
+		       (!dcoord && iNodes > 0 && nbcolumn == 7))
+		{
+		  DBG_fprintf(stderr, "Atomic XYZ: can't read vibration values.\n");
+		  *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
+				       _("Wrong XYZ + vibration format,"
+					 " 'Atom X Y Z vx vy vz' awaited."));
+		}
+	      if (*error)
+		{
 		  g_string_free(line, TRUE);
 		  g_hash_table_destroy(elements);
 		  if (nSets == nSet)
@@ -347,7 +363,7 @@ static int read_Xyz_File(VisuData *data, GIOChannel *flux, int nSet, GError **er
 	      coords[iNodes * 3 + 0] = xu;
 	      coords[iNodes * 3 + 1] = yu;
 	      coords[iNodes * 3 + 2] = zu;
-	      if ( nbcolumn == 7 )
+	      if ( nbcolumn == 7)
 		{
 		  if (!dcoord)
 		    dcoord = g_malloc(sizeof(float) * 3 * nNodes);
