@@ -117,7 +117,7 @@ static gboolean readFile_is_comment(char *str, gboolean *reduced, gboolean *angd
   if ((reduced || angdeg || unit || bc || qpt) &&
       !strncmp(str + 1, "keyword:", 8))
     {
-      tokens = g_strsplit_set(str + 9, " ,\n\t", -1);
+      tokens = g_strsplit_set(str + 9, " ,\n\t\r", -1);
       for (i = 0; tokens[i]; i++)
 	{
 	  DBG_fprintf(stderr, "Atomic ASCII: get keyword '%s'.\n", tokens[i]);
