@@ -68,16 +68,6 @@ struct dataAscii
   int nbNodes;
 };
 
-/**
- * readFile_is_comment:
- * @str: a null terminated string.
- *
- * Lines beginning with a '#' are treated as commentaries.
- *
- * Returns: 1 if the line is a comment.
- */
-int readFile_is_comment(char *str);
-
 /**
  * freeDataAscii:
  * @data: a pointer to #stuct dataAscii instances.
