@@ -43,19 +43,14 @@
 	knowledge of the CeCILL license and that you accept its terms. You can
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <GL/gl.h>
 #include <GL/glu.h> 
 
 #include "renderingSpin.h"
-#include "renderingAtomic.h" /* To get the position file format
-				(and posibly in the future, the read
-				method for position) */
-#include "renderingAtomic_ascii.h"
-#include "renderingAtomic_d3.h"
-#include "atomic_xyz.h"
+#include "renderingAtomic.h"
 #include <visu_basic.h>
 #include <visu_commandLine.h>
 #include <visu_configFile.h>
@@ -74,22 +69,7 @@
 #define FLAG_RESOURCES_SPIN "spin_resources"
 #define DESC_RESOURCES_SPIN "Global or element resource for rendering spin module"
 
-/**
- * pointerOnRenderingSpinMethod:
- *
- * This is a pointer to this rendering method.
- * It should be considered read-only.
- */
-static RenderingMethod *pointerOnRenderingSpinMethod;
-
-/* An identifier for the different shapes to draw elements. */
-enum
-  {
-    shapeArrow1 = 0,
-    shapeArrow2,
-    shapeSpinElipsoid,
-    nb_of_shapes
-  };
+static VisuRendering *spinMethod = (VisuRendering*)0;
 
 /* Parameters for each VisuElement. */
 struct spinResources_struct
@@ -105,13 +85,13 @@ struct spinResources_struct
   gboolean use_element_color, use_element_color_hat;
 
   /* Params for the elipsoid shapes. */
-  /* The radius and the length of the hat. */
+  /* The long and the short axis. */
   float aAxis, bAxis;
   /* The coloring pattern. */
   gboolean elipsoidColor;
 
   /* The shape used. */
-  int shape;
+  SpinArrowShape shape;
 
   /* An id of the list associated with the form. */
   int openGLIdentifier;
@@ -128,10 +108,10 @@ static GType spinElementResourcesTypes[spin_nbElementResources] =
 #define SPIN_ELEMENT_TAIL_LENGTH_DEFAULT 0.8
 #define SPIN_ELEMENT_HAT_COLOR_DEFAULT   FALSE
 #define SPIN_ELEMENT_TAIL_COLOR_DEFAULT  FALSE
-#define SPIN_ELEMENT_AAXIS_DEFAULT       2.0
-#define SPIN_ELEMENT_BAXIS_DEFAULT       0.8
+#define SPIN_ELEMENT_AAXIS_DEFAULT       1.5
+#define SPIN_ELEMENT_BAXIS_DEFAULT       0.6
 #define SPIN_ELEMENT_ELIP_COLOR_DEFAULT  FALSE
-#define SPIN_ELEMENT_SHAPE_DEFAULT       shapeArrow1
+#define SPIN_ELEMENT_SHAPE_DEFAULT       arrow_smooth
 
 /* Global resources for spin rendering and associated flag. */
 static GType spinGlobalResourcesTypes[spin_nbGlobalResources] =
@@ -149,10 +129,11 @@ static GList *allSpinMethods;
 
 /* Names as they appear in the gtk combo box when selecting an element shape. */
 /* tab[i] element matches the above enum i-th element. */
-const char* shapeNameSpin[nb_of_shapes + 1] = {"Rounded", 
-					       "Edged", 
-					       "Elipsoid",
-					       (const char*)0};
+const char* shapeNameSpin[arrow_shape_nb + 1] = {"Rounded", 
+						 "Edged", 
+						 "Elipsoid",
+						 "Torus",
+						 (const char*)0};
 const char** shapeNameSpin_UTF8;
 
 const char* policyNameSpin[policyNbModes + 1] = {"always", 
@@ -171,18 +152,19 @@ static gboolean loadSpin(VisuData *data, FileFormat *format,
 			 int nSet, GError **error);
 static gboolean readSpinResources(gchar **lines, int nbLines,
 				  int position, VisuData *dataObj, GError **error);
-gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
-				      VisuData *dataObj);
-static void onRenderingChanged(GObject *visu, RenderingMethod *method,
+static void exportResourcesRenderingSpin(GString *data, VisuData *dataObj);
+static void onRenderingChanged(GObject *visu, VisuRendering *method,
 			       gpointer data);
+static void onRenderingUpdated(VisuRendering *method, gpointer data);
 static struct spinResources_struct* getSpinResources(VisuElement *ele);
 static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node, gpointer data);
 static void freeSpin(gpointer obj, gpointer data);
 static gpointer newOrCopySpin(gconstpointer obj, gpointer data);
+static float getSize(VisuElement *ele);
 
 /* Local loading methods. */
-static const char* rspin_shape_number_to_name(int n);
-static int rspin_shape_name_to_number(const char *name);
+static const char* rspin_shape_number_to_name(SpinArrowShape n);
+static SpinArrowShape rspin_shape_name_to_number(const char *name);
 static RenderingFormatLoad* spinAsciiInit();
 static RenderingFormatLoad* spinBinaryInit();
 static gboolean read_spin_file(VisuData *data, const char* fileName,
@@ -191,35 +173,28 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
 				 FileFormat *format, int nSet, GError **error);
 
 
-int rspin_get_number_of_shapes()
-{
-  return nb_of_shapes;
-}
-
-static int rspin_shape_name_to_number(const char *name)
+static SpinArrowShape rspin_shape_name_to_number(const char *name)
 {
   int i=0;
-  if(name == NULL) 
-    return -1;
 
-  for(i=0; i<nb_of_shapes; i++)
+  g_return_val_if_fail(name, -1);
+
+  for(i=0; i < arrow_shape_nb; i++)
     if(strcmp(name, shapeNameSpin[i]) == 0)
       return i;
 
   return -1;
 }
 
-static const char* rspin_shape_number_to_name(int n)
+static const char* rspin_shape_number_to_name(SpinArrowShape n)
 {
-  if(n>=nb_of_shapes && n < 0)
-    return NULL;
+  g_return_val_if_fail(n < arrow_shape_nb, (const gchar*)0);
   
   return shapeNameSpin[n];
 }
-const char* rspin_shape_number_to_translated_name(int n)
+const char* rspin_shape_number_to_translated_name(SpinArrowShape n)
 {
-  if(n>=nb_of_shapes && n < 0)
-    return NULL;
+  g_return_val_if_fail(n < arrow_shape_nb, (const gchar*)0);
   
   return shapeNameSpin_UTF8[n];
 }
@@ -254,20 +229,12 @@ const char* rspin_hiding_number_to_translated_name(int n)
   return policyNameSpin_UTF8[n];
 }
 
-float returnOneF()
-{
-  return 1.0f;
-}
-
 /*****************************************************************************/
 /*                                                                           */
 /*****************************************************************************/
-/* This function is responsible for binding the module with the core. A module NEED */
-/* a function of this kind, and it has to be added in the externalRenderingMethods.h file */
-/* for the program to know it exists. */ 
-RenderingMethod* initSpin()
+void rspinInit()
 {
-  RenderingMethod *spin = NULL;
+  VisuRendering *spin = NULL, *atomic;
   /* Name & desc that appear when selecting the "input method" in the config tab. */
   char *methodName = _("Spin visualisation");
   char *methodDescr = _("It draws arrows at given positions to represent an atom "
@@ -291,13 +258,13 @@ RenderingMethod* initSpin()
 
   /* We check that the atomic rendering has been already defined, to use its own
      loader and file formats. */
-  if (!renderingAtomicGet_static())
+  atomic = visuRenderingClassGet_methodByName(VISU_RENDERING_ATOMIC);
+  if (!atomic)
     {
       g_error("Can't initialize the RenderingSpin method,"
 	      " renderingAtomic must be initialized first. Aborting...\n");
     }
-  formatListPos = renderingMethodGet_fileType(renderingAtomicGet_static(),
-					      FILE_KIND_POSITION);
+  formatListPos = visuRenderingGet_fileType(atomic, FILE_KIND_POSITION);
 
   /* We create the list of spin file formats. */
   formatListSpin = (GList*)0;
@@ -308,7 +275,7 @@ RenderingMethod* initSpin()
   meth = spinBinaryInit();
   if (meth)
     allSpinMethods = g_list_prepend(allSpinMethods, meth);
-  allSpinMethods = g_list_sort(allSpinMethods, renderingMethodCompare_priority);
+  allSpinMethods = g_list_sort(allSpinMethods, visuRenderingFormatCompare_priority);
   tmpLst = allSpinMethods;
   while (tmpLst)
     {
@@ -317,20 +284,18 @@ RenderingMethod* initSpin()
       tmpLst = g_list_next(tmpLst);
     }
   
-  /* Calling the constructor to create a new renderingMethod. */
-  spin = renderingMethod_new("Spin visualisation", methodName, methodDescr,
-			     2, loadSpin);
-  renderingMethodSet_fileType(spin, formatListPos, FILE_KIND_POSITION, _("Position files"));
-  renderingMethodSet_fileType(spin, formatListSpin, FILE_KIND_SPIN, _("Spin files"));
+  /* Calling the constructor to create a new visuRendering. */
+  spin = visuRendering_new(VISU_RENDERING_SPIN, methodName, methodDescr,
+			   2, loadSpin, rspin_createShapeSpin,
+			   rspin_placeNodeSpin, getSize);
+  visuRenderingSet_fileType(spin, formatListPos, FILE_KIND_POSITION, _("Position files"));
+  visuRenderingSet_fileType(spin, formatListSpin, FILE_KIND_SPIN, _("Spin files"));
 
   /* Setting up the icon. */
   iconPath = g_build_filename(V_SIM_PIXMAPS_DIR, "stock_spin.png", NULL);
-  renderingMethodSet_icon(spin, iconPath);
+  visuRenderingSet_icon(spin, iconPath);
   g_free(iconPath);
 
-  /* Binding this new method with OpenGL drawing funcs. */
-  setOpenGLMethods(spin, rspin_createShapeSpin, rspin_placeNodeSpin, returnOneF);
-
   /* Ok now it's time to create and register this module as a new one using the variables
      we initialised above. */
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
@@ -342,10 +307,11 @@ RenderingMethod* initSpin()
 				   exportResourcesRenderingSpin);
   
 
-  shapeNameSpin_UTF8 = g_malloc(sizeof(gchar*) * nb_of_shapes);
+  shapeNameSpin_UTF8 = g_malloc(sizeof(gchar*) * arrow_shape_nb);
   shapeNameSpin_UTF8[0] = _("Rounded arrow");
   shapeNameSpin_UTF8[1] = _("Edged arrow");
   shapeNameSpin_UTF8[2] = _("Elipsoid");
+  shapeNameSpin_UTF8[3] = _("Torus");
 
 
   /* Initialise the OpenGl part by reserving space in the display list to draw objects. */
@@ -358,7 +324,7 @@ RenderingMethod* initSpin()
   colorWheel             = 0.;
 
   /* We set a general pointer to retrieve this method later. */
-  pointerOnRenderingSpinMethod = spin;
+  spinMethod = spin;
 
   /* We initialise the container GValue for the spin property. */
   g_value_init(&spinValue, G_TYPE_POINTER);
@@ -367,13 +333,13 @@ RenderingMethod* initSpin()
      when spin is current or not. */
   g_signal_connect(VISU_INSTANCE, "renderingChanged",
 		   G_CALLBACK(onRenderingChanged), (gpointer)spin);
+  g_signal_connect(G_OBJECT(atomic), "fileTypeChanged",
+		   G_CALLBACK(onRenderingUpdated), (gpointer)spin);
 
   /* Register a new NodeData. */
   dataNode = nodeDataNew(SPINVALUES_ID, G_TYPE_FLOAT);
   nodeDataSet_label(dataNode, _("Spin (\316\270, \317\206, mod.)"));
   nodeDataSet_callback(dataNode, onSpinParametersChanged, (gpointer)0);
-
-  return spin;
 }
 
 static RenderingFormatLoad* spinAsciiInit()
@@ -390,7 +356,7 @@ static RenderingFormatLoad* spinAsciiInit()
     {
       g_error("Can't initialize the rendering spin method, aborting...\n");
     }
-  meth->priority = 50;
+  meth->priority = 100;
   meth->load = read_spin_file;
 
   return meth;
@@ -416,14 +382,22 @@ static RenderingFormatLoad* spinBinaryInit()
   return meth;
 }
 
-static void onRenderingChanged(GObject *visu _U_, RenderingMethod *method,
+static void onRenderingChanged(GObject *visu _U_, VisuRendering *method,
 			       gpointer data)
 {
-  if (method == (RenderingMethod*)data)
+  if (method == (VisuRendering*)data)
     visuElementSet_updateNodesOnMaterialChange();
   else
     visuElementUnset_updateNodesOnMaterialChange();
 }
+static void onRenderingUpdated(VisuRendering *atomic, gpointer data)
+{
+  DBG_fprintf(stderr, "Rendering Spin: update the atomic methods.\n");
+
+  visuRenderingSet_fileType((VisuRendering*)data,
+			    visuRenderingGet_fileType(atomic, 0),
+			    FILE_KIND_POSITION, _("Position files"));
+}
 
 
 /*****************************************************************************/
@@ -442,6 +416,9 @@ gboolean rspin_load(VisuData *data, FileFormat *format _U_, int nSet, GError **e
   gboolean loadOk;
   GList *tmpLst;
   FileFormat *fmt;
+#if GLIB_MINOR_VERSION > 5
+  struct stat buf;
+#endif
 
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
 
@@ -456,13 +433,22 @@ gboolean rspin_load(VisuData *data, FileFormat *format _U_, int nSet, GError **e
       return FALSE;
     }
   
-  if (g_file_test(file, G_FILE_TEST_IS_DIR))
+  if (!g_file_test(file, G_FILE_TEST_IS_REGULAR))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("The specified file is a directory indeed. Can't load"
-			     " a directory!\n"));
+			   _("The specified file is"
+			     " not a regular file."));
       return FALSE;
     }
+#if GLIB_MINOR_VERSION > 5
+  if (!g_stat(file, &buf) && buf.st_size == 0)
+    {
+      *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
+			   _("The specified file is"
+			     " an empty file."));
+      return FALSE;
+    }
+#endif
   
   loadOk = FALSE;
   tmpLst = allSpinMethods;
@@ -560,8 +546,8 @@ static gboolean read_spin_file(VisuData *data, const char* fileName,
    
   /* Create a storage for max values of spin modulus for each element. */
   svgMaxSpinModulus = g_malloc(sizeof(float) * data->ntype);
-  visuDataSet_propertyWithDestroyFunc(data, SPINMAXMODULUS_ID,
-				      (gpointer)svgMaxSpinModulus, g_free);
+  g_object_set_data_full(G_OBJECT(data), SPINMAXMODULUS_ID,
+			 (gpointer)svgMaxSpinModulus, g_free);
   spin = visuNodeNew_pointerProperty(visuDataGet_nodeArray(data), SPINVALUES_ID,
 				     freeSpin, newOrCopySpin, (gpointer)0);
   
@@ -638,7 +624,7 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
     }
 
   /* Try to the number of spins. */
-  valid = toolFortranRead_integer(&nspins, 1, readFrom, error, endian, TRUE);
+  valid = toolFortranRead_integer(&nspins, 1, readFrom, error, endian, TRUE, TRUE);
   if (!valid)
     {
       fclose(readFrom);
@@ -657,7 +643,7 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
   
   spins = g_malloc(sizeof(double) * 3 * nspins);
   /* Read module. */
-  valid = toolFortranRead_double(spins, nspins, readFrom, error, endian, TRUE);
+  valid = toolFortranRead_double(spins, nspins, readFrom, error, endian, TRUE, TRUE);
   if (!valid)
     {
       g_free(spins);
@@ -665,7 +651,7 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
       return TRUE;
     }
   /* Read theta. */
-  valid = toolFortranRead_double(spins + nspins, nspins, readFrom, error, endian, TRUE);
+  valid = toolFortranRead_double(spins + nspins, nspins, readFrom, error, endian, TRUE, TRUE);
   if (!valid)
     {
       g_free(spins);
@@ -673,7 +659,7 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
       return TRUE;
     }
   /* Read phi. */
-  valid = toolFortranRead_double(spins + 2 * nspins, nspins, readFrom, error, endian, TRUE);
+  valid = toolFortranRead_double(spins + 2 * nspins, nspins, readFrom, error, endian, TRUE, TRUE);
   if (!valid)
     {
       g_free(spins);
@@ -685,8 +671,8 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
 
   /* Create a storage for max values of spin modulus for each element. */
   svgMaxSpinModulus = g_malloc(sizeof(float) * data->ntype);
-  visuDataSet_propertyWithDestroyFunc(data, SPINMAXMODULUS_ID,
-				      (gpointer)svgMaxSpinModulus, g_free);
+  g_object_set_data_full(G_OBJECT(data), SPINMAXMODULUS_ID,
+			 (gpointer)svgMaxSpinModulus, g_free);
   spin = visuNodeNew_pointerProperty(visuDataGet_nodeArray(data), SPINVALUES_ID,
 				     freeSpin, newOrCopySpin, (gpointer)0);
 
@@ -711,13 +697,15 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
 void rspin_addLoadMethod(RenderingFormatLoad* meth)
 {
   g_return_if_fail(meth && meth->load);
+  g_return_if_fail(spinMethod);
 
-  DBG_fprintf(stderr, "Rendering Spin : adding a new loading method '%s'.\n", meth->name);
+  DBG_fprintf(stderr, "Rendering Spin: adding a new loading method '%s'.\n",
+	      meth->name);
   allSpinMethods = g_list_prepend(allSpinMethods, meth);
-  allSpinMethods = g_list_sort(allSpinMethods, renderingMethodCompare_priority);
+  allSpinMethods = g_list_sort(allSpinMethods, visuRenderingFormatCompare_priority);
   if (meth->fmt)
-    renderingMethodAdd_fileFormat(pointerOnRenderingSpinMethod,
-				  meth->fmt, FILE_KIND_SPIN);
+    visuRenderingAdd_fileFormat(spinMethod,
+				meth->fmt, FILE_KIND_SPIN);
 }
 
 static gboolean loadSpin(VisuData *data, FileFormat *format, int nSet, GError **error)
@@ -727,7 +715,7 @@ static gboolean loadSpin(VisuData *data, FileFormat *format, int nSet, GError **
   if (!data)
     return 0;
   
-  DBG_fprintf(stderr, "Rendering Spin : starting loadSpin procedure...\n");
+  DBG_fprintf(stderr, "Rendering Spin: starting loadSpin procedure...\n");
 
   if(!renderingAtomicLoad(data, format, nSet, error))
     return FALSE;
@@ -873,6 +861,18 @@ gfloat rspin_getGlobalResource_float(SpinGlobalResources property)
 /********************************/
 /* Access to element resources. */
 /********************************/
+static float getSize(VisuElement *ele)
+{
+  struct spinResources_struct *str;
+  
+  g_return_val_if_fail(ele, 1.f);
+
+  str = getSpinResources(ele);
+  if (str->shape == arrow_smooth || str->shape == arrow_sharp)
+    return str->height + str->u_height;
+  else
+    return MAX(str->aAxis, str->bAxis);
+}
 static struct spinResources_struct* getSpinResources(VisuElement *ele)
 {
   struct spinResources_struct *str;
@@ -1398,20 +1398,7 @@ static gboolean readSpinResources(gchar **lines, int nbLines,
   return TRUE;
 }
 
-/**
- * exportResourcesRenderingSpin:
- * @data: an empty GString to store the export ;
- * @nbLinesWritten: a pointer to store the number of lines written.
- *
- * This function is called by the core whenever it's needed to export
- * resources. It's in charge of exporting ALL the spin module resources.
- * First it writes global resources in the given @io structure and then
- * writes each element resources using @write_element_resources
- *
- * Returns: TRUE if everything goes right, FALSE otherwise.
- */
-gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
-				      VisuData* dataObj)
+static void exportResourcesRenderingSpin(GString *data, VisuData* dataObj)
 {
   GList *pos, *eleList;
   unsigned int i;
@@ -1420,34 +1407,28 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 
   /* If dataObj is given and the rendering method is not spin,
      we return. */
-  if (dataObj && pointerOnRenderingSpinMethod != getRenderingMethodInUse())
-    return TRUE;
+  if (dataObj && visuRenderingClassIs_currentByName(VISU_RENDERING_SPIN))
+    return;
 
   DBG_fprintf(stderr, "Rendering Spin: exporting element resources...\n");
 
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_SPIN);
-  *nbLinesWritten = 1;
 
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
   g_string_append_printf(data, "   %s %f %f\n", FLAG_SPIN_CONE_ANGLE,
 			 coneOrientation[0], coneOrientation[1]);
-  *nbLinesWritten += 2;
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
   g_string_append_printf(data, "   %s %f\n", FLAG_SPIN_WHEEL_ANGLE,
 			 colorWheel);
-  *nbLinesWritten += 2;
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
   g_string_append_printf(data, "   %s %s\n", FLAG_SPIN_HIDING_MODE,
 			 policyNameSpin[spinPolicy]);
-  *nbLinesWritten += 2;
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
   g_string_append_printf(data, "   %s %d\n", FLAG_SPIN_AND_ATOMIC,
 			 spinAndAtomicRendering);
-  *nbLinesWritten += 2;
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
   g_string_append_printf(data, "   %s %d\n", FLAG_SPIN_MODULUS,
 			 spinModulusUsage);
-  *nbLinesWritten += 2;
 
   /* We create a list of elements, or get the whole list. */
   eleList = (GList*)0;
@@ -1468,7 +1449,6 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 	  g_string_append_printf(data, "%s:\n", FLAG_RESOURCES_SPIN);
 	  g_string_append_printf(data, "   %s %s %s\n", FLAG_ELEMENT_SHAPE,
 				 ele->name, rspin_shape_number_to_name(str->shape));
-	  *nbLinesWritten += 2;
 	}
       if (ABS(str->height - SPIN_ELEMENT_HAT_LENGTH_DEFAULT) > 1e-6 ||
 	  ABS(str->u_height - SPIN_ELEMENT_TAIL_LENGTH_DEFAULT) > 1e-6 ||
@@ -1482,7 +1462,6 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 				 FLAG_ELEMENT_ARROW, ele->name, 
 				 str->height, str->u_height, str->length, str->u_length,
 				 str->use_element_color, str->use_element_color_hat);
-	  *nbLinesWritten += 2;
 	}
       if (ABS(str->aAxis - SPIN_ELEMENT_AAXIS_DEFAULT) > 1e-6 ||
 	  ABS(str->bAxis - SPIN_ELEMENT_BAXIS_DEFAULT) > 1e-6 ||
@@ -1492,18 +1471,15 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 	  g_string_append_printf(data, "   %s %s %f %f %d\n",
 				 FLAG_ELEMENT_ELIPSOID, ele->name, 
 				 str->aAxis, str->bAxis, str->elipsoidColor);
-	  *nbLinesWritten += 2;
 	}
       pos = g_list_next(pos);
     }
 
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 1;
   if (eleList)
     g_list_free(eleList);
 
   DBG_fprintf(stderr, "Rendering Spin: element resources succesfully exported\n");
-  return TRUE;
 }
 
 static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node,
@@ -1527,97 +1503,6 @@ static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node,
 /*                                                                           */
 /*****************************************************************************/
 
-static void drawRoundedHat(GLUquadricObj *obj, float hat_length,
-			   float hat_height, float radius, int nlat)
-{
-  gluCylinder(obj, hat_length, 0, hat_height, nlat, 1);
-  glRotatef(180, 1, 0, 0);
-  gluDisk(obj, 0, radius, nlat, 1);
-  glRotatef(180, 1, 0, 0);
-}
-static void drawRoundedTail(GLUquadricObj *obj, float tail_length,
-			    float tail_height, int nlat)
-{
-  glTranslatef(0, 0, -tail_height);
-  glRotatef(180, 1, 0, 0);
-  gluDisk(obj, 0, (tail_length)*1.03, nlat, 1);      
-  glRotatef(180, 1, 0, 0);
-  gluCylinder(obj, tail_length, tail_length, tail_height, nlat, 1);
-}
-static void drawEdgedHat(float hat_length, float hat_height)
-{
-  glBegin(GL_TRIANGLE_FAN);
-      
-  glNormal3f(hat_height, 0, hat_length);
-  glVertex3f(0, 0, hat_height); 
-  glVertex3f(hat_length,-hat_length,0);
-  glVertex3f(hat_length,hat_length,0);
-
-  glNormal3f(0, hat_height, hat_length);
-  glVertex3f(-hat_length,hat_length,0);
-       
-  glNormal3f(-hat_height, 0, hat_length);
-  glVertex3f(-hat_length,-hat_length,0);
-
-  glNormal3f(-hat_height, 0, hat_length);
-  glVertex3f(hat_length,-hat_length,0);
-
-  glEnd();
-
-  glBegin(GL_QUADS);
-
-  glNormal3f(0., 0., -1.);
-  glVertex3f(hat_length ,hat_length ,0 );
-  glVertex3f(hat_length ,-hat_length ,0 );
-  glVertex3f(-hat_length ,-hat_length ,0 );
-  glVertex3f(-hat_length ,hat_length ,0 );
-
-  glEnd();
-
-}
-static void drawEdgedTail(float tail_length, float tail_height)
-{
-  glBegin(GL_QUADS);
-
-  glNormal3f(0., 0., 1.);
-  glVertex3f(tail_length , tail_length , 0 );
-  glVertex3f(-tail_length , tail_length , 0 );
-  glVertex3f(-tail_length , -tail_length , 0 );
-  glVertex3f(tail_length , -tail_length , 0 );
-
-  glNormal3f(0., 0., -1.);
-  glVertex3f(tail_length ,tail_length ,-tail_height );
-  glVertex3f(tail_length ,-tail_length ,-tail_height );
-  glVertex3f(-tail_length ,-tail_length ,-tail_height );
-  glVertex3f(-tail_length ,tail_length ,-tail_height );
-
-  glNormal3f(1., 0., 0.);
-  glVertex3f(tail_length ,tail_length ,0 );
-  glVertex3f(tail_length ,-tail_length ,0 );
-  glVertex3f(tail_length ,-tail_length ,-tail_height );
-  glVertex3f(tail_length ,tail_length ,-tail_height );
-
-  glNormal3f(-1., 0., 0.);
-  glVertex3f(-tail_length ,tail_length ,0 );
-  glVertex3f(-tail_length ,tail_length ,-tail_height );
-  glVertex3f(-tail_length ,-tail_length ,-tail_height );
-  glVertex3f(-tail_length ,-tail_length ,0 );
-
-  glNormal3f(0., 1., 0.);
-  glVertex3f(-tail_length ,tail_length ,-tail_height );
-  glVertex3f(-tail_length ,tail_length ,0 );
-  glVertex3f(tail_length ,tail_length ,0 );
-  glVertex3f(tail_length ,tail_length ,-tail_height );
-
-  glNormal3f(0., -1., 0.);
-  glVertex3f(-tail_length ,-tail_length ,-tail_height );
-  glVertex3f(tail_length ,-tail_length ,-tail_height );
-  glVertex3f(tail_length ,-tail_length ,0 );
-  glVertex3f(-tail_length ,-tail_length ,0 );
-
-  glEnd();
-}
-
 int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
 {
   int nlatl=0, nlatul=0, nlatoh=0, iele;
@@ -1649,10 +1534,10 @@ int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
   if (spinModulusUsage == policyNoneModulus)
     maxModulus = (float*)0;
   else if (spinModulusUsage == policyPerTypeModulus)
-    maxModulus = (float*)visuDataGet_property(visuData, SPINMAXMODULUS_ID);
+    maxModulus = (float*)g_object_get_data(G_OBJECT(visuData), SPINMAXMODULUS_ID);
   else if (spinModulusUsage == policyGlobalModulus)
     {
-      maxModulus = (float*)visuDataGet_property(visuData, SPINMAXMODULUS_ID);
+      maxModulus = (float*)g_object_get_data(G_OBJECT(visuData), SPINMAXMODULUS_ID);
       for (iele = 0; iele < (int)visuData->ntype; iele++)
 	globalMax = MAX(maxModulus[iele], globalMax);
     }
@@ -1674,89 +1559,39 @@ int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
     }
   switch (str->shape)
     {
-    case shapeArrow1:
-      if (spinAndAtomicRendering)
-	glTranslatef(0, 0, tailLength / 2.);
-      else
-	glTranslatef(0, 0, -(hatLength - tailLength)/2.);
-      if(str->use_element_color && str->use_element_color_hat)
-	{
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawRoundedHat(obj, hatRadius,
-			 hatLength, MAX(hatRadius, tailRadius)*1.03, nlatl);
-	  drawRoundedTail(obj, tailRadius, tailLength, nlatul);
-	}
-      else if(str->use_element_color)
-	{
-	  drawRoundedHat(obj, hatRadius, hatLength,
-			 MAX(hatRadius, tailRadius)*1.03, nlatl);
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawRoundedTail(obj, tailRadius, tailLength, nlatul);
-	}
-      else if(str->use_element_color_hat)
-	{
-	  drawRoundedTail(obj, tailRadius, tailLength, nlatul);
-	  glTranslatef(0, 0, tailLength);
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawRoundedHat(obj, hatRadius, hatLength,
-			 MAX(hatRadius, tailRadius)*1.03, nlatl);
-	}
-      else
-	{
-	  drawRoundedHat(obj, hatRadius, hatLength,
-			 MAX(hatRadius, tailRadius)*1.03, nlatl);
-	  drawRoundedTail(obj, tailRadius, tailLength, nlatul);
-	}
+    case arrow_smooth:
+      openGLObjectListDraw_smoothArrow(obj, visuElementGet_identifierMaterial(ele),
+				       spinAndAtomicRendering,
+				       tailLength, tailRadius, nlatul, str->use_element_color,
+				       hatLength, hatRadius, nlatl, str->use_element_color_hat);
       break;
-    case shapeArrow2:
-      /* This one is a edged arrow. */
-      if (spinAndAtomicRendering)
-	glTranslatef(0, 0, tailLength / 2.);
-      else
-	glTranslatef(0, 0, -(hatLength - tailLength)/2.);
-      if(str->use_element_color && str->use_element_color_hat)
-	{
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawEdgedHat(hatRadius, hatLength);
-	  drawEdgedTail(tailRadius, tailLength);
-	}
-      else if(str->use_element_color)
-	{
-	  drawEdgedHat(hatRadius, hatLength);
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawEdgedTail(tailRadius, tailLength);
-	}
-      else if(str->use_element_color_hat)
-	{
-	  drawEdgedTail(tailRadius, tailLength);
-	  glCallList(visuElementGet_identifierMaterial(ele));
-	  drawEdgedHat(hatRadius, hatLength);
-	}
-      else
-	{
-	  drawEdgedHat(hatRadius, hatLength);
-	  drawEdgedTail(tailRadius, tailLength);
-	}
+    case arrow_sharp:
+      openGLObjectListDraw_edgeArrow(visuElementGet_identifierMaterial(ele),
+				     spinAndAtomicRendering,
+				     tailLength, tailRadius, str->use_element_color,
+				     hatLength, hatRadius, str->use_element_color_hat);
       break;
-    case shapeElipsoid:
+    case arrow_ellipsoid:
       nlatl = OpenGLViewGet_numberOfFacettes(view, str->bAxis);
-      if (str->bAxis == 0.)
-	glScalef(1.0, 1.0, 10.);
-      else
-	glScalef(1.0, 1.0, str->aAxis / str->bAxis);
-      if (str->elipsoidColor)
-	glCallList(visuElementGet_identifierMaterial(ele));
-      gluSphere(obj, str->bAxis, nlatl, nlatl);
+      openGLObjectListDraw_ellipsoid(obj, visuElementGet_identifierMaterial(ele),
+				     str->aAxis, str->bAxis, nlatl,
+				     str->elipsoidColor);
+      break;
+    case arrow_torus:
+      nlatul = OpenGLViewGet_numberOfFacettes(view, str->aAxis);
+      nlatl = OpenGLViewGet_numberOfFacettes(view, str->bAxis);
+      openGLObjectListDraw_torus(obj, visuElementGet_identifierMaterial(ele),
+				 str->aAxis - str->bAxis, str->aAxis / str->bAxis,
+				 nlatul, nlatl, str->elipsoidColor);
       break;
     default:
-      DBG_fprintf(stderr, "RenderingSpin : ERROR while drawing objects... Aborting...\n");
-      exit(1);
+      g_warning("Unknown shape.");
       break;
     }
   glEndList();
-
+ 
   gluDeleteQuadric(obj);
-   
+  
 /*   ele->openGLIdentifier = identifierSpheresSpin + ele->typeNumber; */
   return str->openGLIdentifier;
 }
@@ -1777,7 +1612,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
   float rgba[4];
   float xyz[3];
 
-  float *spinValues;
+  float *spinValues, scale;
   
   float theta_prime;
   float phi_prime;
@@ -1800,6 +1635,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
   visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
 			    SPINVALUES_ID, &spinValue);
   spinValues = (float*)g_value_get_pointer(&spinValue);
+  scale = visuDataGet_nodeScalingFactor(visuData, node);
   if (spinValues && (spinValues[SPIN_MODULUS] != 0. ||
 		     spinPolicy == policyAlwaysSpin))
     {
@@ -1915,12 +1751,13 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
       if (visuData->setColor)
 	{
 	  visuData->setColor(visuData, rgba, ele, node);
-	  setOpenGlMaterial(ele->material, rgba);
+	  openGLSet_color(ele->material, rgba);
 	}
       else
-	setOpenGlMaterial(ele->material, mm);
+	openGLSet_color(ele->material, mm);
 
       /* We finaly put the spin shape. */
+      glScalef(scale, scale, scale);
       glCallList(ele->openGLIdentifier);
 
       glPopMatrix();
@@ -1936,6 +1773,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
   
 	  glTranslated(xyz[0], xyz[1], xyz[2]);
 
+	  glScalef(scale, scale, scale);
 	  glCallList(visuElementGet_identifierMaterial(ele));
 	  str = (struct spinResources_struct*)getSpinResources(ele);
 	  glCallList(str->openGLIdentifierAtomic);
@@ -1943,8 +1781,3 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
 	}
     }
 }
-
-RenderingMethod* renderingSpinGet_static()
-{
-  return pointerOnRenderingSpinMethod;
-}
