@@ -738,7 +738,11 @@ gboolean visuBasicApply_commandLine(VisuData *visuData, VisuOptionSet *set,
   /* expand argument */
   extension = commandLineGet_extension();
   if (extension)
-    visuData_replicate(visuData, extension, (gboolean*)0);
+    {
+      if (!translations)
+	visuData_constrainedInTheBox(visuData);
+      visuData_replicate(visuData, extension, (gboolean*)0);
+    }
 
   /* Post loading sequence, corresponding to all other arguments given to command line. */
   /* The shade option. */
@@ -1181,12 +1185,12 @@ static gchar* setDir(const gchar* const *sysDirs, const gchar *prefix,
   return dir;
 }
 
-void visuBasicSet_paths(const char *exeLocation _U_)
+void visuBasicSet_paths(const char *exeLocation)
 {
 #if SYSTEM_WIN32 == 1
   #define V_SIM_INI_FILE PACKAGE".ini"
   GIOChannel *iniFile;
-  gchar *iniPath;
+  gchar *iniPath, *tmp, *me, *prefix;
   gchar *buffer, *line;
   GIOStatus res;
   GError *err;
@@ -1199,10 +1203,37 @@ void visuBasicSet_paths(const char *exeLocation _U_)
   v_sim_plugins_dir = g_strdup(V_SIM_PLUGINS_DIR_DEFAULT);
   v_sim_locale_dir  = g_strdup(V_SIM_LOCALE_DIR_DEFAULT);
   iniPath = g_strdup(V_SIM_INI_FILE);
+  /* Try to find the INI file from cwd. */
+  prefix = (gchar*)0;
   iniFile = g_io_channel_new_file(iniPath, "r", (GError**)0);
+  if (iniFile)
+    prefix = g_get_current_dir();
+  /* Try to find the INI file from the name of the executable. */
   if (!iniFile)
     {
-      iniPath = g_build_filename("C:\\WINDOWS", V_SIM_INI_FILE, NULL);
+      g_free(iniPath);
+
+      if (g_file_test(exeLocation, G_FILE_TEST_IS_SYMLINK))
+	tmp = g_file_read_link(exeLocation, (GError**)0);
+      else
+	tmp = g_strdup(exeLocation);
+      me = normalize_path(tmp);
+      g_free(tmp);
+    
+      DBG_fprintf(stderr, "Visu Basic: running program is '%s'.\n", me);
+      /* From the location of the executable, we take the base dir. */
+      prefix = g_path_get_dirname(me);
+      g_free(me);
+
+      iniPath = g_build_filename(prefix, V_SIM_INI_FILE, NULL);
+      iniFile = g_io_channel_new_file(iniPath, "r", (GError**)0);
+    }  
+  /* Try to find the INI file in the Windows directory. */
+  if (!iniFile)
+    {
+      g_free(iniPath);
+      prefix = g_strdup("C:\\WINDOWS");
+      iniPath = g_build_filename(prefix, V_SIM_INI_FILE, NULL);
       iniFile = g_io_channel_new_file(iniPath, "r", (GError**)0);
     }
   if (iniFile)
@@ -1218,27 +1249,47 @@ void visuBasicSet_paths(const char *exeLocation _U_)
 	      if (!strcmp(g_strstrip(tokens[0]), "data_dir") && tokens[1])
 		{
 		  g_free(v_sim_data_dir);
-		  v_sim_data_dir = g_strdup(g_strstrip(tokens[1]));
+		  tmp = g_strstrip(tokens[1]);
+		  if (g_path_is_absolute(tmp))
+		    v_sim_data_dir = g_strdup(tmp);
+		  else
+		    v_sim_data_dir = g_build_filename(prefix, tmp, NULL);
 		}
 	      if (!strcmp(g_strstrip(tokens[0]), "legal_dir") && tokens[1])
 		{
 		  g_free(v_sim_legal_dir);
-		  v_sim_legal_dir = g_strdup(g_strstrip(tokens[1]));
+		  tmp = g_strstrip(tokens[1]);
+		  if (g_path_is_absolute(tmp))
+		    v_sim_legal_dir = g_strdup(tmp);
+		  else
+		    v_sim_legal_dir = g_build_filename(prefix, tmp, NULL);
 		}
 	      if (!strcmp(g_strstrip(tokens[0]), "pixmaps_dir") && tokens[1])
 		{
 		  g_free(v_sim_pixmaps_dir);
-		  v_sim_pixmaps_dir = g_strdup(g_strstrip(tokens[1]));
+		  tmp = g_strstrip(tokens[1]);
+		  if (g_path_is_absolute(tmp))
+		    v_sim_pixmaps_dir = g_strdup(tmp);
+		  else
+		    v_sim_pixmaps_dir = g_build_filename(prefix, tmp, NULL);
 		}
 	      if (!strcmp(g_strstrip(tokens[0]), "plugins_dir") && tokens[1])
 		{
 		  g_free(v_sim_plugins_dir);
-		  v_sim_plugins_dir = g_strdup(g_strstrip(tokens[1]));
+		  tmp = g_strstrip(tokens[1]);
+		  if (g_path_is_absolute(tmp))
+		    v_sim_plugins_dir = g_strdup(tmp);
+		  else
+		    v_sim_plugins_dir = g_build_filename(prefix, tmp, NULL);
 		}
 	      if (!strcmp(g_strstrip(tokens[0]), "locale_dir") && tokens[1])
 		{
 		  g_free(v_sim_locale_dir);
-		  v_sim_locale_dir = g_strdup(g_strstrip(tokens[1]));
+		  tmp = g_strstrip(tokens[1]);
+		  if (g_path_is_absolute(tmp))
+		    v_sim_locale_dir = g_strdup(tmp);
+		  else
+		    v_sim_locale_dir = g_build_filename(prefix, tmp, NULL);
 		}
 	      g_strfreev(tokens);
 	      g_free(line);
