@@ -49,6 +49,7 @@
 #include <glib.h>
 
 #include <coreTools/toolOptions.h>
+#include <coreTools/toolMatrix.h>
 
 /**
  * parseCommandLine:
@@ -63,6 +64,19 @@
  */
 int parseCommandLine(int argc, char **argv);
 
+/**
+ * commandLineExport:
+ * @filename: a path to a filename to create ;
+ * @error: a location to store a possible error.
+ *
+ * Export the known command line options to an XML file.
+ *
+ * Returns: TRUE on success.
+ *
+ * Since: 3.5
+ */
+gboolean commandLineExport(const gchar *filename, GError **error);
+
 /**
  * commandLineGet_ArgFilename:
  *
@@ -262,15 +276,15 @@ OptionTable* commandLineGet_options();
  *
  * Returns: TRUE if a coloured map is requested.
  */
-gboolean commandLineGet_coloredMap(int *planeId);
+int* commandLineGet_coloredMap();
 /**
  * commandLineGet_logScale:
  *
  * Retrieve if a log scale is required for various plots.
  *
- * Returns: TRUE if '--log-scale' has been given, default is FALSE.
+ * Returns: the logscale method.
  */
-gboolean commandLineGet_logScale();
+matrix_scaleflag commandLineGet_logScale();
 /**
  * commandLineGet_nIsoLines:
  *
@@ -288,5 +302,50 @@ guint commandLineGet_nIsoLines();
  * Returns: a string or NULL if option is not used.
  */
 gchar* commandLineGet_bgImage();
+/**
+ * commandLineGet_isoLinesColor:
+ *
+ * Retrieve the chosen colour for the iso-lines.
+ *
+ * Returns: an array of three floats or NULL if the colour is auto.
+ *
+ * Since: 3.5
+ */
+float* commandLineGet_isoLinesColor();
+/**
+ * commandLineGet_windowMode:
+ *
+ * Retrieve the windowing mode for V_Sim, 'classic' with the two
+ * rendering and command panel windows ; 'oneWindow' with a joined
+ * version of the two windows or 'renderOnly' with only the rendering window.
+ *
+ * Returns: one of the three strings.
+ *
+ * Since: 3.5
+ */
+gchar* commandLineGet_windowMode();
+/**
+ * commandLineGet_iSet:
+ *
+ * Retrieve the desired id for multi dataset file to render.
+ *
+ * Returns: 0 as default value.
+ *
+ * Since: 3.5
+ */
+guint commandLineGet_iSet();
+/**
+ * commandLineGet_valueFile:
+ *
+ * Retrieve the name of a possible value file, if any has been given.
+ *
+ * Returns: a path (absolute) or NULL if the option has not been
+ * passed. Data are static.
+ *
+ * Since: 3.5
+ */
+gchar* commandLineGet_valueFile();
+void commandLineFree_all();
+guint commandLineGet_mapPrecision();
 
 #endif
