@@ -105,8 +105,7 @@
 #define DEFAULT_RESOURCES_PATH ""
 static gboolean readResourcesPaths(gchar **lines, int nbLines, int position _U_,
 				   VisuData *dataObj _U_, GError **error _U_);
-static gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj);
+static void exportResourcesPaths(GString *data, VisuData *dataObj);
 
 
 struct _VisuConfigFileEntry
@@ -413,7 +412,7 @@ gboolean visuConfigFileLoad(int kind, const char* fileName,
 		{
 		  if (!message)
 		    message = g_string_new("");
-		  g_string_append_printf(message, "[%s]: %s\n",
+		  g_string_append_printf(message, "[%s]: %s",
 					 key, (*error)->message);
 		  g_error_free(*error);
 		  *error = (GError*)0;
@@ -447,7 +446,16 @@ gboolean visuConfigFileLoad(int kind, const char* fileName,
       return FALSE;
     }
   else
-    return TRUE;
+    {
+      if (kind == VISU_CONFIGFILE_RESOURCE)
+	{
+	  DBG_fprintf(stderr, "Gtk Save: emitting 'resourcesLoaded' signal.\n");
+	  g_signal_emit(VISU_INSTANCE, VISU_SIGNALS[RESOURCESLOADED_SIGNAL],
+			0, dataObj, NULL);
+	  DBG_fprintf(stderr, "Gtk Save: emission done.\n");
+	}
+      return TRUE;
+    }
 }
 
 /* Routine to export the resources (color and
@@ -455,11 +463,10 @@ gboolean visuConfigFileLoad(int kind, const char* fileName,
 gboolean visuConfigFileSave(int kind, const char* fileName, int *lines,
 			    VisuData *dataObj, GError **error)
 {
-  gchar *fileUTF8;
-  GString *exportString, *data, *message;
-  int nbLine, nb;
+  gchar *fileUTF8, *ptCh;
+  GString *exportString, *message;
+  int nbLine;
   GList *pos;
-  gboolean ok;
   int withErrors;
 
   g_return_val_if_fail(error && !*error, FALSE);
@@ -483,7 +490,6 @@ gboolean visuConfigFileSave(int kind, const char* fileName, int *lines,
   g_free(fileUTF8);
 
   withErrors = 0;
-  nbLine = 0;
   exportString = g_string_new("");
   if (kind == VISU_CONFIGFILE_RESOURCE)
     g_string_append_printf(exportString, 
@@ -492,99 +498,59 @@ gboolean visuConfigFileSave(int kind, const char* fileName, int *lines,
     g_string_append_printf(exportString, 
 			   PARAMETER_HEADER);
   g_string_append_printf(exportString, 
-	  VERSION_HEADER
-	  "\n"
-	  "#====================\n"
-	  "\n"
-	  "#WARNING: this file format is DIFFERENT from that for\n"
-	  "#standard v_sim version <= 2.x\n"
-	  "\n");
-  nbLine += 6;
-  g_string_append_printf(exportString, 
-	  "#Line beginning with a # are not parsed.\n"
-	  "\n");
-  nbLine += 2;
-  pos = (GList*)0;
+			 VERSION_HEADER
+			 "\n"
+			 "#====================\n"
+			 "\n"
+			 "#WARNING: this file format is DIFFERENT from that for\n"
+			 "#standard v_sim version <= 2.x\n"
+			 "\n"
+			 "#Line beginning with a # are not parsed.\n"
+			 "\n");
   if (kind == VISU_CONFIGFILE_RESOURCE)
-    {
-      g_string_append_printf(exportString, 
-			     "#The only \"useful\" lines must have the following contents\n"
-			     "#several two or more lines patterns:\n"
-			     "#resource_name:\n"
-			     "#values separeted by blank characters\n"
-			     "\n");
-      nbLine += 5;
-      g_string_append_printf(exportString, 
-			     "#The following resource names are valid :\n");
-      nbLine += 1;
-      pos = registeredResources;
-    }
-  else if (kind == VISU_CONFIGFILE_PARAMETER)
-    {
-      g_string_append_printf(exportString, 
-			     "#The only \"useful\" lines must have the following pattern:\n"
-			     "#parameter_name: value\n"
-			     "\n");
-      nbLine += 3;
-      g_string_append_printf(exportString, 
-			     "#The following parameter names are valid :\n");
-      nbLine += 1;
-      pos = registeredParameters;
-    }
-  while(pos)
-    {
-      g_string_append_printf(exportString, "# %s\n",
-			     ((VisuConfigFileEntry*)(pos->data))->key);
-      nbLine += 1;
-      pos = g_list_next(pos);
-    }
+    g_string_append_printf(exportString, 
+			   "#The only \"useful\" lines must have the following contents\n"
+			   "#several two or more lines patterns:\n"
+			   "#resource_name:\n"
+			   "#values separeted by blank characters\n"
+			   "\n"
+			   "#The following resource names are valid :\n");
+  else
+    g_string_append_printf(exportString, 
+			   "#The only \"useful\" lines must have the following pattern:\n"
+			   "#parameter_name: value\n"
+			   "\n"
+			   "#The following parameter names are valid :\n");
+  for (pos = (kind == VISU_CONFIGFILE_RESOURCE)?
+	 registeredResources:registeredParameters;
+       pos; pos = g_list_next(pos))
+    g_string_append_printf(exportString, "# %s\n",
+			   ((VisuConfigFileEntry*)(pos->data))->key);
   g_string_append_printf(exportString, "\n");
-  nbLine += 1;
-  pos = (GList*)0;
-  if (kind == VISU_CONFIGFILE_RESOURCE)
-    pos = exportResourcesList;
-  else if (kind == VISU_CONFIGFILE_PARAMETER)
-    pos = exportParametersList;
-  while(pos)
-    {
-      data = g_string_new("");
-      ok = ((struct writeFunc_struct*)(pos->data))->writeFunc(data, &nb, dataObj);
-      if (ok)
-	{
-	  g_string_append_printf(exportString, "%s", data->str);
-	  nbLine += nb;
-	}
-      else
-	{
-	  g_string_append_printf(message, _("Error on line %d.\n"), nbLine);
-	  withErrors = 1;
-	}
-      g_string_free(data, TRUE);
-      pos = g_list_next(pos);
-    }
 
-  if (!g_file_set_contents(fileName, exportString->str, -1, error))
-    {
-      g_string_free(exportString, TRUE);
-      g_string_free(message, TRUE);
-      return FALSE;
-    }
+  for (pos = (kind == VISU_CONFIGFILE_RESOURCE)?
+	 exportResourcesList:exportParametersList;
+       pos; pos = g_list_next(pos))
+    ((struct writeFunc_struct*)(pos->data))->writeFunc(exportString, dataObj);
+
+  nbLine = 0;
+  ptCh = exportString->str;
+  while ((ptCh = strchr(ptCh + 1, '\n')))
+    nbLine += 1;
+
+  DBG_fprintf(stderr, "Visu ConfigFile: export OK to string,"
+	      " preparing to write file.\n");
+
+  withErrors = (g_file_set_contents(fileName, exportString->str, -1, error));
   g_string_free(exportString, TRUE);
+  g_string_free(message, TRUE);
+
+  DBG_fprintf(stderr, "Visu ConfigFile: write %d lines.\n", nbLine);
 
   if (lines)
     *lines = nbLine;
   
-  if (withErrors)
-    {
-      g_error_new(G_FILE_ERROR, 0, message->str);
-      g_string_free(message, TRUE);
-      return FALSE;
-    }
-  else
-    {
-      g_string_free(message, TRUE);
-      return TRUE;
-    }
+  return withErrors;
 }
 
 gchar* getValidFileWithHeader(int mode, char* filename, char* header, GList **list)
@@ -806,7 +772,6 @@ int visuConfigFile_init()
   VisuConfigFileEntry *resourceEntry;
 
   DBG_fprintf(stderr, "Visu ConfigFile : initialization process ...");
-  configFileInit();
 
   visuConfigFile_entryList = g_hash_table_new_full(g_str_hash, g_str_equal,
 						   NULL, freeConfigEntry);
@@ -865,8 +830,7 @@ static gboolean readResourcesPaths(gchar **lines, int nbLines, int position _U_,
   g_strfreev(tokens);
   return TRUE;
 }
-static gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj _U_)
+static void exportResourcesPaths(GString *data, VisuData *dataObj _U_)
 {
   GList *pnt;
   
@@ -884,7 +848,4 @@ static gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
       pnt = g_list_next(pnt);
     }
   g_string_append_printf(data, "\n\n");
-  *nbLinesWritten = 3;
-
-  return TRUE;
 }
