@@ -94,7 +94,6 @@ typedef gboolean (*visuConfigFileReadFunc)(gchar **lines, int nbLines,
 /**
  * visuConfigFileExportFunc:
  * @data: an empty GString to store the export ;
- * @nbLinesWritten: a pointer to store the number of lines written ;
  * @dataObj: a #VisuData object.
  *
  * This prototype defines a method that is used to export some resources
@@ -102,11 +101,8 @@ typedef gboolean (*visuConfigFileReadFunc)(gchar **lines, int nbLines,
  * to be written. If the argument @dataObj is not null, only resources related
  * to the #VisuData object should be exported (for parameters files, @dataObj is
  * always NULL).
- *
- * Returns: TRUE if everything goes right, FALSE otherwise.
  */
-typedef gboolean (*visuConfigFileExportFunc)(GString *data, int *nbLinesWritten,
-					     VisuData* dataObj);
+typedef void (*visuConfigFileExportFunc)(GString *data, VisuData* dataObj);
 /**
  * foreachFuncExport_struct:
  * @data: the string where the values are exported to ;
@@ -120,7 +116,6 @@ typedef gboolean (*visuConfigFileExportFunc)(GString *data, int *nbLinesWritten,
 struct foreachFuncExport_struct
 {
   GString *data;
-  int *nbLinesWritten;
   VisuData *dataObj;
 };
 
