@@ -93,7 +93,8 @@ typedef struct _DumpType
   VisuDumpWriteFunc writeFunc;
 } DumpType;
 
-GQuark VISU_ERROR_DUMP;
+#define VISU_ERROR_DUMP visuDumpGet_quark()
+GQuark visuDumpGet_quark();
 /**
  * DumpErrorFlag:
  * @DUMP_ERROR_OPENGL: Error with OpenGL dumping.
