@@ -94,12 +94,13 @@
 /* This is a pointer to the last added VisuElement. */
 /* VisuElement *lastAddedVisuElement; */
 static int identifierMaterials;
+static guint idEle = 0;
 
 /* This hashtable contains a list of all different
    elements loaded in visu. ntype is _not_ the number of items
    in this hashtable. */
-static GHashTable *differentVisuElements;
-static GList *visuElement_list;
+static GHashTable *visuElements_table;
+static GList *visuElements_list;
 
 /* This int stores how many user functions have needs
    to recreate the nodes of an element when its material
@@ -130,46 +131,63 @@ static gboolean readElementProperties(gchar **lines, int nbLines, int position,
 
 
 /* These functions write all the element list to export there associated resources. */
-static gboolean exportResourcesRenderingBase(GString *data, int *nbLinesWritten,
-					     VisuData *dataObj);
-
-static void freeHashValue(gpointer data);
+static void exportResourcesRenderingBase(GString *data, VisuData *dataObj);
 
+static gboolean _add(VisuElement* element)
+{
+  DBG_fprintf(stderr, "Visu Elements: adding %s (%p).\n", element->name,
+	      (gpointer)element);
 
+  g_hash_table_insert(visuElements_table,
+		      (gpointer)element->name, (gpointer)element);
+  visuElements_list = g_list_append(visuElements_list, (gpointer)element);
+  return TRUE;
+}
 
 /* Add a VisuElement to the hashtable. */
 int visuElementAdd(VisuElement* element)
 {
-  int nbElements;
-
   if (element &&
-      !g_hash_table_lookup(differentVisuElements, (gpointer)element->name))
+      !g_hash_table_lookup(visuElements_table, (gpointer)element->name))
     {
-      DBG_fprintf(stderr, "Visu Elements : adding %s (%d).\n", element->name,
-		  GPOINTER_TO_INT(element));
-      nbElements = g_hash_table_size(differentVisuElements);
-      if (nbElements == NMAX_TP)
-	{
-	  g_warning("The system requires more than %d elements. This value is"
-		    " set at compilation time with the variable NMAX_TP. If you"
-		    " need more, ask the maintainer to raise this value.\n", NMAX_TP);
-	  return 1;
-	}
-      element->typeNumber = g_hash_table_size(differentVisuElements);
-      g_hash_table_insert(differentVisuElements, (gpointer)element->name, (gpointer)element);
-      visuElement_list = g_list_append(visuElement_list, (gpointer)element);
-/*       lastAddedVisuElement = element; */
+      if (!_add(element))
+	return 1;
     }
   return 0;
 }
 
 GList* visuElementGet_allElements()
 {
-  return visuElement_list;
+  return visuElements_list;
 }
 VisuElement* visuElementGet_fromName(char* name)
 {
-  return g_hash_table_lookup(differentVisuElements, (gpointer)name);
+  return g_hash_table_lookup(visuElements_table, (gpointer)name);
+}
+VisuElement* visuElementRetrieve_fromName(const gchar *name, gboolean *nw)
+{
+  VisuElement *ele;
+
+  if (nw)
+    *nw = FALSE;
+
+  ele = g_hash_table_lookup(visuElements_table, (gpointer)name);
+  if (ele)
+    return ele;
+
+  if (nw)
+    *nw = TRUE;
+
+  ele = visuElementNew_withName(name);
+  if (!ele)
+    {
+      g_warning("Cannot create a new type for '%s'.", name);
+      return (VisuElement*)0;
+    }
+  if (!_add(ele))
+    return (VisuElement*)0;
+
+  return ele;
 }
 
 
@@ -205,13 +223,10 @@ int visuElementSet_allColorValues(VisuElement* ele, float rgb[4], float material
 
 int visuElementSet_allRGBValues(VisuElement* ele, float rgb[4])
 {
-  if (!ele)
-    return 0;
+  g_return_val_if_fail(ele, 0);
 
-  DBG_fprintf(stderr, "Visu Element : set color (%f, %f, %f, %f)\n",
-	      ele->rgb[0], ele->rgb[1], ele->rgb[2], ele->rgb[3]);
-  DBG_fprintf(stderr, "                      -> (%f, %f, %f, %f).\n",
-	      rgb[0], rgb[1], rgb[2], rgb[3]);
+  DBG_fprintf(stderr, "Visu Element: '%s' set color (%f, %f, %f, %f)\n",
+	      ele->name, ele->rgb[0], ele->rgb[1], ele->rgb[2], ele->rgb[3]);
 
   if (ele->rgb[0] == rgb[0] &&
       ele->rgb[1] == rgb[1] &&
@@ -219,6 +234,9 @@ int visuElementSet_allRGBValues(VisuElement* ele, float rgb[4])
       ele->rgb[3] == rgb[3])
     return 0;
 
+  DBG_fprintf(stderr, "                      -> (%f, %f, %f, %f).\n",
+	      rgb[0], rgb[1], rgb[2], rgb[3]);
+
   ele->rgb[0] = rgb[0];
   ele->rgb[1] = rgb[1];
   ele->rgb[2] = rgb[2];
@@ -290,9 +308,13 @@ int visuElementSet_sensitiveToPlanes(VisuElement *element, gboolean status)
 {
   g_return_val_if_fail(element, 0);
 
+  DBG_fprintf(stderr, "Visu Element: '%s' set sensitiveToMaskingPlanes %d\n",
+	      element->name, element->sensitiveToMaskingPlanes);
+
   if (element->sensitiveToMaskingPlanes == status)
     return 0;
 
+  DBG_fprintf(stderr, " -> %d\n", status);
   element->sensitiveToMaskingPlanes = status;
   return 1;
 }
@@ -323,39 +345,57 @@ VisuElement *visuElementNew_withName(const char *key)
   VisuElement *ele;
   int i;
 
-  ele = g_malloc(sizeof(VisuElement));
-  ele->name = g_strdup(key);
-  ele->typeNumber = -1;
-  ele->materialIsUpToDate = FALSE;
-  ele->rendered = TRUE;
-  ele->sensitiveToMaskingPlanes = TRUE;
-  ele->showNodeInfos = TRUE;
+  idEle += 1;
+  if (idEle == NMAX_TP)
+    {
+      g_warning("The system requires more than %d elements. This value is"
+		" set at compilation time with the variable NMAX_TP. If you"
+		" need more, ask the maintainer to raise this value.\n", NMAX_TP);
+      return (VisuElement*)0;
+    }
 
-  ele->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
-					  NULL, g_free);
+  ele = g_malloc(sizeof(VisuElement));
+  ele->name                     = g_strdup(key);
+  ele->typeNumber               = idEle;
+  ele->materialIsUpToDate       = FALSE;
+  ele->rendered                 = TRUE;
+  ele->sensitiveToMaskingPlanes = TRUE;
+  ele->showNodeInfos            = TRUE;
+  ele->properties               =
+    g_hash_table_new_full(g_str_hash, g_str_equal, NULL, g_free);
   g_return_val_if_fail(ele->properties, (VisuElement*)0);
 
   for (i = 0; i < 4; i++)
     ele->rgb[i] = 1.;
-
   for (i = 0; i < 5; i++)
     ele->material[i] = 0.25;
-  DBG_fprintf(stderr, "Visu Elements : creating a new VisuElement '%s' -> %d.\n",
-	      key, GPOINTER_TO_INT(ele));
+
+  DBG_fprintf(stderr, "Visu Elements: create a new VisuElement '%s' -> %p.\n",
+	      key, (gpointer)ele);
   return ele;
 }
+void visuElementFree(VisuElement *ele)
+{
+  DBG_fprintf(stderr, "Visu Elements: free VisuElement '%s' -> %p.\n",
+	      ele->name, (gpointer)ele);
+  g_free(ele->name);
+  g_hash_table_destroy(ele->properties);
+  g_hash_table_steal(visuElements_table, ele);
+  visuElements_list = g_list_remove(visuElements_list, ele);
+  g_free(ele);
+}
 
 void visuElement_createMaterial(VisuElement *ele)
 {
   g_return_if_fail(ele);
 
-  DBG_fprintf(stderr, "Visu Elements: creating material for %s (ele id %d - OpenGL id %d)\n", ele->name, ele->typeNumber, identifierMaterials + ele->typeNumber);
+  DBG_fprintf(stderr, "Visu Element: create material for %s (ele id %d - OpenGL id %d)\n", ele->name, ele->typeNumber, identifierMaterials + ele->typeNumber);
 /*   fprintf(stderr, "%f %f %f - %f %f %f %f %f\n", ele->rgb[0], ele->rgb[1], ele->rgb[2], ele->material[0], ele->material[1], ele->material[2], ele->material[3], ele->material[4]); */
 
   glDeleteLists(identifierMaterials + ele->typeNumber, 1);
   glNewList(identifierMaterials + ele->typeNumber, GL_COMPILE);
   glColor4fv(ele->rgb);
-  setOpenGlMaterial(ele->material, ele->rgb);
+  openGLSet_color(ele->material, ele->rgb);
   glEndList();
   ele->materialIsUpToDate = TRUE;
 }
@@ -394,12 +434,12 @@ int initVisuElements()
 {
   VisuConfigFileEntry *resourceEntry, *oldEntry;
 
-  differentVisuElements =
+  visuElements_table =
     g_hash_table_new_full(g_str_hash, g_str_equal,
-			  g_free, (GDestroyNotify)freeHashValue);
-  g_return_val_if_fail(differentVisuElements, 0);
+			  NULL, (GDestroyNotify)visuElementFree);
+  g_return_val_if_fail(visuElements_table, 0);
 
-  visuElement_list = (GList*)0;
+  visuElements_list = (GList*)0;
 /*   lastAddedVisuElement = (VisuElement*)0; */
 
   /* Create a VisuModule to registered the new resources as
@@ -436,6 +476,13 @@ int initVisuElements()
 
   return 1;
 }
+void visuElementFree_all()
+{
+  DBG_fprintf(stderr, "Visu Element: free all elements.\n - the hashtable\n");
+  g_hash_table_destroy(visuElements_table);
+  DBG_fprintf(stderr, " - the list\n");
+  g_list_free(visuElements_list);
+}
 
 /******************/
 /* Resources part */
@@ -500,14 +547,12 @@ static gboolean readRendered(gchar **lines, int nbLines, int position,
 }
 
 /* These functions read all the element list to export there associated resources. */
-gboolean exportResourcesRenderingBase(GString *data, int *nbLinesWritten,
-				      VisuData *dataObj)
+static void exportResourcesRenderingBase(GString *data, VisuData *dataObj)
 {
   GList *pos, *eleList;
   unsigned int i;
 
   g_string_append_printf(data, "# %s\n", DESC_ELEMENT_COLOR);
-  *nbLinesWritten = 1;
   /* We create a list of elements, or get the whole list. */
   eleList = (GList*)0;
   if (dataObj)
@@ -533,11 +578,9 @@ gboolean exportResourcesRenderingBase(GString *data, int *nbLinesWritten,
 			     ((VisuElement*)pos->data)->material[2],
 			     ((VisuElement*)pos->data)->material[3],
 			     ((VisuElement*)pos->data)->material[4]);
-      *nbLinesWritten += 2;
       pos = g_list_next(pos);
     }
   g_string_append_printf(data, "# %s\n", DESC_ELEMENT_PROPERTIES);
-  *nbLinesWritten = 1;
   if (dataObj)
     pos = eleList;
   else
@@ -549,24 +592,13 @@ gboolean exportResourcesRenderingBase(GString *data, int *nbLinesWritten,
 			     ((VisuElement*)pos->data)->name,
 			     ((VisuElement*)pos->data)->rendered,
 			     ((VisuElement*)pos->data)->sensitiveToMaskingPlanes);
-      *nbLinesWritten += 2;
       pos = g_list_next(pos);
     }
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 1;
   if (eleList)
     g_list_free(eleList);
-
-  return TRUE;
 }
 
-static void freeHashValue(gpointer data)
-{
-  g_free(((VisuElement*)data)->name);
-  g_free((VisuElement*)data);
-}
-
-
 void visuElementSet_property(VisuElement* element, char* key, gpointer value)
 {
   g_return_if_fail(element && key);
@@ -619,7 +651,7 @@ void debug_VisuElement(gpointer key, gpointer value, gpointer data _U_)
 /* Debug : print all informations about all the stored VisuElements. */
 void debug_AllVisuElements()
 {
-  fprintf(stderr,"DEBUG info : %d elements available.\n", g_hash_table_size(differentVisuElements));
-  g_hash_table_foreach(differentVisuElements, (GHFunc)debug_VisuElement, (gpointer)0);
+  fprintf(stderr,"DEBUG info : %d elements available.\n", g_hash_table_size(visuElements_table));
+  g_hash_table_foreach(visuElements_table, (GHFunc)debug_VisuElement, (gpointer)0);
 }
 
