@@ -93,8 +93,7 @@
 #define DESC_PARAMETER_MODE "Rules the way OpenGl draws extensions (see opengl_render); name (string) value (string)"
 static gboolean readExtensionRendering(gchar **lines, int nbLines, int position,
 				       VisuData *dataObj, GError **error);
-static gboolean exportParametersRendering(GString *data, int *nbLinesWritten,
-					  VisuData *dataObj);
+static void exportParametersRendering(GString *data, VisuData *dataObj);
 
 /* A GHashTable to store all the available OpenGL extensions
    in the system. The keys are the name of each extension. */
@@ -259,7 +258,7 @@ void OpenGLExtensionRemove(OpenGLExtension *extension)
 					    (gpointer)extension);
 }
 
-void OpenGLExtensionCall_list(const char *name)
+void OpenGLExtensionCall_list(const char *name, gboolean lastOnly)
 {
   OpenGLExtension *ext;
   RenderingModeId renderingMode, globalRenderingMode;
@@ -271,7 +270,8 @@ void OpenGLExtensionCall_list(const char *name)
   ext = OpenGLExtensionGet_fromName(name);
   g_return_if_fail(ext);
 
-  if (ext->used)
+  if (ext->used && ((lastOnly && ext->priority == OPENGL_EXTENSION_PRIORITY_LAST) ||
+		    (!lastOnly && ext->priority < OPENGL_EXTENSION_PRIORITY_LAST)))
     {
       callList(ext, &renderingMode, globalRenderingMode);
       if (renderingMode != globalRenderingMode)
@@ -280,24 +280,44 @@ void OpenGLExtensionCall_list(const char *name)
     }
 }
 /* For each extension that has a valid list (id > 1000) a glCallList is raised. */
-void callAllExtensionsLists()
+void OpenGLExtensionCall_allLists()
 {
   GList *pnt;
   RenderingModeId renderingMode, globalRenderingMode;
+  OpenGLExtension *ext;
 
   if (OpenGLExtension_reorderingNeeded)
     {
-      DBG_fprintf(stderr, "Visu Extension : sorting known extension depending on their priority.\n");
+      DBG_fprintf(stderr, "Visu Extension: sorting known extension depending on their priority.\n");
       availableOpenGLExtensions = g_list_sort(availableOpenGLExtensions, compareExtensionPriority);
       OpenGLExtension_reorderingNeeded = FALSE;
     }
   globalRenderingMode = openGLGet_globalRenderingOption();
   renderingMode = globalRenderingMode;
-  pnt = availableOpenGLExtensions;
-  while (pnt)
+  for (pnt = availableOpenGLExtensions; pnt; pnt = g_list_next(pnt))
     {
-      callList((OpenGLExtension*)pnt->data, &renderingMode, globalRenderingMode);
-      pnt = g_list_next(pnt);
+      ext = (OpenGLExtension*)pnt->data;
+      if (ext->priority < OPENGL_EXTENSION_PRIORITY_LAST)
+	callList((OpenGLExtension*)pnt->data, &renderingMode, globalRenderingMode);
+    }
+  if (renderingMode != globalRenderingMode)
+    /* Return the rendering mode to normal. */
+    openGLApply_renderingMode(globalRenderingMode);
+}
+void OpenGLExtensionCall_lastLists()
+{
+  GList *pnt;
+  RenderingModeId renderingMode, globalRenderingMode;
+  OpenGLExtension *ext;
+
+  DBG_fprintf(stderr, "Visu Extension: draw PRIORITY_LAST only.\n");
+  globalRenderingMode = openGLGet_globalRenderingOption();
+  renderingMode = globalRenderingMode;
+  for (pnt = availableOpenGLExtensions; pnt; pnt = g_list_next(pnt))
+    {
+      ext = (OpenGLExtension*)pnt->data;
+      if (ext->priority == OPENGL_EXTENSION_PRIORITY_LAST)
+	callList(ext, &renderingMode, globalRenderingMode);
     }
   if (renderingMode != globalRenderingMode)
     /* Return the rendering mode to normal. */
@@ -407,7 +427,7 @@ static void callList(OpenGLExtension *ext, RenderingModeId *renderingMode,
       /* Save OpenGL state if necessary. */
       if (ext->saveState)
 	{
-	  DBG_fprintf(stderr, "Visu Extension : save state.\n");
+	  DBG_fprintf(stderr, "Visu Extension: save state.\n");
 	  glPushAttrib(GL_ENABLE_BIT);
 	}
 
@@ -415,7 +435,7 @@ static void callList(OpenGLExtension *ext, RenderingModeId *renderingMode,
 
       if (ext->saveState)
 	{
-	  DBG_fprintf(stderr, "Visu Extension : restore state.\n");
+	  DBG_fprintf(stderr, "Visu Extension: restore state.\n");
 	  glPopAttrib();
 	}
     }
@@ -434,7 +454,7 @@ static void drawExtension(OpenGLExtension *extension, RenderingModeId renderingM
     }
 
   /* Call the compiled list. */
-  DBG_fprintf(stderr, "Visu Extension : call list %d (%s).\n",
+  DBG_fprintf(stderr, "Visu Extension: call list %d (%s).\n",
 	      extension->objectListId, extension->name);
   glCallList(extension->objectListId);
 
@@ -495,15 +515,13 @@ static gboolean readExtensionRendering(gchar **lines, int nbLines, int position,
 
   return TRUE;
 }
-static gboolean exportParametersRendering(GString *data, int *nbLinesWritten,
-					  VisuData *dataObj _U_)
+static void exportParametersRendering(GString *data, VisuData *dataObj _U_)
 {
   GList *tmp;
   OpenGLExtension *ext;
   const char **names;
 
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_MODE);
-  *nbLinesWritten = 1;
 
   names = openGLGet_allRenderingModes();
   tmp = availableOpenGLExtensions;
@@ -512,15 +530,9 @@ static gboolean exportParametersRendering(GString *data, int *nbLinesWritten,
       ext = (OpenGLExtension*)tmp->data;
       if (ext->isSensitiveToRenderingMode &&
 	  ext->preferedRenderingMode < nb_renderingModes)
-	{
-	  g_string_append_printf(data, "%s: %s %s\n", FLAG_PARAMETER_MODE,
-				 ext->name, names[ext->preferedRenderingMode]);
-	  *nbLinesWritten += 1;
-	}
+	g_string_append_printf(data, "%s: %s %s\n", FLAG_PARAMETER_MODE,
+			       ext->name, names[ext->preferedRenderingMode]);
       tmp = g_list_next(tmp);
     }
   g_string_append_printf(data, "\n");
-  *nbLinesWritten = 1;
-
-  return TRUE;
 }
