@@ -51,7 +51,6 @@
 #include "opengl.h"
 #include "visu_gtk.h"
 #include "visu_object.h"
-#include "gtk_main.h"
 #include "visu_basic.h"
 #include "visu_commandLine.h"
 #include "visu_configFile.h"
@@ -63,17 +62,13 @@
 #include "coreTools/toolConfigFile.h"
 #include "extraFunctions/plane.h"
 #include "extraFunctions/dataFile.h"
-#include "panelModules/panelBrowser.h"
-#include "panelModules/panelDataFile.h"
-#include "panelModules/panelPlanes.h"
-#include "panelModules/panelMap.h"
 #include "panelModules/panelSurfaces.h"
-#include "panelModules/panelFogBgColor.h"
 #include "gtk_renderingWindowWidget.h"
 #include "extraGtkFunctions/gtk_colorComboBoxWidget.h"
 #include "OSOpenGL/visu_openGL.h"
 #include "extensions/fogAndBGColor.h"
 #include "extensions/box.h"
+#include "extensions/scale.h"
 #include "renderingBackend/visu_windowInterface.h"
 
 /* Parameters. */
@@ -84,11 +79,18 @@ static gboolean usePreview;
 /* Static variables. */
 static gint gtkFileChooserWidth = -1, gtkFileChooserHeight = -1;
 
+/* This hashtable associate a #RenderingMethod with a Gtk dialog to
+   choose files. */
+static GHashTable *visuGtkLoadMethods;
+/* Store the last open directory. It is initialised
+   to current working directory. */
+static gchar *visuGtkLastDir;
+
+static GtkWindow *visuGtkPanel;
+static GtkWindow *visuGtkRender;
+
 /* Local routines. */
-static gboolean initVisuPostGTK(gpointer data);
-static gboolean commandLineLoadWithGtk(gpointer data);
-static gboolean exportParameters(GString *data, int *nbLinesWritten,
-				 VisuData *dataObj);
+static void exportParameters(GString *data, VisuData *dataObj);
 static gboolean readUsePreview(gchar **lines, int nbLines,
 			       int position, VisuData *dataObj, GError **error);
 static void onPreviewToggled(GtkToggleButton *button, gpointer data);
@@ -101,7 +103,10 @@ void visuGtkRaise_warning(gchar *action, gchar *message, GtkWindow *window)
   gchar *str;
   
   if (!window)
-    window = GTK_WINDOW(gtkMainClassGet_currentPanel());
+    window = visuGtkRender;
+
+  DBG_fprintf(stderr, "Visu Gtk: raise the error dialog (parent %p).\n",
+	      (gpointer)window);
 
 #if GTK_MINOR_VERSION > 5
   str = action;
@@ -111,6 +116,7 @@ void visuGtkRaise_warning(gchar *action, gchar *message, GtkWindow *window)
   alert = gtk_message_dialog_new(GTK_WINDOW(window),
 				 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				 GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, str);
+  gtk_window_set_title(GTK_WINDOW(alert), _("V_Sim error message"));
 #if GTK_MINOR_VERSION > 5
   gtk_widget_set_name(alert, "error");
 #else
@@ -133,7 +139,8 @@ void visuGtkRaise_warningLong(gchar *action, gchar *message, GtkWindow *window)
   GtkTextBuffer *buf;
 
   if (!window)
-    window = GTK_WINDOW(gtkMainClassGet_currentPanel());
+    window = visuGtkRender;
+
   alert = gtk_message_dialog_new(GTK_WINDOW(window),
 				 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				 GTK_MESSAGE_WARNING,
@@ -172,6 +179,35 @@ void visuGtkWait()
     gtk_main_iteration();
 }
 
+struct _VisuGtkSetFilesFunc
+{
+  VisuGtkSetFilesFunc load;
+};
+void visuGtkSet_renderingSpecificMethods(VisuRendering *method,
+					 VisuGtkSetFilesFunc methodLoad)
+{
+  struct _VisuGtkSetFilesFunc *ct;
+
+  g_return_if_fail(method && visuGtkLoadMethods);
+
+  DBG_fprintf(stderr, "Visu Gtk: set rendering specific for method '%s'.\n",
+	      visuRenderingGet_name(method, FALSE));
+  ct = g_malloc(sizeof(struct _VisuGtkSetFilesFunc));
+  ct->load = methodLoad;
+  g_hash_table_insert(visuGtkLoadMethods, method, ct);
+}
+VisuGtkSetFilesFunc visuGtkGet_renderingSpecificOpen(VisuRendering *method)
+{
+  struct _VisuGtkSetFilesFunc *ct;
+
+  g_return_val_if_fail(method && visuGtkLoadMethods, (VisuGtkSetFilesFunc)0);
+  
+  DBG_fprintf(stderr, "Visu Gtk: looking for a specific load interface for rendering"
+	      " method '%s'...\n", visuRenderingGet_name(method, FALSE));
+  
+  ct = (struct _VisuGtkSetFilesFunc *)g_hash_table_lookup(visuGtkLoadMethods, method);
+  return (ct && ct->load)?ct->load:visuGtkGet_fileFromDefaultFileChooser;
+}
 
 gchar** visuGtkGet_selectedDirectory(GtkWindow *parent, gboolean multiple,
 				     const gchar *dir)
@@ -184,7 +220,8 @@ gchar** visuGtkGet_selectedDirectory(GtkWindow *parent, gboolean multiple,
 
   /* Create the selector */
   if (!parent)
-    parent = GTK_WINDOW(gtkMainClassGet_currentPanel());
+    parent = visuGtkRender;
+
   file_selector = gtk_file_chooser_dialog_new(_("Choose a directory"), parent,
 					      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -210,7 +247,7 @@ gchar** visuGtkGet_selectedDirectory(GtkWindow *parent, gboolean multiple,
       gtk_widget_show_all(hbox);
     }
   if (!dir)
-    directory = gtkMainClassGet_lastOpenDirectory();
+    directory = visuGtkGet_lastOpenDirectory();
   else
     directory = (char*)dir;
   if (directory)
@@ -266,10 +303,12 @@ void visuGtkPreview_update(VisuGtkPreview *preview, const char *filenames[])
   DumpImage *dumpData;
   guchar* image;
   GdkPixbuf *pixbuf;
-  RenderingMethod *method;
+  visuRenderingLoadFunc loadMethod;
   gint width, height, nRows, nFiles, nb;
   VisuDataIter iter;
   GtkWidget *wd;
+  RenderingWindow *currentWindow;
+  OpenGLView *view;
 
   DBG_fprintf(stderr, "Visu Gtk: update preview with given filenames.\n");
 
@@ -279,10 +318,10 @@ void visuGtkPreview_update(VisuGtkPreview *preview, const char *filenames[])
       preview->table = (GtkWidget*)0;
     }
 
-  method = getRenderingMethodInUse();
-  g_return_if_fail(method && method->loadMethod);
+  loadMethod = visuRenderingClassGet_currentLoadFunc();
+  g_return_if_fail(loadMethod);
 
-  nb = renderingMethodGet_nbFileType(method);
+  nb = visuRenderingGet_nbFileType(visuRenderingClassGet_current());
   for (nFiles = 0; filenames[nFiles]; nFiles++);
 
   if (nFiles < nb)
@@ -298,15 +337,43 @@ void visuGtkPreview_update(VisuGtkPreview *preview, const char *filenames[])
       data = visuDataNew();
       for (nFiles = 0; filenames[nFiles]; nFiles++)
 	visuDataAdd_file(data, (char*)filenames[nFiles], nFiles, (FileFormat*)0);
-      
+
+      /* We save the current rendering context. */
+      currentWindow = visuRenderingWindowGet_current();
+
+      /* We change the context since loading a new data will
+	 generate some changes on the OpenGL rendering. */
+      width = 150;
+      height = 150;
+      dumpData = visuOpenGLNew_pixmapContext((guint)width, (guint)height);
+      if (!dumpData)
+	{
+	  /* We free the VisuData. */
+	  g_object_unref(data);
+
+	  gtk_image_set_from_stock(GTK_IMAGE(preview->image),
+				   GTK_STOCK_DIALOG_ERROR,
+				   GTK_ICON_SIZE_DIALOG);
+	  wd = gtk_label_new(_("Internal error,\nno preview available"));
+	  gtk_box_pack_start(GTK_BOX(preview->vbox), wd,
+			     FALSE, FALSE, 0);
+	  gtk_widget_show_all(wd);
+	  preview->table = wd;
+
+          /* Set the rendering window current for OpenGL. */
+          renderingWindowSet_current(currentWindow, TRUE);
+
+	  return;
+	}
+
       error = (GError*)0;
-      valid = method->loadMethod(data, (FileFormat*)0, 0, &error);
+      valid = loadMethod(data, (FileFormat*)0, 0, &error);
       if (!valid)
 	{
 	  gtk_image_set_from_stock(GTK_IMAGE(preview->image),
 				   GTK_STOCK_DIALOG_QUESTION,
 				   GTK_ICON_SIZE_DIALOG);
-	  wd = gtk_label_new(_("Not an V_Sim file"));
+	  wd = gtk_label_new(_("Not a V_Sim file"));
 	  gtk_box_pack_start(GTK_BOX(preview->vbox), wd,
 			     FALSE, FALSE, 0);
 	  gtk_widget_show_all(wd);
@@ -328,119 +395,107 @@ void visuGtkPreview_update(VisuGtkPreview *preview, const char *filenames[])
 	    }
 	  else
 	    {
-	      width = 150;
-	      height = 150;
-	      dumpData = visuOpenGLNew_pixmapContext((guint)width, (guint)height);
-	      if (dumpData)
+	      view = visuDataGet_openGLView(data);
+	      /* We set the glViewport of this new context. */
+	      glViewport(0, 0, width, height);
+	      /* We call the given draw method. */
+	      openGLInit_context();
+	      visuDataSet_sizeOfView(data, (guint)width, (guint)height);
+	      openGLViewCompute_matrixAndView(view);
+	      glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
+	      OpenGLExtensionRebuild_list(data, EXT_FOG_AND_BG_ID);
+	      OpenGLExtensionRebuild_list(data, EXT_VISU_DATA_ID);
+	      OpenGLExtensionRebuild_list(data, EXT_BOX_ID);
+	      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+	      OpenGLExtensionCall_list(EXT_FOG_AND_BG_ID, FALSE);
+	      OpenGLExtensionCall_list(EXT_VISU_DATA_ID, FALSE);
+	      OpenGLExtensionCall_list(EXT_BOX_ID, FALSE);
+	      /* We copy the pixmap into generic data. */
+	      image = visuOpenGLGet_pixmapData((guint)width,
+					       (guint)height, FALSE);
+	      pixbuf =
+		gdk_pixbuf_new_from_data((guchar*)image, GDK_COLORSPACE_RGB,
+					 FALSE, 8, width, height, 3 * width,
+					 free_image, (gpointer)0);
+	      gtk_image_set_from_pixbuf(GTK_IMAGE(preview->image), pixbuf);
+	      gdk_pixbuf_unref(pixbuf);
+	      /* We reset the flag of material. */
+	      comment = visuDataGet_fileCommentary(data, 0);
+	      visuDataIter_new(data, &iter);
+	      nRows = (comment && comment[0])?
+		iter.nElements + 3:iter.nElements + 1;
+	      preview->table = gtk_table_new(iter.nElements + 1, 2, FALSE);
+	      wd = gtk_label_new(_("<i>Box composition:</i>"));
+	      gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
+	      gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
+	      gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2, 0, 1,
+			       GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 5);
+	      for (visuDataIter_start(data, &iter); iter.element;
+		   visuDataIter_nextElement(data, &iter))
 		{
-		  /* We set the glViewport of this new context. */
-		  glViewport(0, 0, width, height);
-		  /* We call the given draw method. */
-		  openGLInit_context();
-		  visuDataSet_sizeOfView(data, (guint)width, (guint)height);
-		  openGLViewCompute_matrixAndView(visuDataGet_openGLView(data));
-		  OpenGLExtensionRebuild_list(data, EXT_FOG_AND_BG_ID);
-		  OpenGLExtensionRebuild_list(data, EXT_VISU_DATA_ID);
-		  OpenGLExtensionRebuild_list(data, EXT_BOX_ID);
-		  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-		  OpenGLExtensionCall_list(EXT_FOG_AND_BG_ID);
-		  OpenGLExtensionCall_list(EXT_VISU_DATA_ID);
-		  OpenGLExtensionCall_list(EXT_BOX_ID);
-		  /* We copy the pixmap into generic data. */
-		  image = visuOpenGLGet_pixmapData((guint)width,
-						   (guint)height, FALSE);
-		  /* We free the pixmap context. */
-		  visuOpenGLFree_pixmapContext(dumpData);
-		  pixbuf =
-		    gdk_pixbuf_new_from_data((guchar*)image, GDK_COLORSPACE_RGB,
-					     FALSE, 8, width, height, 3 * width,
-					     free_image, (gpointer)0);
-		  gtk_image_set_from_pixbuf(GTK_IMAGE(preview->image), pixbuf);
-		  gdk_pixbuf_unref(pixbuf);
-		  /* Set the rendering window current for OpenGL. */
-		  renderingWindowSet_current(currentRenderingWindow, TRUE);
-		  /* We reset the flag of material. */
-		  comment = visuDataGet_fileCommentary(data, 0);
-		  visuDataIter_new(data, &iter);
-		  nRows = (comment && comment[0])?
-		    iter.nElements + 3:iter.nElements + 1;
-		  preview->table = gtk_table_new(iter.nElements + 1, 2, FALSE);
-		  wd = gtk_label_new(_("<i>Box composition:</i>"));
+		  iter.element->materialIsUpToDate = FALSE;
+		  wd = gtk_label_new("");
+		  text = g_markup_printf_escaped
+		    (_("<span size=\"small\"><b>%s:</b></span>"),
+		     iter.element->name);
+		  gtk_label_set_markup(GTK_LABEL(wd), text);
+		  gtk_misc_set_alignment(GTK_MISC(wd), 1., 0.5);
+		  g_free(text);
+		  gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 1,
+				   iter.iElement + 1, iter.iElement + 2,
+				   GTK_FILL, GTK_SHRINK, 2, 0);
+		  wd = gtk_label_new("");
+		  if (iter.nStoredNodes[iter.iElement] > 1)
+		    text = g_markup_printf_escaped
+		      (_("<span size=\"small\">%d nodes</span>"),
+		       iter.nStoredNodes[iter.iElement]);
+		  else
+		    text = g_markup_printf_escaped
+		      (_("<span size=\"small\">%d node</span>"),
+		       iter.nStoredNodes[iter.iElement]);
+		  gtk_label_set_markup(GTK_LABEL(wd), text);
+		  gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
+		  g_free(text);
+		  gtk_table_attach(GTK_TABLE(preview->table), wd, 1, 2,
+				   iter.iElement + 1, iter.iElement + 2,
+				   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
+		}
+	      if (comment && comment[0])
+		{
+		  wd = gtk_label_new(_("<i>Description:</i>"));
 		  gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
 		  gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
-		  gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2, 0, 1,
+		  gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2,
+				   iter.nElements + 2, iter.nElements + 3,
 				   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 5);
-		  for (visuDataIter_start(data, &iter); iter.element;
-		       visuDataIter_nextElement(data, &iter))
-		    {
-		      iter.element->materialIsUpToDate = FALSE;
-		      wd = gtk_label_new("");
-		      text = g_markup_printf_escaped
-			(_("<span size=\"small\"><b>%s:</b></span>"),
-			 iter.element->name);
-		      gtk_label_set_markup(GTK_LABEL(wd), text);
-		      gtk_misc_set_alignment(GTK_MISC(wd), 1., 0.5);
-		      g_free(text);
-		      gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 1,
-				       iter.iElement + 1, iter.iElement + 2,
-				       GTK_FILL, GTK_SHRINK, 2, 0);
-		      wd = gtk_label_new("");
-		      if (iter.nStoredNodes[iter.iElement] > 1)
-			text = g_markup_printf_escaped
-			  (_("<span size=\"small\">%d nodes</span>"),
-			   iter.nStoredNodes[iter.iElement]);
-		      else
-			text = g_markup_printf_escaped
-			  (_("<span size=\"small\">%d node</span>"),
-			   iter.nStoredNodes[iter.iElement]);
-		      gtk_label_set_markup(GTK_LABEL(wd), text);
-		      gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
-		      g_free(text);
-		      gtk_table_attach(GTK_TABLE(preview->table), wd, 1, 2,
-				       iter.iElement + 1, iter.iElement + 2,
-				       GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
-		    }
-		  if (comment && comment[0])
-		    {
-		      wd = gtk_label_new(_("<i>Description:</i>"));
-		      gtk_label_set_use_markup(GTK_LABEL(wd), TRUE);
-		      gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
-		      gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2,
-				       iter.nElements + 2, iter.nElements + 3,
-				       GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 5);
-		      wd = gtk_label_new("");
-		      text = g_markup_printf_escaped("<span size=\"small\">%s</span>",
-						     comment);
-		      gtk_label_set_markup(GTK_LABEL(wd), text);
-		      g_free(text);
-		      gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
-		      gtk_label_set_justify(GTK_LABEL(wd), GTK_JUSTIFY_FILL);
-		      gtk_label_set_line_wrap(GTK_LABEL(wd), TRUE);
-		      gtk_widget_set_size_request(wd, 150, -1);
-		      gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2,
-				       iter.nElements + 3, iter.nElements + 4,
-				       GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
-		    }
-		  gtk_box_pack_start(GTK_BOX(preview->vbox), preview->table,
-				     FALSE, FALSE, 0);
-		  gtk_widget_show_all(preview->table);
-		}
-	      else
-		{
-		  gtk_image_set_from_stock(GTK_IMAGE(preview->image),
-					   GTK_STOCK_DIALOG_ERROR,
-					   GTK_ICON_SIZE_DIALOG);
-		  wd = gtk_label_new(_("Internal error, no preview available"));
-		  gtk_box_pack_start(GTK_BOX(preview->vbox), wd,
-				     FALSE, FALSE, 0);
-		  gtk_widget_show_all(wd);
-		  preview->table = wd;
+		  wd = gtk_label_new("");
+		  text = g_markup_printf_escaped("<span size=\"small\">%s</span>",
+						 comment);
+		  gtk_label_set_markup(GTK_LABEL(wd), text);
+		  g_free(text);
+		  gtk_misc_set_alignment(GTK_MISC(wd), 0., 0.5);
+		  gtk_label_set_justify(GTK_LABEL(wd), GTK_JUSTIFY_FILL);
+		  gtk_label_set_line_wrap(GTK_LABEL(wd), TRUE);
+		  gtk_widget_set_size_request(wd, 150, -1);
+		  gtk_table_attach(GTK_TABLE(preview->table), wd, 0, 2,
+				   iter.nElements + 3, iter.nElements + 4,
+				   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
 		}
+	      gtk_box_pack_start(GTK_BOX(preview->vbox), preview->table,
+				 FALSE, FALSE, 0);
+	      gtk_widget_show_all(preview->table);
 	    }
 	}
+      /* Set the rendering window current for OpenGL. */
+      renderingWindowSet_current(currentWindow, TRUE);
+
+      /* We free the pixmap context. */
+      visuOpenGLFree_pixmapContext(dumpData);
+
       /* We free the VisuData. */
       g_object_unref(data);
     }
+  DBG_fprintf(stderr, "Visu Gtk: end of preview creation.\n");
 }
 static void onPreviewToggled(GtkToggleButton *button, gpointer data _U_)
 {
@@ -495,7 +550,7 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
 {
   GtkWidget *fileSelection;
   GList *filters, *tmpLst;
-  RenderingMethod *method;
+  VisuRendering *method;
   gchar* directory, *filename;
   FileFormat *selectedFormat;
   GtkFileFilter *filterDefault;
@@ -506,7 +561,7 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
 
   DBG_fprintf(stderr, "Visu Gtk: default filechooser for file opening.\n");
 
-  method = getRenderingMethodInUse();
+  method = visuRenderingClassGet_current();
   g_return_val_if_fail(method, FALSE);
 
   fileSelection = gtk_file_chooser_dialog_new(_("Load session"),
@@ -518,7 +573,7 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
   if (gtkFileChooserWidth > 0 || gtkFileChooserHeight > 0)
     gtk_window_set_default_size(GTK_WINDOW(fileSelection), gtkFileChooserWidth,
 				gtkFileChooserHeight);
-  directory = gtkMainClassGet_lastOpenDirectory();
+  directory = visuGtkGet_lastOpenDirectory();
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fileSelection), directory);
 
@@ -527,7 +582,8 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
   gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);
 
   /* Create and add the filters. */
-  filters = visuGtkCreate_fileChooserFilter(method->fileType[0], fileSelection);
+  filters = visuGtkCreate_fileChooserFilter
+    (visuRenderingGet_fileType(visuRenderingClassGet_current(), 0), fileSelection);
 
   /* Create and add the preview. */
   visuGtkPreview_add(&preview, GTK_FILE_CHOOSER(fileSelection));
@@ -556,7 +612,7 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
 
   directory =
     gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(fileSelection));
-  gtkMainClassSet_lastOpenDirectory((char*)directory);
+  visuGtkSet_lastOpenDirectory((char*)directory, VISU_DIR_FILE);
   g_free(directory);
 
   gtk_window_get_size(GTK_WINDOW(fileSelection), &gtkFileChooserWidth,
@@ -577,26 +633,42 @@ gboolean visuGtkGet_fileFromDefaultFileChooser(VisuData *data, GtkWindow *parent
   return res;
 }
 
+void visuGtkCreate_main(GtkWindow **panel, GtkWindow **render)
+{
+  int width, height;
+  GtkWidget *renderingWindow;
 
-int visuGtkMain(int argc, char *argv[])
+  /* Force the creation of the Scale class. */
+  commandLineGet_XWindowGeometry(&width, &height);
+  DBG_fprintf(stderr,"Visu Gtk: create a rendering window (%dx%d).\n",
+	      width, height);
+  renderingWindow =
+    renderingWindow_newEmbedded(width, height);
+  *render = GTK_WINDOW(g_object_get_data(G_OBJECT(renderingWindow),
+					 RENDERING_WINDOW_ID));
+  g_signal_connect(G_OBJECT(*render), "delete-event",
+		   G_CALLBACK(gtk_main_quit), (gpointer)0);
+  g_signal_connect(G_OBJECT(*render), "destroy-event",
+		   G_CALLBACK(gtk_main_quit), (gpointer)0);
+  gtk_widget_show(GTK_WIDGET(*render));
+  visuRenderingWindowSet_default(renderingWindow);
+  *panel = (GtkWindow*)0;
+
+  return;
+}
+
+void visuGtkMain(VisuGtkInitWidgetFunc panelFunc)
 {
-  GtkWidget *commandPanel;
   VisuConfigFileEntry *entry;
+  GString *message;
 
-  DBG_fprintf(stderr, "Visu Gtk: initialise Gtk.\n");
-  gtk_init(&argc, &argv);
+  g_return_if_fail(panelFunc);
 
-#ifdef HAVE_GTKGLEXT
-  DBG_fprintf(stderr, "Visu Gtk: initialise GtkGlExt.\n");
-  gdk_gl_init(&argc, &argv);
-#endif
-
-  /* Force the creation of the main object class. */
-  g_type_class_ref(visu_get_type());
+  /* Retrieve the current working directory. */
+  visuGtkLastDir = g_get_current_dir();
 
   /* Set private variables. */
   usePreview = TRUE;
-  visuBasicInit();
   entry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
 				  FLAG_PARAMETER_PREVIEW,
 				  DESC_PARAMETER_PREVIEW,
@@ -604,328 +676,96 @@ int visuGtkMain(int argc, char *argv[])
   visuConfigFileSet_version(entry, 3.4f);
   visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_PARAMETER,
 				   exportParameters);
+  visuGtkLoadMethods = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+					     NULL, g_free);
+  panelFunc(&visuGtkPanel, &visuGtkRender);
+  g_return_if_fail(visuGtkRender);
 
-  DBG_fprintf(stderr,"--- Initialising the new gtk classes ---\n");
-  /* Force the creation of the ColorComboBoxClass. */
-  g_type_class_ref(colorComboBox_get_type());
-  /* Force the creation of the VisuData class. */
-  g_type_class_ref(visu_data_get_type());
+  /* Add the gtk tag as a known tag to allow to read such parameters. */
+  message = visuBasicParse_configFiles();
+  if (message)
+    {
+      visuGtkRaise_warningLong(_("Reading the configuration files"), message->str,
+			       visuGtkRender);
+      g_string_free(message, TRUE);
+    }
 
-  commandPanel = gtkMainNew();
-  gtkMainClassSet_currentPanel(GTK_MAIN(commandPanel));
-
-  DBG_fprintf(stderr, "Visu Gtk: adding loading parameters and resources"
-	      " callbacks in the queue.\n");
-  g_idle_add(initVisuPostGTK, (gpointer)commandPanel);
-
-  DBG_fprintf(stderr, "Visu Gtk: starting main GTK loop.\n");
-  gtk_main();
-
-  return 0;
+  return;
 }
 
-static gboolean initVisuPostGTK(gpointer data)
+gboolean visuGtkRun_commandLine(gpointer data _U_)
 {
-  char* path;
-  int res;
-  char *filename, *spin_filename;
-  gchar *dirname, *normDir, *resFile;
-  VisuData *newData;
-  GError *error;
-  GtkMain *commandPanel;
-
-  commandPanel = GTK_MAIN(data);
-  g_return_val_if_fail(commandPanel, FALSE);
-
-  DBG_fprintf(stderr,"--- Initialising the parameters ---\n");
-  /* Save the gtk tag as a known tag to allow to read such parameters. */
-  visuConfigFileAdd_knownTag("gtk");
-  /* Look for parameters file */
-  path = visuConfigFileGet_validPath(VISU_CONFIGFILE_PARAMETER, R_OK, 0);
-  if (path)
-    {
-      error = (GError*)0;
-      res = visuConfigFileLoad(VISU_CONFIGFILE_PARAMETER, path, (VisuData*)0, &error);
-      if (error)
-	{
-	  visuGtkRaise_warningLong(_("Loading a parameter file"), error->message,
-				   GTK_WINDOW(commandPanel));
-	  g_error_free(error);
-	}
-      g_free(path);
-    }
-  else
-    visuGtkRaise_warning(_("Loading a file"),
-			 _("Unable to find a valid parameters file."
-			   " Starting with defaults."),
-			 GTK_WINDOW(commandPanel));
-
-  DBG_fprintf(stderr,"--- Initialising resources ---\n");
-  /* Look for resources file */
-  resFile = commandLineGet_resourcesFile();
-  if (!resFile)
-    path = visuConfigFileGet_validPath(VISU_CONFIGFILE_RESOURCE, R_OK, 0);
-  else
-    path = g_strdup(resFile);
-  if (path)
-    {
-      error = (GError*)0;
-      res = visuConfigFileLoad(VISU_CONFIGFILE_RESOURCE, path, (VisuData*)0, &error);
-      if (error)
-	{
-	  visuGtkRaise_warningLong(_("Loading a resource file"), error->message,
-				   GTK_WINDOW(commandPanel));
-	  g_error_free(error);
-	}
-      g_free(path);
-      DBG_fprintf(stderr, "Visu Gtk: emit 'resource-loaded' signal.\n");
-      g_signal_emit(VISU_INSTANCE,
-		    VISU_GET_CLASS(VISU_INSTANCE)->resourcesLoaded_signal_id,
-		    0, (GObject*)0, NULL);
-    }
-
-  newData = (VisuData*)0;
-  filename = commandLineGet_ArgFilename();
-  spin_filename = commandLineGet_ArgSpinFileName();
-  if (spin_filename && filename)
-    {
-      setRenderingMethodInUse(renderingSpinGet_static());
-      newData = visuDataNew();
-      if (!newData)
-	exit(1);
-
-      dirname = g_path_get_dirname(filename);
-      normDir = normalize_path(dirname);
-      gtkMainClassSet_lastOpenDirectory((char*)normDir);
-      g_free(dirname);
-      g_free(normDir);
-
-      visuDataAdd_file(newData, filename, FILE_KIND_POSITION, (FileFormat*)0);
-      visuDataAdd_file(newData, spin_filename, FILE_KIND_SPIN, (FileFormat*)0);
-      g_free(spin_filename);
-      g_free(filename);
-      spin_filename = (char*)0;
-      filename = (char*)0;
-
-      DBG_fprintf(stderr, "Visu Gtk: adding the load call back in the queue.\n");
-      g_idle_add(commandLineLoadWithGtk, (gpointer)newData);
-    }
-  else if (filename)
-    {
-      setRenderingMethodInUse(renderingAtomicGet_static());
-      newData = visuDataNew();
-      if (!newData)
-	exit(1);
-
-      dirname = g_path_get_dirname(filename);
-      normDir = normalize_path(dirname);
-      gtkMainClassSet_lastOpenDirectory((char*)normDir);
-      g_free(dirname);
-      g_free(normDir);
-
-      visuDataAdd_file(newData, filename, FILE_KIND_POSITION, (FileFormat*)0);
-      g_free(filename);
-      filename = (char*)0;
-
-      DBG_fprintf(stderr, "Visu Gtk: adding the load call back in the queue.\n");
-      g_idle_add(commandLineLoadWithGtk, (gpointer)newData);
-    }
-  else
-    panelBrowserSet_currentDirectory(gtkMainClassGet_lastOpenDirectory());
-
-  if (spin_filename)
-    g_free(spin_filename);
-  if (filename)
-    g_free(filename);
-
-  gtk_widget_show(GTK_WIDGET(gtkMainClassGet_currentPanel()));
-
-  return FALSE;
-}
-
-gboolean commandLineLoadWithGtk(gpointer data)
-{
-  int i, somethingIsLoaded, presetShade, nb, mapPlaneId;
-  gboolean res, new;
-  gchar *colorizeFilename, *planeFile, *surfFile, *bgFile;
-  float *values;
-  int *colUsed;
-  float *translations, *extension;
-  GList *list;
-  Shade *shade;
+  RenderingWindow *window;
   VisuData *obj;
+  VisuOptionSet *set;
   GError *error;
-  OptionTable *table;
-  gchar **names;
-  Plane **planes;
-  GtkMain *commandPanel;
 
-  obj = (VisuData*)data;
-  commandPanel = gtkMainClassGet_currentPanel();
-
-  error = (GError*)0;
-  res = visuBasicLoad_dataFromFile(obj, (FileFormat*)0, 0, &error);
-  if (error)
-    {
-      visuGtkRaise_warning(_("Loading a file"), error->message,
-			   GTK_WINDOW(commandPanel));
-      g_error_free(error);
-    }
-  if (!res)
-    {
-      g_object_unref(obj);
-      obj = (VisuData*)0;
-      if (!error)
-	g_warning("No error message!");
-    }
-
-  /* Post loading sequence, corresponding to all other arguments
-     given to command line. */
-  /* translate argument */
-  if (obj)
-    {
-      translations = commandLineGet_translation();
-      if (translations)
-	{
-	  visuDataSet_XYZtranslation(obj, translations);
-	  visuData_constrainedInTheBox(obj);
-	}
-    }
-
-  visuRenderingWindowSet_visuData(RENDERING_WINDOW(commandPanel->renderingWindow), obj);
-  panelBrowserSet_currentDirectory(gtkMainClassGet_lastOpenDirectory());
+  window = RENDERING_WINDOW(visuRenderingWindowGet_current());
+  g_return_val_if_fail(window, FALSE);
 
+  obj = renderingWindowGet_visuData(window);
   if (!obj)
     return FALSE;
 
-  /* Post loading sequence, corresponding to all other arguments given to command line. */
-  /* A possible shade. */
-  presetShade = commandLineGet_colorizePresetColor();
-  if (presetShade < 0)
-    presetShade = 0;
-  /* colorize argument */
-  colorizeFilename = commandLineGet_colorizeFileName();
-  if (colorizeFilename)
+  set = g_malloc(sizeof(VisuOptionSet));
+  error = (GError*)0;
+  if (!visuBasicApply_commandLine(obj, set, &error))
     {
-      panelDataFileSet_use(TRUE);
-      somethingIsLoaded = panelDataFileLoad_file(obj, colorizeFilename, &new);
-      if (somethingIsLoaded)
-	{
-	  list = toolShadeBuild_presetList();
-	  if (list)
-	    {
-	      shade = (Shade*)g_list_nth_data(list, presetShade);
-	      if (shade)
-		dataFileSet_shade(obj, shade);
-	    }
-	  colUsed = commandLineGet_colorizeColUsed();
-	  if (colUsed)
-	    for (i = 0; i < 3; i++)
-	      dataFileSet_colUsed(obj, colUsed[i] - 1, i);
-	}
+      visuGtkRaise_warning(_("Parsing command line"), error->message, (GtkWindow*)0);
+      g_error_free(error);
     }
-  /* plane file argument */
-  planeFile = commandLineGet_planesFileName();
-  if (planeFile)
-    {
-      panelPlanesSet_use(obj, TRUE);
-      error = (GError*)0;
-      res = panelPlanesLoad_file(obj, planeFile, &error);
-      if (error)
-	{
-	  visuGtkRaise_warning(_("Loading a file"), error->message, (GtkWindow*)0);
-	  g_clear_error(&error);
-	}
-      else if (res)
-	panelPlanesApply_hidingScheme(obj);
-    }
-  /* iso-surface argument. */
-  table = commandLineGet_options();
-  surfFile = commandLineGet_isoSurfacesFileName();
-  if (surfFile)
-    {
-      res = panelIsosurfacesLoad_file(surfFile, commandLineGet_fitToBox(), obj, table);
-      if (!res)
-	{
-	  panelIsosurfacesShow_all(TRUE);
-	  planes = panelPlanesGet_listOfAllPlanes(TRUE);
-	  panelIsosurfacesHide(planes);
-	  g_free(planes);
-	  panelIsosurfacesSet_used(obj, TRUE);
-	}
-    }
-  /* scalar field argument. */
-  surfFile = commandLineGet_scalarFieldFileName();
-  if (surfFile)
-    {
-      res = panelIsosurfacesLoad_file(surfFile, commandLineGet_fitToBox(), obj, table);
-      if (!res)
-	{
-	  values = commandLineGet_isoValues(&nb);
-	  names = commandLineGet_isoNames(&nb);
-	  for (i = 0; i < nb; i++)
-	    panelIsosurfacesAdd_withValue(surfFile, values[i], names[i]);
-	  panelIsosurfacesShow_all(TRUE);
-	  planes = panelPlanesGet_listOfAllPlanes(TRUE);
-	  panelIsosurfacesHide(planes);
-	  g_free(planes);
-	  panelIsosurfacesSet_used(obj, TRUE);
-	}
-    }
-  /* The extension. */
-  extension = commandLineGet_extension();
-  if (extension)
-    visuData_replicate(obj, extension, (gboolean*)0);
-  /* The coloured map argument. */
-  if (commandLineGet_coloredMap(&mapPlaneId))
-    {
-      panelMapSet_logScale(commandLineGet_logScale());
-      panelMapSet_nIsoLines(commandLineGet_nIsoLines());
-      panelMapSet((guint)mapPlaneId, 0, (guint)presetShade);
-    }
-  /* The background image. */
-  bgFile = commandLineGet_bgImage();
-  if (bgFile)
-    panelFogBgColorSet_bgImage(bgFile);
-  
+  else
+    visuBasicCreate_extensions(obj, set, TRUE);
+
   visuData_createAllNodes(obj);
+
   g_idle_add(visuObjectRedraw, (gpointer)0);
   
   return FALSE;
 }
 
-gboolean visuGtkLoadAndRender_file(gpointer data)
+struct _load_struct
+{
+  VisuData *data;
+  guint iSet;
+};
+static gboolean _visuGtkLoad_file(gpointer data)
 {
-  GtkMain *commandPanel;
   VisuData *obj;
   GError *error;
   VisuData *prevData;
   gboolean changeElement, res;
+  RenderingWindow *window;
+  struct _load_struct *pt;
+  int iSet;
 
   /* obj is the new object and main the panel that handle the
      loading. */
-  obj = VISU_DATA(data);
-  commandPanel = gtkMainClassGet_currentPanel();
+  pt = (struct _load_struct*)data;
+  obj = pt->data;
+  iSet = (int)pt->iSet;
+  g_free(pt);
+  window = visuRenderingWindowGet_current();
 
-  prevData = renderingWindowGet_visuData(RENDERING_WINDOW(commandPanel->renderingWindow));
+  prevData = renderingWindowGet_visuData(RENDERING_WINDOW(window));
   DBG_fprintf(stderr, "Visu Gtk: loading process ... %p points to"
 	      " previous VisuData.\n", (gpointer)prevData);
 
   error = (GError*)0;
-  res = visuBasicLoad_dataFromFile(obj, (FileFormat*)0, 0, &error);
-  if (error)
-    {
-      visuGtkRaise_warning(_("Loading a file"), error->message,
-			   GTK_WINDOW(commandPanel));
-      g_error_free(error);
-    }
+  res = visuBasicLoad_dataFromFile(obj, (FileFormat*)0, iSet, &error);
+  DBG_fprintf(stderr, "Visu Gtk: basic load OK, continue with Gtk loading parts.\n");
 
   if (!res)
     {
       g_object_unref(obj);
       obj = (VisuData*)0;
-      if (!error)
-	g_warning("No error message!");
+      if (error)
+	{
+	  visuGtkRaise_warning(_("Loading a file"), error->message, NULL);
+	  g_error_free(error);
+	}
+      else
+	g_warning("No error message.");
     }
   else
     if (prevData && obj)
@@ -936,18 +776,29 @@ gboolean visuGtkLoadAndRender_file(gpointer data)
 
   DBG_fprintf(stderr, "Visu Gtk: loading process ... try to load %p.\n",
 	      (gpointer)obj);
-  visuRenderingWindowSet_visuData(RENDERING_WINDOW(commandPanel->renderingWindow), obj);
-
-  panelBrowserSet_currentDirectory(gtkMainClassGet_lastOpenDirectory());
+  renderingWindowSet_visuData(RENDERING_WINDOW(window), obj);
 
   if (!obj)
     return FALSE;
 
+  /* We release a ref on obj, since
+     renderingWindowSet_visuData has increased it. */
+  g_object_unref(G_OBJECT(obj));
+	 
   visuData_createAllNodes(obj);
-  g_idle_add(visuObjectRedraw, (gpointer)0);
 
   return FALSE;
 }
+void visuGtkLoad_file(VisuData *data, guint iSet)
+{
+  struct _load_struct *pt;
+
+  pt = g_malloc(sizeof(struct _load_struct));
+  pt->data = data;
+  pt->iSet = iSet;
+  g_idle_add(_visuGtkLoad_file, pt);
+}
+
 
 GList* visuGtkCreate_fileChooserFilter(GList *list, GtkWidget *fileChooser)
 {
@@ -1008,14 +859,11 @@ GList* visuGtkCreate_fileChooserFilter(GList *list, GtkWidget *fileChooser)
   return returnedFilters;
 }
 
-static gboolean exportParameters(GString *data, int *nbLinesWritten,
-				 VisuData *dataObj _U_)
+static void exportParameters(GString *data, VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_PREVIEW);
   g_string_append_printf(data, "%s[gtk]: %d\n\n", FLAG_PARAMETER_PREVIEW,
 			 usePreview);
-  *nbLinesWritten = 3;
-  return TRUE;
 }
 static gboolean readUsePreview(gchar **lines, int nbLines,
 			       int position, VisuData *dataObj _U_, GError **error)
@@ -1040,3 +888,58 @@ static gboolean readUsePreview(gchar **lines, int nbLines,
 
   return TRUE;
 }
+
+char* visuGtkGet_lastOpenDirectory()
+{
+  DBG_fprintf(stderr, "Visu Gtk: get the last open directory : '%s'.\n",
+	      visuGtkLastDir);
+  return visuGtkLastDir;
+}
+void visuGtkSet_lastOpenDirectory(char* directory, VisuGtkDirType type)
+{
+  if (visuGtkLastDir)
+    g_free(visuGtkLastDir);
+
+  if (!g_path_is_absolute(directory))
+    visuGtkLastDir = g_build_filename(g_get_current_dir(),
+					 directory, NULL);
+  else
+    visuGtkLastDir = g_build_filename(directory, NULL);
+  DBG_fprintf(stderr, "Visu Gtk: set the last open directory to '%s', emit signal.\n",
+	      visuGtkLastDir);
+
+  g_signal_emit(VISU_INSTANCE, VISU_SIGNALS[DIR_SIGNAL],
+		0 /* details */, type, NULL);
+  DBG_fprintf(stderr, "Visu Gtk: emission done (DirectoryChanged).\n");
+}
+
+GdkPixbuf* visuGtkCreate_pixbuf(const gchar *filename)
+{
+  gchar *pathname = NULL;
+  GdkPixbuf *pixbuf;
+  GError *error = NULL;
+
+  g_return_val_if_fail(filename && filename[0], (GdkPixbuf*)0);
+
+  pathname = g_build_filename(V_SIM_PIXMAPS_DIR, filename, NULL);
+
+  pixbuf = gdk_pixbuf_new_from_file(pathname, &error);
+  if (!pixbuf)
+    {
+      g_warning(_("failed to load pixbuf file '%s': %s\n"),
+		pathname, error->message);
+      g_error_free(error);
+    }
+  g_free(pathname);
+
+  return pixbuf;
+}
+
+GtkWindow* visuGtkGet_render()
+{
+  return visuGtkRender;
+}
+GtkWindow* visuGtkGet_panel()
+{
+  return visuGtkPanel;
+}
