@@ -176,7 +176,8 @@ VisuNodeArray* visuNodeNew_nodes(unsigned int nTypes, unsigned int *nNodes)
   g_return_val_if_fail(nTypes > 0 && nNodes, (VisuNodeArray*)0);
 
   array = g_malloc(sizeof(VisuNodeArray));
-  DBG_fprintf(stderr, "Visu Node: creating a new VisuNodeArray %p", (gpointer)array);
+  DBG_fprintf(stderr, "Visu Node: creating a new VisuNodeArray %p (%d types).\n",
+	      (gpointer)array, nTypes);
 
   array->ntype                = nTypes;
   array->idCounter            = 0;
@@ -188,6 +189,8 @@ VisuNodeArray* visuNodeNew_nodes(unsigned int nTypes, unsigned int *nNodes)
   for(i = 0; i < array->ntype; i++)
     {
       g_return_val_if_fail(nNodes[i] > 0, (VisuNodeArray*)0);
+      DBG_fprintf(stderr, " | for type %d -> %d nodes allocated.\n",
+		  i, nNodes[i]);
 
       array->nodes[i]               = g_malloc(sizeof(VisuNode) * nNodes[i]);
       array->numberOfNodes[i]       = nNodes[i];
@@ -200,13 +203,14 @@ VisuNodeArray* visuNodeNew_nodes(unsigned int nTypes, unsigned int *nNodes)
 	  array->nNodes += 1;
 	}
     }
+  DBG_fprintf(stderr, " | total number of allocated nodes %d.\n",
+	      array->nNodes);
   array->nbOfAllStoredNodes = 0;
   array->nodeTableSize      = array->nNodes;
   array->nodeTable          = g_malloc(sizeof(VisuNode*) * array->nNodes);
 
   array->properties = g_hash_table_new_full(g_str_hash, g_str_equal,
 					    NULL, freePropertiesStruct);
-  DBG_fprintf(stderr, " ... OK\n");
   DBG_fprintf(stderr, " | size = %do\n",
 	      (int)(array->nNodes * sizeof(VisuNode) +
 		    sizeof(unsigned int) * array->ntype * 2 +
@@ -217,7 +221,8 @@ VisuNodeArray* visuNodeNew_nodes(unsigned int nTypes, unsigned int *nNodes)
      node and negative if not. The value is the node id that the
      duplicate refers to. */
   DBG_fprintf(stderr, " | create the original node property.\n");
-  array->origProp = visuNodeNew_intProperty(array, ORIGINAL_ID);
+  array->origProp   = visuNodeNew_intProperty(array, ORIGINAL_ID);
+  array->nOrigNodes = 0;
 
   return array;
 }
@@ -274,7 +279,7 @@ void visuNodeRemove_nodes(VisuNodeArray *nodeArray, int *nodeNumbers)
   /* We get the last non NULL node pointer in the nodeTable array to make the
      idCounter having this value to avoid to much reallocation of the nodeTable
      array. */
-  for (; !nodeArray->nodeTable[nodeArray->idCounter - 1] && nodeArray->idCounter > 0;
+  for (; nodeArray->idCounter > 0 && !nodeArray->nodeTable[nodeArray->idCounter - 1];
        nodeArray->idCounter -= 1);
   DBG_fprintf(stderr, "Visu Node: idCounter is set to %d.\n", nodeArray->idCounter);
   DBG_fprintf(stderr, " | size = %do\n",
@@ -313,6 +318,41 @@ gboolean visuNodeRemove_allDuplicateNodes(VisuNodeArray *nodeArray,
   else
     return FALSE;
 }
+gint visuNodeGet_original(VisuNodeArray *nodeArray, guint nodeId)
+{
+  VisuNode *node;
+  gint orig;
+
+  g_return_val_if_fail(nodeArray && nodeArray->origProp, -1);
+  g_return_val_if_fail(nodeId < nodeArray->idCounter, -1);
+
+  orig = (gint)nodeId;
+  do
+    {
+      node = nodeArray->nodeTable[orig];
+      orig = nodeArray->origProp->data_int[node->posElement][node->posNode];
+    }
+  while (orig >= 0);
+  DBG_fprintf(stderr, "Visu Node: get original from %d: %d.\n",
+	      nodeId, nodeArray->origProp->data_int[node->posElement][node->posNode]);
+  return (node->number == nodeId)?-1:(gint)node->number;
+}
+gboolean visuNodeSet_original(VisuNodeArray *nodeArray, guint nodeId)
+{
+  VisuNode *node;
+  gint orig;
+
+  g_return_val_if_fail(nodeArray && nodeArray->origProp, -1);
+  g_return_val_if_fail(nodeId < nodeArray->idCounter, -1);
+
+  node = nodeArray->nodeTable[nodeId];
+  orig = nodeArray->origProp->data_int[node->posElement][node->posNode];
+  nodeArray->origProp->data_int[node->posElement][node->posNode] = -1;
+
+  DBG_fprintf(stderr, "Visu Node: set original for %d (%d).\n",
+	      nodeId, orig);
+  return (orig != -1);
+}
 void visuNodeFree_nodes(VisuNodeArray *nodeArray)
 {
   unsigned int i;
@@ -406,9 +446,12 @@ VisuNode* visuNodeGet_newNode(VisuNodeArray *nodeArray, unsigned int iEle)
 }
 VisuNode* visuNodeGet_copyNode(VisuNodeArray *nodeArray, VisuNode *node)
 {
-  DBG_fprintf(stderr, "Visu Node: copy a new node from node %d-%d.\n",
-	      node->posElement, node->posNode);
-  return newOrCopyNode(nodeArray, node->posElement, node->number);
+  VisuNode* out;
+
+  out = newOrCopyNode(nodeArray, node->posElement, node->number);
+  DBG_fprintf(stderr, "Visu Node: copy a new node from node %d (%d-%d) -> %d.\n",
+	      node->number, node->posElement, node->posNode, out->number);
+  return out;
 }
 static VisuNode* newOrCopyNode(VisuNodeArray *nodeArray, int iEle,
 			       int oldNodeId)
@@ -465,6 +508,7 @@ static VisuNode* newOrCopyNode(VisuNodeArray *nodeArray, int iEle,
   g_value_init(&idValue, G_TYPE_INT);
   g_value_set_int(&idValue, oldNodeId);
   visuNodePropertySet_value(nodeArray->origProp, node, &idValue);
+  nodeArray->nOrigNodes += (oldNodeId < 0)?1:0;
 
   return node;
 }
@@ -778,12 +822,14 @@ void visuNodeTrace_property(VisuNodeArray *array, const gchar *id)
     {
       for (i = 0; i < prop->array->ntype; i++)
 	for (j = 0; j < prop->array->numberOfStoredNodes[i]; j++)
-	  fprintf(stderr, " | %3d %7d -> %d\n", i, j, prop->data_int[i][j]);
+	  fprintf(stderr, " | %7d %3d %7d -> %d\n", array->nodes[i][j].number,
+		  i, j, prop->data_int[i][j]);
     }
   if (prop->data_pointer)
     {
       for (i = 0; i < prop->array->ntype; i++)
 	for (j = 0; j < prop->array->numberOfStoredNodes[i]; j++)
-	  fprintf(stderr, " | %3d %7d -> %p\n", i, j, prop->data_pointer[i][j]);
+	  fprintf(stderr, " | %7d %3d %7d -> %p\n", array->nodes[i][j].number,
+		  i, j, prop->data_pointer[i][j]);
     }
 }
