@@ -102,6 +102,12 @@ struct VisuNodeProperty_struct;
  */
 typedef struct VisuNodeProperty_struct VisuNodeProperty;
 
+typedef struct VisuNodeInfo_struct
+{
+  guint id;
+  float dist;
+} VisuNodeInfo;
+
 /**
  * VisuNodeArray:
  * @ntype: number of #VisuElement used in this object.
@@ -152,6 +158,7 @@ typedef struct nodes_struct
 
   /* A convenient pointer on the original node property. */
   VisuNodeProperty *origProp;
+  guint nOrigNodes;
 } VisuNodeArray;
 
 /**
@@ -226,6 +233,18 @@ void visuNodeRemove_nodes(VisuNodeArray *nodeArray, int *nodeNumbers);
  */
 gboolean visuNodeRemove_allDuplicateNodes(VisuNodeArray *nodeArray,
 					  int **nodeNumbers);
+/**
+ * visuNodeTest_original:
+ * @nodeArray: a #VisuNodeArray object.
+ * @nodeId: a node id.
+ *
+ * Test if the given @nodeId is an original or a replica for the
+ * periodisation.
+ *
+ * Returns: TRUE for an original node.
+ */
+gint visuNodeGet_original(VisuNodeArray *nodeArray, guint nodeId);
+gboolean visuNodeSet_original(VisuNodeArray *nodeArray, guint nodeId);
 /**
  * visuNodeFree_nodes:
  * @nodeArray: a #VisuNodeArray object.
