@@ -60,7 +60,7 @@ G_BEGIN_DECLS
  *
  * return the type of #VisuObject.
  */
-#define VISU_TYPE		  (visu_get_type ())
+#define VISU_TYPE		  (visuObject_get_type ())
 /**
  * VISU:
  * @obj: a #GObject to cast.
@@ -103,42 +103,21 @@ G_BEGIN_DECLS
  * A short way to identify #_VisuObject structure.
  */
 typedef struct _VisuObject VisuObject;
-/**
- * VisuObjectClass:
- *
- * A short way to identify #_VisuObjectClass structure.
- */
-typedef struct _VisuObjectClass VisuObjectClass;
-/**
- * VisuObjectPrivate:
- *
- * A short way to identify #_VisuObjectPrivate structure.
- */
-typedef struct _VisuObjectPrivate VisuObjectPrivate;
 
-/**
- * VisuObject:
- * @parent: an object to inherit from (NULL here).
- * @privateDt: a pointer to the private data.
- *
- * This structure describes a #VisuObject object.
- */
-struct _VisuObject {
-	GObject parent;
-	/* instance members */
-        VisuObjectPrivate *privateDt;
-};
-
-/**
- * visuObjectRedraw:
- * @data: some data (unused).
- *
- * Call the signal OpenGLAskForReDraw. The API is adapted to the routine to be added
- * in the gloop.
- *
- * Returns: FALSE to stop the emission as soon as done.
- */
-gboolean visuObjectRedraw(gpointer data);
+enum
+  {
+    COLORNEWAVAILABLE_SIGNAL,
+    DATANEW_SIGNAL,
+    DATALOADED_SIGNAL,
+    DATAREADYFORRENDERING_SIGNAL,
+    RENDERINGCHANGED_SIGNAL,
+    RESOURCESLOADED_SIGNAL,
+    OPENGLASKFORREDRAW_SIGNAL,
+    OPENGLFORCEREDRAW_SIGNAL,
+    SPINAXES_SIGNAL,
+    DIR_SIGNAL,
+    VISU_NB_SIGNAL
+  };
 
 /**
  * VisuObjectClass:
@@ -165,22 +144,27 @@ struct _VisuObjectClass
 {
   GObjectClass parent;
 
-  guint colorNewAvailable_signal_id;
-        
-  guint dataNew_signal_id;
-  guint dataLoaded_signal_id;
-  guint dataReadyForRendering_signal_id;
-/*   guint dataNotReadyForRendering_signal_id; */
-
-  guint renderingChanged_signal_id;
-  guint resourcesLoaded_signal_id;
-
-  guint OpenGLAskForReDraw_signal_id;
-  guint OpenGLForceReDraw_signal_id;
-
-  guint SpinAxes_signal_id;
+  guint signals[VISU_NB_SIGNAL];
 };
 
+/**
+ * VisuObjectClass:
+ *
+ * A short way to identify #_VisuObjectClass structure.
+ */
+typedef struct _VisuObjectClass VisuObjectClass;
+
+/**
+ * visuObjectRedraw:
+ * @data: some data (unused).
+ *
+ * Call the signal OpenGLAskForReDraw. The API is adapted to the routine to be added
+ * in the gloop.
+ *
+ * Returns: FALSE to stop the emission as soon as done.
+ */
+gboolean visuObjectRedraw(gpointer data);
+
 /**
  * visu_get_type:
  *
@@ -188,11 +172,13 @@ struct _VisuObjectClass
  *
  * Returns: the type of #VisuObject.
  */
-GType visu_get_type (void);
+GType visuObject_get_type (void);
 
 #define VISU_INSTANCE visuObjectGet_static()
 GObject* visuObjectGet_static();
 
+#define VISU_SIGNALS VISU_GET_CLASS(VISU_INSTANCE)->signals
+
 G_END_DECLS
 
 #endif
