@@ -141,8 +141,9 @@ static VisuPair* newPair(VisuElement *ele1, VisuElement *ele2);
 /* Callbacks. */
 static void onNewData(GObject* obj, VisuData *dataObj, gpointer data);
 static void onDataReady(GObject* obj, VisuData *dataObj, gpointer data);
-void rebuildPairs(VisuData *dataObj);
-void facetteChanged(VisuData* obj, gpointer data);
+static void onNodesReady(VisuData *dataObj, VisuNodeArray *nodes, gpointer data);
+static void rebuildPairs(VisuData *dataObj);
+static void facetteChanged(VisuData* obj, gpointer data);
 void createPairsOnNodeRenderedChanged(VisuData *dataObj, gpointer data);
 void createPairsOnElementRenderedChanged(VisuData *dataObj, VisuElement *element, gpointer data);
 void createPairsOnNodePositionChanged(VisuData *dataObj, gpointer data);
@@ -160,8 +161,7 @@ static gboolean readPairsAreOn(gchar **lines, int nbLines, int position,
 static gboolean readFavPairsMethod(gchar **lines, int nbLines, int position,
 				   VisuData *dataObj, GError **error);
 /* This function save the resources. */
-static gboolean exportResourcesPairs(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj);
+static void exportResourcesPairs(GString *data, VisuData *dataObj);
 #define FLAG_RESOURCES_PAIRS      "pairs_are_on"
 #define DESC_RESOURCES_PAIRS      "Ask the opengl engine to draw pairs between elements ; boolean 0 or 1"
 #define RESOURCES_PAIRS_DEFAULT FALSE
@@ -419,10 +419,22 @@ gboolean visuPairSet_color(VisuPairData *data, Color* destColor)
   rebuildPairsNeeded = TRUE;
   return extensionPairs->used;
 }
+Color* visuPairGet_color(VisuPairData *data)
+{
+  Color *color;
+
+  g_return_val_if_fail(data, defaultPairColor);
+
+  color = (Color*)g_hash_table_lookup(data->properties, (gpointer)"color");
+  if (!color)
+    color = defaultPairColor;
+/*   DBG_fprintf(stderr, "Visu Pairs: get color [%g;%g;%g] for %p.\n", */
+/* 	      color->rgba[0], color->rgba[1], color->rgba[2], */
+/* 	      (gpointer)data); */
+  return color;
+}
 gboolean visuPairSet_distance(VisuPairData *data, float val, int minOrMax)
 {
-  gboolean drawn;
-
   g_return_val_if_fail(data && (minOrMax == PAIRS_MIN ||
 				minOrMax == PAIRS_MAX), FALSE);
 
@@ -430,10 +442,7 @@ gboolean visuPairSet_distance(VisuPairData *data, float val, int minOrMax)
     return FALSE;
 
   data->minMax[minOrMax] = val;
-  drawn = (data->minMax[PAIRS_MIN] < data->minMax[PAIRS_MAX] &&
-	   data->minMax[PAIRS_MAX] > 0.f);
-  rebuildPairsNeeded = (drawn || (drawn != data->drawn));
-  data->drawn = drawn;
+  rebuildPairsNeeded = TRUE;
 
   return rebuildPairsNeeded && extensionPairs->used;
 }
@@ -723,6 +732,11 @@ VisuPairDistribution* visuPairGet_distanceDistribution(VisuPair *pair,
     if (iter2.node->rendered)
       dd->nNodesEle2 += 1;
 
+  if (DEBUG)
+    for (nRef = 0; nRef < dd->nValues; nRef++)
+      fprintf(stderr, " | %03d -> %6.3f, %5d\n", nRef, dd->stepValue * nRef,
+	      dd->histo[nRef]);
+
   return dd;
 }
 
@@ -767,8 +781,6 @@ gboolean visuPairBuild(VisuData *dataObj)
 
   rebuildPairsNeeded = FALSE;
   glNewList(openGlListPairsId, GL_COMPILE);
-  glPushMatrix();
-  glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
 
   if (currentPairsExtension->initOpenGl)
     {
@@ -883,7 +895,7 @@ gboolean visuPairBuild(VisuData *dataObj)
 					(xyz1[1] + xyz2[1]) / 2.,
 					(xyz1[2] + xyz2[2]) / 2.);
 			  sprintf(distStr, "%7.3f", sqrt(d2));
-			  openGLText_drawChars(distStr);
+			  openGLText_drawChars(distStr, TEXT_NORMAL);
 			}
 		    }
 		}
@@ -901,7 +913,6 @@ gboolean visuPairBuild(VisuData *dataObj)
       currentPairsExtension->stopOpenGl();
     }
       
-  glPopMatrix();
   glEndList();
 
   /* Emit the valueChanged signal of dataNode. */
@@ -910,10 +921,11 @@ gboolean visuPairBuild(VisuData *dataObj)
   return TRUE;
 }
 
-void rebuildPairs(VisuData *dataObj)
+static void rebuildPairs(VisuData *dataObj)
 {
   if (dataObj && extensionPairs->used)
     {
+      openGLText_rebuildFontList();
       /* Force to recreate pairs. */
       rebuildPairsNeeded = TRUE;
       visuPairBuild(dataObj);
@@ -977,28 +989,33 @@ static void onNewData(GObject* obj _U_, VisuData *dataObj, gpointer data _U_)
                    G_CALLBACK(createPairsOnNodePopulationChanged), (gpointer)0);
   g_signal_connect(G_OBJECT(dataObj), "NodePopulationIncrease",
                    G_CALLBACK(createPairsOnNodePopulationChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(dataObj), "NodePopulationDefined",
+		   G_CALLBACK(onNodesReady), (gpointer)0);
   g_signal_connect(G_OBJECT(dataObj), "OpenGLFacetteChanged",
 		   G_CALLBACK(facetteChanged), (gpointer)0);
 }
 
-static void onDataReady(GObject* obj _U_, VisuData *dataObj, gpointer data _U_)
+static void onNodesReady(VisuData *dataObj, VisuNodeArray *nodes, gpointer data _U_)
 {
-  if (dataObj)
+  if (nodes)
     {
       /* Create the Node property to count the number of bonds. */
       DBG_fprintf(stderr, "Visu Pairs: create the bond node property.\n");
-      visuNodeNew_pointerProperty(visuDataGet_nodeArray(dataObj), BONDNUMBER_ID,
-				  freeData, newOrCopyData, (gpointer)0);
+      visuNodeNew_pointerProperty(nodes, BONDNUMBER_ID, freeData,
+				  newOrCopyData, (gpointer)0);
       nodeDataSet_used(dataNode, dataObj, (extensionPairs->used)?1:0);
-    }
-    
+    }    
+}
+
+static void onDataReady(GObject* obj _U_, VisuData *dataObj, gpointer data _U_)
+{
   rebuildPairsNeeded = TRUE;
 
   if (extensionPairs->used)
     visuPairBuild(dataObj);
 }
 
-void facetteChanged(VisuData* obj, gpointer data _U_)
+static void facetteChanged(VisuData* obj, gpointer data _U_)
 {
   if (extensionPairs->used && currentPairsExtension->sensitiveToFacette)
     visuPairBuild(obj);
@@ -1279,15 +1296,11 @@ static void exportLevel1(gpointer key, gpointer value, gpointer user_data)
       g_string_append_printf(str->data, "    %s %4.3f %4.3f\n"
 			     "    %4.3f %4.3f %4.3f  %d  %d\n", (gchar*)key,
 			     data->minMax[PAIRS_MIN], data->minMax[PAIRS_MAX],
-			     color->rgba[PAIRS_COLOR_R], color->rgba[PAIRS_COLOR_G],
-			     color->rgba[PAIRS_COLOR_B], data->drawn,
-			     data->printLength);
-
-      *str->nbLinesWritten += 3;
+			     color->rgba[0], color->rgba[1], color->rgba[2],
+			     data->drawn, data->printLength);
     }  
 }
-static gboolean exportResourcesPairs(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj)
+static void exportResourcesPairs(GString *data, VisuData *dataObj)
 {
   struct foreachFuncExport_struct str;
   GList *pairsMeth;
@@ -1295,7 +1308,6 @@ static gboolean exportResourcesPairs(GString *data, int *nbLinesWritten,
   g_string_append_printf(data, "# %s\n", DESC_RESOURCES_PAIRS);
   g_string_append_printf(data, "%s:\n    %i\n", FLAG_RESOURCES_PAIRS,
 			 (int)extensionPairs->used);
-  *nbLinesWritten += 3;
 
   if (currentPairsExtension)
     {
@@ -1311,21 +1323,15 @@ static gboolean exportResourcesPairs(GString *data, int *nbLinesWritten,
       g_string_append_printf(data, ")\n");
       g_string_append_printf(data, "%s:\n    %s\n", FLAG_RESOURCES_FAV_PAIRS,
 			     currentPairsExtension->name);
-      *nbLinesWritten += 3;
     }
 
   g_string_append_printf(data, "# %s\n", DESC1_RESOURCES_PAIR_LINK);
   g_string_append_printf(data, "# %s\n", DESC2_RESOURCES_PAIR_LINK);
-  *nbLinesWritten = 2;
   str.data = data;
-  str.nbLinesWritten = nbLinesWritten;
   str.dataObj = dataObj;
   g_hash_table_foreach(DminDmax, (GHFunc)exportLevel1, (gpointer)&str);
 
   g_string_append_printf(data, "\n");
-  *nbLinesWritten += 1;
-
-  return TRUE;
 }
 
 
