@@ -64,28 +64,6 @@ G_BEGIN_DECLS
  */
 #define PAIRS_MAX 1
 
-/**
- * PAIRS_COLOR_R
- *
- * Flag used to define the red color of a pair. This is useful to access
- * the rgba array of the #Color object return by setPairsColor().
- */
-#define PAIRS_COLOR_R 0
-/**
- * PAIRS_COLOR_G
- *
- * Flag used to define the green color of a pair. This is useful to access
- * the rgba array of the #Color object return by setPairsColor().
- */
-#define PAIRS_COLOR_G 1
-/**
- * PAIRS_COLOR_B
- *
- * Flag used to define the blue color of a pair. This is useful to access
- * the rgba array of the #Color object return by setPairsColor().
- */
-#define PAIRS_COLOR_B 2
-
 /**
  * pairsData_struct:
  * @minMax: storage for the length bounds for drawn pairs ;
@@ -363,6 +341,7 @@ gboolean visuPairSet_distance(VisuPairData *data, float val, int minOrMax);
  * Returns: TRUE if visuPairBuild() should be called or not.
  */
 gboolean visuPairSet_color(VisuPairData *data, Color* destColor);
+Color* visuPairGet_color(VisuPairData *data);
 /**
  * visuPairSet_printLength:
  * @data: a #PairsData object ;
@@ -453,7 +432,6 @@ gboolean visuPairGet_printLength(VisuPairData *data);
  *          If something is returned it should not be freed.
  */
 gpointer visuPairGet_linkProperty(VisuPairData *data, const gchar* key);
-/* float getPairsColor(VisuElement *ele1, VisuElement *ele2, int color); */
 /**
  * visuPairGet_pair:
  * @ele1: a #VisuElement object ;
