@@ -146,13 +146,19 @@ static gboolean visuPluginLoad_byPath(gchar* path)
       if (!res)
 	plugin->getIcon = (pluginsIconFunc)0;
       g_free(func);
+      func = g_strdup_printf("%sInitGtk", name);
+      res = g_module_symbol(plugin->hook, func, (void*)&(plugin->initGtk));
+      g_free(func);
+      if (!res)
+	plugin->initGtk = (pluginsInitGtkFunc)0;
       plugin->name = g_strdup(name);
       g_free(name);
       presentPlugins = g_list_prepend(presentPlugins, (gpointer)plugin);
     }
   else
     {
-      g_warning("The plugin '%s' is not a loadable module.", path);
+      g_warning("The plugin '%s' is not a loadable module, error:\n%s.",
+		path, g_module_error());
       g_free(plugin);
       return FALSE;
     }
