@@ -70,23 +70,8 @@ G_BEGIN_DECLS
     - one method to write to a file the values of a specified
     resource. */
 
-
-/***************/
-/* Public area */
-/***************/
-
-/* This structure allows the user to store data
-   about a rendering method. */
-struct RenderingMethod_struct;
-/**
- * RenderingMethod:
- *
- * Short name to address #RenderingMethod_struct objects.
- */
-typedef struct RenderingMethod_struct RenderingMethod;
-
-#define VISU_ERROR_RENDERING renderingMethodGet_quark()
-GQuark renderingMethodGet_quark();
+#define VISU_ERROR_RENDERING visuRenderingClassGet_quark()
+GQuark visuRenderingClassGet_quark();
 /**
  * RenderingErrorFlag:
  * @RENDERING_ERROR_METHOD: Error from the rendering method.
@@ -102,9 +87,87 @@ typedef enum
     RENDERING_ERROR_FORMAT    /* Wrongness in format. */
   } RenderingErrorFlag;
 
+/**
+ * VISU_RENDERING_TYPE:
+ *
+ * return the type of #VisuRendering.
+ */
+#define VISU_RENDERING_TYPE	     (visu_rendering_get_type ())
+/**
+ * VISU_RENDERING:
+ * @obj: a #GObject to cast.
+ *
+ * Cast the given @obj into #VisuRendering type.
+ */
+#define VISU_RENDERING(obj)	     (G_TYPE_CHECK_INSTANCE_CAST(obj, VISU_RENDERING_TYPE, VisuRendering))
+/**
+ * VISU_RENDERING_CLASS:
+ * @klass: a #GClassObject to cast.
+ *
+ * Cast the given @klass into #VisuRenderingClass.
+ */
+#define VISU_RENDERING_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST(klass, VISU_RENDERING_TYPE, VisuRenderingClass))
+/**
+ * IS_VISU_RENDERING_TYPE:
+ * @obj: a #GObject to test.
+ *
+ * Test if the given @ogj is of the type of #VisuRendering object.
+ */
+#define IS_VISU_RENDERING_TYPE(obj)    (G_TYPE_CHECK_INSTANCE_TYPE(obj, VISU_RENDERING_TYPE))
+/**
+ * IS_VISU_RENDERING_CLASS:
+ * @klass: a #GClassObject to test.
+ *
+ * Test if the given @klass is of the type of #VisuRenderingClass class.
+ */
+#define IS_VISU_RENDERING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE(klass, VISU_RENDERING_TYPE))
+/**
+ * VISU_RENDERING_GET_CLASS:
+ * @obj: a #GObject to get the class of.
+ *
+ * It returns the class of the given @obj.
+ */
+#define VISU_RENDERING_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS(obj, VISU_RENDERING_TYPE, VisuRenderingClass))
 
 /**
- * renderingMethodLoadFunc:
+ * VisuRendering:
+ *
+ * A short way to identify #_VisuRendering structure.
+ */
+typedef struct _VisuRendering VisuRendering;
+/**
+ * VisuRenderingClass:
+ *
+ * A short way to identify #_VisuRenderingClass structure.
+ */
+typedef struct _VisuRenderingClass VisuRenderingClass;
+
+/**
+ * visu_rendering_get_type:
+ *
+ * This method returns the type of #VisuRendering, use VISU_RENDERING_TYPE instead.
+ *
+ * Returns: the type of #VisuRendering.
+ */
+GType visu_rendering_get_type(void);
+
+
+
+
+
+
+
+
+
+
+/***************/
+/* Public area */
+/***************/
+
+
+
+/**
+ * visuRenderingLoadFunc:
  * @data: an already allocated #VisuData ;
  * @format: a pointer on a format (can be NULL if format is to be guessed) ;
  * @nSet: an integer ;
@@ -122,8 +185,8 @@ typedef enum
  *
  * Returns: TRUE if everything goes right.
  */
-typedef gboolean (*renderingMethodLoadFunc) (VisuData *data, FileFormat *format,
-					     int nSet, GError **error);
+typedef gboolean (*visuRenderingLoadFunc) (VisuData *data, FileFormat *format,
+					   int nSet, GError **error);
 
 
 /**
@@ -141,7 +204,7 @@ typedef gboolean (*renderingMethodLoadFunc) (VisuData *data, FileFormat *format,
  * context). If the load method fails (because the format is wrong or
  * anything else), the @data argument should not be modified. If some
  * errors occur, the pointer @error will be instanciated. A
- * #RenderingMethod can have several #renderingFormatLoadFunc methods
+ * #VisuRendering can have several #renderingFormatLoadFunc methods
  * for each format they support. The @nSet argument is used to load a
  * specific set of nodes if the input format supports it. If @nSet is
  * 0, then the default set of nodes is loaded.
@@ -178,20 +241,6 @@ struct RenderingFormatLoad_struct
  */
 typedef struct RenderingFormatLoad_struct RenderingFormatLoad;
 
-/**
- * renderingMethodCompare_priority:
- * @a: a #RenderingFormatLoad ;
- * @b: another #RenderingFormatLoad.
- *
- * Compare the priority field of the structures of the two arguments.
- *
- * Returns: -1 if priority of a is higher (i.e. lower value) than the one of b.
- */
-gint renderingMethodCompare_priority(gconstpointer a, gconstpointer b);
-
-
-/* Interfaces for OpenGl functions. */
-
 /**
  * createOpenGLElementFunc:
  * @visuData: a #VisuData object ;
@@ -202,7 +251,7 @@ gint renderingMethodCompare_priority(gconstpointer a, gconstpointer b);
  * Returns: an id representing an OpenGL list in which the element
  *          has been created.
  */
-typedef int (*createOpenGLElementFunc) (VisuData *visuData, VisuElement* ele);
+typedef int (*visuRenderingCreate_elementFunc) (VisuData *visuData, VisuElement* ele);
 /**
  * createOpenGLNodeFunc:
  * @visuData: a #VisuData object ;
@@ -213,8 +262,8 @@ typedef int (*createOpenGLElementFunc) (VisuData *visuData, VisuElement* ele);
  * The @ele parameter is the #VisuElement of the given @node and the @visuData one
  * points to the #VisuData object that contains this node.
  */
-typedef void (*createOpenGLNodeFunc) (VisuData *visuData, VisuNode* node,
-				      VisuElement* ele);
+typedef void (*visuRenderingCreate_nodeFunc) (VisuData *visuData, VisuNode* node,
+					      VisuElement* ele);
 /**
  * getExtensOfNodeFunc:
  * @ele: a @VisuElement.
@@ -224,10 +273,10 @@ typedef void (*createOpenGLNodeFunc) (VisuData *visuData, VisuNode* node,
  *
  * Returns: the geometrical size of the element.
  */
-typedef float (*getExtensOfNodeFunc) (VisuElement* ele);
+typedef float (*visuRenderingGet_nodeExtendFunc) (VisuElement* ele);
 
 /**
- * renderingMethod_new:
+ * visuRendering_new:
  * @name: a string to label the method ;
  * @printName: a string to label the method that can be safely translated ;
  * @description: a string to describe the method ;
@@ -236,22 +285,19 @@ typedef float (*getExtensOfNodeFunc) (VisuElement* ele);
  *
  * Method used to create rendering methods. The @nbFileType argument is used to declare
  * the number of files that the method needs to read to render a set of data.
- * Use renderingMethodSet_fileType() method to set some #FileFormat for each kind.
+ * Use visuRenderingSet_fileType() method to set some #FileFormat for each kind.
  *
  * Returns: the newly created method of a null pointer if something goes
  * wrong.
  */
-RenderingMethod* renderingMethod_new(char* name, char* printName, char* description,
-				     int nbFileType, renderingMethodLoadFunc loadMethod);
+VisuRendering* visuRendering_new(const char* name, const char* printName,
+				 const char* description, guint nbFileType,
+				 visuRenderingLoadFunc           loadMethod,
+				 visuRenderingCreate_elementFunc createElement,
+				 visuRenderingCreate_nodeFunc    createNode,
+				 visuRenderingGet_nodeExtendFunc getExtend);
 /**
- * renderingMethod_free:
- * @method: the method to delete.
- *
- * Free all the allocated attributes of the specified method.
- */
-void renderingMethod_free(RenderingMethod* method);
-/**
- * renderingMethodSet_fileType:
+ * visuRenderingSet_fileType:
  * @method: a method ;
  * @fileTypeList: a list of #FileFormat ;
  * @fileType: an integer used as a key, must >= 0 and < method->nbFilesType ;
@@ -260,10 +306,10 @@ void renderingMethod_free(RenderingMethod* method);
  * Store a list of #FileFormat for the kind of file @fileType. The @name
  * argument is copied. but warning, the @fileTypeList GList* is not copied.
  */
-void renderingMethodSet_fileType(RenderingMethod *method, GList *fileTypeList,
-				 int fileType, gchar *name);
+void visuRenderingSet_fileType(VisuRendering *method, GList *fileTypeList,
+			       guint fileType, const gchar *name);
 /**
- * renderingMethodAdd_fileFormat:
+ * visuRenderingAdd_fileFormat:
  * @method: a method ;
  * @fmt: a #FileFormat ;
  * @fileType: an integer used as a key, must >= 0 and < method->nbFilesType.
@@ -271,21 +317,21 @@ void renderingMethodSet_fileType(RenderingMethod *method, GList *fileTypeList,
  * Add a file format descriptor to the list of already known file formats
  * of the key @fileType.
  */
-void renderingMethodAdd_fileFormat(RenderingMethod *method, FileFormat *fmt,
-				   int fileType);
+void visuRenderingAdd_fileFormat(VisuRendering *method, FileFormat *fmt,
+				 guint fileType);
 
 /**
- * renderingMethodGet_nbFileType:
+ * visuRenderingGet_nbFileType:
  * @method: a method.
  *
  * This method is used to get the number of kind of files needed to
  * render a set of data.
  *
- * Returns: how many kind of files are handled by the given RenderingMethod.
+ * Returns: how many kind of files are handled by the given VisuRendering.
  */
-int renderingMethodGet_nbFileType(RenderingMethod *method);
+guint visuRenderingGet_nbFileType(VisuRendering *method);
 /**
- * renderingMethodGet_fileType:
+ * visuRenderingGet_fileType:
  * @method: a method ;
  * @fileType: an integer used as a key, must >= 0 and < method->nbFilesType.
  *
@@ -294,10 +340,10 @@ int renderingMethodGet_nbFileType(RenderingMethod *method);
  *
  * Returns: a GList* with the #FileFormat. This GList should been considered read-only.
  */
-GList* renderingMethodGet_fileType(RenderingMethod *method, int fileType);
+GList* visuRenderingGet_fileType(VisuRendering *method, guint fileType);
 
 /**
- * renderingMethodGet_fileTypeName:
+ * visuRenderingGet_fileTypeName:
  * @method: a method ;
  * @fileType: an integer used as a key, must >= 0 and < method->nbFilesType.
  *
@@ -306,10 +352,12 @@ GList* renderingMethodGet_fileType(RenderingMethod *method, int fileType);
  *
  * Returns: a string own by V_Sim. This string should been considered read-only.
  */
-gchar* renderingMethodGet_fileTypeName(RenderingMethod *method, int fileType);
+const gchar* visuRenderingGet_fileTypeName(VisuRendering *method, guint fileType);
+const gchar* visuRenderingGet_name(VisuRendering *method, gboolean UTF8);
+const gchar* visuRenderingGet_description(VisuRendering *method);
 
 /**
- * renderingMethodGet_AllMethods:
+ * visuRenderingGet_AllMethods:
  *
  * This method gives a GList with pointers to each rendering method.
  * Warning : the returned GList is not a copy, it must not be modified,
@@ -317,10 +365,10 @@ gchar* renderingMethodGet_fileTypeName(RenderingMethod *method, int fileType);
  *
  * Returns: A GList containing all the registered rendering methods.
  */
-GList* renderingMethodGet_AllMethods();
+GList* visuRenderingClassGet_Allobjects();
 
 /**
- * renderingMethodSet_icon:
+ * visuRenderingSet_icon:
  * @method: a method ;
  * @path: a path to an image file.
  *
@@ -328,10 +376,11 @@ GList* renderingMethodGet_AllMethods();
  * The path is copied, and the given name can be freed freely after a call to
  * this method.
  */
-void renderingMethodSet_icon(RenderingMethod *method, char *path);
+void visuRenderingSet_icon(VisuRendering *method, const gchar *path);
+const gchar* visuRenderingGet_iconPath(VisuRendering *method);
 
 /**
- * renderingMethodGet_sizeOfElement:
+ * visuRenderingGet_sizeOfElement:
  * @method: a method ;
  * @ele: a #VisuElement to get the size of.
  *
@@ -340,127 +389,42 @@ void renderingMethodSet_icon(RenderingMethod *method, char *path);
  *
  * Returns: the radius of the given element.
  */
-float renderingMethodGet_sizeOfElement(RenderingMethod *method, VisuElement *ele);
+float visuRenderingGet_sizeOfElement(VisuRendering *method, VisuElement *ele);
+void visuRenderingEmit_elementSize(VisuRendering *method, float value);
 
 /**
- * setOpenGLMethods:
- * @method: a RenderingMethod ;
- * @createElement: a @createOpenGLElementFunc ;
- * @createNode: a @createOpenGLNodeFunc ;
- * @getSize: a @getExtensOfNodeFunc.
- *
- * This method is used to set the callback functions for the specified
- * rendering method. These callbacks are used by the OpenGl part of the code.
- * The @createElement parameter is a method called to create an OpenGl list
- * that draw the symbol corresponding to the element. The @createNode parameter
- * is called to position and drw the specified node. It should not create a list.
- */
-void setOpenGLMethods(RenderingMethod* method,
-		      createOpenGLElementFunc createElement,
-		      createOpenGLNodeFunc createNode,
-		      getExtensOfNodeFunc getSize);
-
-/**
- * registerRenderingMethod:
- * @method: a RenderingMethod.
- *
- * A method used by user to registered a new method.
- */
-void registerRenderingMethod(RenderingMethod *method);
-
-/**
- * setRenderingMethodInUse:
- * @method: a RenderingMethod.
+ * setVisuRenderingInUse:
+ * @method: a VisuRendering.
  *
  * Choose the method used to render the data.
  */
-void setRenderingMethodInUse(RenderingMethod* method);
+void visuRenderingClassSet_current(VisuRendering* method);
 
 /**
- * setRenderingMethodByName:
+ * setVisuRenderingByName:
  * @name: the name of a rendering method.
  *
  * Choose the method used to render the data.
  *
  * Returns: 1 if the method exists and is set, 0 otherwise.
  */
-int setRenderingMethodByName(char* name);
+gboolean visuRenderingClassSet_currentByName(const gchar* name);
 
 /**
- * getRenderingMethodInUse:
+ * getVisuRenderingInUse:
  *
  * Get the current method used to render the data.
  *
  * Returns: the name of the currently used rendering method.
  */
-RenderingMethod* getRenderingMethodInUse();
-/**
- * initRenderingMethods:
- *
- * Initialise all the variable of this part. It is called
- * by the main program at startup and should not be used by
- * users.
- *
- * Returns: true if everything goes right.
- */
-int initRenderingMethods();
+VisuRendering* visuRenderingClassGet_current();
+visuRenderingCreate_elementFunc visuRenderingClassGet_currentElementFunc();
+visuRenderingCreate_nodeFunc visuRenderingClassGet_currentNodeFunc();
+visuRenderingLoadFunc visuRenderingClassGet_currentLoadFunc();
+VisuRendering* visuRenderingClassGet_methodByName(const gchar *name);
+gboolean visuRenderingClassIs_currentByName(const gchar *name);
 
-
-
-
-/****************/
-/* Private area */
-/****************/
-
-/**
- * RenderingMethod_struct:
- * @name: a string describing the method (not NULL) ;
- * @printName: a string describing the method (not NULL) that can be safely translated ;
- * @description: a string to give a longer description to the method (can be NULL) ;
- * @icon: a path to a small 16x16 image to represent the method,
- *   if null, no image is used ;
- * @nbFilesType: the number of files required to render one set of data,
- *   for example 2 for one file with positions and one file with orientations
- *   to represent a vector field ;
- * @fileType: an array of GList* that contains #FileFormat ;
- * @fileTypeLabel: an array of string that describes each kind of file ;
- * @loadMethod: a pointer to a method used to read input data ;
- * @createElement: a pointer to a method used to create OpenGL lists for #VisuElement ;
- * @createNode: a pointer to a method used to draw a given node ;
- * @getExtensForElement: not used.
- *
- * This structure is used to describe a rendering method. Besides
- * names, representing icon... this structure stores pointers to
- * method technically used to draw elements with this method.
- */
-struct RenderingMethod_struct
-{
-  /* Some variable to describe this rendering method.
-     The attribute name is mandatory since it is
-     used to identify the method. */
-  char* name;
-  char* printName;
-  char* description;
-  /* A path to a small 16x16 image to represent the method.
-     If null, no image is used. */
-  char* icon;
-
-  /* Dealing with file type. */
-  /* Number of files required to render one set of data. */
-  int nbFilesType;
-  /* Types of files described by this rendering method. */
-  GList **fileType;
-  /* Label associated to this each kind of file. */
-  gchar **fileTypeLabel;
-
-  /* Pointer to the file functions. */
-  renderingMethodLoadFunc loadMethod;
-  
-  /* Pointers to useful openGL methods. */
-  createOpenGLElementFunc createElement;
-  createOpenGLNodeFunc createNode;
-  getExtensOfNodeFunc getExtensForElement;
-};
+gint visuRenderingFormatCompare_priority(gconstpointer a, gconstpointer b);
 
 G_END_DECLS
 
