@@ -218,6 +218,33 @@ gboolean g_file_set_contents(const gchar *fileName, const gchar *str,
 			     gsize len, GError **error);
 #endif
 
+/**
+ * visuToolsSubstitute_XML:
+ * @output: a GString to store the substitution ;
+ * @filename: the file to read the data from ;
+ * @tag: the tag to substitute ;
+ * @error: a location to store possible errors.
+ *
+ * Read @filename (must be XML file) and remove the @tag zone from
+ * it. At that place, it puts the contain of @output.
+ *
+ * Returns: TRUE if no error occured.
+ */
+gboolean visuToolsSubstitute_XML(GString *output, const gchar *filename,
+				 const gchar *tag, GError **error);
+
+typedef enum
+  {
+    unit_undefined,
+    unit_bohr,
+    unit_angstroem,
+    unit_nanometer,
+    nb_units
+  } VisuUnits;
+const gchar** visuToolsGet_unitNames();
+float visuToolsGet_unitValueInMeter(VisuUnits unit);
+VisuUnits visuToolsGet_unitFromName(const gchar *name);
+
 G_END_DECLS
 
 #endif
